#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#include <gtk/gtk.h>

#include <gdk/gdkx.h>
#include <X11/Xlib.h>

#define COMMON extern
#include "glg/main.h"

#include "atilo.h"

#include "glg/modules.h"


static char **szaFontNames;
static int nb_fonts;
typedef struct
{
	unsigned char width, height;
	char *name;
}
FONT_INFO;
static FONT_INFO *font_infos;

static GdkGC *penBlack = NULL;
static GdkGC *penForegroundBoard = NULL;


void
scan_fonts(void)
{
	int n, k;
	int sz, max_width, max_height;
	GdkFont *font;
	FONT_INFO *font_info;
	static char *hor[8] = { "A", "B", "C", "D", "E", "F", "G", "H" };
	static char *vert[8] = { "1", "2", "3", "4", "5", "6", "7", "8" };

	/* --- Get the font names --- */
#if 0
	if (!read_fonts())
#endif
	{

		szaFontNames =
			XListFonts(GDK_DISPLAY(), "*bold-r-normal-*-p*iso8859-1", 1000,
				   &nb_fonts);
		if (nb_fonts == 0)
			szaFontNames =
				XListFonts(GDK_DISPLAY(), "*bold-r-normal-*iso8859-1", 1000,
					   &nb_fonts);
		if (nb_fonts == 0)
			szaFontNames =
				XListFonts(GDK_DISPLAY(), "*-r-normal-*iso8859-1", 1000,
					   &nb_fonts);
		font_infos = malloc(sizeof(FONT_INFO) * nb_fonts);

		/* --- Compute size for each font --- */
		for (font_info = font_infos, n = 0; n < nb_fonts; n++, font_info++)
		{
			font = gdk_font_load(szaFontNames[n]);
			for (max_width = 0, k = 0; k < 8; k++)
				if ((sz = gdk_string_width(font, hor[k])) > max_width)
					max_width = sz;
			for (max_height = 0, k = 0; k < 8; k++)
				if ((sz = gdk_string_height(font, vert[k])) > max_height)
					max_height = sz;
			font_info->width = max_width;
			font_info->height = max_height;
			font_info->name = strdup(szaFontNames[n]);
		}
		free(szaFontNames);

#if 0
		/*--- Write all fonts to a file ---*/
		write_fonts();
#endif

	}

}				// scan_fonts


char *
find_max_font(const int width, const int height)
{
	FONT_INFO *font_info, *info = NULL;
	int max_width, max_height, n;

	for (max_width = max_height = 0, font_info = font_infos, n = 0;
	     n < nb_fonts; n++, font_info++)
	{
		if ((font_info->width <= width) && (font_info->height <= height) &&
		    (font_info->width > max_width) && (font_info->height > max_height))
		{
			max_width = font_info->width;
			max_height = font_info->height;
			info = font_info;
		}
	}

	if (info == NULL)
		return "*bold*-r*140*";
	else
		return info->name;

}				// find_max_font


/*
 * NewColor
 *
 * Create and allocate a GdkColor with the color 
 * specified in the parameter list.
 */
static GdkColor *
NewColor(const long red, const long green, const long blue)
{
	/* --- Get the color --- */
	GdkColor *c = (GdkColor *) g_malloc(sizeof(GdkColor));

	/* --- Fill it in. --- */
	c->red = red;
	c->green = green;
	c->blue = blue;

	gdk_color_alloc(gdk_colormap_get_system(), c);

	return (c);
}				// NewColor


/*
 * GetPen
 *
 * Get a pen using the GdkColor passed in.  The "pen"
 * (just a GdkGC) is created and returned ready for 
 * use.
 */
static GdkGC *
GetPen(GdkColor * c)
{
	GdkGC *gc;

	/* --- Create a gc --- */
	gc = gdk_gc_new(main_wnd->window);

	/* --- Set the forground to the color --- */
	gdk_gc_set_foreground(gc, c);

	/* --- Return it. --- */
	return (gc);
}				// GetPen


static GtkStyle *new_style = NULL;

void
compute_size_elements_board(void)
{
	int histlist_width;
	int vbox_width, vbox_height;
	int area_width, area_height;
	GtkStyle *style;
	char *font_name;

	/* --- Size of the complete area --- */
	histlist_width = hist_list->allocation.width;
	vbox_width = main_hbox->allocation.width;
	vbox_height = main_hbox->allocation.height;
	area_width = vbox_width - histlist_width;
	area_height = vbox_height;
	if (area_width < area_height)
		board_width = board_height = area_width;
	else
		board_width = board_height = area_height;

	board_height -= 2 * BOARD_HEIGHT_FENCE;
	board_width -= 2 * BOARD_WIDTH_FENCE;

	board_height = board_height * 8 / 9;
	board_width = board_width * 8 / 9;

	board_height -= (board_height % 8);
	board_width -= (board_width % 8);

	cell_width = board_width / 8;
	cell_height = board_height / 8;

	board_xstart = (area_width - board_width) / 2;
	board_ystart = (area_height - board_height) / 2;

	font_name = find_max_font(cell_width * 3 / 5, cell_height * 3 / 5);
	style = gtk_widget_get_style(board_drawing_area);
	new_style = gtk_style_copy(style);
	if ((new_style->font = gdk_font_load(font_name)) == NULL)
		new_style->font = style->font;

}				// compute_size_elements_board


void
draw_pawn(GdkDrawable * drawable, const int r, const int c, const int pawn)
{
	int cx, cy, x, y;

	cx = cell_width - 2 * CELL_WIDTH_FENCE;
	cy = cell_height - 2 * CELL_HEIGHT_FENCE;
	x = board_xstart + (c - 1) * cell_width + CELL_WIDTH_FENCE;
	y = board_ystart + (r - 1) * cell_height + CELL_HEIGHT_FENCE;

	gdk_draw_arc(drawable,
		     (pawn == CODE_EMPTY) ? penForegroundBoard :
		     ((pawn == CODE_BLACK) ? penBlackCell : penWhiteCell),
		     TRUE, x, y, cx, cy, 0, 360 * 64);

}				// draw_pawn


void
redraw_board(void)
{
	int x, y;
	GdkDrawable *drawable = board_drawing_area->window;
	static int first = TRUE;
	char str[2];
	int px, py;
	int w, h;
	char *font_name;
	GdkFont *font;

	if (first)
	{
		first = FALSE;
		return;
	}

	/*--- Draw each pawn ---*/
	for (x = 1; x <= 8; x++)
		for (y = 1; y <= 8; y++)
			draw_pawn(drawable, x, y, board[XY(x, y)]);

	/*--- Draw horizontal A-H ---*/
	font_name = find_max_font(cell_width * 3 / 5, cell_height * 3 / 5);
	font = gdk_font_load(font_name);
	for (x = 1; x <= 8; x++)
	{
		str[0] = '@' + x;
		str[1] = 0;
		w = gdk_string_width(font, str);
		h = gdk_string_height(font, str);
		px = board_xstart + (x - 1) * cell_width;
		py = board_ystart + (0 - 1) * cell_height;
		px += (cell_width - w) / 2;
		py += (cell_height - h - font->descent) / 2;
		py += new_style->font->ascent;
		gdk_draw_text(drawable, new_style->font, penBlackCell, px, py, str, 1);
	}

	/*--- Draw vertical 1-8 ---*/
	for (y = 1; y <= 8; y++)
	{
		str[0] = '0' + y;
		str[1] = 0;
		w = gdk_string_width(font, str);
		h = gdk_string_height(font, str);
		px = board_xstart + (0 - 1) * cell_width;
		py = board_ystart + (y - 1) * cell_height;
		px += (cell_width - w) / 2;
		py += (cell_height - h - font->descent) / 2;
		py += new_style->font->ascent;
		gdk_draw_text(drawable, new_style->font, penBlackCell, px, py, str, 1);
	}

	gdk_flush();

}				// redraw_board


void
draw_board(GdkDrawable * drawable)
{
	int n, x, y;

	/* --- Draw green board --- */
	gdk_draw_rectangle(drawable,
			   penForegroundBoard,
			   TRUE, board_xstart, board_ystart, board_width, board_height);

	/* --- Draw lines to form cells --- */
	for (x = board_xstart, y = board_ystart, n = 0;
	     n <= 8; n++, x += cell_width, y += cell_height)
	{
		gdk_draw_line(drawable,
			      penBlack, x, board_ystart, x, board_ystart + board_height);
		gdk_draw_line(drawable,
			      penBlack, board_xstart, y, board_xstart + board_width, y);
	}


	redraw_board();

}				// draw_board


static void
expose_init(void)
{

	penBlack = GetPen(NewColor(0, 0, 0));
	penForegroundBoard = GetPen(NewColor(0, 0xE000, 0));
	penWhiteCell = GetPen(NewColor(0xFF00, 0xFF00, 0xFF00));
	penBlackCell = GetPen(NewColor(0xFF, 0xFF, 0xFF));

}				// expose_init


/* Callback "button_press_event" for GtkDrawingArea widget wgt_board_area */
gboolean
cb_main_wnd_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{

	printf( "cb_main_wnd_press...\n" );
	return FALSE;
	
}								// cb_main_wnd_press


/* Callback "button_release_event" for GtkDrawingArea widget wgt_board_area */
gboolean
cb_main_wnd_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{

	printf( "cb_main_wnd_release...\n" );
	return FALSE;
	
}								// cb_main_wnd_release


/* Callback "expose_event" for GtkDrawingArea widget wgt_board_area */
gboolean
cb_main_wnd_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	static int first = TRUE;

	if (first)
	{
		expose_init();
		first = FALSE;
	}

	compute_size_elements_board();
	draw_board( widget->window );

	return FALSE;
}								// cb_main_wnd_expose


/* Callback "configure_event" for GtkDrawingArea widget wgt_board_area */
gboolean
cb_main_wnd_resized(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{

	printf( "cb_main_wnd_resized..." );
	return FALSE;
	
}								// cb_main_wnd_resized

/* Callback "motion_notify_event" for GtkDrawingArea widget wgt_board_area */
gboolean
cb_main_wnd_motion(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{

	printf( "cb_main_wnd_motion..." );
	return FALSE;
	
}								// cb_main_wnd_motion
