#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <inttypes.h>
#include <unistd.h>

#include <gnome.h>

#include "glg/main.h"
#include "glg/modules.h"


static void
about_destroy_cb(GtkObject * obj, GtkWidget ** about)
{
	*about = NULL;
}								// about_destroy_cb


void
about_dialog( void )
{
	static GtkWidget *about = NULL;
	static const gchar *authors[] = {
		"Olivier Singla <lo_gafet@users.sourceforget.net>",
		NULL
	};
	static gchar version[80];
//	#include "graphics/atilo_about.xpm"

	if ( !about )
	{
		g_snprintf( version, sizeof(version),
			"version %d.%d.%c %s",
			(VERSION >> 16), (VERSION >> 8) & 0xFF,
			(VERSION & 0xFF) + 'a', __DATE__ );

		about = gnome_about_new( _("Atilo, Reversi game"), version,
			"Free Software", authors,
			_("Released under the terms of GNU Public License"),
			NULL);
		gtk_signal_connect(GTK_OBJECT(about), "destroy",
			GTK_SIGNAL_FUNC(about_destroy_cb), &about);
		gtk_widget_show(about);
	}
	else
	{
		gdk_window_raise(GTK_WIDGET(about)->window);
	}

}								// about_dialog


//* Callback "expose_event" for GtkDrawingArea widget board_drawing_area */
gboolean
main_wnd_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	static int first = TRUE;

	if ( first )
	{
		expose_init();
		first = FALSE;
	}

	compute_size_elements_board();
	draw_board(widget->window);

	return FALSE;
	
}								// main_wnd_expose


/* Callback "configure_event" for GtkDrawingArea widget board_drawing_area */
gboolean
main_wnd_resized(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
#if 0
	printf("new size = %d , %d\n",
	       main_wnd->allocation.width, main_wnd->allocation.height);
#endif
	return TRUE;
}								// main_wnd_resized


void
clear_hist_list(void)
{
	gtk_clist_clear( GTK_CLIST( hist_list) );
}								// clear_hist_list


void
init_new_game(const int f_draw, const int f_reset_history)
{
	int k, x, y;

	/*--- By default, computer has black, play has white ---*/
	code_computer = CODE_WHITE;
	code_human = CODE_BLACK;
	color_to_play = CODE_BLACK;	// Next color to play

	/*--- Initialize the board ---*/
	for (k = 0; k < 9; k++)
		board[XY(0, k)] =
			board[XY(k, 9)] = board[XY(9, 9 - k)] = board[XY(9 - k, 0)] =
			CODE_AROUND;
	for (x = 1; x <= 8; x++)
		for (y = 1; y <= 8; y++)
			board[XY(x, y)] = CODE_EMPTY;
	board[XY(4, 4)] = CODE_WHITE;
	board[XY(4, 5)] = CODE_BLACK;
	board[XY(5, 4)] = CODE_BLACK;
	board[XY(5, 5)] = CODE_WHITE;

	/*--- Cumul time played by each color ---*/
	if (f_reset_history)
	{
		time(&t_start);	// Time when player starts thinking
		ctime_black = ctime_white = 0;
		show_horo();
	}

	/*--- Draw the complete board ---*/
	if (f_draw)
		if (board_drawing_area->window)
			redraw_board();
	compute_nb_pawns(f_draw);

	/*--- Show the possibilities of the player ---*/
	if (f_draw)
		show_poss_play_cursor(code_human, code_computer);

	/*--- Reset history of the game ---*/
	if (f_reset_history)
		clear_hist_list();

	/*--- Reset msg ---*/
	computer_infos_msg("");
	player_infos_msg("Choose a position and click to play.");

	/*--- Disable the menu option 'Save the game' ---*/
	enable_SaveGame(FALSE);

}				// init_new_game

/* Callback "select_row" for GtkCList widget hist_list */
void
hist_list_selected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gchar *text;
	unsigned char player, enemy;
	GList *list;
	int cnt;
	int x, y;
	PCELL ptr1, ptr2, p;
	CELL possib[256];
	int nb_poss, nb_took;
	int k;
	int valid_move;
	int pass;

	gtk_clist_get_text(GTK_CLIST(clist), row, column, &text);

	/*--- Reset the game ---*/
	init_new_game(FALSE, FALSE);

	/*--- Bring the game to this point ---*/
	player = CODE_BLACK;
	enemy = CODE_WHITE;
	list = GTK_CLIST(hist_list)->selection;
	for (cnt = 0; cnt <= row; cnt++)
	{

		/*--- Get the position ---*/
		gtk_clist_get_text(GTK_CLIST(hist_list), cnt, 0, &text);
		y = text[11] - '@';
		x = text[12] - '0';

		/*--- Generate all the possible move ---*/
		ptr1 = ptr2 = &possib[0];
		if ((nb_poss = do_build_possib(&ptr2, player, enemy)) == 0)
		{
			player = (CODE_BLACK + CODE_WHITE) - player;
			enemy = (CODE_BLACK + CODE_WHITE) - enemy;
			ptr1 = ptr2 = &possib[0];
			if ((nb_poss = do_build_possib(&ptr2, player, enemy)) == 0)
				return;
		}		// if

		/*--- Is-it a valid move ? ---*/
		for (valid_move = FALSE, p = ptr1, p++; p != ptr2;)
		{
			if (*p++ == &board[XY(x, y)])
			{
				valid_move = TRUE;
				break;
			}
			nb_took = (int) *p++;
			p += nb_took;
		}		// for (p)
		if (!valid_move)
			return;

		/*--- Play this new move ---*/
		board[XY(x, y)] = player;
		nb_took = (int) *p++;
		for (k = 0; k < nb_took; k++)
			*(unsigned char *) *p++ = player;

		/*--- Next player ---*/
		player = (CODE_BLACK + CODE_WHITE) - player;
		enemy = (CODE_BLACK + CODE_WHITE) - enemy;

	}			// for (cnt)

	/*--- Refresh the display ---*/
	redraw_board();
	compute_nb_pawns(TRUE);

	/*--- Which player next, if any ? ---*/
	pass = FALSE;
	ptr1 = ptr2 = &possib[0];
	if ((nb_poss = do_build_possib(&ptr2, player, enemy)) == 0)
	{
		ptr2 = &possib[0];
		if ((nb_poss = do_build_possib(&ptr2, enemy, player)) == 0)
			return;
		pass = TRUE;
		if (player == code_human)
			player_infos_msg("Sorry, no possibility left, you pass next turn!");
		else
			computer_infos_msg("No possibility left, computer pass next turn!");
		player = (CODE_BLACK + CODE_WHITE) - player;
	}			// if

	/*--- Next player ---*/
	if (player == code_computer)
	{
		nb_poss = show_poss_play_cursor(code_computer, code_human);
		computer_infos_msg
			("%d possibilit%s. Press <SPACE> to play Computer", nb_poss,
			 (nb_poss == 1) ? "y" : "ies");
		install_key_press_handler(TRUE);
		if (!pass)
			player_infos_msg("");
	}
	else
	{
		nb_poss = show_poss_play_cursor(code_human, code_computer);
		player_infos_msg
			("%d possibilit%s. Choose a position and click to play",
			 nb_poss, (nb_poss == 1) ? "y" : "ies");
		install_key_press_handler(FALSE);
		if (!pass)
			computer_infos_msg("");
	}
	color_to_play = player;

}								// hist_list_selected
