
#define VERSION	0x000900

/* ---  Generate a pseudo-random number between [a..b] --- */
#define RANDOM( a, b ) \
	a + (rand() % (b - a + 1))

/*--- Board pawns codes ---*/
#define	CODE_AROUND			-1
#define	CODE_EMPTY			0
#define	CODE_BLACK			1
#define	CODE_WHITE			2

/*--- Convert a board coordonnate into a linar position ---*/
#define XY( x, y )	((x)*10+(y))

/*--- Show a move name in a form A1-H8 ---*/
#define SHOW_COORD_Y( p )	((((UINT)p - (UINT)board) % 10) + '@')

/*--- Board position and size ---*/
#define BOARD_HEIGHT_FENCE 10
#define BOARD_WIDTH_FENCE 10
#define CELL_HEIGHT_FENCE 5
#define CELL_WIDTH_FENCE 5
#define BOARD_RC( r, c ) \
	BOARD_C0 + BOARDCELL_W*(c-1), \
	BOARD_R0 + BOARDCELL_H*(r-1)
#define PAWN_RC( r, c ) \
	BOARD_C0 + (BOARDCELL_W - PAWN_W)/2 + BOARDCELL_W*(c-1), \
	BOARD_R0 + (BOARDCELL_H - PAWN_H)/2 + BOARDCELL_H*(r-1)

/*--- ---*/
typedef unsigned char *CELL;
typedef CELL *PCELL;

/*--- Position to play ---*/
typedef struct
{
	int		x, y;						// Position to play, set to (-1, -1) if pass
	int		coef_best;					// Best coef. 
	long	cnt;						// Nb of positions examined
	int		max_level;					// Max level reached
	time_t	elapsed;					// Time set for calculation
}	WHERE_TO_PLAY;

/*--- Used to sort scores into the minimax function ---*/
typedef struct
{
	int		score;
	int		index;
}	SCORE_TO_SORT;

/*--- Config file ---*/
enum { LEVEL1=1, LEVEL2, LEVEL3, LEVEL4, MAX_LEVELS };
enum { STRATEGY1=1, STRATEGY2, STRATEGY3, STRATEGY4, STRATEGY5, STRATEGY6, MAX_STRATEGIES };
typedef struct
{
	gint	level;
	gint	strategy;
	gint	mainwnd_w, mainwnd_h;
}	CONFIG;
COMMON CONFIG	config;

/*--- The game ---*/
COMMON unsigned char	code_computer;			// Either CODE_BLACK or CODE_WHITE
COMMON unsigned char	code_human;				// Either CODE_BLACK or CODE_WHITE
COMMON unsigned char	color_to_play;			// Next color to play
COMMON unsigned char	board[ (8+2) * (8+2) ];
COMMON int		nb_pawns[ 3 ];					// CODE_BLACK+1 = 3
COMMON CELL		cancel_last_human_move[ 40 ];	// To cancel the last human move
COMMON int		ctime_black, ctime_white;		// Cumul time played by each color
COMMON time_t	t_start;						// Time when player starts thinking

/*--- List (stack) of possibilities *and* enclosed pawns ---*/
COMMON CELL	possib[ 4096 ];

COMMON	GtkWidget	*main_wnd;
COMMON	GtkWidget	*main_hbox;

COMMON	GdkGC		*penWhiteCell;
COMMON	GdkGC		*penBlackCell;

/*--- If this variable is TRUE, show the possibilities of the user ---*/
/*--- Meaning:	0 means no show, else the color to show ---*/
COMMON	int		which_show_poss;
COMMON	char	state_selectable[ 8 ][ 8 ];

/* --- Used by minimax --- */
COMMON	int	max_depth;
COMMON	int	cnt;
COMMON	int	chosen_move;

/*--- Current ponderation function to execute ---*/
COMMON	inline int	_ponderation1( const int the_player, const int the_enemy );
COMMON	inline int	_ponderation2( const int the_player, const int the_enemy );
COMMON	inline int	_ponderation3( const int the_player, const int the_enemy );
COMMON	inline int	_ponderation4( const int the_player, const int the_enemy );

COMMON	int board_width, board_height;
COMMON	int board_xstart, board_ystart;
COMMON	int cell_width, cell_height;

/*--- Public functions in: atilo.c --- */
extern char *find_max_font( const int width, const int height );
extern void init_new_game( const int f_draw, const int f_reset_history );

/*--- Public functions in: util.c --- */
extern long filelength( FILE *fp );
void warning_msg (gchar * message);


/*--- Public functions in; menu.c ---*/
extern void enable_NewGame( const int flag );
extern void enable_SaveGame( const int flag );
extern void enable_Undo( const int flag );
extern void enable_ChangeSide( const int flag );
extern void enable_SuggestMove( const int flag );
extern void create_main_menu( GtkWidget *vbox );

/*--- Public functions in: layout.c --- */
extern void install_key_press_handler( const int enable );
extern void install_motion_handler( const int enable );
extern void expose_init( void );
extern void compute_size_elements_board( void );
extern void redraw_board( void );
extern void draw_pawn( GdkDrawable *drawable, const int r, const int c, const int pawn );
extern void draw_complete_board;
extern void draw_board( GdkDrawable *drawable );
extern void create_layout( void );
extern void computer_infos_msg( const char *fmt, ... );
extern void player_infos_msg( const char *fmt, ... );

/*--- Public functions in: status.c --- */
extern void set_progressbar( const int val, const int max_val );
extern void set_owners( void );
extern void show_horo( void );

/*--- Public functions in: loadsave.c --- */
extern void file_load_game( void );
extern void file_save_game( void );

/*--- Public functions in: play.c --- */
extern inline char *lib_colors( const int n );
extern void compute_nb_pawns( const int show );
extern inline int do_build_possib( PCELL *ptr, const int cplayer, const int cenemy );
extern inline int do_nb_possib( const int cplayer, const int cenemy );
extern int show_poss_play_cursor( const unsigned char cplayer, const unsigned char cenemy );
extern gint main_wnd_motion( GtkWidget * signal_widget, GdkEventMotion * event, gpointer data);
extern void computer_turn( void );
extern void computer_play( WHERE_TO_PLAY *where_to_play,
	const unsigned char the_player, const unsigned char the_enemy,
	const int f_suggest );
extern void suggest_move( void );
extern void change_side( void );

/*--- Public functions in: game.c --- */
extern int max_level_depth();
extern void do_game( const unsigned char the_player, const unsigned char the_enemy,
	WHERE_TO_PLAY *where_to_play );
extern void show_nb_pawns( void );

/*--- Public functions in: endgame.c --- */
extern void do_end_game( const unsigned char the_player,
	const unsigned char the_enemy,
	WHERE_TO_PLAY *where_to_play );

/*--- Public functions in: settings.c --- */
extern void settings_set_level( void );
extern void settings_set_strategy( void );
extern void dialog_settings( void );
extern void init_config( void );

/*--- Public functions in: config.c --- */
extern char *find_max_font( const int width, const int height );
extern void scan_fonts( void );
extern int load_config( void );
extern void save_config( void );

/*--- Public functions in: about.c ---*/
extern void about_dialog( void );
