#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <inttypes.h>
#include <unistd.h>

#include <gnome.h>

#include "glg/main.h"
#include "glg/modules.h"


static GdkGC *penBlack = NULL;
static GdkGC *penForegroundBoard = NULL;


/* Callback "delete_event" for GtkWindow widget main_wnd */
gboolean
close_app(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	save_config();
	gtk_main_quit();
	return (TRUE);
}								// close_app


/* Callback "destroy" for GtkWindow widget main_wnd */
void
destroy_app(GtkObject *object, gpointer user_data)
{
	save_config();
	gtk_main_quit();
}								// destroy_app


static void
main_wnd_key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	GList *list;
	int hist_sel, hist_nb;

	if (event->keyval != ' ')
		return;

	install_key_press_handler(FALSE);

	/*--- Update the list of move ---*/
	list = GTK_CLIST(hist_list)->selection;
	hist_sel = (list) ? (int) list->data : -1;
	hist_nb = GTK_CLIST(hist_list)->rows;
	if (hist_sel + 1 != hist_nb)
	{
		int cnt;
		// Remove all elements after the one selected in the list
		for (cnt = hist_sel + 1; cnt < hist_nb; cnt++)
			gtk_clist_remove(GTK_CLIST(hist_list), hist_sel + 1);
	}

	computer_turn();

}				// main_wnd_key_press


void
install_key_press_handler(const int enable)
{
	static guint id = 0;

	/* --- We care if a key is pressed --- */
	if (enable)
	{
		if (id == 0)
			id = gtk_signal_connect(GTK_OBJECT(main_wnd), "key_press_event",
						GTK_SIGNAL_FUNC(main_wnd_key_press), NULL);
	}
	else
	{
		if (id > 0)
		{
			gtk_signal_disconnect(GTK_OBJECT(main_wnd), id);
			id = 0;
		}
	}

}				// install_key_press_handler


void
install_motion_handler(const int enable)
{
	static guint id = 0;

	/* --- We care if a key is pressed --- */
	if (enable)
	{
		if (id == 0)
			id =
				gtk_signal_connect(GTK_OBJECT(board_drawing_area),
						   "motion_notify_event",
						   (GtkSignalFunc) main_wnd_motion, NULL);
	}
	else
	{
		if (id > 0)
		{
			gtk_signal_disconnect(GTK_OBJECT(board_drawing_area), id);
			id = 0;
		}
	}
}				// install_motion_handler


/*
 * NewColor
 *
 * Create and allocate a GdkColor with the color 
 * specified in the parameter list.
 */
static GdkColor *
NewColor(const long red, const long green, const long blue)
{
	/* --- Get the color --- */
	GdkColor *c = (GdkColor *) g_malloc(sizeof(GdkColor));

	/* --- Fill it in. --- */
	c->red = red;
	c->green = green;
	c->blue = blue;

	gdk_color_alloc(gdk_colormap_get_system(), c);

	return (c);
}				// NewColor


/*
 * GetPen
 *
 * Get a pen using the GdkColor passed in.  The "pen"
 * (just a GdkGC) is created and returned ready for 
 * use.
 */
static GdkGC *
GetPen(GdkColor * c)
{
	GdkGC *gc;

	/* --- Create a gc --- */
	gc = gdk_gc_new(main_wnd->window);

	/* --- Set the forground to the color --- */
	gdk_gc_set_foreground(gc, c);

	/* --- Return it. --- */
	return (gc);
}				// GetPen


void
expose_init(void)
{

	penBlack = GetPen(NewColor(0, 0, 0));
	penForegroundBoard = GetPen(NewColor(0, 0xE000, 0));
	penWhiteCell = GetPen(NewColor(0xFF00, 0xFF00, 0xFF00));
	penBlackCell = GetPen(NewColor(0xFF, 0xFF, 0xFF));

	gtk_widget_queue_draw( black_pawn );
	gtk_widget_queue_draw( white_pawn );

}								// expose_init


static GtkStyle *new_style = NULL;

void
compute_size_elements_board(void)
{
	int			area_width, area_height;
	GtkStyle	*style;
	char		*font_name;

	/* --- Size of the complete area --- */
	area_width  = board_drawing_area->allocation.width;
	area_height = board_drawing_area->allocation.height;
	if ( area_width < area_height )
		board_width = board_height = area_width;
	else
		board_width = board_height = area_height;

	board_height -= 2 * BOARD_HEIGHT_FENCE;
	board_width  -= 2 * BOARD_WIDTH_FENCE;

	board_height = board_height * 8 / 9;
	board_width  = board_width * 8 / 9;

	board_height -= (board_height % 8);
	board_width  -= (board_width % 8);

	cell_width  = board_width / 8;
	cell_height = board_height / 8;

	board_xstart = (area_width - board_width) / 2;
	board_ystart = (area_height - board_height) / 2;

	font_name = find_max_font(cell_width * 3 / 5, cell_height * 3 / 5);
	style = gtk_widget_get_style(board_drawing_area);
	new_style = gtk_style_copy(style);
	if ((new_style->font = gdk_font_load(font_name)) == NULL)
		new_style->font = style->font;

}								// compute_size_elements_board


void
draw_pawn(GdkDrawable * drawable, const int r, const int c, const int pawn)
{
	int cx, cy, x, y;

	cx = cell_width - 2 * CELL_WIDTH_FENCE;
	cy = cell_height - 2 * CELL_HEIGHT_FENCE;
	x = board_xstart + (c - 1) * cell_width + CELL_WIDTH_FENCE;
	y = board_ystart + (r - 1) * cell_height + CELL_HEIGHT_FENCE;

	gdk_draw_arc(drawable,
		     (pawn == CODE_EMPTY) ? penForegroundBoard :
		     ((pawn == CODE_BLACK) ? penBlackCell : penWhiteCell),
		     TRUE, x, y, cx, cy, 0, 360 * 64);

}				// draw_pawn


void
redraw_board( void )
{
	int x, y;
	GdkDrawable *drawable = board_drawing_area->window;
	static int first = TRUE;
	char str[2];
	int px, py;
	int w, h;
	char *font_name;
	GdkFont *font;

	if (first)
	{
		first = FALSE;
		return;
	}

	/*--- Draw each pawn ---*/
	for (x = 1; x <= 8; x++)
		for (y = 1; y <= 8; y++)
			draw_pawn(drawable, x, y, board[XY(x, y)]);

	/*--- Draw horizontal A-H ---*/
	font_name = find_max_font(cell_width * 3 / 5, cell_height * 3 / 5);
	font = gdk_font_load(font_name);
	for (x = 1; x <= 8; x++)
	{
		str[0] = '@' + x;
		str[1] = 0;
		w = gdk_string_width(font, str);
		h = gdk_string_height(font, str);
		px = board_xstart + (x - 1) * cell_width;
		py = board_ystart + (0 - 1) * cell_height;
		px += (cell_width - w) / 2;
		py += (cell_height - h - font->descent) / 2;
		py += new_style->font->ascent;
		gdk_draw_text(drawable, new_style->font, penBlackCell, px, py, str, 1);
	}

	/*--- Draw vertical 1-8 ---*/
	for (y = 1; y <= 8; y++)
	{
		str[0] = '0' + y;
		str[1] = 0;
		w = gdk_string_width(font, str);
		h = gdk_string_height(font, str);
		px = board_xstart + (0 - 1) * cell_width;
		py = board_ystart + (y - 1) * cell_height;
		px += (cell_width - w) / 2;
		py += (cell_height - h - font->descent) / 2;
		py += new_style->font->ascent;
		gdk_draw_text(drawable, new_style->font, penBlackCell, px, py, str, 1);
	}

	gdk_flush();

}								// redraw_board


void
draw_board(GdkDrawable * drawable)
{
	int n, x, y;

	/* --- Draw green board --- */
	gdk_draw_rectangle( drawable,
		penForegroundBoard,
		TRUE, board_xstart, board_ystart, board_width, board_height );

	/* --- Draw lines to form cells --- */
	for (x = board_xstart, y = board_ystart, n = 0;
	     n <= 8; n++, x += cell_width, y += cell_height)
	{
		gdk_draw_line(drawable,
			      penBlack, x, board_ystart, x, board_ystart + board_height);
		gdk_draw_line(drawable,
			      penBlack, board_xstart, y, board_xstart + board_width, y);
	}


	redraw_board();

}								// draw_board


void
computer_infos_msg(const char *fmt, ...)
{
	va_list		ap;
	char		msg[200];

	va_start( ap, fmt );
	vsprintf( msg, fmt, ap );
	va_end( ap );

	gtk_label_set_text( GTK_LABEL( computer_infos_label ), msg );

}								// computer_infos_msg


void
player_infos_msg(const char *fmt, ...)
{
	va_list		ap;
	char		msg[200];

	va_start( ap, fmt );
	vsprintf( msg, fmt, ap );
	va_end( ap );

	gtk_label_set_text( GTK_LABEL( player_infos_label ), msg );

}								// player_infos_msg
