#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include <assert.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"

#include <gdk/gdkkeysyms.h>

#include "graphics/cannon.0.xpm"
#include "graphics/cannon.1.xpm"
#include "graphics/cannon.2.xpm"
#include "graphics/cannon.unarmed.xpm"


/*--- -------------------------------------------------------------------- ---*/
/*--- Variables used only by start_new_cannon() and move_starting_cannon() ---*/
/*--- -------------------------------------------------------------------- ---*/

typedef enum
{
	STARTING_CANNON_PHASE_BLINKING,
	STARTING_CANNON_PHASE_BLASTING
}	starting_cannon_phase_t;

static	now_t					starting_cannon_when;
static	starting_cannon_phase_t	starting_cannon_phase;
static	int						starting_cannon_blinking_cnt;
static	double					yinc, ystep;


PUBLIC void
install_starting_cannon( starting_cannon_t *starting_cannon, int index )
{

	starting_cannon->x = (frame_start_width - cannon.w) / 2;	
	starting_cannon->y = frame_start_height - ((index + 1) * (cannon.h + 12));
	
	starting_cannon->is_visible = TRUE;

}								// install_starting_cannon

		
PUBLIC void 
start_new_cannon( void )
{

	/*--- Which cannon to start ? ---*/
	nb_cannons_left--;
	is_starting_cannon = TRUE;

	starting_cannon_when = now();
	starting_cannon_phase = STARTING_CANNON_PHASE_BLINKING;
	starting_cannon_blinking_cnt = 0;

}								// start_new_cannon


static void
blink_starting_cannon( void )
{
	double				diff;
	starting_cannon_t	*starting_cannon;

	diff = now() - starting_cannon_when;
	if ( diff < MSEC( 75 ) )
		return;

	starting_cannon = &starting_cannons[ nb_cannons_left ];
	
	starting_cannon_when = now();
	if ( starting_cannon->is_visible )
	{
		starting_cannon->is_visible = FALSE;
	}
	else
	{
		starting_cannon->is_visible = TRUE;
		if ( ++starting_cannon_blinking_cnt == 6 )
		{
			starting_cannon_phase = STARTING_CANNON_PHASE_BLASTING;
			yinc  = 0.95;
			ystep = .175;
		}
	}

}								// blink_starting_cannon

static void
blast_starting_cannon( void )
{
	double				diff;
	starting_cannon_t	*starting_cannon;

	diff = now() - starting_cannon_when;
	if ( diff < 100 )
		return;

	starting_cannon = &starting_cannons[ nb_cannons_left ];
	
	starting_cannon->y -= yinc;
	yinc += ystep;

	if ( starting_cannon->y <= -cannon.h )
	{
		is_starting_cannon = FALSE;
		starting_cannon->is_visible = FALSE;
	}

}								// blast_starting_cannon


PUBLIC void
move_starting_cannon( void )
{

	if ( starting_cannon_phase == STARTING_CANNON_PHASE_BLINKING )
		blink_starting_cannon();
	else if ( starting_cannon_phase == STARTING_CANNON_PHASE_BLASTING )
		blast_starting_cannon();

}								// move_starting_cannon


PUBLIC void
draw_starting_cannon( void )
{
	starting_cannon_t	*starting_cannon;
	int					index;

	/*--- Erase everything into the frame buffer ---*/
	gdk_draw_rectangle( pixmap_start_frame, 
		BACKGROUND_GC,
		TRUE,
		0, 0, frame_start_width, frame_start_height );

	for ( starting_cannon = starting_cannons, index = 0; 
		  index < NB_STARTING_CANNONS; 
		  index++, starting_cannon++ )
	{
		if ( starting_cannon->is_visible )
		{
			gdk_draw_pixmap( pixmap_start_frame, 
				BACKGROUND_GC,
				cannon.img_des[ CANNON_ARMED_0 ],
				0, 0, starting_cannon->x, starting_cannon->y,
				cannon.w, cannon.h );
		}						// if
	}							// for (n, starting_cannon)

	gdk_draw_pixmap( wgt_start->window, 
		BACKGROUND_GC,
		pixmap_start_frame,
		0, 0, 0, 0,
		frame_start_width, frame_start_height );

}								// draw_starting_cannon


PUBLIC void
init_keys_state( void )
{
	keys_state.left		= FALSE;
	keys_state.right	= FALSE;
	keys_state.shoot	= FALSE;
	keys_state.escape	= FALSE;
	keys_state.exit		= FALSE;
}								// init_keys_state


PUBLIC void
place_cannon( void )
{

	cannon.state = CANNON_ARMED_0;
	cannon.can_shoot = TRUE;
	cannon.dir = 1;
	cannon.x = (frame_width - cannon.w) >> 1;
	cannon.y = (frame_height - cannon.h);
	cannon.is_hit = FALSE;

}								// place_cannon


PUBLIC void
load_cannon( void )
{
	GdkBitmap	*mask;

	/*--- Initializations ---*/
	memset( &cannon, 0, sizeof( cannon_t ) );

#define LOAD( IMG, XPM ) \
	IMG = gdk_pixmap_create_from_xpm_d( wgt_spacearea->window, \
		&mask, &BACKGROUND, XPM ); \
	assert( IMG != NULL )

	LOAD( cannon.img_des[ CANNON_ARMED_0 ], cannon_0_xpm );
	LOAD( cannon.img_des[ CANNON_ARMED_1 ], cannon_1_xpm );
	LOAD( cannon.img_des[ CANNON_ARMED_2 ], cannon_2_xpm );
	LOAD( cannon.img_des[ CANNON_UNARMED ], cannon_unarmed_xpm );

#undef LOAD

	gdk_window_get_size( (GdkWindow *)cannon.img_des[ CANNON_ARMED_0 ],
		&cannon.w, &cannon.h );

	/*--- How many pixels by frame ---*/
	cannon.xinc = CANNON_XSPEED;

}								// load_cannon


static void
make_hit_cannon_bitmap( void )
{
		int			n, nb, sz;
		double		elapsed;
static	GdkPoint	*points = NULL;
static	int			maxnb = 0;
		GdkPoint	*p;

	/*--- Compute how many percentage of the ship to destroy ---*/
	sz = cannon.w * cannon.h;
	elapsed = _now - cannon.hit_start;

	/*--- Ship is dead ? ---*/
	if ( elapsed >= CANNON_HIT_LONG )
	{
		if ( nb_cannons_left == 0 )
		{
			refresh_frame( FALSE );
			set_label( wgt_info_nbships, 
				"Sorry, you loose! All the galaxy curse your name now..." );
			gtk_widget_set( GTK_WIDGET( wgt_btn_play ), 
				"GtkWidget::sensitive", TRUE,
				NULL );
			return;
		}
		restart_same_level( TRUE );
		return;
	}						// if

	/*--- Points storage ---*/
	nb = (int)( (elapsed / CANNON_HIT_LONG) * (sz * .22) );
	if ( nb == 0 )
		return;
	if ( maxnb < nb )
	{
		maxnb = nb;
		if ( points == NULL )
			points = malloc( sizeof( GdkPoint ) * nb );
		else
			points = realloc( points, sizeof( GdkPoint ) * nb );
		assert( points != NULL );
	}

	for ( p = points, n = 0; n < nb; n++, p++ )
	{
		p->x = RND( 0, cannon.w - 1 );
		p->y = RND( 0, cannon.h - 1 );
	}							// for (n)
	gdk_draw_points( (GdkWindow *)cannon.destroyed_img_des,
		BACKGROUND_GC,
		points, nb );

}								// make_hit_cannon_bitmap


PUBLIC void 
draw_cannon( void )
{
	double 	xinc;

	/*--- Always at the same Y position: bottom of the screen ---*/
	cannon.y = (frame_height - cannon.h);

	/*--- How many pixels by frame ---*/
	xinc = frame_duration * cannon.xinc;

	/*--- Move the cannon ? ---*/
	if ( keys_state.left )
	{
		if ( !is_starting_cannon && !cannon.is_hit )
			if ( (cannon.x -= xinc) < 0 )
				cannon.x = 0;
	}
	else if ( keys_state.right && !cannon.is_hit )
	{
		if ( !is_starting_cannon )
			if ( (cannon.x += xinc) > (frame_width - cannon.w) )
				cannon.x = (frame_width - cannon.w);
	}
	else if ( keys_state.shoot && !cannon.is_hit )
	{
		if ( !is_starting_cannon )
			if ( !laser.is_overheat )
				try_launch_missile();
	}
	else if ( keys_state.escape )
	{
#if 0
		if ( keys_state.escape_down )
			if ( !is_frozen )
				freeze();
			else
				unfreeze();
#endif
	}
	else if ( keys_state.exit )
	{
#if 0
		PtDestroyWidget( ABW_phreshold );
		exit( 0 );
#endif
	}

	/*--- Draw the cannon to the frame buffer ---*/
	if ( !cannon.is_hit )
	{

		if ( cannon.state != CANNON_UNARMED )
		{
			if ( !(cannon.cnt++ % 10) )
			{
				if ( cannon.dir == 1 )
				{
					if ( ++cannon.state == 3 )
					{
						cannon.state -= 2;
						cannon.dir = -1;
					}
				}
				else
				{
					if ( !cannon.state-- )
					{
						cannon.state += 2;
						cannon.dir = 1;
					}
				}	
			}					// if
		}

		gdk_draw_pixmap( pixmap_frame, 
			BACKGROUND_GC,
			cannon.img_des[ cannon.state ],
			0, 0, cannon.x, cannon.y,
			cannon.w, cannon.h );

	}
	else
	{

		make_hit_cannon_bitmap();		
	
		gdk_draw_pixmap( pixmap_frame, 
			BACKGROUND_GC,
			cannon.destroyed_img_des,
			0, 0, cannon.x, cannon.y,
			cannon.w, cannon.h );

	}

}								// draw_cannon


PUBLIC void 
cannon_is_hit( void )
{
	GdkBitmap	*mask;

	if ( testing_level == -1 )
	{

		cannon.is_hit = TRUE;
		cannon.hit_start = now();			// When the cannon started to be destroyed

		cannon.destroyed_img_des = gdk_pixmap_create_from_xpm_d( wgt_spacearea->window,
			&mask, &BACKGROUND, cannon_unarmed_xpm );
		assert( cannon.destroyed_img_des != NULL );

	}							// if

}								// cannon_is_hit



/* Callback "key_press_event" for GtkWindow widget wgt_wnd */
gboolean
cb_wnd_keypress(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{

	if ( is_paused )
		return TRUE;

	switch ( event->keyval )
	{
		case GDK_Left :
			keys_state.left = TRUE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_press_event" );
			return FALSE;
		case GDK_Right :
			keys_state.right = TRUE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_press_event" );
			return FALSE;
		case GDK_Escape :
			cb_btn_pause_pressed( NULL, NULL );
			break;
		case GDK_Control_L :
		case GDK_space :
			keys_state.shoot = TRUE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_press_event" );
			return FALSE;
	}

	return TRUE;
	
}								// cb_wnd_keypress


/* Callback "key_release_event" for GtkWindow widget wgt_wnd */
gboolean
cb_wnd_keyrelease(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{

	if ( is_paused )
		return TRUE;

	switch ( event->keyval )
	{
		case GDK_Left :	
			keys_state.left = FALSE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_release_event" );
			return FALSE;
		case GDK_Right :
			keys_state.right = FALSE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_release_event" );
			return FALSE;
		case GDK_Control_L :
		case GDK_space :
			keys_state.shoot = FALSE;
			gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "key_release_event" );
			return FALSE;
	}

	return TRUE;
	
}								// cb_wnd_keyrelease
