#include <time.h>

#ifdef DEBUG
extern void toto( void );
#endif

/* 
 *  *******************
 *	* Utilities		  *
 *  *******************
 */

#define RND( N1, N2 ) \
	((random() % ((N2) - (N1) +1)) + (N1))

#define FRND( N1, N2 ) \
	((frandom() * ((N2) - (N1))) + (N1))


#define BACKGROUND		wgt_spacearea->style->bg[ 0 ]
#define BACKGROUND_GC	wgt_spacearea->style->bg_gc[ 0 ]

#define PIXELS_SEC( N ) \
	( (double)(N) / (double)1000000)

#define PIXELS_SEC_MM( N1, N2 ) \
	PIXELS_SEC(N1), PIXELS_SEC(N2)

#define MSEC( N ) \
	(((double)N) * (double)1000)

#define SEC( N ) \
	(((double)N) * (double)1000000)
	
#define UNITS_MICROSEC( N ) \
	((double)(N))

typedef double now_t;


typedef GdkPoint point_t;

typedef struct 
{
	gint		w;
	gint		h;
}	dim_t;

typedef struct
{
	point_t	ul;
	point_t	lr;
}	rect_t;


/* 
 *  *******************
 *	* Games Constants *
 *  *******************
 */

#define	MAXNB_SHIPS_PER_LEVEL	20
#define	MAXNB_SHIP_STATES		24
#define	MAXNB_SHELLS			72
#define	MAXNB_STARS				16


/* 
 *  *******************
 *	* Testing mode ?  *
 *  *******************
 */

COMMON	int	testing_level;		// Set to -1 if not active


/* 
 *  *******************
 *	* Keys State	  *
 *  *******************
 */

typedef struct
{
	int		left;
	int		right;
	int		shoot;
	int		escape;
	int		exit;
}	keys_state_t;

COMMON	keys_state_t	keys_state;


/* 
 *  *******************
 *	* Spacearea		  *
 *  *******************
 */
COMMON	GdkPixmap	*pixmap_frame	INIT( NULL )
COMMON	gint		frame_width		INIT( -1 )
COMMON	gint		frame_height	INIT( -1 )
COMMON	double		frame_duration;					// In micro-seconds
COMMON	double		avg_frame_duration;				// In micro-seconds
COMMON	double		frame_duration_cumul;
COMMON	double		frame_duration_nb;
COMMON	now_t		_now;


/* 
 *  *******************
 *	* Starting Cannon *
 *  *******************
 */

#define	NB_STARTING_CANNONS	5

typedef struct
{
	double		x, 	y;				// Current position
	int			is_visible;
}	starting_cannon_t;

COMMON	GdkPixmap	*pixmap_start_frame	INIT( NULL )
COMMON	gint		frame_start_width	INIT( -1 )
COMMON	gint		frame_start_height	INIT( -1 )

COMMON	starting_cannon_t	starting_cannons[ NB_STARTING_CANNONS ];


/* 
 *  *******************
 *	* Ships (enemies) *
 *  *******************
 */

#define MAXNB_BEZIER_CTRL		10		// Max nb of control points on bezier curve
#define	CTRLPTNB_SHIPFORCVIS	10		// Control points

typedef struct
{
	
	/*--- Pointer to configuration description ---*/
	struct conf_ship	*des;

	/*--- Running state ---*/
	int			is_dead;						// Ship is dead ?
	int			prev_x, prev_y, prev_state;		// Previous position and state
	double		x, y;							// Current position
	int			state, inc;						// Current state and increment (-1, +1)
	now_t		last_state_when;
	int			idle_cnt;

	/*--- Moving to this new position ---*/
	double		move_dx, move_dy;				// Offset
	double		cnt_movex, cnt_movey;
	double		alpha, x0, y0;

	/*--- Attacking ---*/
	int			is_attacking;					// Ship is now attacking ?
	int			is_downCalcAttacking;			// Computing points for next attack (down)
	int			downCtrlPtNb;					// Nb of ctrl point
	int			downPtNb, downPtIndex;			// Points nb and index
	point_t		downPt[MAXNB_BEZIER_CTRL+2];	// Each	position of	control	points (relativly to the ship)
	int			is_upCalcAttacking;				// Computing points for next attack (down)
	int			upCtrlPtNb;						// Nb of ctrl point
	int			upPtNb, upPtIndex;				// Points nb and index
	point_t		upPt[MAXNB_BEZIER_CTRL+2];		// Each	position of	control	points (relativly to the ship)

	/*--- Back to be visible ---*/
	time_t		when_nonvisible;				// Time 1st time non visible
	int			is_nonvisible;					// Is the ship now visible ?
	int			forcing_visible;				// Ship is now descending (to be not hidden) ?
	int			forcVisPtNb, forcVisPtIndex;	// Points nb and index
	point_t		forcVisPt[ CTRLPTNB_SHIPFORCVIS+2 ];	// Control points

	/*--- Ship is hit - Destruction process ---*/
	int			is_hit;		 					// Ship is hit
	now_t		hit_start;	 					// When hit started - See HIT_LONG

	/*--- Pixmap ---*/
	GdkPixmap	*img_des[ MAXNB_SHIP_STATES ];
	dim_t		sz[ MAXNB_SHIP_STATES ];		// Size of the ships (for each state)

}	ship_t;

COMMON	ship_t	ships[ MAXNB_SHIPS_PER_LEVEL ];


/* 
 *  *******************
 *	* SHELLS		  *
 *  *******************
 */

// Current nb of active shells
COMMON	int		nb_active_shells;

// Description of a shell
typedef struct
{
	int			active;				// Is this shell active ?
	double		x, y;				// Current position
	double		yinc;				// Speed, in pixels / sec
}	shell_t;

// Pool of all shells (active or not)
COMMON	shell_t		shells[ MAXNB_SHELLS ];

// Description of the bitmap
COMMON	GdkPixmap	*shell_img;
COMMON	gint		shell_w, shell_h;


/* 
 *  *******************
 *	* CANNON		  *
 *  *******************
 */

#define CANNON_ARMED_0	0
#define CANNON_ARMED_1	1
#define CANNON_ARMED_2	2
#define	CANNON_UNARMED	3

// Description of the cannon
typedef struct
{
	double		x, y;				// Current position
	int			cnt;				// 
	int			state;				// CANNON_ARMED_0-2 | CANNON_UNARMED
	int			can_shoot;			// Can shoot ?
	int			dir;
	int			is_hit;				// Cannon is hit ?
	gint		w, h;
	double		xinc;				// Pixel increment on each frame
	double		hit_start;			// When the cannon started to be destroyed
	GdkPixmap	*img_des[ 4 ];
	GdkPixmap	*destroyed_img_des;
}	cannon_t;

// The cannon
COMMON	cannon_t	cannon;

// Speed of the cannon
#define CANNON_XSPEED		PIXELS_SEC(280)

#define CANNON_HIT_LONG		MSEC(3500)


/* 
 *  *******************
 *	* MISSILES		  *
 *  *******************
 */

// Description of the Missile
typedef struct
{
	GdkPixmap	*img_des;
	int			w, h;
	double		x, y;				// Current position
	int			is_active;			// Active or not
	double		yinc;				// Pixel increment on each frame
}	missile_t;

// Current number of missiles
#define MAXNB_MISSILES			4
COMMON	int		nb_active_missiles;

// The Missile
COMMON	missile_t	missiles[ MAXNB_MISSILES ];

// Description of the bitmap

// Missile speed
#define MISSILE_YSPEED		PIXELS_SEC(700)
// #define MISSILE_YSPEED		PIXELS_SEC(100)


/* 
 *  ***************
 *	* LASER 	  *
 *  ***************
 */

#define LASER_MAX		1000 
#define	LASER_SHOOT		35
#define	LASER_OVERHEAT	900
#define	LASER_OK		650
#define	LASER_DEC		32				/* 32 each second */

typedef struct
{
	now_t		prev_now;
	double		level;					// Current level, from 0 to 1000
	int			is_overheat;			// Is laser overheat ?
	GtkStyle	*style;					// Either wgt_style_laser_ok or wgt_style_laser_overheat
}	laser_t;

COMMON	laser_t	laser;


/* 
 *  *******************
 *	* FUEL LEVEL	  *
 *  *******************
 */

typedef struct
{
	now_t	start;						// Starting time
	now_t	nb_microsec;				// Nb of micro-seconds given at startup
	double	level;						// Current gauge value (0..1000)
}	fuel_t;

COMMON	fuel_t	fuel;


/*--- ------ ---*/
/*--- SCORES ---*/
/*--- ------ ---*/

#define	SCORE_HIT_SHIP				100
#define	SCORE_HIT_SHIP_ATTACKING	200


/* 
 *  *************************
 *	* Level configuration	*
 *  *************************
 */
// Function to place ships
struct conf_ship;
typedef	void (*POS_FUNCT) (struct conf_ship *des);

#define TYPMOV_RANDOM	1
#define TYPMOV_ELLIPSE	2

typedef struct conf_ship
{

	/*--- Global parameters ---*/
	int			moving_grouped;				// Do the ships move grouped ?
	int			scroll_lr, scroll_bt;		// Scrolling left-right, Scrolling bottom-top
	int			same_state;					// Do ships have the same state ?
	POS_FUNCT	pos_funct;					// Function to place ships
	int			nbships1, nbships2;			// Nb of ships (btw [n1..n2])
	int			type_movment;				// Type of movement
	double		xspeed1, xspeed2;			// Generate a random X movement btw [x1, x2] px/sec
	double		xlen1, xlen2;
	double		yspeed1, yspeed2;			// Generate a random Y movement btw [y1, y2] px/sec
	double		ylen1, ylen2;
	int			bottom_ymax;				// Bottom gap btw the bottom and ship

	/*--- Description of the ships ---*/
	int			rounding;					// State transition: 01232101232... or 012301230...
	int			nb_states;					// Nb of states of each ship
	double		state_trans_time;			// State transition time
	point_t		offs[ MAXNB_SHIP_STATES ];	// Offset (transition btws each state)
	int			maxnb_sec_hidden;			// Max nb of sec. of ship being hidden
	int			perc_reforce_inside_x,
				perc_reforce_inside_y;		// % of chance to force ship inside if computed outside
	double		ship_hit_long;				// Time for a ship to be destroyed
	int			destroyed_ships_active;		// Destroyed ships continue to drop shells ?

	/*--- Attack ---*/
	double		shells_drop_avg;			// Shells drop: avg number / minute
	double		drops1, drops2;				// Speed of shells (between), in pixels/sec
	double		attack_avg;					// Frequency of the attacks: avg nb / minute
	double		attack_time_descent_min,
				attack_time_descent_max;	// Time (msec) for attack (descent)
	double		attack_time_up_min,
				attack_time_up_max;			// Time (msec) for attack (come back)
	int			attack_cannon_xdist_min,	
				attack_cannon_xdist_max;	// Distance to the cannon (X)
	int			attack_cannon_ydist_min,	
				attack_cannon_ydist_max;	// Distance to the cannon (X)

	/*--- XPM description *---*/
	char		**img[ MAXNB_SHIP_STATES ];

	/*--- This part is filled at run-time ---*/
	int			nbships;						// Nb of ships (initial)
}	conf_ship_t;

typedef struct conf_level
{

	/*--- Nb of different ships ---*/
	int			nbdiff_ships;

	/*--- Ship decription ---*/
	int			fuel_time;					// Fuel time (in seconds)
	int			maxnb_attacks;				// Max nb of ships attacking at same time
	int			maxnb_shells;				// Max number of shells for this level
	int			maxnb_missiles;				// Max number of missiles (cannon)
	conf_ship_t	shipdes[ 4 ];

	/*--- This part is filled at run-time ---*/
	int			nbships;					// Nb of ships (initial)
	int			nb_ships_attacking;			// Nb of ships attacking now
	int			nb_ships_alive;				// Nb of ships still alive

}	conf_level_t;

COMMON	int	nb_levels;

COMMON	int	level;
COMMON	int	score;
COMMON	int	nb_cannons_left;

// Used by the low-level function blaster_or
COMMON	int	is_scrolling_lr;
COMMON	int	is_scrolling_bt;

COMMON	int	is_starting_cannon;


/* 
 *  **********
 *	* MOTHER *
 *  **********
 */

// Description of the mother ship
typedef struct
{
	GdkPixmap	*img_des;
	double		x, y;					// Current position
	int			w, h;
	int			is_active;				// Is the mother currently active ?
	int			start_a_cannon;			// Parameter given to start_new_level()
	int			going_down;
	int			refill_fuel;
	int			going_up;
}	mother_t;

// The mother ship
COMMON	mother_t		mother;

#define MOTHER_GOING_DOWN_YSPEED		PIXELS_SEC(150)
#define MOTHER_REFILL_FUEL_SPEED		UNITS_MICROSEC(2500)	/* One unit each 5,000 microsec */
#define MOTHER_GOING_UP_YSPEED			PIXELS_SEC(250)


/* 
 *  *********
 *	* Stars *
 *  *********
 */

/*--- Description of a star ---*/
typedef struct
{
	double		x, y;					// Current position
	int			state, inc;
	double		yinc;					// Pixel increment on each frame
}	star_t;

/*--- The stars ---*/
COMMON	star_t			stars[ MAXNB_STARS ];
COMMON	int				nb_stars;
#define	STAR_NBSTATES	6
#define	STAR_HEIGHT		9
#define	STAR_WIDTH		9
COMMON	GdkPixmap		*pixmap_stars[ STAR_NBSTATES ];

// Speed of the stars
#define STAR_YSPEED_MIN		PIXELS_SEC(10)		// 10 pixels / sec
#define STAR_YSPEED_MAX		PIXELS_SEC(30)		// 10 pixels / sec


/*--- ------------------------------- ---*/
/*--- Create the configuration levels ---*/
/*--- ------------------------------- ---*/

COMMON	conf_level_t	currlevel;
#ifdef __STARTUP__

#include "levels.h"

int		nb_levels = sizeof( conf_levels ) / sizeof( conf_level_t );

#else

extern	conf_level_t	conf_levels[];

extern	int	nb_levels;

#endif


/*
 *
 */

COMMON	gint		frame_width, frame_height;


/*--- menu.c ---*/
COMMON	guint	draw_frame_id	INIT( 0 )
COMMON	int		is_frame_id		INIT( FALSE )
COMMON	int		is_paused		INIT( FALSE )


/*--- menu.c ---*/
void restart_same_level( const int start_a_cannon );
void start_new_level( const int start_a_cannon );
void start_play( void );
void refresh_frame( const int on );

/*--- ships.c ---*/
void  level2_pos_funct( conf_ship_t *des );
void init_gen_ships( void );
void ship_is_hit( ship_t *ship );
void move_ships( void );

/*--- shell.c ---*/
void reset_shells( void );
void load_shell( void );
void drop_shell( const ship_t *ship );
void move_shells( void );

/*--- cannon.c ---*/
void init_keys_state( void );
void place_cannon( void );
void install_starting_cannon( starting_cannon_t *starting_cannon, int index );
void move_starting_cannon( void );
void draw_starting_cannon( void );
void start_new_cannon( void );
void load_cannon( void );
void init_cannon( void );
void draw_cannon( void );
void cannon_is_hit( void );

/*--- spacearea.c ---*/
gint draw_frame( gpointer data );

/*--- missile.c ---*/
void reset_missiles( void );
void load_missiles( void );
void try_launch_missile( void );
void draw_missiles( void );

/*--- game.c ---*/
double frandom ( void );
now_t now( void );
void set_label( GtkWidget *wgt, char *fmt, ... );
void draw_score( void );
void set_gauge_minmax( GtkWidget *gauge, int vmin, int vmax );
void set_gauge_val( GtkWidget *gauge, int val );
int rect_intersect( rect_t *r1, rect_t *r2 );

/*--- fuel.c ---*/
void update_fuel( void );

/*--- laser.c ---*/
void update_laser( void );

/*--- stars.c ---*/
void load_stars( void );
void put_stars( void );
void draw_stars( void );

/*--- mother.c ---*/
void load_mother( void );
void start_mother( const int start_a_cannon );
void draw_mother( void );
void end_mother( void );
