#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include <assert.h>
#include <math.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"


#ifdef DEBUG
void toto( void ) {}
#endif


/* Callback "pressed" for GtkButton widget wgt_btn_play */
void
cb_btn_play_pressed(GtkButton *button, gpointer user_data)
{

	gtk_widget_set( GTK_WIDGET( wgt_btn_play ), 
		"GtkWidget::sensitive", FALSE,
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_btn_pause ), 
		"GtkWidget::sensitive", TRUE,
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_btn_about ), 
		"GtkWidget::sensitive", FALSE,
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_btn_quit ), 
		"GtkWidget::sensitive", FALSE,
		NULL );

	is_paused = FALSE;
	start_play();
	
}								// cb_btn_playpressed


/* Callback "pressed" for GtkButton widget wgt_btn_pause */
void
cb_btn_pause_pressed(GtkButton *button, gpointer user_data)
{

	gtk_widget_set( GTK_WIDGET( wgt_btn_play ), 
		"GtkWidget::sensitive", TRUE,
		NULL );
	is_paused = (is_paused) ? FALSE : TRUE;
	
	gtk_widget_set( GTK_WIDGET( wgt_btn_pause ), 
		"GtkButton::label", (is_paused) ? "Resume" : "Pause",
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_btn_about ), 
		"GtkWidget::sensitive", (is_paused) ? TRUE : FALSE,
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_btn_quit ), 
		"GtkWidget::sensitive", (is_paused) ? TRUE : FALSE,
		NULL );

	refresh_frame( (is_paused) ? FALSE : TRUE );

}								// cb_btn_pause_pressed


/* Callback "pressed" for GtkButton widget wgt_btn_about */
void
cb_btn_about_pressed(GtkButton *button, gpointer user_data)
{
	double	x, y;
	double	a, b;
	double	alpha;
	
	a = 200;
	b = 100;

	y = 150;

		gdk_draw_point( pixmap_frame,
			wgt_spacearea->style->bg_gc[ 1 ],
			200, 200 );

	for ( alpha = -M_PI; alpha < M_PI; alpha += 0.01 )
	{
		x = 200 + cos( alpha ) * a;
		y = 200 + sin( alpha ) * b;
		gdk_draw_point( pixmap_frame,
			wgt_spacearea->style->bg_gc[ 1 ],
			x, y );
	}

	gtk_widget_queue_draw( wgt_spacearea );
	
}								// cb_btn_about_pressed


/* Callback "pressed" for GtkButton widget wgt_btn_quit */
void
cb_btn_quit_pressed(GtkButton *button, gpointer user_data)
{

	printf( "Quit...\n");

}								// cb_btn_quit_pressed


/*--- ----------------------------------- ---*/
/*--- Initializations prior to a new play ---*/
/*--- ----------------------------------- ---*/

PUBLIC void 
restart_same_level( const int start_a_cannon )
{

	/*--- Erase everything into the frame buffer ---*/
	gdk_draw_rectangle( pixmap_frame, 
		BACKGROUND_GC,
		TRUE,
		0, 0, frame_width, frame_height );

	/*--- Reset shells ---*/
	reset_shells();

	if ( start_a_cannon )
	{
	
		/*--- Start a new cannon ---*/	
		if ( testing_level == -1 )
			start_new_cannon();

		/*--- Initialize the initial position/state of the cannon ---*/
		place_cannon();
		
		/*--- Laser ---*/
		laser.level = 0;
		laser.is_overheat = FALSE;
		laser.style = NULL;
		laser.prev_now = 0;
	
	}							// if

	/*--- No missile running ---*/
	reset_missiles();

	/*--- State of the keys (used for the cannon movement) ---*/
	init_keys_state();

	/*--- This part CONF_LEVEL is filled at run-time ---*/
	currlevel.nb_ships_attacking = 0;		// Nb of ships attacking now

}								// restart_same_level


PUBLIC void
start_new_level( const int start_a_cannon )
{

	/*--- Stop drawing frame ---*/
	refresh_frame( FALSE );

	/*--- Mother ship ? ---*/
	if ( testing_level == -1 )
	{
		level++;
		if ( (level != 0) && ((level % 4) == 0) )
		{
			refresh_frame( TRUE );
			start_mother( start_a_cannon );
			return;
		}
	}

	/*--- Finish, no more level available ? ---*/
	if ( level == nb_levels )
	{
		set_label( wgt_info_nbships, "Sorry, no more level available !" );
		return;
	}

	/*--- Show the level ---*/
	set_label( wgt_level, "Level %d", level + 1 );

	/*--- Initializations prior to a new play ---*/
	memmove( &currlevel, &conf_levels[ level ], sizeof( conf_level_t ) );

	/*--- Fuel ---*/
	if ( (testing_level != -1) || (level % 4) == 0 )
	{
		fuel.start = _now;
		fuel.nb_microsec = 1000000 * currlevel.fuel_time;
		set_gauge_minmax( wgt_fuel, 0, 1000 );
		set_gauge_val( wgt_fuel, 1000 );
	}

	/*--- Laser ---*/
	if ( (testing_level != -1) || (level % 4) == 0 )
	{
		set_gauge_minmax( wgt_laser, 0, 1000 );
		set_gauge_val( wgt_laser, 0 );
	}

	/*--- Initialize a new generation of ships ---*/
	init_gen_ships();
	currlevel.nb_ships_alive = currlevel.nbships;

	/*--- Show how many ships still alive ---*/
	set_label( wgt_info_nbships, "%d ship%s still alive",
		currlevel.nb_ships_alive,
		(currlevel.nb_ships_alive > 1) ? "s" : "" );

	/*--- Install some stars ---*/
	put_stars();

	/*--- Initialize the game to play level ---*/
	restart_same_level( start_a_cannon );

	/*--- Now the cannon is started, time to play the real game ! ---*/
	refresh_frame( TRUE );

}								// start_new_level


PUBLIC void
start_play( void )
{
	starting_cannon_t	*starting_cannon;
	int					n;

	/*--- Initialize state of playing keys ---*/
	init_keys_state();

	/*--- Initialize score ---*/
	score = 0;
	draw_score();

	/*--- Reset starting cannons ---*/
	nb_cannons_left = NB_STARTING_CANNONS;
	for ( starting_cannon = starting_cannons, n = 0; 
		  n < NB_STARTING_CANNONS; 
		  n++, starting_cannon++ )
	{
		install_starting_cannon( starting_cannon, n );
	}							// for (n, starting_cannon)

	/*--- Start at level #0 ---*/
	if ( testing_level == -1 )
		level = -1;
	else
		level = testing_level;
	start_new_level( TRUE );	// Starting cannon

}								// start_play


PUBLIC void
refresh_frame( const int on )
{

	if ( on )
	{
		assert( is_frame_id == FALSE );
//		draw_frame_id = gtk_timeout_add( 50, draw_frame, NULL );
		draw_frame_id = g_idle_add( draw_frame, NULL );
		is_frame_id = TRUE;
	}
	else
	{
		if ( is_frame_id )
			gtk_idle_remove( draw_frame_id );
//			gtk_timeout_remove( draw_frame_id );
		is_frame_id = FALSE;
	}

}								// refresh_frame
