#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include <assert.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"

#include <gdk/gdkkeysyms.h>

#include "graphics/missile.0.xpm"


PUBLIC void
reset_missiles( void )
{
	missile_t	*missile;
	int			index;

	nb_active_missiles = 0;
	for ( missile = missiles, index = 0; index < MAXNB_MISSILES; index++, missile++ )
	{

		missile->yinc = MISSILE_YSPEED;
		missile->is_active = FALSE;
	
	}							// for (missile, index)

}								// reset_missiles


PUBLIC void
load_missiles( void )
{
	GdkBitmap	*mask;
	missile_t	*missile;
	int			index;

	for ( missile = missiles, index = 0; index < MAXNB_MISSILES; index++, missile++ )
	{

		missile->img_des = gdk_pixmap_create_from_xpm_d( wgt_spacearea->window,
			&mask, &BACKGROUND, missile_0_xpm );
		assert( missile->img_des != NULL );
		gdk_window_get_size( (GdkWindow *)missile->img_des,
			&missile->w, &missile->h );

	}							// for (missile, index)

	reset_missiles();

}								// load_missiles


PUBLIC void 
try_launch_missile( void )
{
	missile_t	*missile;
	int			index;

	/*--- Check if missile is allowed ---*/
	if ( (nb_active_missiles == currlevel.maxnb_missiles) || !cannon.can_shoot )
		return;

	/*--- Found an empty slot ---*/
	for ( missile = missiles, index = 0; index < MAXNB_MISSILES; index++, missile++ )
		if ( !missile->is_active )
			break;
	assert( index < MAXNB_MISSILES );

	/*--- Create the new missile ---*/
	nb_active_missiles++;
	missile->is_active = TRUE;
	missile->x = cannon.x + ((cannon.w - missile->w) >> 1);
	missile->y = cannon.y;

	/*--- Change the state of the cannon while missile is active ---*/
	cannon.state = CANNON_UNARMED;
	cannon.can_shoot = FALSE;
	cannon.dir = 1;

	/*--- Update the laser status ---*/
	if ( (laser.level += LASER_SHOOT) >= LASER_MAX )
		laser.level = LASER_MAX;

}								// try_launch_missile


PUBLIC void 
draw_missiles( void )
{
static	ship_t		*ship;
static	conf_ship_t	*des;
static	int			n;
static	rect_t		rect1, rect2;
		double		yinc;
		missile_t	*missile, *miss;
		int			index;
		int			can_shoot;

	for ( missile = missiles, index = 0; index < MAXNB_MISSILES; index++, missile++ )
	{

		/*--- Non active ? ---*/
		if ( !missile->is_active )
			continue;

		/*--- How many pixels by frame ---*/
		yinc = frame_duration * missile->yinc;

		/*--- Draw the missile to the frame buffer ---*/
		gdk_draw_pixmap( pixmap_frame, 
			BACKGROUND_GC,
			missile->img_des,
			0, 0, missile->x, missile->y,
			missile->w, missile->h );

		/*--- Missile disappears ? ---*/
		if ( !missile->y )
		{
			missile->is_active = FALSE;
			nb_active_missiles--;
			continue;
		}

		/*--- Move the missile ---*/
		rect2.lr.x = missile->x + missile->w;
		rect2.lr.y = missile->y + missile->h;
		if ( (missile->y -= yinc) < 0 )
			missile->y = 0;
		rect2.ul.x = missile->x;
		rect2.ul.y = missile->y;

		/*--- Any ship hit ? ---*/
		for ( ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
		{

			des = ship->des;
			if ( !ship->is_hit )
			{
				rect1.ul.x = ship->x;
				rect1.ul.y = ship->y;
				rect1.lr.x = ship->x + ship->sz[ ship->state ].w;
				rect1.lr.y = ship->y + ship->sz[ ship->state ].h;

				if ( rect_intersect( &rect1, &rect2 ) )
				{
					if ( ship->is_attacking )
						score += SCORE_HIT_SHIP_ATTACKING;
					else
						score += SCORE_HIT_SHIP;
					draw_score();
					ship_is_hit( ship );
					missile->is_active = FALSE;
					nb_active_missiles--;
					continue;
				}				// if

			}					// if

		}						// for (ship, n)
		
	}							// for (missile, index)

	/*--- Another missile can be launch ? ---*/
	for ( miss = NULL, missile = missiles, index = 0; 
		  index < MAXNB_MISSILES;
		  index++, missile++ )
	{
		if ( missile->is_active )
			if ( !miss || (missile->y > miss->y) )
				miss = missile;
	}							// for (missile, index)
	if ( !miss || (miss->y <= ((frame_height * (currlevel.maxnb_missiles - 1)) / currlevel.maxnb_missiles)) )
		can_shoot = TRUE;
	else
		can_shoot = FALSE;

	cannon.can_shoot = can_shoot;
	if ( can_shoot  )
	{
		cannon.state = CANNON_ARMED_0;
		cannon.dir = 1;
	}
	else
	{
		cannon.state = CANNON_UNARMED;
	}							// if		

}								// draw_missiles
