#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include <assert.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"

#include <gdk/gdkkeysyms.h>

#include "graphics/shell.0.xpm"


PUBLIC void
reset_shells( void )
{
	shell_t		*shell;
	int			index;
	
	nb_active_shells = 0;
	for ( shell = shells, index = 0; index < MAXNB_SHELLS; index++, shell++ )
	{
	
		shell->active = FALSE;
		
	}							// for (shell, index)
	
}								// reset_shells


PUBLIC void
load_shell( void )
{
	GdkBitmap	*mask;

	/*--- Create the cannon -armed- pixmap ---*/
	shell_img = gdk_pixmap_create_from_xpm_d( wgt_spacearea->window,
		&mask, &BACKGROUND, shell_0_xpm );
	assert( shell_img != NULL );
	gdk_window_get_size( (GdkWindow *)shell_img,
		&shell_w, &shell_h );

}								// load_shell


PUBLIC void 
drop_shell( const ship_t *ship )
{
	shell_t	   		*shell;
	conf_ship_t		*des;
	int				n;

	/*--- No room for a new shell ? ---*/
	if ( nb_active_shells == currlevel.maxnb_shells )
		return;

	/*--- This level is finished ? ---*/
	if ( !currlevel.nb_ships_alive )
		return;

	/*--- Found an empty slot ---*/
	for ( shell = shells, n = 0; n < currlevel.maxnb_shells; n++, shell++ )
		if ( !shell->active )
			break;

	/*--- Create the shell ---*/
	des = ship->des;
	shell->y = ship->y + ship->sz[ ship->state ].h;
	shell->x = ship->x + ((ship->sz[ ship->state ].w - shell_w) >> 1);
	shell->yinc = FRND( des->drops1, des->drops2 );
	shell->active = TRUE;
	nb_active_shells++;
	
}								// drop_shell


PUBLIC void 
move_shells( void )
{
	shell_t		*shell;
	int			n;
	double		yinc;

	/*--- This level is finished ? ---*/
	if ( !currlevel.nb_ships_alive && !nb_active_shells )
	{
		if ( testing_level == -1 )
			start_new_level( FALSE );
		else
			refresh_frame( FALSE );
		return;
	}

	/*--- Move all active shells ---*/
	for ( shell = shells, n = 0; n < currlevel.maxnb_shells; n++, shell++ )
	{

		/*--- Not active ? ---*/
		if ( !shell->active )
			continue;

		/*--- Shell to be lost ? ---*/
		if ( shell->y == frame_height - shell_h )
		{
			shell->active = FALSE;
			nb_active_shells--;
			continue;
		}

		/*--- Draw the shell to the frame buffer ---*/
		gdk_draw_pixmap( pixmap_frame, 
			BACKGROUND_GC,
			shell_img,
			0, 0, shell->x, shell->y,
			shell_w, shell_h );

		/*--- How many pixels by frame ---*/
		yinc = frame_duration * shell->yinc;

		/*--- Compute new position of the shell ---*/
		if ( (shell->y += yinc) > (frame_height - shell_h) )
			shell->y = frame_height - shell_h;

		/*--- Test if shell hit the cannon ---*/
		if ( shell->y < (frame_height - cannon.h) )
			continue;
		if ( !cannon.is_hit )
		{
			if ( (shell->x >= cannon.x) && (shell->x + shell_w <= cannon.x + cannon.w) )
			{
				cannon_is_hit();
				shell->active = FALSE;
				nb_active_shells--;
			}
		}						// if

	}							// for (shell, n)
	
}								// move_shells
