#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/time.h>
#include <math.h>
#include <assert.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"

#include <gdk/gdkkeysyms.h>



void 
level2_pos_funct( conf_ship_t *des )
{
	ship_t		*ship;
	int			cpt, n;
	int			a, c;
	
	for ( ship = ships, cpt = 0, n = 0; n < currlevel.nbships; n++, ship++ )
	{

		/*--- Not this set ? ---*/
		if ( ship->des != des )
			continue;
			
		if ( (c = cpt%7) < 4 )
		{
			a = (frame_width - 4 * ship->sz[ ship->state ].w) / 5;
			ship->x = (c+1)*a + (c * ship->sz[ ship->state ].w);
			ship->y = (cpt/7) * 100 + 20;
		}
		else
		{
			c -= 4;
			a = (frame_width - 3 * ship->sz[ ship->state ].w) / 4;
			ship->x = (c+1)*a + (c * ship->sz[ ship->state ].w);
			ship->y = 50 + (cpt/7) * 100 + 20;
		}
		cpt++;

	}							// for (n, ship)

}								// level2_pos_funct


/*----------------------------*/
/*--- Compute Bezier curve ---*/
/*----------------------------*/

static long 
C( const int n, const int i )
{
	long	a;
	int		j;
	int		ni = n - i;

	a = 1;
	for (j = i + 1; j <= n; j++)
		a *= j;

	for (j = 1; j <= ni; j++)
		a /= j;

	return (a);

}								// C


static double 
BBlend( const int i, const int n, const double u )
{
	int			j;
	double		v;
    double		u1 = (1 - u);
	int			ni = n - i;

	v = C( n, i );

	for ( j = 0; j < i; j++ )
		v *= u;

	for ( j = 0; j < ni; j++ )
		v *= u1;;

	return ( v ); 

}								// BBlend


void 
Bezier( double *x, double *y, double u, int n, point_t *cc )
{
	int		i;
	double	b;

	*x = 0;
	*y = 0;

	for ( i = 0; i <= n; i++ )
	{
		b = BBlend(i, n, u);
		*x += (cc[ i ].x * b);
		*y += (cc[ i ].y * b);
	}							// for (i)

}								// Bezier


/*------------------------------*/
/*--- Compute B-Spline curve ---*/
/*------------------------------*/

static long 
Knot( const int i, int *knotK, int *knotN )
{

	if (i < *knotK)
		return (0);
	else
	{
		if (i > *knotN)
			return (*knotN - *knotK + 2);
		else
			return (i - *knotK + 1); 
	}							// else

}								// Knot


static double 
NBlend( const int i, const int k, const double u, int *knotK, int *knotN )
{
	int		t;
	double	v;

	v = 0;
	if (k == 1)
	{
		if ( (Knot(i, knotK, knotN) <= u) && (u < Knot(i+1, knotK, knotN)) )
			v = 1; 
	}							// if
	else
	{
		t = (int)(Knot(i+k-1, knotK, knotN) - Knot(i, knotK, knotN));
		if (t != 0)
			v = (u - Knot(i, knotK, knotN)) * NBlend(i, k-1, u, knotK, knotN) / t;
		t = (int)(Knot(i+k, knotK, knotN) - Knot(i+1, knotK, knotN));
		if (t != 0)
			v += ((Knot(i+k, knotK, knotN) - u) * NBlend(i+1, k-1, u, knotK, knotN) / t);
	}							// else

    return (v);

}								// NBlend


void 
BSpline( double *x, double *y, double u, int n, int k, point_t *cc )
{
	int		knotK, knotN;
	int		i;
    double	b;

	*x = 0;
	*y = 0;

	knotK = k;
    knotN = n;

	for ( i = 0; i <= n; i++ )
	{
		b = NBlend(i, k, u, &knotK, &knotN);
		*x += (cc[i].x * b);
		*y += (cc[i].y * b);
	} 							// for (i)

}								// BSpline


PUBLIC void 
ship_is_hit( ship_t *ship )
{

	/*--- Update internal state of the ship ---*/
	ship->is_hit = TRUE;
	ship->hit_start = _now;

	/*--- Update attacking ships nb ---*/
	if ( ship->is_attacking )
		currlevel.nb_ships_attacking--;

}								// ship_is_hit


static void 
is_cannon_hit_by_ship( ship_t *ship )
{
	rect_t	rect1, rect2;

	/*--- ------------------------------------------------ ---*/
	/*--- Check now if the new ship has not hit the cannon ---*/
	/*--- ------------------------------------------------ ---*/

	if ( !ship->is_hit && !cannon.is_hit && !is_starting_cannon && !testing_level )
	{

		/*--- The cannon area ---*/
		rect2.ul.x = cannon.x;
		rect2.ul.y = cannon.y;
		rect2.lr.x = cannon.x + cannon.w;
		rect2.lr.y = cannon.y + cannon.h;

		/*--- The ship area ---*/	
		rect1.ul.x = ship->x;
		rect1.ul.y = ship->y;
		rect1.lr.x = ship->x + ship->sz[ ship->state ].w;
		rect1.lr.y = ship->y + ship->sz[ ship->state ].h;

		if ( rect_intersect( &rect1, &rect2 ) )
		{
	
			/*--- Cannon is dead ---*/
			cannon_is_hit();

			/*--- But also the ship... ---*/
			if ( ship->is_attacking )
				score += SCORE_HIT_SHIP_ATTACKING;
			else
				score += SCORE_HIT_SHIP;
			draw_score();
			ship_is_hit( ship );

		}						// if
			
	}							// if

}								// is_cannon_hit_by_ship



static void
make_hit_ship_bitmap( ship_t *ship, conf_ship_t *des )
{
		int			n, nb, sz;
		double		elapsed;
static	GdkPoint	*points = NULL;
static	int			maxnb = 0;
		GdkPoint	*p;
		int			state;

	/*--- Compute how many percentage of the ship to destroy ---*/
	state = ship->state;
	sz = ship->sz[ state ].w * ship->sz[ state ].h;
	elapsed = _now - ship->hit_start;

	/*--- Ship is dead ? ---*/
	if ( elapsed >= des->ship_hit_long )
	{
		ship->is_dead = TRUE;
		currlevel.nb_ships_alive--;
		set_label( wgt_info_nbships, "%d ship%s still alive",
			currlevel.nb_ships_alive,
			(currlevel.nb_ships_alive > 1) ? "s" : "" );
		ship->prev_state = ship->state;		
		return;
	}						// if

	/*--- Points storage ---*/
	nb = (int)( (elapsed / des->ship_hit_long) * (sz * 0.33) );
	if ( nb == 0 )
		return;
	if ( maxnb < nb )
	{
		maxnb = nb;
		if ( points == NULL )
			points = malloc( sizeof( GdkPoint ) * nb );
		else
			points = realloc( points, sizeof( GdkPoint ) * nb );
		assert( points != NULL );
	}

	for ( p = points, n = 0; n < nb; n++, p++ )
	{
		p->x = RND( 0, ship->sz[ state ].w - 1 );
		p->y = RND( 0, ship->sz[ state ].h - 1 );
	}							// for (n)
	gdk_draw_points( (GdkWindow *)ship->img_des[ state ],
		BACKGROUND_GC,
		points, nb );

}								// make_hit_ship_bitmap


static int
sort_ships( const void *s1, const void *s2 )
{
	ship_t	*ship1 = (ship_t *)s1;
	ship_t	*ship2 = (ship_t *)s2;
	
	if ( ship1->y < ship2->y )
		return 1;
	else if ( ship1->y > ship2->y )
		return -1;
	else return 0;
	
}								// sort_ships


/*--- ------------------------------------ ---*/
/*--- Initialize a new generation of ships ---*/
/*--- ------------------------------------ ---*/

PUBLIC void 
init_gen_ships( void )
{
	ship_t			*ship;
	int				cpt, w, n, k;
	int				set;
	conf_ship_t		*des;
	GdkBitmap  		*mask;

	/*--- ---------------- ---*/
	/*--- How many ships ? ---*/
	/*--- ---------------- ---*/

	currlevel.nbships = 0;	
	for ( w = 0; w < currlevel.nbdiff_ships; w++ )
	{
		currlevel.shipdes[ w ].nbships = 
			RND( currlevel.shipdes[ w ].nbships1, currlevel.shipdes[ w ].nbships2 );
		currlevel.nbships += currlevel.shipdes[ w ].nbships;
	}

	/*--- ---------------------------------- ---*/
	/*--- Found a position for all the ships ---*/	
	/*--- ---------------------------------- ---*/

	memset( &ships, 0, sizeof( ships ) );	
	for ( ship = ships, cpt = 0, set = 0, n = 0; n < currlevel.nbships; n++, ship++ )
	{

		/*--- Which ship set ? ---*/
		if ( cpt++ == currlevel.shipdes[ set ].nbships )
		{
			set++;
			cpt = 0;
		}
		ship->des = des = &currlevel.shipdes[ set ];

		/*--- Create pixmap ---*/
		for ( k = 0; k < currlevel.shipdes[ set ].nb_states; k++ )
		{
			ship->img_des[ k ] = gdk_pixmap_create_from_xpm_d( 
				wgt_spacearea->window, &mask, &BACKGROUND, 
				currlevel.shipdes[ set ].img[ k ] );
			assert( ship->img_des[ k ] != NULL );
			gdk_window_get_size( (GdkWindow *)ship->img_des[ k ],
				&ship->sz[ k ].w,
				&ship->sz[ k ].h );
		}						// for (k)

		/*--- Ship state ---*/
		ship->is_dead = FALSE;
		ship->is_hit  = FALSE;
		ship->hit_start = 0;
		ship->when_nonvisible = 0;

		/*--- Create an initial state ---*/
		ship->last_state_when = now();
		if ( !des->same_state || !n )
		{
			if ( des->nb_states == 1 )
			{
				ship->state = 0;
				ship->inc	= 0;
			}
			else if ( des->nb_states == 2 )
			{
				ship->state = RND( 0, 1 );
				ship->inc	= 1 - 2*ship->state;
			}
			else
			{
				ship->state = RND( 1, des->nb_states - 2 );
				ship->inc	= 2 * RND( 0, 1) - 1;
			}
		}						// if
		if ( des->same_state && n )
		{
			ship->state = ships[ 0 ].state;
			ship->inc   = ships[ 0 ].inc;
		}
		ship->cnt_movex = 0;
		ship->cnt_movey = 0;
		ship->alpha = M_PI;

		/*--- Found a position for this ship ---*/
		if ( des->pos_funct == NULL )
		{
			ship->x = RND( 1, frame_width - 1 );
			ship->y = RND( 1, frame_height - des->bottom_ymax - 1 );
		}

	}							// for (ship, n)

	/*--- Function to position all ships ? ---*/
	for ( set = 0; set < currlevel.nbdiff_ships; set++ )
	{
		if ( currlevel.shipdes[ set ].pos_funct )
		{
			currlevel.shipdes[ set ].pos_funct( &currlevel.shipdes[ set ] );
		}						// if
	}

	/*--- Sort the ship to have as 1st ship the one at the most bottom ---*/
	qsort( ships, currlevel.nbships, sizeof( ship_t ), sort_ships );

}								// init_gen_ships


static void 
start_new_attack( ship_t *ship )
{
	conf_ship_t		*des;
	int				n;
	int				nCtrlPtNb;
	point_t			*q;					// Control points
	int				cx, cy;

	des = ship->des;

	ship->is_attacking			= TRUE;			// Attack in progress
	ship->is_downCalcAttacking	= TRUE;			// Computing points for next attack (down)
	ship->is_upCalcAttacking	= FALSE;		// Computing points for next attack (down)

	/*----------------*/
	/*--- (1) Down ---*/
	/*----------------*/

	/*--- Generate points of control of the bezier curve ---*/
	q = ship->downPt;
	q->x = ship->x;
	q->y = ship->y;
	nCtrlPtNb = RND( 3, 9 );
	for ( q++, n = 0; n < nCtrlPtNb; n++, q++ )
	{
		q->x = RND( -(frame_width  >> 1), (3 * frame_width)  >> 1 );
		q->y = RND( -(frame_height >> 1), (3 * frame_height) >> 2 );
	}							// for (n)
	cx = cannon.x + (cannon.w >> 1);
	cy = cannon.y + (cannon.h >> 1);
	q->x = RND( cx + des->attack_cannon_xdist_min, cx + des->attack_cannon_xdist_max );
	q->y = RND( cy + des->attack_cannon_ydist_min, cy + des->attack_cannon_ydist_max );
 
	/*--- Number of	points in the curve	---*/
	ship->downPtNb     = FRND( des->attack_time_descent_min, des->attack_time_descent_max ) / avg_frame_duration;
    ship->downPtIndex  = 0;
	ship->downCtrlPtNb = nCtrlPtNb;


	/*--------------*/
	/*--- (2) Up ---*/
	/*--------------*/
 
	/*--- Generate points of control of the bezier curve ---*/
	q = ship->upPt;
	q->x = cannon.x + (cannon.w >> 1);
	q->y = cannon.y + (cannon.h >> 1);
	nCtrlPtNb = RND( 3, 9 );
	for ( q++, n = 0; n < nCtrlPtNb; n++, q++ )
	{
		q->x = RND( -(frame_width  >> 1), (3 * frame_width)  >> 1 );
		q->y = RND( -(frame_height >> 1), (3 * frame_height) >> 2 );
	}	// for (n)
	q->x = ship->x + RND( -20, 20 );
	q->y = ship->y + RND( -15, 15 );
 
	/*--- Number of	points in the curve	---*/
	ship->upPtNb     = FRND( des->attack_time_up_min, des->attack_time_up_max ) / avg_frame_duration;
	ship->upCtrlPtNb = nCtrlPtNb;

	/*--- Nb of ships attacking now ---*/
	currlevel.nb_ships_attacking++;

}								// start_new_attack


static void
ship_attacking( ship_t *ship )
{
	double		x, y;			// Points computed

	/*---------------------------------------------*/
	/*--- Is the ship attacking ? (down to gun) ---*/
	/*---------------------------------------------*/

	if ( ship->is_downCalcAttacking )
	{

		/*--- End of the curve ? ---*/
		if (ship->downPtNb == ship->downPtIndex)
		{

			ship->is_downCalcAttacking 	= FALSE;	// Computing points for next attack (down)
			ship->is_upCalcAttacking    = TRUE;		// Computing points for next attack (up)
			ship->upPtIndex		   		= 0;		// Index

		}						// if
		else
		{

			Bezier( &x, &y,
				ship->downPtIndex++ / (double)ship->downPtNb, 
				ship->downCtrlPtNb + 1, 
				ship->downPt );
			ship->x = x;
			ship->y = y;

		}						// else

	}							// if (ship->is_downCalcAttacking)


	/*-------------------------------------------*/
	/*--- Is the ship attacking ? (up to pos) ---*/
	/*-------------------------------------------*/

	else if (ship->is_upCalcAttacking)
	{

		/*--- End of the curve ? ---*/
		if (ship->upPtNb == ship->upPtIndex)
		{

			ship->is_upCalcAttacking = FALSE;	// Computing points for next attack (down)
			ship->is_attacking		 = FALSE;	// End of attack
			ship->forcing_visible	 = FALSE;

			/*--- Update attacking ships nb ---*/
			currlevel.nb_ships_attacking--;

		}						// if
		else
		{

			Bezier( &x, &y, 
				ship->upPtIndex++ / (double)ship->upPtNb, 
				ship->upCtrlPtNb + 1, 
				ship->upPt);
			ship->x = x;
			ship->y = y;

		}						// else

	}							// if (ship->is_upCalcAttacking)

	/*--- Does the ship hit the cannon ? ---*/
	if ( !cannon.is_hit )
		is_cannon_hit_by_ship( ship );

}								// ShipAttacking


static void 
force_ship_to_be_visible( ship_t *ship )
	{
	point_t		*p;				// Pointer into ctrl point table
	int			 n;				// Loop	index
	double		 dx, dy;		// Distance in X-Y
	int			 dist;			// The distance between source and destination
	int				w, h;

	/*--- The original starting point ---*/
	p = ship->forcVisPt;
	p[ 0 ].x = ship->x;
	p[ 0 ].y = ship->y;

	/*--- Define control points	of bezier curve	---*/
	for ( n = 1; n <= CTRLPTNB_SHIPFORCVIS; n++ )
	{
		p[ n ].x = RND( 1,	frame_width );
		p[ n ].y = RND( 1,	frame_height );
	} 							// for (n)

	/*--- Where the ship go ---*/
	w = ship->sz[ ship->state ].w;
	h = ship->sz[ ship->state ].h;
	p[ n ].x = RND( w, frame_width - w );
	p[ n ].y = RND( h, frame_height >> 1 );

	/*--- The distance between source and destination ---*/
	dx   = fabs( p[ n ].x - p[ 0 ].x);
	dy   = fabs( p[ n ].y - p[ 0 ].y);
	dist = (int)sqrt( dx*dx + dy*dy );

	/*--- Number of	points in the curve	---*/
	ship->forcVisPtNb     = (int)(dist / (RND( 75, 150 ) / 100.0));
    ship->forcVisPtIndex  = 0;
	ship->forcing_visible = TRUE;			// To be not hidden

	/*--- Does the ship hit the cannon ? ---*/
	if ( !cannon.is_hit )
		is_cannon_hit_by_ship( ship );

}								// force_ship_to_be_visible


static void 
force_ship_visible( ship_t *ship )
{
	double		x, y;			// Points computed

	/*--- End of the curve ? ---*/
    if ( ship->forcVisPtIndex >= ship->forcVisPtNb )
	{

		ship->forcing_visible = FALSE;

	} 							// if
	else
	{

		Bezier( &x, &y, 
			ship->forcVisPtIndex++ / (double)ship->forcVisPtNb, 
			CTRLPTNB_SHIPFORCVIS + 1, 
			ship->forcVisPt);
		ship->x = x;
		ship->y = y;

	} 							// else

}								// force_ship_visible
	

static void 
found_x( ship_t *ship, conf_ship_t *des, int w )
{
	int		cpt;
	double	x;

	for ( cpt = 0; ; cpt++ )
	{
		if ( cpt == 100 )
		{
			if ( x < 0 )
				ship->move_dx = des->xspeed2;
			else
				ship->move_dx = des->xspeed1;
			break;
		}
		ship->move_dx = FRND( des->xspeed1, des->xspeed2 );
		if ( des->perc_reforce_inside_x == 0 )
			break;
		x = ship->x + (ship->move_dx * frame_duration);
		if ( (x >= 0) && (x <= frame_width - w) )
			break;
		if ( RND( 1, 100 ) > des->perc_reforce_inside_x ) 
			break;
	}

}								// found_x


static void 
new_position_ship_random( ship_t *ship, const ship_t *first_ship, 
	conf_ship_t *des,
	const int ydec )
{
	int				cnt;
	double			y;
	int				cpt;
	int				w, h;

	w = ship->sz[ ship->state ].w;
	h = ship->sz[ ship->state ].h;
	if ( !des->moving_grouped || (ship == first_ship) )
	{

		ship->cnt_movex -= frame_duration;
		if ( ship->cnt_movex <= 0 )
		{
			found_x( ship, des, w );
			ship->cnt_movex = FRND( des->xlen1, des->xlen2 );
		}

		ship->cnt_movey -= frame_duration;
		if ( ship->cnt_movey < 0 )
		{
			if ( !des->scroll_bt )
			{
				cnt = 0;
				do
				{
					ship->move_dy = FRND( des->yspeed1, des->yspeed2 );
					y = ship->y + (ship->move_dy * frame_duration);
				}
				while
					((++cnt != 10) && y > (frame_height - des->bottom_ymax) );
				if ( cnt == 10 )
					ship->move_dy = des->yspeed1;
			}					// if
			else
			{
				for ( cpt = 0; ; cpt++ )
				{
					if ( cpt == 100 )
					{
						if ( y < 0 )
							ship->move_dy = des->yspeed2;
						else
							ship->move_dy = des->yspeed1;
						break;
					}
					ship->move_dy = FRND( des->yspeed1, des->yspeed2 );
					if ( des->perc_reforce_inside_y == 0 )
						break;
					y = ship->y + (ship->move_dy * frame_duration);
					if ( (y >= 0) && (y <= frame_height - h) )
						break;
					if ( RND( 1, 100 ) > des->perc_reforce_inside_y ) 
						break;
				}
			}					// else
			ship->cnt_movey = FRND( des->ylen1, des->ylen2 );
		}

		ship->x += (ship->move_dx * frame_duration);
		ship->y += (ship->move_dy * frame_duration);

	}
	else
	{

		ship->x += (first_ship->move_dx * frame_duration);
		if ( ydec != INT_MAX )
			ship->y += (first_ship->move_dy * frame_duration);
		else
			ship->y -= ydec;

	}

}								// new_position_ship_random


static void 
new_position_ship_ellipse( ship_t *ship, const ship_t *first_ship, 
	conf_ship_t *des,
	const int ydec )
{
	int				w, h;

	w = ship->sz[ ship->state ].w;
	h = ship->sz[ ship->state ].h;
	if ( !des->moving_grouped || (ship == first_ship) )
	{
		ship->alpha += FRND( 0.0040, 0.035 );
		if ( ship->alpha >= M_PI )
		{
			ship->alpha = -M_PI;
			ship->x0 = ship->x;
			ship->y0 = ship->y;
			found_x( ship, des, w );
			ship->move_dx *= 1000000.0;
			ship->move_dy = FRND( des->yspeed1, des->yspeed2 ) * 1000000.0;
		}
		ship->x = (ship->x0 + ship->move_dx) + cos( ship->alpha ) * (ship->move_dx);
		ship->y = (ship->y0) + sin( ship->alpha ) * (ship->move_dy);
	}
	else
	{

		assert( 0 );

	}


}								// new_position_ship_ellipse


static void 
new_position_ship( ship_t *ship, const ship_t *first_ship, const int ydec )
{
	conf_ship_t		*des;

	des = ship->des;
	switch ( des->type_movment )
	{
		case TYPMOV_RANDOM :
			new_position_ship_random( ship, first_ship, ship->des, ydec );
			break;
		case TYPMOV_ELLIPSE :
			new_position_ship_ellipse( ship, first_ship, ship->des, ydec );
			break;
		default :
			assert ( 0 );
	}

	/*--- In case of scrolling ---*/
	if ( des->scroll_lr )
	{
		if ( ship->x + ship->sz[ ship->state ].w < 0 )
			ship->x = frame_width - ship->sz[ ship->state ].w;
		else
			ship->x = fmod( ship->x, frame_width );
	}
	if ( des->scroll_bt )
	{
		ship->y = fmod( ship->y, frame_height );
		ship->y = abs( ship->y );
	}

	/*--- Does the ship hit the cannon ? ---*/
	if ( !cannon.is_hit )
		is_cannon_hit_by_ship( ship );

}								// new_position_ship


/*--- --------------------- ---*/
/*--- Move the ships armada ---*/
/*--- --------------------- ---*/

PUBLIC void 
move_ships( void )
{
	ship_t			*ship;
	conf_ship_t		*des;
	int				n;
	ship_t			*first_ship;
	int				ydec;
	time_t			currtime;
	int				change_state;
	int				bottom_ymax;

	/*--- Ships too low ? ---*/
	for ( ydec = INT_MIN, ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
	{
		bottom_ymax = frame_height - ship->des->bottom_ymax;
		if ( ship->y > bottom_ymax )
			if ( ship->y - bottom_ymax > ydec )
				ydec = (ship->y - bottom_ymax);
	}

	/*--- For each ship ---*/
	currtime = time( NULL );
	for ( first_ship = NULL, ship = ships, n = 0; n < currlevel.nbships; n++, ship++ )
	{

		/*--- ----------------------------- ---*/
		/*--- Nothing to do if ship is dead ---*/
		/*--- ----------------------------- ---*/

		if ( ship->is_dead )
			continue;
		des = ship->des;
		if ( _now - ship->last_state_when < des->state_trans_time )
			change_state = FALSE;
		else
		{
			change_state = TRUE;
			ship->last_state_when = _now;
		}
		if ( !first_ship )
			first_ship = ship;
			

		/*--- ------------ ---*/
		/*--- New position ---*/
		/*--- ------------ ---*/

		if ( ship->is_attacking )
		{
			ship_attacking( ship );
		}
		else if ( ship->forcing_visible )
		{
			force_ship_visible( ship );
		}
		else
		{
			int	skip = FALSE;
			if ( des->maxnb_sec_hidden )
			{
				ship->is_nonvisible = (ship->x < 0) || (ship->x >= frame_width) ||
					(ship->y < 0) || (ship->y >= frame_height);
				if ( !ship->is_nonvisible )
					ship->when_nonvisible = 0;
				else
					if ( !ship->when_nonvisible )
						ship->when_nonvisible = currtime;

				if ( ship->is_nonvisible && 
					((currtime - ship->when_nonvisible) >= des->maxnb_sec_hidden) )
				{
					force_ship_to_be_visible( ship );
					skip = TRUE;
				}
			}
			if ( !skip )
				new_position_ship( ship, first_ship, ydec );
		}


		/*--- ---------------------------------------- ---*/
		/*--- Copy the ship bitmap to the frame buffer ---*/
		/*--- ---------------------------------------- ---*/

		/*--- Used by the low-level function blaster_or ---*/
		is_scrolling_lr = des->scroll_lr;
		is_scrolling_bt = des->scroll_bt;

		if ( ship->is_hit )
			make_hit_ship_bitmap( ship, des );
		gdk_draw_pixmap( pixmap_frame, 
			BACKGROUND_GC,
			ship->img_des[ ship->state ],
			0, 0, ship->x, ship->y,
			ship->sz[ ship->state ].w, ship->sz[ ship->state ].h );


		/*--- --------------------------------- ---*/
		/*--- Compute the new state of the ship ---*/
		/*--- --------------------------------- ---*/

		if ( change_state )
		{
			if ( des->nb_states > 1 )
			{
				ship->prev_state = ship->state;
				if ( des->rounding )
				{
					if ( ship->inc == 1 )
					{
						if ( ++ship->state == des->nb_states )
						{
							ship->state -= 2;
							ship->inc = -1;
						}
					}
					else
					{
						if ( !ship->state-- )
						{
							ship->state += 2;
							ship->inc = 1;
						}
					}
				}				// if
				else
				{
					ship->state = ((ship->state + 1) % des->nb_states);
				}
			}					// if
		}						// if

		/*--- --------------------------- ---*/
		/*--- Adjust position of the ship ---*/
		/*--- --------------------------- ---*/

		if ( change_state )
		{
			if ( des->nb_states > 1 )
			{
				if ( !ship->state && (ship->prev_state == (des->nb_states-1)) )
				{
					ship->x += des->offs[ 0 ].x;
					ship->y += des->offs[ 0 ].y;
				}
				else
				{
					if ( ship->state > ship->prev_state )
					{
						ship->x += des->offs[ ship->state ].x;
						ship->y += des->offs[ ship->state ].y;
					}
					else
					{
						ship->x -= des->offs[ ship->prev_state ].x;
						ship->y -= des->offs[ ship->prev_state ].y;
					}
				}					// if
			}						// if
		}


#if 1
		/*--- -------------------------- ---*/
		/*--- Time to drop a new shell ? ---*/
		/*--- -------------------------- ---*/

		if ( !cannon.is_hit && !is_starting_cannon &&
			 (!ship->is_hit || des->destroyed_ships_active) )
		{
			if ( (ship->x >= 0) && (ship->x < frame_width) )
			{
				double k = 60000000.0 / des->shells_drop_avg;
				if ( FRND( 0, k ) < frame_duration )
					drop_shell( ship );
			}
		}
#endif


#if 1
		/*--- ---------------------------- ---*/
		/*--- Time to start a new attack ? ---*/
		/*--- ---------------------------- ---*/

		if ( !ship->forcing_visible && !cannon.is_hit && !is_starting_cannon )
		{
			if ( currlevel.nb_ships_attacking < currlevel.maxnb_attacks )
			{
				double k = 60000000.0 / des->attack_avg;
				if ( FRND( 0, k ) < frame_duration )
					start_new_attack( ship );
			}
		}
#endif


		ship->prev_state = ship->state;		

	}							// for (ship, n)

}								// move_ships
