#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include <sched.h>
#include <assert.h>

#include <gtk/gtk.h>

#define COMMON extern
#include "glg/main.h"

#include "glg/modules.h"


static void
init_first( void )
{

	gtk_widget_set( GTK_WIDGET( wgt_btn_pause ), 
		"sensitive", FALSE,
		NULL );

	/*--- Get the bitmap of the shell ---*/
	load_shell();

	/*--- Get the bitmaps of the cannon ---*/
	load_cannon();	
	place_cannon();

	/*--- Get the bitmaps of the missile ---*/
	load_missiles();

	/*--- Initialize the Mother Ship ---*/
	load_mother();

	/*--- Get the bitmaps for the stars ---*/
	load_stars();

}								// init_first


static void
create_frame( void )
{

	if ( pixmap_frame != NULL )
		gdk_pixmap_unref( pixmap_frame );

	pixmap_frame = gdk_pixmap_new( wgt_spacearea->window, 
		frame_width, frame_height, -1 );
	gdk_pixmap_ref( pixmap_frame );

	/*--- Erase everything into the frame buffer ---*/
	gdk_draw_rectangle( pixmap_frame, 
		BACKGROUND_GC,
		TRUE,
		0, 0, frame_width, frame_height );

}								// create_frame


PUBLIC gint
draw_frame( gpointer data )
{
static now_t	prev = 0;

	if ( is_paused )
		return FALSE;

	_now = now();
	frame_duration = _now - prev;
	prev = _now;

	/*--- Erase everything into the frame buffer ---*/
	gdk_draw_rectangle( pixmap_frame, 
		BACKGROUND_GC,
		TRUE,
		0, 0, frame_width, frame_height );

	/*--- Draw the stars ---*/
	draw_stars();

	if ( mother.is_active )
	{
	
		draw_mother();
	
	}							// if
	else
	{

		/*--- Move the ships armada ---*/
		move_ships();

		/*--- Draw the starting cannon ---*/
		if ( is_starting_cannon )
		{
			move_starting_cannon();
			draw_starting_cannon();
		}

		/*--- Move the shells ---*/
		move_shells();

		/*--- Draw the cannon ---*/
		if ( !is_starting_cannon )
			draw_cannon();

		/*--- Move the missile, if any ---*/
		draw_missiles();

		/*--- Update the fuel level ---*/
		update_fuel();

		/*--- Update the laser level ---*/
		update_laser();

	}							// else

	/*--- Now draw the frame buffer to the screen ---*/
	gdk_draw_pixmap( wgt_spacearea->window, 
		BACKGROUND_GC,
		pixmap_frame,
		0, 0, 0, 0,
		frame_width, frame_height );
	gdk_flush();

	if ( frame_duration_nb >= 1000 )
	{
		frame_duration_nb = 0;
		frame_duration_cumul = 0;
	}
	frame_duration_cumul += (now() - _now);
	frame_duration_nb++;
	avg_frame_duration = frame_duration_cumul  / frame_duration_nb;
	if ( !( ((int)frame_duration_nb - 1) % 100 ) )
		set_label( wgt_info_frame, "%.0f micro-seconds/frame", 
			avg_frame_duration );

	return TRUE;

}								// draw_frame


/* Callback "expose_event" for GtkDrawingArea widget wgt_spacearea */
gboolean
cb_wgt_spacearea_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{

	/*--- Draw the frame buffer to the screen ---*/
	gdk_draw_pixmap( wgt_spacearea->window, 
		BACKGROUND_GC,
		pixmap_frame,
		0, 0, 0, 0,
		frame_width, frame_height );

	return FALSE;
}								// cb_wgt_spacearea_expose


/* Callback "configure_event" for GtkDrawingArea widget wgt_spacearea */
gboolean
cb_wgt_spacearea_configure(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
	static int	first = TRUE;

	frame_width  = widget->allocation.width;
	frame_height = widget->allocation.height;
	create_frame();

	if ( first )
	{
		init_first();
		first = FALSE;
	}

	return FALSE;
	
}								// cb_wgt_spacearea_configure
