#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include <gtk/gtk.h>

#include "glg/main.h"

#include "glg/modules.h"


static GdkColor *
NewColor(const long red, const long green, const long blue)
{
	/* --- Get the color --- */
	GdkColor *c = (GdkColor *) g_malloc(sizeof(GdkColor));

	/* --- Fill it in. --- */
	c->red = red;
	c->green = green;
	c->blue = blue;

	gdk_color_alloc(gdk_colormap_get_system(), c);

	return (c);
}				// NewColor

static GdkGC *
GetPen(GdkColor * c)
{
	GdkGC *gc;

	/* --- Create a gc --- */
	gc = gdk_gc_new(matches->window);

	/* --- Set the forground to the color --- */
	gdk_gc_set_foreground(gc, c);

	/* --- Return it. --- */
	return (gc);
}				// GetPen


/* Callback "expose_event" for GtkDrawingArea widget wgt_field */
gboolean
cb_field_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
		int		x, z, n;
static	GdkGC	*gc_orange = NULL;
static	GdkGC	*gc_red = NULL;

	if (gc_orange == NULL)
	{
		gc_orange = GetPen( NewColor(0xE800, 0xE800, 0) );
		gc_red 	  = GetPen( NewColor(0xE800, 0, 0) );
	}

	gdk_window_clear_area (widget->window,
		event->area.x, event->area.y,
		event->area.width, event->area.height);
	gdk_gc_set_clip_rectangle (widget->style->fg_gc[widget->state],
		&event->area);

#define MATCH_WIDTH	14
#define MATCH_INTER	7
	for ( n = 0; n < nb_matches; n++ )
	{
		z = widget->allocation.height / 8;	
		x = ((n+1)*MATCH_INTER) + (n * MATCH_WIDTH);
    	gdk_draw_rectangle(widget->window,
			gc_red,
			TRUE,
			x, 1 * z, 
			MATCH_WIDTH, z);
    	gdk_draw_rectangle(widget->window,
			gc_orange,
			TRUE,
			x, 2 * z, 
			MATCH_WIDTH, z * 5 );
	}

    gdk_gc_set_clip_rectangle (widget->style->fg_gc[widget->state],
		NULL);

	return FALSE;
	
}								// cb_field_expose


static void
computer_play( void )
{
	int		nb;
	char	tmp[80];

	/*--- How many matches to blow ? ---*/
	if ( (nb_matches-1)%4 == 0 )
		nb = MIN( (rand() % 3) + 1, nb_matches );
	else
		nb = (nb_matches-1)%4;
	nb_matches -= nb;
	gtk_widget_queue_draw( wgt_field );
	sprintf( tmp, "I blow %d match%s%s",
		nb, (nb > 1) ? "es" : "",
		(nb_matches == 0) ? " and I have LOST!" : "" );
	gtk_label_set_text( GTK_LABEL(wgt_computer_play), tmp );

	gtk_widget_set_sensitive( wgt_user_play, (nb_matches >= 1) );
	gtk_widget_set_sensitive( wgt_btn_blow1, (nb_matches >= 1) );
	gtk_widget_set_sensitive( wgt_btn_blow2, (nb_matches >= 2) );
	gtk_widget_set_sensitive( wgt_btn_blow3, (nb_matches >= 3) );

	if ( nb_matches == 0 )
		gtk_widget_show_all( wgt_whobegins );

}								// computer_play


static void
blow( const int blowed )
{

	nb_matches -= blowed;
	gtk_widget_queue_draw( wgt_field );
	gtk_widget_set_sensitive( wgt_user_play, FALSE );

	if ( nb_matches == 0 )
	{
		gtk_widget_show_all( wgt_whobegins );
		gtk_label_set_text( GTK_LABEL(wgt_computer_play), "You have LOST!" );
	}
	else
		computer_play();
}								// blow


/* Callback "clicked" for GtkButton widget wgt_btn_blow1 */
void
cb_blow_match1(GtkButton *button, gpointer user_data)
{
	blow(1);
}								// cb_blow_match1

/* Callback "clicked" for GtkButton widget wgt_btn_blow2 */
void
cb_blow_match2(GtkButton *button, gpointer user_data)
{
	blow(2);
}								// cb_blow_match2

/* Callback "clicked" for GtkButton widget wgt_btn_blow3 */
void
cb_blow_match3(GtkButton *button, gpointer user_data)
{
	blow(3);
}								// cb_blow_match3


/* Callback "clicked" for GtkButton widget wgt_4_16 */
void
cb_start_user(GtkButton *button, gpointer user_data)
{
	gtk_widget_hide_all( wgt_whobegins );
	nb_matches = 21;
	gtk_widget_set_sensitive( wgt_user_play, TRUE );

	gtk_widget_set_sensitive( wgt_btn_blow1, (nb_matches >= 1) );
	gtk_widget_set_sensitive( wgt_btn_blow2, (nb_matches >= 2) );
	gtk_widget_set_sensitive( wgt_btn_blow3, (nb_matches >= 3) );

}								// cb_start_user


/* Callback "clicked" for GtkButton widget wgt_4_17 */
void
cb_start_computer(GtkButton *button, gpointer user_data)
{
	gtk_widget_hide_all( wgt_whobegins );
	nb_matches = 21;
	gtk_widget_set_sensitive( wgt_user_play, TRUE );

	gtk_widget_set_sensitive( wgt_btn_blow1, (nb_matches >= 1) );
	gtk_widget_set_sensitive( wgt_btn_blow2, (nb_matches >= 2) );
	gtk_widget_set_sensitive( wgt_btn_blow3, (nb_matches >= 3) );

	computer_play();

}								// cb_start_computer

/* Callback "delete_event" for GtkWindow widget matches */
gboolean
cb_window_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	exit(0);	
	return FALSE;	
}								// cb_window_delete_event
