#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <errno.h>
#include <inttypes.h>
#include <assert.h>

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif /* HAVE_SYS_SELECT_H */

#ifndef NO_FD_SET
#  define SELECT_MASK fd_set
#else
#  ifndef _AIX
     typedef long fd_mask;
#  endif
#  if defined(_IBMR2)
#    define SELECT_MASK void
#  else
#    define SELECT_MASK int
#  endif
#endif

#include "glg.h"

#define FIELD_EMPTY ' '


static GtkWidget	*wgt_window_comprun = NULL;
static GtkWidget	*wgt_comprun_term	= NULL;
static GtkWidget	*wgt_comprun_btn_generate;
static GtkWidget	*wgt_comprun_btn_make;
static GtkWidget	*wgt_comprun_btn_run;
static GtkWidget	*wgt_comprun_btn_stop;
static GtkWidget	*wgt_comprun_btn_edit;
static GtkWidget	*wgt_comprun_btn_ok;
static GtkWidget	*wgt_comprun_statusbar;

static	int			f_done;
static	pid_t 		pid_run;

#define DEFT_ROWS		25
#define DEFT_COLUMNS	80
static	GdkFont		*term_font;
static	int			term_width, term_height;
static	int			char_width, char_height;
static	char		*term_field = NULL;
static	int			term_row, term_col;
static	int			term_nb_rows, term_nb_cols;
static	int			prev_term_nb_rows, prev_term_nb_cols;

#define TERM_DISPLAY( R, C ) \
	gdk_window_clear_area( \
		wgt_comprun_term->window, \
		(C) * char_width, \
		(R) * char_height, \
		char_width, \
		char_height ); \
   	gdk_draw_text( \
		wgt_comprun_term->window, \
		term_font, \
		wgt_comprun_term->style->fg_gc[wgt_comprun_term->state], \
		(C) * char_width, \
		(R) * char_height + char_height, \
		&term_field[ (R)*term_nb_cols+(C) ], 1 )


static void
term_clear( void )
{

	memset( term_field, FIELD_EMPTY, term_nb_rows * term_nb_cols );
	term_row = 0;
	term_col = 0;

	gtk_widget_queue_draw( wgt_comprun_term );
	gdk_flush();

	while ( gtk_events_pending() )
		gtk_main_iteration();
	
}								// term_clear


static void
term_putc( char c )
{

	if (c != '\n')
	{
		term_field[ term_row * term_nb_cols + term_col ] = c;
		TERM_DISPLAY( term_row, term_col );
	}
	else
	{
		term_col = term_nb_cols - 1;
	}

	if (++term_col == term_nb_cols)
	{
		term_col = 0;
		if (++term_row < term_nb_rows )
		{
		}
		else
		{
			int		row, x, y;
			
			for ( row = 1; row < term_nb_rows; row++ )
			{
				memmove( &term_field[ (row - 1) * term_nb_cols ],
						 &term_field[ row * term_nb_cols ],
						 term_nb_cols );
			}
			memset( &term_field[ (row - 1) * term_nb_cols ], FIELD_EMPTY, term_nb_cols );

			for ( row = 0; row < term_nb_rows; row++)
			{
				x = 0;
				y = (row+1) * char_height;
				gdk_window_clear_area(
					wgt_comprun_term->window,
					0,
					y - char_height,
					char_width * term_nb_cols,
					char_height );
    			gdk_draw_text(
					wgt_comprun_term->window,
					term_font,
					wgt_comprun_term->style->fg_gc[wgt_comprun_term->state],
					x, 
					y,
					&term_field[ row * term_nb_cols ], term_nb_cols );
			}
			term_row--; 

		}
		gdk_flush();
	}

}								// term_putc


static void
term_printf( char *fmt, ... )
{
	va_list		ap;
	char		buff[300], *p;

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	for ( p = buff; *p; p++ )
		term_putc( *p );

	gdk_flush();

}								// term_printf


static void
sigchld_sighandler( int signum )
{
	f_done = TRUE;
}								// sigchld_sighandler


static int
launch( char *cmd )
{
	char			objdir[ PATH_MAX+NAME_MAX + 80 ];
	pid_t			pid;
	int				in_fd[2], out_fd[2];
	SELECT_MASK		fdset;
	SELECT_MASK		readset;
	struct timeval	tv;
	int 			sel;
	char 			c;
	char			*args_make[] = { "make", NULL };
	char			*args_gmake[] = { "gmake", NULL };
	GdkCursor		*cursor;

	/*--- Where to run make ? ---*/
	sprintf( objdir, "%s/obj", project_dirname );
	if ( chdir(objdir) == -1 )
	{
		return -1;
	}

	signal( SIGCHLD, sigchld_sighandler );
	
	if ((pipe (in_fd) == -1) || (pipe (out_fd) == -1))
	{
		perror ("unable to open pipe");
		_exit (0);
	}

	f_done = FALSE;
	pid = fork();
	if ( pid == 0 )
    {
		close( 0 ); dup( in_fd[ 0 ] );		/* set the stdin to the in pipe */
		close( 1 ); dup( out_fd[ 1 ] );		/* set the stdout to the out pipe */
		close( 2 ); dup( out_fd[ 1 ] );		/* set the stderr to the out pipe */

		if ( syst == SYST_FREEBSD )
			execvp( args_gmake[ 0 ], args_gmake );	/* exec gmake */
		else
			execvp( args_make[ 0 ], args_make );	/* exec make */
		perror( "exec failed" );
		_exit( 0 );
    }
	else if ( pid == (pid_t) -1 )
    {
		perror ("unable to fork");
		_exit (0);
    }

	/*--- Change the cursor to 'Busy' ---*/
	cursor = gdk_cursor_new( GDK_WATCH );
	gdk_window_set_cursor( wgt_window_comprun->window, cursor );
	gdk_cursor_destroy( cursor );

	FD_ZERO( &fdset );
	FD_SET( out_fd[ 0 ], &fdset );

	while (1)
    {
		readset = fdset;
		tv.tv_sec = 0;
		tv.tv_usec = 250000;

		sel = select (FD_SETSIZE, &readset, NULL, NULL, &tv);
		if (sel == -1)
			break;

		if ((sel > 0) && (FD_ISSET (out_fd[0], &readset)))
		{
			if (read (out_fd[0], &c, 1))
			{
				term_putc( c );
			}
			else
			{
				while ( gtk_events_pending() )
					gtk_main_iteration();
			}
		}
		else if (f_done)
			break;
	}

	/*--- Change the cursor to 'Busy' ---*/
	cursor = gdk_cursor_new( GDK_ARROW );
	gdk_window_set_cursor( wgt_window_comprun->window, cursor );
	gdk_cursor_destroy( cursor );

	/*--- Ok ---*/
	close( in_fd[0] );
	close( in_fd[1] );
	close( out_fd[0] );
	close( out_fd[1 ]);
	return 0;

}								// launch


static void
update_btn_run( void )
{
	char	exe_name[PATH_MAX+NAME_MAX + 100];
	int		ok;

	sprintf( exe_name, "%s/obj/%s", project_dirname, config.exe_name );
	ok = (access( exe_name, F_OK ) == 0) ? TRUE : FALSE;
	
	INIT_ARGS;
	SETARG_BOOL( "sensitive", ok );
	gtk_widget_setv( GTK_WIDGET(wgt_comprun_btn_run), nb_args, args );

}								// update_btn_run


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_ok */
static void
cb_comprun_btn_ok(GtkButton *button, gpointer user_data)
{
	gtk_widget_destroy( wgt_window_comprun );
	wgt_window_comprun = NULL;
}								// cb_comprun_btn_ok


/* Callback "expose_event" for GtkDrawingArea widget wgt_comprun_term */
static gboolean
cb_comprun_term_expose( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	int		row;
	int		x, y;

	for ( row = 0; row < term_nb_rows; row++)
	{
		x = 0;
		y = (row+1) * char_height;
		gdk_window_clear_area(
			wgt_comprun_term->window,
			0,
			y - char_height,
			char_width * term_nb_cols,
			char_height );
    	gdk_draw_text(
			wgt_comprun_term->window,
			term_font,
			wgt_comprun_term->style->fg_gc[wgt_comprun_term->state],
			x, 
			y,
			&term_field[ row * term_nb_cols ], term_nb_cols );
 	}

	gdk_flush();
		
	return FALSE;
}								// cb_comprun_term_expose


/* Callback "expose_event" for GtkDrawingArea widget wgt_comprun_term */
static void
cb_comprun_term_size_allocate( GtkWidget *widget, GtkAllocation *allocation, gpointer user_data )
{
	/*--- Recompute the number of rows, columns ---*/
	term_nb_rows = allocation->height / char_height;
	term_nb_cols = allocation->width / char_width;

	/*--- Allocate space to hold terminal text ---*/
	if ( (term_nb_rows != prev_term_nb_rows) || (term_nb_cols != prev_term_nb_cols) )
	{
		if ( term_field != NULL )
			free( term_field );
		term_field = malloc( term_nb_rows * term_nb_cols );
		memset( term_field, FIELD_EMPTY, term_nb_rows * term_nb_cols );
		prev_term_nb_rows = term_nb_rows;
		prev_term_nb_cols = term_nb_cols;
	}

}								// cb_comprun_term_size_allocate


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_generate */
static void
cb_comprun_btn_generate(GtkButton *button, gpointer user_data)
{
	term_clear();
	generate( project_dirname, term_printf );
}								// cb_comprun_btn_generate


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_make */
static void
cb_comprun_btn_make(GtkButton *button, gpointer user_data)
{
	term_clear();
	if ( syst == SYST_FREEBSD )
		launch( "gmake" );
	else
		launch( "make" );
	update_btn_run();
}								// cb_comprun_btn_make


static void
enable_btn( GtkWidget *wgt, int ok )
{

	INIT_ARGS;
	SETARG_BOOL( "sensitive", ok );
	gtk_widget_setv( wgt, nb_args, args );

}								// enable_btn


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_run */
static void
cb_comprun_btn_run(GtkButton *button, gpointer user_data)
{
	char	 	exe_name[PATH_MAX+NAME_MAX + 100];
	char	 	*args[2];

	sprintf( exe_name, "%s/obj/%s", project_dirname, config.exe_name );
	args[ 0 ] = exe_name;
	args[ 1 ] = NULL;

	enable_btn( wgt_comprun_btn_generate, FALSE );
	enable_btn( wgt_comprun_btn_make, FALSE );
	enable_btn( wgt_comprun_btn_run, FALSE );
	enable_btn( wgt_comprun_btn_stop, TRUE );
	enable_btn( wgt_comprun_btn_ok, FALSE );

	signal( SIGCHLD, sigchld_sighandler );

	f_done = FALSE;
	pid_run = fork ();
	if (pid_run == 0)
    {
		execvp( args[0], args );
		modal_error( "Exec failed\n%s\nerrno=%d", args[0], errno );
		return;
    }
	else if (pid_run == (pid_t) -1)
    {
		modal_error( "Unable to fork!\nerrno=%d", errno );
		return;
    }

	while ( !f_done )
		gtk_main_iteration();

	f_done = FALSE;
	enable_btn( wgt_comprun_btn_generate, TRUE );
	enable_btn( wgt_comprun_btn_make, TRUE );
	enable_btn( wgt_comprun_btn_run, TRUE );
	enable_btn( wgt_comprun_btn_stop, FALSE );
	enable_btn( wgt_comprun_btn_ok, TRUE );

}								// cb_comprun_btn_run


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_stop */
static void
cb_comprun_btn_stop(GtkButton *button, gpointer user_data)
{
	kill( pid_run, SIGINT );
}								// cb_comprun_btn_stop


/* Callback "clicked" for GtkButton widget wgt_comprun_btn_edit */
static void
cb_comprun_btn_edit(GtkButton *button, gpointer user_data)
{
}								// cb_comprun_btn_edit


static GtkWidget *
create_window( GtkWidget *parent, const int realize )
{
	GtkWidget	*wgt_window;
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_2_4, *wgt_2_5;
	
	INIT_ARGS;
	SETARG_STRING( "title", "Compile and Run" );
	SETARG_BOOL( "allow_grow", 1 );
	SETARG_BOOL( "allow_shrink", 1 );
	wgt_window = gtk_widget_newv( gtk_window_get_type(), nb_args, args );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_window );
	SETARG_ULONG( "border_width", 2 );
	SETARG_INT( "spacing", 4 );
	wgt_1_2 = gtk_widget_newv( gtk_vbox_get_type(), nb_args, args );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_1_2 );
	wgt_2_3 = gtk_widget_newv( gtk_hbox_get_type(), nb_args, args );

	INIT_ARGS;
	SETARG_INT( "width", term_width );
	SETARG_INT( "height", term_height );
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_3 );
	wgt_comprun_term = gtk_widget_newv( gtk_drawing_area_get_type(), nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_term ), "expose_event",
		GTK_SIGNAL_FUNC( cb_comprun_term_expose ), NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_term ), "size_allocate",
		GTK_SIGNAL_FUNC( cb_comprun_term_size_allocate ), NULL );
		
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_1_2 );
	wgt_2_4 = gtk_widget_newv( gtk_hbox_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_BOOL( "expand", 0 );
	SETARG_LONG( "position", 1 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_1_2 ), wgt_2_4, nb_args, args );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_4 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	SETARG_STRING( "label", "Generate" );
	wgt_comprun_btn_generate = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_generate ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_generate ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_4 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	if ( syst == SYST_FREEBSD )
	{
		SETARG_STRING( "label", "gmake" );
	}
	else
	{
		SETARG_STRING( "label", "make" );
	}
	wgt_comprun_btn_make = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_LONG( "position", 1 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_2_4 ), wgt_comprun_btn_make, nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_make ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_make ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_4 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	SETARG_STRING( "label", "Run" );
	wgt_comprun_btn_run = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_LONG( "position", 2 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_2_4 ), wgt_comprun_btn_run, nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_run ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_run ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_4 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	SETARG_STRING( "label", "Stop" );
	SETARG_BOOL( "sensitive", FALSE );
	wgt_comprun_btn_stop = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_LONG( "position", 3 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_2_4 ), wgt_comprun_btn_stop, nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_stop ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_stop ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_4 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	SETARG_BOOL( "sensitive", 0 );
	SETARG_STRING( "label", "Edit" );
	wgt_comprun_btn_edit = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_LONG( "position", 4 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_2_4 ), wgt_comprun_btn_edit, nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_edit ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_edit ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_1_2 );
	wgt_2_5 = gtk_widget_newv( gtk_hbox_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_BOOL( "expand", 0 );
	SETARG_LONG( "position", 2 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_1_2 ), wgt_2_5, nb_args, args );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_2_5 );
	SETARG_BOOL( "can_focus", 1 );
	SETARG_BOOL( "receives_default", 1 );
	SETARG_STRING( "label", "Close" );
	wgt_comprun_btn_ok = gtk_widget_newv( gtk_button_get_type(), nb_args, args );
	gtk_signal_connect( GTK_OBJECT( wgt_comprun_btn_ok ), "clicked",
		GTK_SIGNAL_FUNC( cb_comprun_btn_ok ), NULL );

	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_1_2 );
	SETARG_INT( "spacing", 2 );
	wgt_comprun_statusbar = gtk_widget_newv( gtk_statusbar_get_type(), nb_args, args );
	INIT_ARGS;
	SETARG_BOOL( "expand", 0 );
	SETARG_LONG( "position", 3 );
	gtk_container_child_setv( GTK_CONTAINER( wgt_1_2 ), wgt_comprun_statusbar, nb_args, args );

	update_btn_run();

	if ( realize )
		gtk_widget_show_all( wgt_window );

	return wgt_window;

}								// create_window


PUBLIC void
comprun_dlg( void )
{
	gint	lbearing, rbearing, width, ascent, descent;

	/*--- Load font to display term ---*/
	if ( (term_font = gdk_font_load("*courier-*-r*120*")) == NULL )
	{
		if ((term_font = gdk_font_load("*courier-*-r*100*")) == NULL)
		{
			fprintf( stderr, "Unable to load courier font\n" );
			return;;
		}
	}

	/*--- Compute term size (25x80) ---*/
	gdk_string_extents( term_font, "X", &lbearing, &rbearing, &width, &ascent, &descent );
#if 0
trace( " %d %d %d %d %d (%d)\n",
lbearing, rbearing, width, ascent, descent,
gdk_string_height( term_font, "X" ) );
#endif
	char_width = width;
	char_height = (ascent + descent) + 10;
	term_nb_cols = DEFT_COLUMNS;
	term_nb_rows = DEFT_ROWS;
	prev_term_nb_cols = -1;
	prev_term_nb_rows = -1;
	term_width  = term_nb_cols * char_width + 3;
	term_height = term_nb_rows * char_height;
	term_field  = NULL;
	term_row = 0;
	term_col = 0;

	wgt_window_comprun = create_window( NULL, TRUE );
	gtk_window_set_modal( GTK_WINDOW(wgt_window_comprun), TRUE );
	push_block_all_windows();
	while ( wgt_window_comprun != NULL )
		gtk_main_iteration();
	pop_block_all_windows();

}								// comprun_dlg

