#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include <inttypes.h>

#include "glg.h"


typedef struct
{
	gtk_wgtdef_t	*wgtdef;
	int				depth;
	int				index;
}	specialres_t;


static void
cb_treemenu_insertnewwgt( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkWidget		*wgt_container = GTK_WIDGET(user_data);
	module_info_t	*module;
	gtk_wgtdef_t	*wgtdef_newwgt;

	module = get_object_data( GTK_WIDGET(wgt_container), WGTDATA_MODULE );
	assert( module != NULL );

	assert( wgt_selected_wgt_palette != NULL );
	wgtdef_newwgt = get_object_data( GTK_WIDGET(wgt_selected_wgt_palette), WGTDATA_WGTDEF );
	assert( wgtdef_newwgt != NULL );

	INIT_ARGS;
	insert_new_widget( wgt_container, wgtdef_newwgt,
		TRUE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		NULL );

	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
}								// cb_treemenu_insertnewwgt


#ifdef GLG_SUPPORT_COPYPASTE

static void
cb_treemenu_copy( GtkMenuItem *menuitem, gpointer user_data )
{
}								// cb_treemenu_copy


static void
cb_treemenu_cut( GtkMenuItem *menuitem, gpointer user_data )
{
}								// cb_treemenu_cut


static void
cb_treemenu_paste( GtkMenuItem *menuitem, gpointer user_data )
{
}								// cb_treemenu_paste

#endif

static void
cb_treemenu_delete( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkWidget	*wgt = GTK_WIDGET(user_data);
	module_info_t	*module;

	/*--- Get the module_info stored into the line ---*/	
	module = get_object_data( wgt, WGTDATA_MODULE );
	assert( module != NULL );

	select_widget( module->wgt_window, 
		TRUE,
		TRUE );					// Select CB, if any
	gtk_widget_destroy( wgt );
	update_tree();

	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
}								// cb_treemenu_delete


static void
cb_treemenu_expand( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkCTreeNode	*node = (GtkCTreeNode *)user_data;
	
	gtk_ctree_expand_recursive( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(node) );
	
}								// cb_treemenu_expand


static void
cb_treemenu_collapse( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkCTreeNode	*node = (GtkCTreeNode *)user_data;
	
	gtk_ctree_collapse_recursive( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(node) );
	
}								// cb_treemenu_collapse


#ifdef GLG_SUPPORT_WIDGET_MAKE_DEFAULT
static void
cb_treemenu_makedefault( GtkMenuItem *menuitem, gpointer user_data )
{
	g_warning( "make default widget... \n" );
}								// cb_treemenu_makedefault


static void
cb_treemenu_resetdefault( GtkMenuItem *menuitem, gpointer user_data )
{
	g_warning( "reset default widget... \n" );
}								// cb_treemenu_resetdefault
#endif


#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
static void
cb_treemenu_createtemplate( GtkMenuItem *menuitem, gpointer user_data )
{
	g_warning( "create template widget... \n" );
}								// cb_treemenu_createtemplate
#endif

static void
cb_treemenu_displaygtktype( GtkCheckMenuItem *menuitem, gpointer user_data )
{
	config.tree_is_display_gtktype = !config.tree_is_display_gtktype;
	is_project_touched = TRUE;
	update_tree();
}								// cb_treemenu_displaygtktype


static void
cb_treemenu_displayicon( GtkCheckMenuItem *menuitem, gpointer user_data )
{
	config.tree_is_display_icon = !config.tree_is_display_icon;
	is_project_touched = TRUE;
	update_tree();
}								// cb_treemenu_displayicon


static void
cb_treemenu_editstyle( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkStyle	*style = (GtkStyle *)user_data;
	
	edit_styles( style );
	
}								// cb_treemenu_editstyle


static void
cb_treemenu_edit_cb( GtkMenuItem *menuitem, gpointer user_data )
{
	def_cb_t	*defcb = (def_cb_t *)user_data;
	GtkWidget	*wgt_selected;

	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	
	edit_cb( wgt_selected, defcb );

}								// cb_treemenu_edit_cb


static void
cb_treemenu_edit_specialres( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkWidget			*wgt_selected;
	specialres_t		*specialres;
	gtk_wgtdef_t		*wgtdef;
	gtk_wgtdef_t		*wgtdef_depth;
	int					depth, k, xdepth;
	defclass_t			*class=NULL;
	defclasssignal_t	*class_sig=NULL;
	int					found;

	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );

	specialres = get_object_data( GTK_WIDGET( menuitem ), "<specialres>" );
	wgtdef      = specialres->wgtdef;
	depth       = specialres->depth;
	wgtdef_depth = wgtdef->def[ depth].wgtdef_depth;

	found = FALSE;
	for ( xdepth = 0; 
		  (xdepth < MAXNB_HIERARCHIES) && (wgtdef->def[ xdepth ].classnames != NULL); 
		  xdepth++ )
	{
		for ( class  = wgtdef->def[ xdepth ].classes, k = 0; 
			  k < wgtdef->def[ xdepth ].nb_classes; 
			  k++, class++ )
		{
			if ( !strcmp( class->resname, wgtdef_depth->rightclickmenu[ specialres->index ] ) )
			{
				found = TRUE;
				break;
			}
		}
		if ( found )
			break;
	}
	if ( found )
	{
		switch ( class->restype )
		{
			case GTK_TYPE_STRING :
				dlg_res_editor_string( NULL, NULL,
					class,
					NULL,
					wgtdef, wgtdef_depth );
				break;
			default :
				fprintf( stderr, "Resource type [%s] not yet supported\n",
					class->resname );
				assert( 0 );
		}
		return;
	}

	/*--- Is-it a signal-type resource ? ---*/	
	for ( xdepth = 0; 
		  (xdepth < MAXNB_HIERARCHIES) && (wgtdef->def[ xdepth ].classnames != NULL); 
		  xdepth++ )
	{
		for ( class_sig  = wgtdef->def[ xdepth ].classes_signal, k = 0; 
			  k < wgtdef->def[ xdepth ].nb_classes_signal; 
			  k++, class_sig++ )
		{
			if ( !strcmp( class_sig->resname, wgtdef_depth->rightclickmenu[ specialres->index ] ) )
			{
				found = TRUE;
				break;
			}
		}
		if ( found )
			break;
	}
	if ( !found )
	{
		fprintf( stderr, "Resource [%s] not found\n",
			wgtdef_depth->rightclickmenu[ specialres->index ] );
		return;
	}
	do_dialog_cb( class_sig );
	
}								// cb_treemenu_edit_specialres


PUBLIC int
get_first_callbacks( int maxnb_cb, get_first_callback_t *cbget,
	GtkWidget *wgt, gtk_wgtdef_t *wgtdef )
{
	int						nb_cbget;
	int						depth, k, index;
	defclasssignal_t		*q;
	get_first_callback_t	*p;

	nb_cbget = 0;
	p = cbget;
	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
		  depth++ )
	{
		for ( q = wgtdef->def[ depth ].classes_signal, k = 0; 
			  k < wgtdef->def[ depth ].nb_classes_signal; 
			  k++, q++, index++ )
		{
			data_cb_t	*old_datacb;
			def_cb_t	*defcb;
			int			n;
			
			old_datacb = get_object_data( wgt, q->resname );
			if (old_datacb == NULL)
				continue;

			for ( defcb = old_datacb->cb, n = 0; 
				  n < old_datacb->nb_cb; 
				  n++, defcb++ )
			{
				p->resname = q->resname;
				p->defcb   = defcb;
				p++;
				if ( ++nb_cbget == maxnb_cb )
					break;
			}					// for (n)
			if ( nb_cbget == maxnb_cb )
				break;

		}						// for (k)
		if ( nb_cbget == maxnb_cb )
			break;
	}							// for (depth)

	return nb_cbget;

}								// get_first_callbacks


static void
add_first_callbacks_to_popup_menu( int maxnb_cb, GtkWidget *wgt, gtk_wgtdef_t *wgtdef,
	int f_add_sep )
{
	int						n, nb_cbget;
	get_first_callback_t	*cbget, *p;
	char					label[ 200 ];

	cbget = malloc( sizeof( get_first_callback_t ) * maxnb_cb );	
	nb_cbget = get_first_callbacks( maxnb_cb, cbget, wgt, wgtdef );
	for ( p = cbget, n = 0; n < nb_cbget; n++, p++ )
	{

		if ( f_add_sep && (n == 0) )
			create_menu_sep( wgt_popup_menu );

		if ( !strcmp( p->defcb->cbname, p->defcb->cbfile ) )
			sprintf( label, "%s::%s - %s",
				wgtdef->widget_name, p->resname, 
				p->defcb->cbname );
		else
			sprintf( label, "%s::%s - %s@%s",
				wgtdef->widget_name, p->resname, 
				p->defcb->cbname, p->defcb->cbfile );
		create_menu_item( wgt_popup_menu, label, 
			(!strcmp(project_dirname, "")) ? FALSE : TRUE, 
			0, 0, cb_treemenu_edit_cb, p->defcb );

	}							// for (n)

	free( cbget );

}								// add_first_callbacks_to_popup_menu


static void
add_some_resources_to_popup_menu( GtkWidget *wgt, gtk_wgtdef_t *wgtdef,
	int f_add_sep )
{
	int				depth, k;
	gtk_wgtdef_t	*w;
	int				first;
	char			label[ 100 ];
	GtkWidget		*wgt_item;
	specialres_t	*specialres;
	
	for ( first = TRUE, depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
		  depth++ )
	{
		w = wgtdef->def[ depth ].wgtdef_depth;
		if ( w->nb_rightclickmenu > 0 )
		{
			if ( f_add_sep && first )
			{
				create_menu_sep( wgt_popup_menu );
				first = FALSE;
			}
			for ( k = 0; k < w->nb_rightclickmenu; k++ )
			{
				sprintf( label, "%s::%s", 
					w->widget_name, w->rightclickmenu[ k ] );
				wgt_item = create_menu_item( wgt_popup_menu, label, 
					TRUE, 
					0, 0, cb_treemenu_edit_specialres, NULL );
				specialres = malloc( sizeof( specialres_t ) );
				specialres->wgtdef = wgtdef;
				specialres->depth  = depth;
				specialres->index  = k;
				set_object_data( wgt_item, "<specialres>", specialres, TRUE );
			}
		}						// if
	}							// for (depth)
	
}								// add_some_resources_to_popup_menu


PUBLIC void
tree_select_popup_menu( GtkWidget *wgt, GdkEventButton *event,
	GtkCTreeNode *node, int from_ctree )

{
	GtkWidget		*wgt_treemenu_expand, *wgt_treemenu_collapse; 
	GtkWidget		*wgt_treemenu_insertnewwgt; 
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	GtkWidget		*wgt_treemenu_copy, *wgt_treemenu_cut, *wgt_treemenu_paste;
#endif
	GtkWidget		*wgt_treemenu_delete;
#ifdef GLG_SUPPORT_WIDGET_MAKE_DEFAULT
	GtkWidget		*wgt_treemenu_makedefault, *wgt_treemenu_resetdefault;
#endif
#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
	GtkWidget		*wgt_treemenu_createtemplate;
#endif
	GtkWidget		*wgt_treemenu_displayicon, *wgt_treemenu_displaygtktype;
	GtkWidget		*wgt_treemenu_editstyle;
	gtk_wgtdef_t	*wgtdef;
	module_info_t	*module;
	int				ok;
	GtkArg			arg;
	GtkStyle		*style;
	int				n;

	if ( wgt_popup_menu )
		gtk_widget_destroy( wgt_popup_menu );	
	wgt_popup_menu = gtk_menu_new();

	wgtdef = get_object_data( GTK_WIDGET(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	module = get_object_data( GTK_WIDGET(wgt), WGTDATA_MODULE );
	assert( module != NULL );

	if (from_ctree)
	{

		ok = (GTK_CTREE_ROW(node)->children != NULL);
		wgt_treemenu_expand = create_menu_item( wgt_popup_menu, "Expand", 
			ok, 0, 0, cb_treemenu_expand, node );

		ok = (GTK_CTREE_ROW(node)->children != NULL);
		wgt_treemenu_collapse = create_menu_item( wgt_popup_menu, "Collapse", 
			TRUE, 0, 0, cb_treemenu_collapse, node );
		create_menu_sep( wgt_popup_menu );
		
	}							// if

	if ( wgt_selected_wgt_palette )
	{
		int				ok;
		char			label[200];
		gtk_wgtdef_t	*wgtdef_newwgt;
		if ( IS_WGT_PLACEHOLDER( wgt) || GTK_IS_CONTAINER(wgt) )
			ok = TRUE;
		else
			ok = FALSE;
		if ( ok && (wgtdef->max_nb_children != -1) )
		{
			GtkWidget	*wgt_container;
			GList		*list_children;
			wgt_container = wgt;
			assert( GTK_IS_CONTAINER(wgt_container) );
			list_children = gtk_container_children(GTK_CONTAINER(wgt_container));
			if ( list_children && (g_list_length(list_children) >= wgtdef->max_nb_children))
				ok = FALSE;
		}
		wgtdef_newwgt = get_object_data( GTK_WIDGET(wgt_selected_wgt_palette), WGTDATA_WGTDEF );
		assert( wgtdef_newwgt != NULL );
		if ( IS_WGT_PLACEHOLDER( wgt) )
			sprintf( label, "Put new %s", wgtdef_newwgt->widget_name );
		else
			sprintf( label, "Insert new %s", wgtdef_newwgt->widget_name );
		wgt_treemenu_insertnewwgt = create_menu_item( wgt_popup_menu, label, 
			ok, 0, 0, cb_treemenu_insertnewwgt, wgt );
		create_menu_sep( wgt_popup_menu );
	}

	if ( wgt != module->wgt_window )
	{
		int	ok_del;
#ifdef GLG_SUPPORT_COPYPASTE
		wgt_treemenu_copy = create_menu_item( wgt_popup_menu, "Copy", 
			FALSE, GDK_C, GDK_CONTROL_MASK, cb_treemenu_copy, wgt );
		wgt_treemenu_cut = create_menu_item( wgt_popup_menu, "Cut", 
			FALSE, GDK_X, GDK_CONTROL_MASK, cb_treemenu_cut, wgt );
		wgt_treemenu_paste = create_menu_item( wgt_popup_menu, "Paste", 
			FALSE, GDK_V, GDK_CONTROL_MASK, cb_treemenu_paste, wgt );
#endif
#if !defined(GLG_SUPPORT_EDIT_COMPOSITE) 
		if ( !IS_WGT_DONTDELETE(GTK_WIDGET(wgt)) )
			ok_del = TRUE;
		else
			ok_del = FALSE;
#else
		if ( !IS_WGT_DONTDELETE(GTK_WIDGET(wgt)) )
		{
			if ( !get_object_data(GTK_WIDGET(wgt), WGTDATA_HIERARCHY_CHILD) )
				ok_del = TRUE;
			else
				ok_del = FALSE;
		}
#endif
		wgt_treemenu_delete = create_menu_item( wgt_popup_menu, "Delete", 
			ok_del, GDK_Delete, GDK_CONTROL_MASK, cb_treemenu_delete, wgt );
		create_menu_sep( wgt_popup_menu );
	}
#ifdef GLG_SUPPORT_WIDGET_MAKE_DEFAULT
	wgt_treemenu_makedefault = create_menu_item( wgt_popup_menu, "Make default", 
		TRUE, 0, 0, cb_treemenu_makedefault, wgt );
	wgt_treemenu_resetdefault = create_menu_item( wgt_popup_menu, "Reset default", 
		TRUE, 0, 0, cb_treemenu_resetdefault, wgt );
		create_menu_sep( wgt_popup_menu );
#endif
#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
	wgt_treemenu_createtemplate = create_menu_item( wgt_popup_menu, "Create template", 
		TRUE, 0, 0, cb_treemenu_createtemplate, wgt );
		create_menu_sep( wgt_popup_menu );
#endif

	if ( from_ctree )
	{
		wgt_treemenu_displayicon = create_toggle_item( wgt_popup_menu, "Display Icon", 
			TRUE, config.tree_is_display_icon,
			0, 0, cb_treemenu_displayicon, wgt );
		wgt_treemenu_displaygtktype = create_toggle_item( wgt_popup_menu, "Display ::GtkType", 
			TRUE, config.tree_is_display_gtktype,
			0, 0, cb_treemenu_displaygtktype, wgt );
	}							// if (from_ctree)

	/*--- Is there a style attached to this widget ? ---*/
	arg.name = "style";
	gtk_widget_get( GTK_WIDGET( wgt ), &arg );
	style = (GtkStyle *)( GTK_VALUE_OBJECT( arg ) );
	for ( n = 0; n < nb_styles; n++ )
	{
		if ( style == styles[ n ] )
		{
			char	label[ 100 ];
			sprintf( label, "Edit style %s", styles_infos[ n ]->name );
			create_menu_sep( wgt_popup_menu );
			wgt_treemenu_editstyle = create_menu_item( wgt_popup_menu, label, 
				TRUE, 0, 0, cb_treemenu_editstyle, styles[ n ] );
			break;
		}
	}

	/*--- Add some resources for this widget ---*/
	add_some_resources_to_popup_menu( wgt, wgtdef, TRUE );

	/*--- Add the first 8 callbacks, if any ---*/
	add_first_callbacks_to_popup_menu( 8, wgt, wgtdef, TRUE );

	gtk_menu_popup( GTK_MENU(wgt_popup_menu), NULL, NULL, NULL, NULL,
		event->button, event->time );

}								// tree_select_popup_menu


static void 
cb_tree_select_row( GtkCTree *wgt_tree, GtkCTreeNode *wgt_node, 
	gint column, gpointer dummy )
{
	GtkWidget		*widget;

	widget = gtk_ctree_node_get_row_data( GTK_CTREE(wgt_tree), 
		GTK_CTREE_NODE(wgt_node) );
	if ( widget == NULL )
		return;

#if 0
{
gint	x, y, width, height, depth, width2, height2, width3, height3;
gint	x2, y2, x3, y3, x4, y4, x5, y5, x6, y6;
gdk_window_get_geometry( widget->window, &x, &y, &width, &height, &depth );
gdk_window_get_size( widget->window, &width2, &height2 );
width3  = widget->allocation.width;
height3 = widget->allocation.height;
gdk_window_get_origin( widget->window, &x2, &y2 );
gdk_window_get_root_origin( widget->window, &x3, &y3 );
gdk_window_get_deskrelative_origin( widget->window, &x4, &y4 );
gdk_window_get_position( widget->window, &x5, &y5 );
x6 = widget->allocation.x;
y6 = widget->allocation.y;
trace( "1) %p %16s %d,%d-%d,%d %d,%d %d,%d 2:%d,%d 3:%d,%d 4:%d,%d 5:%d,%d 6:%d,%d (%d)\n", 
widget, gtk_widget_get_name(widget),
x, y, width, height,
width2, height2,
width3, height3,
x2, y2, 
x3, y3, 
x4, y4, 
x5, y5, 
x6, y6,
GTK_WIDGET_NO_WINDOW(widget) );
}
#endif
#if 0
{
GtkStyle	*style;
GtkArg		arg;
arg.name = "style";
gtk_widget_get( GTK_WIDGET( widget ), &arg );
style = (GtkStyle *)( GTK_VALUE_OBJECT( arg ) );
trace( "%p %s %p \n", widget, gtk_widget_get_name(widget), style );
}
#endif

	select_widget( widget, 
		FALSE,
		TRUE );					// Select CB, if any

}								// cb_tree_select_row


static void
callbacks_select_popup_menu( GtkWidget *wgt, GdkEventButton *event,
	GtkCTreeNode *node )

{
	gtk_wgtdef_t	*wgtdef;

	if ( wgt_popup_menu )
		gtk_widget_destroy( wgt_popup_menu );	
	wgt_popup_menu = gtk_menu_new();

	wgtdef = get_object_data( GTK_WIDGET(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	/*--- Add the first 12 callbacks, if any ---*/
	add_first_callbacks_to_popup_menu( 12, wgt, wgtdef, FALSE );

	gtk_menu_popup( GTK_MENU(wgt_popup_menu), NULL, NULL, NULL, NULL,
		event->button, event->time );

}								// callbacks_select_popup_menu


static int 
cb_tree_button_press_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkCTreeNode	*node;
	GtkWidget		*widget;
	gint			row, col;

	gtk_clist_get_selection_info (GTK_CLIST (ctree), event->x, event->y,
		&row, &col);
	node = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (ctree)->row_list, row));

	if ( event->button == 3 )
	{
		if ( node == NULL )
			return FALSE;

		gtk_ctree_select( GTK_CTREE(ctree), GTK_CTREE_NODE(node) );
		widget = GTK_WIDGET (gtk_ctree_node_get_row_data( ctree, node ));
		tree_select_popup_menu( GTK_WIDGET(widget), event, GTK_CTREE_NODE(node), TRUE );
		return TRUE;
	}

	else if ( event->button == 2 )
	{
		if ( node == NULL )
			return FALSE;

		gtk_ctree_select( GTK_CTREE(ctree), GTK_CTREE_NODE(node) );
		widget = GTK_WIDGET (gtk_ctree_node_get_row_data (ctree, node));
		gtk_widget_hide_all( widget );
		return TRUE;
	}

	return FALSE;
  	
}								// cb_tree_button_press_event


static int 
cb_tree_button_release_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkCTreeNode	*node;
	GtkWidget		*widget;
	gint			row, col;
	
	gtk_clist_get_selection_info (GTK_CLIST (ctree), event->x, event->y,
		&row, &col);
	node = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (ctree)->row_list, row));

	if ( event->button == 2 )
	{
		if ( node == NULL )
			return FALSE;

		widget = GTK_WIDGET (gtk_ctree_node_get_row_data (ctree, node));
		gtk_widget_show_all( widget );
		return TRUE;
	}

	return FALSE;

}								// cb_tree_button_release_event


static void 
cb_callbacks_select_row( GtkCTree *wgt_tree, GtkCTreeNode *wgt_node, 
	gint column, gpointer dummy )
{
	GtkWidget	*widget;

	widget = gtk_ctree_node_get_row_data( GTK_CTREE( wgt_tree_callbacks ), 
		GTK_CTREE_NODE(wgt_node) );
	if ( widget == NULL )
		return;

	select_widget( widget, 
		TRUE,
		FALSE );				// Don't select CB, if any

}								// cb_callbacks_select_row


static int 
cb_callbacks_button_press_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkCTreeNode	*node;
	GtkWidget		*widget;
	gint			row, col;

	if (event->button == 3)
	{
		gtk_clist_get_selection_info (GTK_CLIST (ctree), event->x, event->y,
			&row, &col);
		node = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (ctree)->row_list, row));
		if (node == NULL)
			return FALSE;

		gtk_ctree_select( GTK_CTREE(ctree), GTK_CTREE_NODE(node) );
		widget = GTK_WIDGET (gtk_ctree_node_get_row_data (ctree, node));
		callbacks_select_popup_menu( GTK_WIDGET(widget), event, GTK_CTREE_NODE(node) );
		return TRUE;
	}

	return FALSE;
  	
}								// cb_callbacks_button_press_event


static void 
cb_styles_select_row( GtkCTree *wgt_tree, GtkCTreeNode *wgt_node, 
	gint column, gpointer dummy )
{
	GtkWidget	*widget;

	widget = gtk_ctree_node_get_row_data( GTK_CTREE( wgt_tree_styles ), 
		GTK_CTREE_NODE(wgt_node) );
	if ( widget == NULL )
		return;

	select_widget( widget, 
		TRUE,
		FALSE );				// Don't select CB, if any

}								// cb_styles_select_row


static int 
cb_styles_button_press_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkCTreeNode	*node;
	GtkWidget		*widget;
	gint			row, col;

	if (event->button == 3)
	{
		gtk_clist_get_selection_info (GTK_CLIST (ctree), event->x, event->y,
			&row, &col);
		node = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (ctree)->row_list, row));
		if (node == NULL)
			return FALSE;

		gtk_ctree_select( GTK_CTREE(ctree), GTK_CTREE_NODE(node) );
		widget = GTK_WIDGET (gtk_ctree_node_get_row_data (ctree, node));
//		callbacks_select_popup_menu( GTK_WIDGET(widget), event, GTK_CTREE_NODE(node) );
		return TRUE;
	}

	return FALSE;
  	
}								// cb_callbacks_button_press_event


static void
update_cb( GtkWidget *widget, gtk_wgtdef_t *wgtdef )
{
	GtkCTreeNode		*wgt_node;
	defclasssignal_t	*q;
	int					depth, k, index;
	char				*text[1], label[200];

	assert( wgtdef != NULL );
	
	/*--- Add the first 8 callbacks, if any ---*/
	text[0] = label;
	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[depth].classnames != NULL); 
		  depth++ )
	{
		for ( q = wgtdef->def[depth].classes_signal, k = 0; 
			  k < wgtdef->def[depth].nb_classes_signal; 
			  k++, q++, index++ )
		{
			data_cb_t	*old_datacb;
			def_cb_t	*defcb;
			int			n;
			
			old_datacb = get_object_data( widget, q->resname );
			if (old_datacb == NULL)
				continue;

			for ( defcb = old_datacb->cb, n = 0; 
				  n < old_datacb->nb_cb; 
				  n++, defcb++ )
			{
				if ( !strcmp(defcb->cbname, defcb->cbfile) )
					sprintf( label, "%s - %s",
						defcb->cbname, q->resname );
				else
					sprintf( label, "%s@%s - %s", 
						defcb->cbname, defcb->cbfile, q->resname );
				if ( config.callbacks_is_display_icon )
					wgt_node = gtk_ctree_insert_node( GTK_CTREE( wgt_tree_callbacks ), 
						NULL, NULL,
						text, 5,
						wgtdef->gdkpixmap, wgtdef->gdkbitmap,
						wgtdef->gdkpixmap, wgtdef->gdkbitmap,
						FALSE, FALSE );
				else
					wgt_node = gtk_ctree_insert_node( GTK_CTREE( wgt_tree_callbacks ), 
						NULL, NULL,
						text, 5,
						NULL, NULL,
						NULL, NULL,
						FALSE, FALSE );

				gtk_ctree_node_set_row_data( GTK_CTREE( wgt_tree_callbacks), 
					GTK_CTREE_NODE(wgt_node),  widget );

				/*--- Link the widget to this tree item ---*/
				set_object_data( widget, CALLBACKSNODE_WGTDATA, wgt_node, FALSE );

			}					// for (n)

		}						// for (k)
	}							// for (depth)

}								// update_cb


static void
build_wgt_name( GtkWidget *widget, char *label )
{
	int		type_wgt;

	type_wgt = get_nametype_wgt( widget );
	if (type_wgt == NAMETYPE_ANONYMOUS)
	{
		if (IS_WGT_PLACEHOLDER( widget ))
			strcpy( label, "placeholder" );
		else
			sprintf( label, "%s", 
				gtk_widget_get_name(widget) );
	}
	else if (type_wgt == NAMETYPE_GLOBAL)
	{
		if ( config.tree_is_display_gtktype )
			sprintf( label, "%s: %s", 
				gtk_widget_get_name( widget ),
				get_name_wgt( widget ) );
		else
			sprintf( label, "%s", 
				get_name_wgt( widget ) );
	}
	else assert(0);
	
}								// build_wgt_name


static void
update_styles( GtkWidget *widget, gtk_wgtdef_t *wgtdef )
{
	int					n, found;
	char				*text[1], label[200];
	GtkStyle			*style, **pstyle;
	GtkArg				arg;
	GtkCTreeNode		*wgt_node;

	assert( wgtdef != NULL );

	/*--- Is there any style attached to this widget ? ---*/
	arg.name = "style";
	gtk_widget_get( GTK_WIDGET( widget ), &arg );
	style = (GtkStyle *)( GTK_VALUE_OBJECT( arg ) );
	for ( pstyle = &styles[ 0 ], found = FALSE, n = 0; n < nb_styles; n++, pstyle++ )
	{
		if ( !found && (*pstyle == style) )
		{
			found = TRUE;
			break;
		}
	}
	if ( !found )
		return;
	
	/*--- Add the first 8 callbacks, if any ---*/
	build_wgt_name( widget, label );
	text[0] = label;
	if ( config.callbacks_is_display_icon )
		wgt_node = gtk_ctree_insert_node( 
			GTK_CTREE( wgt_tree_styles ), 
			styles_infos[ n ]->wgt_node, NULL,
			text, 5,
			wgtdef->gdkpixmap, wgtdef->gdkbitmap,
			wgtdef->gdkpixmap, wgtdef->gdkbitmap,
			FALSE, FALSE );
	else
		wgt_node = gtk_ctree_insert_node( 
			GTK_CTREE( wgt_tree_styles ), 
			styles_infos[ n ]->wgt_node, NULL,
			text, 5,
			NULL, NULL,
			NULL, NULL,
			FALSE, FALSE );
	styles_infos[ n ]->nb_ref++;

	gtk_ctree_node_set_row_data( GTK_CTREE( wgt_tree_styles), 
		GTK_CTREE_NODE(wgt_node),  widget );

	/*--- Link the widget to this tree item ---*/
	set_object_data( widget, CALLBACKSNODE_WGTDATA, wgt_node, FALSE );

}								// update_styles


typedef struct
{
	int				level;
	int				*cpt;
	GtkCTreeNode	*parent;
}	TREE_DATA;

static void
forall_update_tree( GtkWidget *widget, TREE_DATA *data )
{
	char			*text[1], label[80];
	GtkCTreeNode	*wgt_node;
	gtk_wgtdef_t	*wgtdef;
	int				type_wgt;

#if 0
	trace( "*level=%-3d cpt=%-3d (%d) %d %d %d %s \n",
		data->level, *data->cpt, 
		(!IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget)) ? 1 : 0,
		IS_WGT_NOCOMPOSITE(widget), IS_WGT_DONTDELETE(widget),
		(GTK_WIDGET_FLAGS( widget ) & GTK_COMPOSITE_CHILD) ? 1 : 0,
		gtk_widget_get_name( widget ) );
#endif

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if (!config.show_composite_childs)
#endif
	{
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget) )
			return;
	}

	data->level++;
	(*data->cpt)++;

	/*--- Don't save blocked container ---*/
	if ( !IS_WGT_PLACEHOLDER(widget) )
	{
		wgtdef = gtk_object_get_data( GTK_OBJECT(widget), WGTDATA_WGTDEF );
		assert( wgtdef != NULL );
	}
	else
	{
		wgtdef = NULL;
	}

	type_wgt = get_nametype_wgt(widget);
	if (type_wgt == NAMETYPE_ANONYMOUS)
	{
		char	name[ 80 ];
		sprintf( name, "wgt_%d_%d", data->level, *(data->cpt) );
		set_name_wgt( widget, name );
	}

	build_wgt_name( widget, label );
	text[ 0 ] = label;
	
	if ( wgtdef && config.tree_is_display_icon )
	{
		wgt_node = gtk_ctree_insert_node( GTK_CTREE( wgt_tree_widgets ), 
			data->parent, NULL,
			text, 5,
			wgtdef->gdkpixmap, wgtdef->gdkbitmap,
			wgtdef->gdkpixmap, wgtdef->gdkbitmap,
			FALSE, FALSE );
	}
	else
	{
		wgt_node = gtk_ctree_insert_node( GTK_CTREE( wgt_tree_widgets ), 
			data->parent, NULL,
			text, 5,
			NULL, NULL,
			NULL, NULL,
			FALSE, FALSE );
	}
	gtk_ctree_node_set_row_data( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(wgt_node), 
		widget );

	if ( IS_WGT_PLACEHOLDER(widget) )
	{
		if ( properties_style_oblique )
			gtk_ctree_node_set_row_style( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(wgt_node),
				properties_style_oblique );
	}
	else if ( type_wgt == NAMETYPE_ANONYMOUS )
	{
		if ( properties_style_normal )
			gtk_ctree_node_set_row_style( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(wgt_node),
				properties_style_normal );
	}
	else if ( type_wgt == NAMETYPE_GLOBAL )
	{
		if ( properties_style_bold )
			gtk_ctree_node_set_row_style( GTK_CTREE( wgt_tree_widgets ), GTK_CTREE_NODE(wgt_node),
				properties_style_bold );
	}

	/*--- Link the widget to this tree item ---*/
	set_object_data( widget, TREENODE_WGTDATA, wgt_node, FALSE );

	/*--- Add callbaks into the list of callbacks ---*/
	if ( wgtdef != NULL )
		update_cb( widget, wgtdef );
	
	/*--- Add callcaks into the list of callbacks ---*/
	if ( wgtdef != NULL )
		update_styles( widget, wgtdef );
	
	if ( GTK_IS_CONTAINER(widget) )
	{
		TREE_DATA	new_data;
		
		new_data.level   = data->level;
		new_data.parent  = wgt_node;
		new_data.cpt	 = data->cpt;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_update_tree, &new_data );
	}

	data->level--;

}								// forall_update_tree


PUBLIC void
update_tree( void )
{
		TREE_DATA		data;
		GtkWidget		*window;
		gtk_wgtdef_t	*wgtdef;
static	int				first = TRUE;
		int				cpt;
		int				n;
		char			*text[ 1 ], label[ 200 ];

	/*--- No module ? ---*/
	if ( curr_module == NULL )
	{

		reset_selection();
		clear_resources( TRUE );

		gtk_clist_clear( GTK_CLIST( wgt_tree_widgets ) );
		gtk_clist_clear( GTK_CLIST( wgt_tree_callbacks ) );
		gtk_clist_clear( GTK_CLIST( wgt_tree_styles ) );

		return;
	}

	/*--- Window of the current module ---*/
	window = curr_module->wgt_window;
	wgtdef = gtk_object_get_data( GTK_OBJECT(window), WGTDATA_WGTDEF );

	/*--- First Clear the whole tree ---*/
	gtk_clist_freeze( GTK_CLIST( wgt_tree_widgets ) );
	gtk_clist_clear( GTK_CLIST( wgt_tree_widgets ) );
	gtk_clist_freeze( GTK_CLIST( wgt_tree_callbacks ) );
	gtk_clist_clear( GTK_CLIST( wgt_tree_callbacks ) );
	gtk_clist_freeze( GTK_CLIST( wgt_tree_styles ) );
	gtk_clist_clear( GTK_CLIST( wgt_tree_styles ) );

	if ( first)
	{

		/*--- List of widgets ---*/
		gtk_signal_connect( GTK_OBJECT( wgt_tree_widgets ), "tree_select_row",
			GTK_SIGNAL_FUNC(cb_tree_select_row),  wgt_tree_widgets );
		gtk_signal_connect_after( GTK_OBJECT( wgt_tree_widgets ), "button_press_event",
			GTK_SIGNAL_FUNC( cb_tree_button_press_event ),  wgt_tree_widgets );
		gtk_signal_connect_after( GTK_OBJECT( wgt_tree_widgets ), "button_release_event",
			GTK_SIGNAL_FUNC( cb_tree_button_release_event ),  wgt_tree_widgets );

		/*--- List of CB ---*/
		gtk_signal_connect( GTK_OBJECT( wgt_tree_callbacks ), "tree_select_row",
			GTK_SIGNAL_FUNC( cb_callbacks_select_row ), wgt_tree_callbacks );
		gtk_signal_connect_after( GTK_OBJECT( wgt_tree_callbacks ), "button_press_event",
			GTK_SIGNAL_FUNC( cb_callbacks_button_press_event ), wgt_tree_callbacks );

		/*--- List of Styles ---*/
		gtk_signal_connect( GTK_OBJECT( wgt_tree_styles ), "tree_select_row",
			GTK_SIGNAL_FUNC( cb_styles_select_row ), wgt_tree_styles );
		gtk_signal_connect_after( GTK_OBJECT( wgt_tree_styles ), "button_press_event",
			GTK_SIGNAL_FUNC( cb_styles_button_press_event ), wgt_tree_styles );

		first = FALSE;
	}

	/*--- First add the defined styles ---*/
	text[0] = label;
	for ( n = 0; n < nb_styles; n++ )
	{
		sprintf( label, "%s", styles_infos[ n ]->name );
		styles_infos[n]->wgt_node = gtk_ctree_insert_node( 
			GTK_CTREE( wgt_tree_styles ), 
			NULL, NULL,
			text, 5,
			NULL, NULL,
			NULL, NULL,
			FALSE, FALSE );
		styles_infos[n]->nb_ref = 0;
		gtk_ctree_node_set_selectable( GTK_CTREE( wgt_tree_styles ), 
			styles_infos[n]->wgt_node, FALSE );
	}

	data.level   = -1;
	cpt = 0;
	data.cpt     = &cpt;
	data.parent  = NULL;
	forall_update_tree( window, &data );

	gtk_clist_thaw( GTK_CLIST( wgt_tree_widgets ) );
	gtk_clist_thaw( GTK_CLIST( wgt_tree_callbacks ) );
	gtk_clist_thaw( GTK_CLIST( wgt_tree_styles ) );
		
}								// update_tree
