#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static	GtkWidget	*wgt_box_titles[ MAXNB_HIERARCHIES ];
static	GtkWidget	*wgt_box_res[ MAXNB_HIERARCHIES ];


PUBLIC void
clear_resources( int clear_widget_name_area )
{
	int	n;
	
	for ( n = 0; n < MAXNB_HIERARCHIES; n++ )
		gtk_notebook_remove_page( GTK_NOTEBOOK(wgt_res_notebook), -1 );

	/*--- Reset the widget to anonymous ---*/
	if ( clear_widget_name_area )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_resources_btn_anonymous), TRUE );
		gtk_widget_set( GTK_WIDGET(wgt_resources_label_name), "sensitive", FALSE, NULL );
		gtk_widget_set( GTK_WIDGET(wgt_resources_entry_name), "sensitive", FALSE, NULL );
		gtk_signal_handler_block( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
		gtk_entry_set_text( GTK_ENTRY(wgt_resources_entry_name), "" );
		gtk_signal_handler_unblock( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
	}							// if

}								// clear_resources


static void
add_new_page( int depth, char *label_page )
{
	GtkWidget	*wgt_label;
	GtkWidget	*wgt_scrolled_wnd;
	GtkWidget	*wgt_viewport;
	GtkWidget	*wgt_hpaned;

	if ( (strlen( label_page ) > 3) && !strncmp( label_page, "Gtk", 3 ) )	
		wgt_label = gtk_label_new( &label_page[ 3 ] );
	else
		wgt_label = gtk_label_new( label_page );
	gtk_widget_show( wgt_label );
	
	wgt_scrolled_wnd = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (wgt_scrolled_wnd), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	wgt_viewport = gtk_viewport_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (wgt_scrolled_wnd), wgt_viewport);

	wgt_hpaned = gtk_hpaned_new ();
	gtk_container_add (GTK_CONTAINER (wgt_viewport), wgt_hpaned);

	wgt_box_titles[ depth ] = gtk_vbox_new (FALSE, 0);
	gtk_box_set_homogeneous( GTK_BOX( wgt_box_titles[ depth ]), FALSE );
	gtk_container_add (GTK_CONTAINER (wgt_hpaned), wgt_box_titles[ depth ]);

	wgt_box_res[ depth ] = gtk_vbox_new (FALSE, 0);
	gtk_box_set_homogeneous( GTK_BOX( wgt_box_res[ depth ] ), FALSE );
	gtk_container_add (GTK_CONTAINER (wgt_hpaned), wgt_box_res[ depth ]);
  
	gtk_widget_show_all( wgt_scrolled_wnd );;
	gtk_notebook_append_page( GTK_NOTEBOOK (wgt_res_notebook), 
		wgt_scrolled_wnd, wgt_label );

}								// add_new_page


static void
_disp_title_res( GtkWidget *wgt, 
	int nb_res, defclass_t *classes, 
	int nb_signal, defclasssignal_t *classes_signal, 
	gtk_wgtdef_t *wgtdef, 
	gtk_wgtdef_t *wgtdef_depth, 
	int depth, 
	GtkWidget *wgt_parent, gtk_wgtdef_t *wgtdef_parent )
{
	defclass_t			*p;
	int					k, index;
	GtkWidget			*wgt_labels[100], *wgt_editor[100];
	char				*classname;
	defclasssignal_t	*q;
	GtkRequisition 		req1, req2;

	classname = wgtdef->def[ depth ].classnames;
	add_new_page( depth, classname );

	/*--- Then create the titles ---*/
	index = 0;
	for ( p = classes, k = 0; k < nb_res; k++, p++, index++ )
	{

		if ( !p->f_user )
		{
			wgt_labels[index] = NULL;
			continue;
		}

		assert( index < sizeof(wgt_labels)/sizeof(wgt_labels[0]) );
		if ( p->title )
			wgt_labels[index] = gtk_label_new( p->title );
		else
			wgt_labels[index] = gtk_label_new( p->resname );
		assert( wgt_labels[index] != NULL );
		gtk_misc_set_alignment( GTK_MISC( wgt_labels[index] ), 0, 0.5 );
		gtk_misc_set_padding( GTK_MISC( wgt_labels[index] ), 4, 0 );
		gtk_box_pack_start( GTK_BOX( wgt_box_titles[depth] ), wgt_labels[index], FALSE, FALSE, 0 );
		if ( is_default_value( wgt, wgt_parent, wgtdef_depth, p ) )
		{
			if ( properties_style_normal )
				gtk_widget_set_style( wgt_labels[index], properties_style_normal );
		}
		else
		{
			if ( properties_style_bold )
				gtk_widget_set_style( wgt_labels[index], properties_style_bold );
		}

	}							// for (n)

	/*--- Create also the titles for the signals (callbacks) ---*/
	for ( q = classes_signal, k = 0; k < nb_signal; k++, q++, index++ )
	{
		assert( index < sizeof(wgt_labels)/sizeof(wgt_labels[0]) );
		if ( q->title )
			wgt_labels[index] = gtk_label_new( q->title );
		else
			wgt_labels[index] = gtk_label_new( q->resname );
		assert( wgt_labels[index] != NULL );
		gtk_misc_set_alignment( GTK_MISC( wgt_labels[index] ), 0, 0.5 );
		gtk_misc_set_padding( GTK_MISC( wgt_labels[index] ), 4, 0 );
		gtk_box_pack_start( GTK_BOX( wgt_box_titles[depth] ), wgt_labels[index], FALSE, FALSE, 0 );
		if ( properties_style_normal )
			gtk_widget_set_style( wgt_labels[index], properties_style_normal );
	}

	/*--- Then create the resource entry ---*/
	index = 0;
	for ( p = classes, k = 0; k < nb_res; k++, p++, index++ )
	{
		int	f_optional;
		
		if ( !p->f_user )
		{
			wgt_editor[index] = NULL;
			continue;
		}
		assert( index < sizeof(wgt_editor)/sizeof(wgt_editor[0]) );
		gtk_widget_size_request( GTK_WIDGET(wgt_labels[index]), &req1 );

		f_optional = FALSE;
		switch ( p->restype )
		{

			case GTK_TYPE_STRING :
				wgt_editor[index] = res_editor_string( wgtdef, wgtdef_depth, 
					wgt_box_res[depth], 
					p, wgt_parent );
				break;

			case GTK_TYPE_BOOL :
				wgt_editor[index] = res_editor_bool( wgtdef, wgtdef_depth, 
					wgt_box_res[depth], 
					p, wgt_parent );
				break;

			case GTK_TYPE_INT :
			case GTK_TYPE_UINT :
			case GTK_TYPE_LONG :
			case GTK_TYPE_ULONG :
			case GTK_TYPE_FLOAT :
				if (wgtdef_parent)
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") || !strcmp(p->resname, "~GtkWidget::y") )
						if ( wgtdef_parent->f_children_autopos )
							f_optional = TRUE;
					if ( !strcmp(p->resname, "~GtkWidget::width") || !strcmp(p->resname, "~GtkWidget::height") )
						if ( wgtdef_parent->f_children_autodim )
							f_optional = TRUE;
				}
				wgt_editor[index] = res_editor_int( 
					f_optional, 
					wgtdef, wgtdef_depth, wgt_box_res[depth], 
					p, wgt_parent );
				break;

			case GTK_TYPE_ENUM :
				wgt_editor[index] = res_editor_enum( wgtdef, wgtdef_depth, 
					wgt_box_res[depth], 
					p, wgt_parent );
				break;

			case GTK_TYPE_FLAGS :
				wgt_editor[index] = res_editor_flags( wgtdef, wgtdef_depth, 
					wgt_box_res[depth], 
					p, wgt_parent );
				break;

			case GTK_TYPE_OBJECT :
				if ( !strcmp( p->object_restype_str, "GTK_TYPE_STYLE" ) )
					wgt_editor[index] = res_editor_style( wgtdef, wgtdef_depth, 
						wgt_box_res[depth], 
						p, wgt_parent );
				else assert(0);
				break;

			case GTK_TYPE_POINTER :
				if ( !strcmp( p->object_restype_str, "GTK_TYPE_ADJUSTMENT" ) )
					wgt_editor[ index ] = res_editor_adjustment( wgtdef, wgtdef_depth, 
						wgt_box_res[ depth ], 
						p, wgt_parent );
				else if ( !strcmp( p->object_restype_str, "GTK_USERTYPE_STRINGS" ) )
					wgt_editor[ index ] = res_editor_strings( wgtdef, wgtdef_depth, 
						wgt_box_res[ depth ], 
						p, wgt_parent );
				else assert(0);
				break;

			default :
				wgt_editor[index] = NULL;
				RES_NOT_YET_HANDLED( wgtdef, p );
				assert(0);
				break;

		}						// switch

		if (wgt_editor[index] == NULL)
		{
			gtk_widget_destroy( wgt_labels[index] );
			wgt_labels[index] = NULL;
		}
		else
		{
			set_object_data( wgt, PROPERTIES_WGTDATA_LABEL, wgt_labels[index], FALSE );
		}
	}							// for (k)

	/*--- Create also the resources for the signals (callbacks) ---*/
	for ( q = classes_signal, k = 0; k < nb_signal; k++, q++, index++ )
	{
		wgt_editor[index] = 
			res_editor_signal( wgtdef, wgtdef_depth, wgt_box_res[ depth ], q, 
				wgt_labels[index] );
		if (wgt_editor[index] == NULL)
		{
			gtk_widget_destroy( wgt_labels[index] );
			wgt_labels[index] = NULL;
		}
		else
		{
			set_object_data( wgt, PROPERTIES_WGTDATA_LABEL, wgt_labels[index], FALSE );
		}
	}							// for(n)

	/*--- Then create the resource entry ---*/
	for ( p = classes, index = 0; index < nb_res+nb_signal; index++, p++ )
	{

		if ( (index < nb_res) && !p->f_user )
			continue;
		
		if ( !wgt_labels[index] || !wgt_editor[index] )
			continue;

		gtk_widget_size_request( GTK_WIDGET(wgt_labels[index]), &req1 );
		gtk_widget_size_request( GTK_WIDGET(wgt_editor[index]), &req2 );
		if ( req1.height < req2.height )
			gtk_widget_set_usize( GTK_WIDGET(wgt_labels[index]), -1, req2.height );		
		else if ( req1.height > req2.height )
			gtk_widget_set_usize( GTK_WIDGET(wgt_editor[index]), -1, req1.height );		

	}							// for (n)
	gtk_widget_show_all( wgt_box_titles[depth] );
	gtk_widget_show_all( wgt_box_res[depth] );
	
}								// _disp_title_res


static void
add_hierarchy( GtkWidget *wgt, gtk_wgtdef_t *wgtdef, int depth, 
	gtk_wgtdef_t *wgtdef_parent )
{
	if ( wgtdef->def[depth].nb_classes + wgtdef->def[depth].nb_classes_signal > 0 )
	{
		_disp_title_res( wgt, 
			wgtdef->def[depth].nb_classes, wgtdef->def[depth].classes, 
			wgtdef->def[depth].nb_classes_signal, wgtdef->def[depth].classes_signal, 
			wgtdef, wgtdef->def[depth].wgtdef_depth, depth, 
			NULL, wgtdef_parent );
	}
			
}								// add_hierarchy 


static void
add_hierarchy_child( GtkWidget *wgt, 
	gtk_wgtdef_t *wgtdef, 
	GtkWidget *wgt_parent, gtk_wgtdef_t *wgtdef_parent, 
	int depth )
{
	int				nb_res;
	defclass_t		*classes;

	nb_res = wgtdef->def[ depth ].nb_classes_child;
	if ( nb_res > 0 )
	{
		classes = wgtdef->def[depth].classes_child;
		_disp_title_res( wgt, 
			nb_res, classes, 
			0, NULL, 
			wgtdef, wgtdef->def[ depth ].wgtdef_depth, depth, 
			wgt_parent, wgtdef_parent );
	}
		
}								// add_hierarchy_child


PUBLIC void
display_resource_one_widget( GtkWidget *widget, int page )
{
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_parent;
	int				depth;
	GtkWidget		*wgt_parent=NULL;
	GtkArg			args[1];
	char			*name_wgt;
	int				nametype_wgt;
#ifdef DEBUG
	double			t0;
#endif

	gdk_flush();

	/*--- First erase all pages ---*/	
#ifdef DEBUG
	t0 = now();
#endif
	clear_resources( FALSE );
	
	/*--- Find the description of this widget ---*/
	wgtdef = gtk_object_get_data( GTK_OBJECT( widget ), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	/*--- Find the parent of this widget ---*/
	if ( GTK_WIDGET_TYPE( widget ) == GTK_TYPE_MENU )
	{
		wgtdef_parent = NULL;
	}
	else
	{
		args[ 0 ].name = "GtkWidget::parent";
		args[ 0 ].type = GTK_TYPE_CONTAINER;	
		gtk_widget_getv( GTK_WIDGET( widget ), 1, args );
		wgt_parent = (GtkWidget *)GTK_VALUE_OBJECT(args[0]);
		if (wgt_parent)
		{
			wgtdef_parent = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
			assert( wgtdef_parent != NULL );
		}
		else
		{
			wgtdef_parent = NULL;
		}
	}
	
	/*--- Then recreate a new page for each object into the widget hierarchy ---*/
	for ( depth = 0;
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL);
		  depth++ )
	{
		add_hierarchy( widget, wgtdef, depth, wgtdef_parent );
	}
	
	/*--- Then recreate a new page for each object into the widget hierarchy ---*/
	if ( wgtdef_parent != NULL )
	{
		gtk_wgtdef_t *_wgtdef;
		_wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
		assert( _wgtdef != NULL );
		assert( _wgtdef == wgtdef_parent );
		for ( depth = 0;
			  (depth < MAXNB_HIERARCHIES) && (wgtdef_parent->def[ depth ].classnames != NULL);
			  depth++ )
		{
			add_hierarchy_child( widget, _wgtdef, wgt_parent, wgtdef_parent, depth );
		}
	}

	gtk_notebook_set_page( GTK_NOTEBOOK( wgt_res_notebook ), page );

	/*--- Display the type/name of the widget ---*/
	nametype_wgt = get_nametype_wgt( widget );
	name_wgt = get_name_wgt( widget );
	switch (nametype_wgt)
	{

		case NAMETYPE_ANONYMOUS :
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_resources_btn_anonymous), TRUE );
			gtk_widget_set( GTK_WIDGET(wgt_resources_label_name), "sensitive", FALSE, NULL );
			gtk_widget_set( GTK_WIDGET(wgt_resources_entry_name), "sensitive", FALSE, NULL );
			gtk_signal_handler_block( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
			gtk_entry_set_text( GTK_ENTRY(wgt_resources_entry_name), "" );
			gtk_signal_handler_unblock( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
			break;

		case NAMETYPE_GLOBAL :
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_resources_btn_global), TRUE );
			gtk_widget_set( GTK_WIDGET(wgt_resources_label_name), "sensitive", TRUE, NULL );
			gtk_widget_set( GTK_WIDGET(wgt_resources_entry_name), "sensitive", TRUE, NULL );
			gtk_signal_handler_block( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
			gtk_entry_set_text( GTK_ENTRY(wgt_resources_entry_name), name_wgt );
			gtk_signal_handler_unblock( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
			break;

		default :
			assert(0);

	}

	if ( GTK_WIDGET_TYPE(widget) == GTK_TYPE_WINDOW )
	{
		gtk_widget_set( GTK_WIDGET(wgt_resources_btn_anonymous), "sensitive", FALSE, NULL );
		gtk_widget_set( GTK_WIDGET(wgt_resources_btn_global), "sensitive", FALSE, NULL );
	}
	else
	{
		gtk_widget_set( GTK_WIDGET(wgt_resources_btn_anonymous), "sensitive", TRUE, NULL );
		gtk_widget_set( GTK_WIDGET(wgt_resources_btn_global), "sensitive", TRUE, NULL );
	}

#ifdef DEBUG
	{
	char	tmp[ 100 ];
	double	t1, d;
	t1 = now();
	d = t1 - t0;
	sprintf( tmp, "%d widgets - %.0f microsec", 
		count_nb_widgets( wgt_res_notebook ), d );
	gtk_label_set_text( GTK_LABEL( wgt_debug_infos_properties ), tmp );
	}
#endif

}								// display_resource_one_widget
