#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include <X11/Xlib.h>

#include "gdk/gdkx.h"

#include "glg.h"

static GtkWidget	*wgt_dlg_styles	INIT( NULL );
static GtkWidget	*wgt_dlgstyles_scrolledwindow;
static GtkWidget	*wgt_dlgstyles_list;
static GtkWidget	*wgt_dlgstyles_display;
static GtkWidget	*wgt_dlgstyles_fg0;
static GtkWidget	*wgt_dlgstyles_fg1;
static GtkWidget	*wgt_dlgstyles_fg2;
static GtkWidget	*wgt_dlgstyles_fg3;
static GtkWidget	*wgt_dlgstyles_fg4;
static GtkWidget	*wgt_dlgstyles_bg0;
static GtkWidget	*wgt_dlgstyles_bg1;
static GtkWidget	*wgt_dlgstyles_bg2;
static GtkWidget	*wgt_dlgstyles_bg3;
static GtkWidget	*wgt_dlgstyles_bg4;
static GtkWidget	*wgt_dlgstyles_text0;
static GtkWidget	*wgt_dlgstyles_text1;
static GtkWidget	*wgt_dlgstyles_text2;
static GtkWidget	*wgt_dlgstyles_text3;
static GtkWidget	*wgt_dlgstyles_text4;
static GtkWidget	*wgt_dlgstyles_base0;
static GtkWidget	*wgt_dlgstyles_base1;
static GtkWidget	*wgt_dlgstyles_base2;
static GtkWidget	*wgt_dlgstyles_base3;
static GtkWidget	*wgt_dlgstyles_base4;
static GtkWidget	*wgt_dlgstyles_font_example;
static GtkWidget	*wgt_dlgstyles_btn_new;
static GtkWidget	*wgt_dlgstyles_btn_edit;
static GtkWidget	*wgt_dlgstyles_btn_delete;
static GtkWidget	*wgt_dlgstyles_btn_cancel;
static GtkWidget	*wgt_dlgstyles_btn_ok;

#define STYLE_DATA_INFO "style_info"


typedef struct
{
	int			level;
	int			cpt;			// Cpt for anonymous variables, same level
	int			nb_old_styles;
	GtkStyle	**old_styles;
	int			nb_new_styles;
	GtkStyle	**new_styles;
}	UPDATEMPDULESSTYLE_DATA;


PUBLIC char *
get_font_name( GtkStyle *style )
{
	XFontStruct		**font_structs;
	gint			num_fonts;
	gchar			**font_names;
	int				status;
	char			*name;
	Atom			font_atom, atom;

	font_atom = gdk_atom_intern( "FONT", FALSE );
	if ( style->font->type == GDK_FONT_FONTSET)
	{
		num_fonts = XFontsOfFontSet( GDK_FONT_XFONT( style->font ),
			&font_structs, &font_names );
	  status = XGetFontProperty( font_structs[0], font_atom, &atom );
	}
	else
	{
		status = XGetFontProperty( GDK_FONT_XFONT( style->font ), font_atom,
			&atom );
	}

	name = gdk_atom_name (atom);
	return name;
	
}								// get_font_name


static void
forall_update_modules_style( GtkWidget *widget, UPDATEMPDULESSTYLE_DATA *data )
{
	GtkArg		arg;
	GtkStyle	*style;
	int			n;

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if (!config.show_composite_childs )
#endif
		if ( !IS_WGT_NOCOMPOSITE(widget) )
			return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	arg.name = "style";
	gtk_widget_get( GTK_WIDGET( widget ), &arg );
	style = (GtkStyle *)( GTK_VALUE_OBJECT( arg ) );
	for ( n = 0; n < data->nb_old_styles; n++ )
	{
		if ( style == data->old_styles[ n ] )
		{
			gtk_widget_set( widget, "style", data->new_styles[ n ], NULL );
			break;
		}
	}

	if ( GTK_IS_CONTAINER(widget) )
	{
		UPDATEMPDULESSTYLE_DATA	new_data;
		
		new_data.level         = data->level;
		new_data.cpt   	       = data->cpt;
		new_data.nb_old_styles = data->nb_old_styles;
		new_data.old_styles    = data->old_styles;
		new_data.nb_new_styles = data->nb_new_styles;
		new_data.new_styles    = data->new_styles;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_update_modules_style, &new_data );
	}

	data->level--;

}								// forall_update_modules_style


PUBLIC void
update_modules_style( int nb_old_styles, GtkStyle **old_styles, 
	int nb_new_styles, GtkStyle **new_styles )
{
	module_info_t			*module;
	UPDATEMPDULESSTYLE_DATA	data;

	module = next_module( FIRST );
	while ( module )
	{

		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			case MODULE_TYPE_MENU :
				data.level         = -1;
				data.cpt           = 0;
				data.nb_old_styles = nb_old_styles;
				data.old_styles    = old_styles;
				data.nb_new_styles = nb_new_styles;
				data.new_styles    = new_styles;
				forall_update_modules_style( module->wgt_window, &data );
				break;
			default :
				assert( 0 );
		}

		module = next_module( NEXT );

	}

}								// update_modules_style


static int
list_length( void )
{
	GList	*list_children;

	list_children = gtk_container_children( GTK_CONTAINER( wgt_dlgstyles_list ) );
	if ( list_children == NULL )
		return 0;
	else
		return g_list_length( list_children );

}								// list_length


static void
cb_list_select_child( GtkList *list, GtkWidget *widget, gpointer user_data )
{
	GtkStyle	*style;
	GtkStyle	*s;
	GList		*selected_list;
	char		*font_name;

	selected_list = list->selection;
	if ( g_list_length( selected_list ) == 0 )
	{
		gtk_widget_hide_all( wgt_dlgstyles_display );
		return;
	}

	gtk_widget_show_all( wgt_dlgstyles_display );

	style = get_object_data( widget, "style" );
	assert( style != NULL );

#define SET_STYLE( C, WGT ) \
	s = gtk_style_copy( style ); \
	s->bg [ 0 ] = style->##C; \
	gtk_widget_set_style( WGT, s )

	SET_STYLE( fg [ 0 ], wgt_dlgstyles_fg0 );
	SET_STYLE( fg [ 1 ], wgt_dlgstyles_fg1 );
	SET_STYLE( fg [ 2 ], wgt_dlgstyles_fg2 );
	SET_STYLE( fg [ 3 ], wgt_dlgstyles_fg3 );
	SET_STYLE( fg [ 4 ], wgt_dlgstyles_fg4 );

	SET_STYLE( bg [ 0 ], wgt_dlgstyles_bg0 );
	SET_STYLE( bg [ 1 ], wgt_dlgstyles_bg1 );
	SET_STYLE( bg [ 2 ], wgt_dlgstyles_bg2 );
	SET_STYLE( bg [ 3 ], wgt_dlgstyles_bg3 );
	SET_STYLE( bg [ 4 ], wgt_dlgstyles_bg4 );

	SET_STYLE( base [ 0 ], wgt_dlgstyles_base0 );
	SET_STYLE( base [ 1 ], wgt_dlgstyles_base1 );
	SET_STYLE( base [ 2 ], wgt_dlgstyles_base2 );
	SET_STYLE( base [ 3 ], wgt_dlgstyles_base3 );
	SET_STYLE( base [ 4 ], wgt_dlgstyles_base4 );

	SET_STYLE( text [ 0 ], wgt_dlgstyles_text0 );
	SET_STYLE( text [ 1 ], wgt_dlgstyles_text1 );
	SET_STYLE( text [ 2 ], wgt_dlgstyles_text2 );
	SET_STYLE( text [ 3 ], wgt_dlgstyles_text3 );
	SET_STYLE( text [ 4 ], wgt_dlgstyles_text4 );

#undef SET_STYLE

	font_name = get_font_name( style );
	gtk_widget_set( wgt_dlgstyles_font_example, 
		"label", font_name, NULL );

	gtk_widget_set( GTK_WIDGET( wgt_dlgstyles_btn_edit ), "sensitive", 
		TRUE, 
		NULL );

	gtk_widget_set( GTK_WIDGET( wgt_dlgstyles_btn_delete ), "sensitive", 
		(!strcmp( get_object_data( widget, "delete" ), "true" )) ? TRUE : FALSE,
		NULL );

}								// cb_list_select_child


/* Callback "clicked" for GtkButton widget wgt_dlgstyles_btn_new */
static void
cb_styles_btn_new( GtkButton *button, gpointer user_data )
{
	GtkStyle		*new_style;
	int				ok;
	GtkWidget		*list_item;
	GList			*item_list;
	char			label[ 80 ];
	style_info_t	*style_info;
	GList			*selected_list;
	GtkWidget		*wgt_current_item;

	selected_list = GTK_LIST(wgt_dlgstyles_list)->selection;
	if ( selected_list != NULL )
		wgt_current_item = GTK_WIDGET( selected_list->data );
	else
		wgt_current_item = NULL;

	sprintf( label, "wgt_style_%d", list_length() + 1 );
	if ( wgt_current_item != NULL )
	{
		GtkStyle *style = get_object_data( wgt_current_item, "style" );
		assert( style != NULL );
		new_style = gtk_style_copy( style );
	}
	else
	{
		new_style = gtk_style_new();
	}
	ok = edit_style( new_style );
	if ( !ok )
	{
		free( new_style );
		return;
	}

	list_item = gtk_list_item_new_with_label( label );
	set_object_data( list_item, "style", new_style, TRUE );
	gtk_widget_show( list_item );
	item_list = g_list_append( NULL, list_item );
	gtk_list_append_items( GTK_LIST( wgt_dlgstyles_list ), item_list );
	set_object_data( list_item, "delete", "true", FALSE );
	gtk_list_select_child( GTK_LIST( wgt_dlgstyles_list ), list_item );

	/*--- Attach some info to this new style ---*/	
	style_info = malloc( sizeof( style_info_t ) );
	memset( style_info, 0, sizeof( style_info_t ) );
	strcpy( style_info->name, label );
	set_object_data( list_item, STYLE_DATA_INFO, style_info, TRUE );
			
}								// cb_styles_btn_new


/* Callback "clicked" for GtkButton widget wgt_dlgstyles_btn_edit */
static void
cb_styles_btn_edit( GtkButton *button, gpointer user_data )
{
	GtkStyle		*style, *new_style;
	int				ok;
	GtkWidget		*wgt_current_item;
	GList			*selected_list;

	selected_list = GTK_LIST(wgt_dlgstyles_list)->selection;
	assert( selected_list != NULL );
	wgt_current_item = GTK_WIDGET( selected_list->data );
	
	style = get_object_data( wgt_current_item, "style" );
	assert( style != NULL );

	new_style = gtk_style_copy( style );
	ok = edit_style( new_style );
	if ( !ok )
	{
		free( new_style );
		return;
	}

	set_object_data( wgt_current_item, "style", new_style, TRUE );
	gtk_list_select_child( GTK_LIST( wgt_dlgstyles_list ), wgt_current_item );

}								// cb_styles_btn_edit


/* Callback "clicked" for GtkButton widget wgt_dlgstyles_btn_delete */
static void
cb_styles_btn_delete(GtkButton *button, gpointer user_data)
{
	GtkStyle		*style;
	GtkWidget		*wgt_current_item;
	GList			*selected_list;

	selected_list = GTK_LIST(wgt_dlgstyles_list)->selection;
	assert( selected_list != NULL );
	wgt_current_item = GTK_WIDGET( selected_list->data );

	style = get_object_data( wgt_current_item, "style" );
	assert( style != NULL );

	set_object_data( wgt_current_item, "style", NULL, TRUE );
	gtk_widget_hide_all( wgt_current_item );
	
	gtk_widget_set( GTK_WIDGET( wgt_dlgstyles_btn_edit ), "sensitive", 
		FALSE, 
		NULL );
	gtk_widget_set( GTK_WIDGET( wgt_dlgstyles_btn_delete ), "sensitive", 
		FALSE, 
		NULL );

}								// cb_styles_btn_delete


/* Callback "clicked" for GtkButton widget wgt_dlgstyles_btn_cancel */
static void
cb_styles_btn_cancel( GtkButton *button, gpointer user_data )
{

	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_dlg_styles );
	wgt_dlg_styles = NULL;
	
}								// cb_styles_btn_cancel


/* Callback "clicked" for GtkButton widget wgt_dlgstyles_btn_ok */
static void
cb_styles_btn_ok( GtkButton *button, gpointer user_data )
{
	int				n, nb;
	GtkList			*list;
	GList			*children;
	GtkWidget		*wgt;
	GtkStyle		*style, **new_styles;
	style_info_t	*style_info, **new_styles_infos;
	int				nb_new_styles;

	/*--- Now remember the edited styles ---*/
	new_styles       = NULL;
	new_styles_infos = NULL;
	nb_new_styles = 0;
	nb = list_length();
	if ( nb > 0 )
	{

		new_styles       = malloc( nb * sizeof( GtkStyle * ) );
		new_styles_infos = malloc( nb * sizeof( style_info_t * ) );

		list = GTK_LIST( wgt_dlgstyles_list );
		children = list->children;
		for ( n = 0; n < nb; n++ )
		{
		
			wgt = children->data;
			if ( GTK_WIDGET_VISIBLE( wgt ) )
			{
				style = get_object_data( wgt, "style" );
				assert( style != NULL );

				new_styles[ n ] = gtk_style_copy( style );

				style_info = get_object_data( GTK_WIDGET( wgt ), STYLE_DATA_INFO );
				assert( style_info != NULL );
				new_styles_infos[ n ] = malloc( sizeof( style_info_t ) );
				memmove( new_styles_infos[ n ], style_info, sizeof( style_info_t ) );

				nb_new_styles++;
			}

			children = children->next;
		}

		update_modules_style( nb_styles, styles, nb_new_styles, new_styles );
	}

	/*--- Forget the previous styles ---*/
	reset_styles();
	nb_styles = nb_new_styles;
	styles = new_styles;
	styles_infos = new_styles_infos;

	/*--- Reselect current selected widget (refresh styles, combo box) ---*/
	refresh_properties_selected_widgets();

	touch_all_modules();

	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_dlg_styles );
	wgt_dlg_styles = NULL;

}								// cb_styles_btn_ok


/*--- This CB is used to avoid the close of window when user clicks X - top/right corner ---*/
static gint
cb_close_window( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_close_window


static GtkWidget *
create_window( GtkWidget *parent, const int realize )
{
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_3_4, *wgt_3_7, 
				*wgt_5_9, *wgt_6_10, *wgt_6_11, *wgt_5_17, *wgt_6_18, 
				*wgt_6_19, *wgt_5_25, *wgt_6_26, *wgt_6_27, *wgt_5_33, 
				*wgt_6_34, *wgt_6_35, *wgt_5_41, *wgt_6_42, *wgt_2_44;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new ();

	wgt_dlg_styles = gtk_widget_new( gtk_window_get_type(),
		"width", 400,
		"height", 300,
		"visible", TRUE,
		"resize_mode", GTK_RESIZE_QUEUE,
//		"window_position", GTK_WIN_POS_MOUSE,
		"title", "Styles Editor",
		"allow_grow", TRUE,
		"modal", TRUE,
		"default_width", 400,
		"default_height", 300,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"parent", wgt_dlg_styles,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_hpaned_get_type(),
		"parent", wgt_1_2,
		"visible", TRUE,
		NULL );

	wgt_3_4 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_2_3,
		"width", 210,
		"visible", TRUE,
		"label", "Styles",
		NULL );

	wgt_dlgstyles_scrolledwindow = gtk_widget_new( gtk_scrolled_window_get_type(),
		"parent", wgt_3_4,
		"visible", TRUE,
		"resize_mode", GTK_RESIZE_QUEUE,
		NULL );

	wgt_dlgstyles_list = gtk_widget_new( gtk_list_get_type(),
		"parent", wgt_dlgstyles_scrolledwindow,
		"visible", TRUE,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_list ), "select_child",
		GTK_SIGNAL_FUNC( cb_list_select_child ), NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_list ), "unselect_child",
		GTK_SIGNAL_FUNC( cb_list_select_child ), NULL );

	wgt_3_7 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_2_3,
		"visible", TRUE,
		"label", "Properties",
		NULL );

	wgt_dlgstyles_display = gtk_widget_new( gtk_vbox_get_type(),
		"parent", wgt_3_7,
		"visible", TRUE,
		"homogeneous", TRUE,
		NULL );

	wgt_5_9 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_dlgstyles_display,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );

	wgt_6_10 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_9,
		"width", 28,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Fg",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_9 ), wgt_6_10,
		"expand", FALSE,
		NULL );

	wgt_6_11 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_5_9,
		"visible", TRUE,
		"border_width", 6,
		"spacing", 8,
		"homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_9 ), wgt_6_11,
		"expand", TRUE,
		"position", 1,
		NULL );

	wgt_dlgstyles_fg0 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_11,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_fg0, "GTK_STATE_NORMAL", NULL );

	wgt_dlgstyles_fg1 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_11,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_11 ), wgt_dlgstyles_fg1,
		"position", 1,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_fg1, "GTK_STATE_ACTIVE", NULL );

	wgt_dlgstyles_fg2 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_11,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_11 ), wgt_dlgstyles_fg2,
		"position", 2,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_fg2, "GTK_STATE_PRELIGHT", NULL );

	wgt_dlgstyles_fg3 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_11,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_11 ), wgt_dlgstyles_fg3,
		"position", 3,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_fg3, "GTK_STATE_SELECTED", NULL );

	wgt_dlgstyles_fg4 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_11,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_11 ), wgt_dlgstyles_fg4,
		"position", 4,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_fg4, "GTK_STATE_INSENSITIVE", NULL );

	wgt_5_17 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_dlgstyles_display,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_dlgstyles_display ), wgt_5_17,
		"position", 1,
		NULL );

	wgt_6_18 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_17,
		"width", 28,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Bg",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_17 ), wgt_6_18,
		"expand", FALSE,
		NULL );

	wgt_6_19 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_5_17,
		"visible", TRUE,
		"border_width", 6,
		"spacing", 8,
		"homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_17 ), wgt_6_19,
		"expand", TRUE,
		"position", 1,
		NULL );

	wgt_dlgstyles_bg0 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_19,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_bg0, "GTK_STATE_NORMAL", NULL );

	wgt_dlgstyles_bg1 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_19,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_19 ), wgt_dlgstyles_bg1,
		"position", 1,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_bg1, "GTK_STATE_ACTIVE", NULL );

	wgt_dlgstyles_bg2 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_19,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_19 ), wgt_dlgstyles_bg2,
		"position", 2,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_bg2, "GTK_STATE_PRELIGHT", NULL );

	wgt_dlgstyles_bg3 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_19,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_19 ), wgt_dlgstyles_bg3,
		"position", 3,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_bg3, "GTK_STATE_SELECTED", NULL );

	wgt_dlgstyles_bg4 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_19,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_19 ), wgt_dlgstyles_bg4,
		"position", 4,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_bg4, "GTK_STATE_INSENSITIVE", NULL );

	wgt_5_25 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_dlgstyles_display,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_dlgstyles_display ), wgt_5_25,
		"position", 2,
		NULL );

	wgt_6_26 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_25,
		"width", 28,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Text",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_25 ), wgt_6_26,
		"expand", FALSE,
		NULL );

	wgt_6_27 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_5_25,
		"visible", TRUE,
		"border_width", 6,
		"spacing", 8,
		"homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_25 ), wgt_6_27,
		"expand", TRUE,
		"position", 1,
		NULL );

	wgt_dlgstyles_text0 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_27,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_text0, "GTK_STATE_NORMAL", NULL );

	wgt_dlgstyles_text1 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_27,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_27 ), wgt_dlgstyles_text1,
		"position", 1,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_text1, "GTK_STATE_ACTIVE", NULL );

	wgt_dlgstyles_text2 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_27,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_27 ), wgt_dlgstyles_text2,
		"position", 2,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_text2, "GTK_STATE_PRELIGHT", NULL );

	wgt_dlgstyles_text3 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_27,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_27 ), wgt_dlgstyles_text3,
		"position", 3,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_text3, "GTK_STATE_SELECTED", NULL );

	wgt_dlgstyles_text4 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_27,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_27 ), wgt_dlgstyles_text4,
		"position", 4,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_text4, "GTK_STATE_INSENSITIVE", NULL );

	wgt_5_33 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_dlgstyles_display,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_dlgstyles_display ), wgt_5_33,
		"position", 3,
		NULL );

	wgt_6_34 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_33,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Base",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_33 ), wgt_6_34,
		"expand", FALSE,
		NULL );

	wgt_6_35 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_5_33,
		"visible", TRUE,
		"border_width", 6,
		"spacing", 8,
		"homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_33 ), wgt_6_35,
		"expand", TRUE,
		"position", 1,
		NULL );

	wgt_dlgstyles_base0 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_35,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_base0, "GTK_STATE_NORMAL", NULL );

	wgt_dlgstyles_base1 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_35,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_35 ), wgt_dlgstyles_base1,
		"position", 1,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_base1, "GTK_STATE_ACTIVE", NULL );

	wgt_dlgstyles_base2 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_35,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_35 ), wgt_dlgstyles_base2,
		"position", 2,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_base2, "GTK_STATE_PRELIGHT", NULL );

	wgt_dlgstyles_base3 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_35,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_35 ), wgt_dlgstyles_base3,
		"position", 3,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_base3, "GTK_STATE_SELECTED", NULL );

	wgt_dlgstyles_base4 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_6_35,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_6_35 ), wgt_dlgstyles_base4,
		"position", 4,
		NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgstyles_base4, "GTK_STATE_INSENSITIVE", NULL );

	wgt_5_41 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_dlgstyles_display,
		"visible", TRUE,
		"border_width", 2,
		"spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_dlgstyles_display ), wgt_5_41,
		"position", 4,
		NULL );

	wgt_6_42 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_41,
		"width", 28,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Font",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_41 ), wgt_6_42,
		"expand", FALSE,
		NULL );

	wgt_dlgstyles_font_example = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_5_41,
		"visible", TRUE,
		"xalign", 0.000000,
		"label", "Example",
		"justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_41 ), wgt_dlgstyles_font_example,
		"position", 1,
		NULL );

	wgt_2_44 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_1_2,
		"height", 30,
		"visible", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_44,
		"expand", FALSE,
		"position", 1,
		NULL );

	wgt_dlgstyles_btn_new = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_44,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "New",
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_btn_new ), "clicked",
		GTK_SIGNAL_FUNC( cb_styles_btn_new ), NULL );

	wgt_dlgstyles_btn_edit = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_44,
		"visible", TRUE,
		"sensitive", FALSE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "Edit",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_44 ), wgt_dlgstyles_btn_edit,
		"position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_btn_edit ), "clicked",
		GTK_SIGNAL_FUNC( cb_styles_btn_edit ), NULL );

	wgt_dlgstyles_btn_delete = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_44,
		"visible", TRUE,
		"sensitive", FALSE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "Delete",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_44 ), wgt_dlgstyles_btn_delete,
		"position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_btn_delete ), "clicked",
		GTK_SIGNAL_FUNC( cb_styles_btn_delete ), NULL );

	wgt_2_44 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_1_2,
		"height", 30,
		"visible", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_44,
		"expand", FALSE,
		"position", 2,
		NULL );

	wgt_dlgstyles_btn_cancel = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_44,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "Cancel",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_dlgstyles_btn_ok,
		"expand", FALSE,
		"position", 0,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_btn_cancel ), "clicked",
		GTK_SIGNAL_FUNC( cb_styles_btn_cancel ), NULL );

	wgt_dlgstyles_btn_ok = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_44,
		"visible", TRUE,
		"can_focus", TRUE,
		"receives_default", TRUE,
		"label", "Ok",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_dlgstyles_btn_ok,
		"expand", FALSE,
		"position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgstyles_btn_ok ), "clicked",
		GTK_SIGNAL_FUNC( cb_styles_btn_ok ), NULL );

	gtk_signal_connect( GTK_OBJECT(wgt_dlg_styles), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_window), NULL );

	if ( realize )
		gtk_widget_show_all( wgt_dlg_styles );

	return wgt_dlg_styles;

}								// create_window


PUBLIC void
edit_styles( GtkStyle *style )
{
	GtkWidget		*list_item;
	GList			*item_list;
	int				n;
	style_info_t	*style_info;
	GtkStyle		*new_style;

	create_window( NULL, FALSE );
	gtk_widget_realize( wgt_dlg_styles );

	/*--- Populate the list of available styles ---*/
	for ( n = 0; n < nb_styles; n++ )
	{

		new_style = gtk_style_copy( styles[ n ] );

		style_info = malloc( sizeof( style_info_t ) );
		memmove( style_info, styles_infos[ n ], sizeof( style_info_t ) );

		list_item = gtk_list_item_new_with_label( style_info->name );
		set_object_data( list_item, "style", new_style, TRUE );
		gtk_widget_show( list_item );
		item_list = g_list_append( NULL, list_item );
		gtk_list_append_items( GTK_LIST( wgt_dlgstyles_list ), item_list );
		
		set_object_data( list_item, STYLE_DATA_INFO, style_info, TRUE );
			
		if ( styles_infos[ n ]->nb_ref == 0 )
			set_object_data( list_item, "delete", "true", FALSE );
		else
			set_object_data( list_item, "delete", "false", FALSE );

		if ( ((style == NULL) && (n == 0)) || (style == styles[ n ]) )
			gtk_list_select_child(GTK_LIST( wgt_dlgstyles_list ), list_item );

	}

	/*--- Select the given style, or the first one if none is given ---*/
	if ( style != NULL )
		gtk_button_clicked( GTK_BUTTON( wgt_dlgstyles_btn_edit ) );

	gtk_widget_show_all( wgt_dlg_styles );

	push_block_all_windows();
	while ( wgt_dlg_styles != NULL )
		gtk_main_iteration();
	pop_block_all_windows();

	/*--- Rebuild tree (styles) ---*/
	if (nb_modules() > 0)
		update_tree();

}								// edit_styles
