#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <sys/select.h>
#include <utime.h>
#include <sys/time.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

#ifdef DLG_EMAIL

static GtkWidget	*wgt_dlg_email;
static GtkWidget	*wgt_dlgemail_host;
static GtkWidget	*wgt_dlgemail_from;
static GtkWidget	*wgt_dlgemail_subject;
static GtkWidget	*wgt_dlgemail_msg;
static GtkWidget	*wgt_dlgemail_btn_send;
static GtkWidget	*wgt_dlgemail_btn_cancel;

// #define MAIL_HOST	"mail.qnx.com"
#define MAIL_HOST	"smtp.mail.yahoo.com"
#define OS_EMAIL	"lo_gafet@yahoo.com"
#define SMTP_PORT	25

static GtkWidget	*wgt_dlg_smtp	INIT( NULL );
static GtkWidget	*wgt_dlgsmtp_list;

static GtkWidget *
create_wgt_dlg_smtp( GtkWidget *parent, const int realize )
{
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_3_4;

	wgt_dlg_smtp = gtk_widget_new( gtk_window_get_type(),
		"width", 400,
		"height", 300,
		"visible", 1,
		"resize_mode", GTK_RESIZE_QUEUE,
		"title", "SMTP dialog",
		"allow_grow", 1,
		"default_width", 400,
		"default_height", 300,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"parent", wgt_dlg_smtp,
		"visible", 1,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_1_2,
		"visible", 1,
		"label", "SMTP log",
		NULL );

	wgt_3_4 = gtk_widget_new( gtk_scrolled_window_get_type(),
		"parent", wgt_2_3,
		"visible", 1,
		"resize_mode", GTK_RESIZE_QUEUE,
		"hscrollbar_policy", GTK_POLICY_AUTOMATIC,
		"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
		NULL );

	wgt_dlgsmtp_list = gtk_widget_new( gtk_list_get_type(),
		"visible", 1,
		"selection_mode", GTK_SELECTION_SINGLE,
		NULL );
	gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW(wgt_3_4), wgt_dlgsmtp_list );
	gtk_container_set_focus_vadjustment( GTK_CONTAINER (wgt_dlgsmtp_list),
		gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (wgt_3_4)) );
	gtk_container_set_focus_hadjustment( GTK_CONTAINER (wgt_dlgsmtp_list),
		gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW (wgt_3_4)) );

	if ( realize )
		gtk_widget_show_all( wgt_dlg_smtp );

	return wgt_dlg_smtp;

}								// create_wgt_dlg_smtp


static void
clear_smtp_log( void )
{
	gtk_list_clear_items( GTK_LIST(wgt_dlgsmtp_list), 0, -1 );
}								// clear_smtp_log


static void
smtp_log( char *fmt, ... )
{
	va_list			ap;
	char			buff[512];
	GtkWidget		*list_item;
	GtkContainer	*container;
	int				len;

	/*--- Develop message to display ---*/ 
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );
	
	/*--- remove CRLF ? ---*/
	len = strlen(buff);
	if ( (len >= 2) && buff[len-2]==13 && buff[len-1]==10 )
		buff[len-2] = 0;

	container = GTK_CONTAINER (wgt_dlgsmtp_list);
	list_item = gtk_list_item_new_with_label (buff);
	gtk_widget_show (list_item);
	gtk_container_add (container, list_item);

	/*--- Flush and display ---*/
	gdk_flush();
	while (g_main_iteration(FALSE));
	
}								// smtp_log


static int
smtp_read( int sock, char *buff, int buffsz, int until_crlf )
{
	int 			rval;
	fd_set 			fds,efds;
	struct timeval	tmo;
	int				status;
	int				index = 0;

	buff[ index = 0 ] = 0;
	for (;;)
	{

		tmo.tv_sec = (!index) ? 30 : 10;
		tmo.tv_usec = 0;
		FD_ZERO( &fds );		/* initialize selection vector */
		FD_ZERO( &efds );		/* handle errors too */
		FD_SET( sock, &fds );	/* set bit in selection vector */
		FD_SET( sock, &efds );	/* set bit in error selection vector */
		errno = 0;	   			/* block and read */

		status = select( sock+1, &fds, 0, &efds, &tmo );
		if ( status <= 0 )
		{
			smtp_log( buff );
			return ((index) ? index : -1);
		}
		rval = read( sock, &buff[index], buffsz - index );
		if ( rval != -1 )
		{
			buff[ index+rval ] = 0;
			index += rval;
		}
		else
		{
			buff[ index ] = 0;
			break;
		}
		if ( until_crlf && (rval >= 2) && (buff[rval-2] == '\015') && (buff[rval-1] == '\012') )
		{
			smtp_log( buff );
			return index+rval;
		}

		if ( !until_crlf )
		{
			int k = index;
			if ( (k >= 5) && !strncmp( &buff[k-5], "\015\012.\015\012", 5 ) )
			{
				smtp_log( buff );
				return index;
			}
		}

	}

	smtp_log( buff );
	return index;

}								// smtp_read


static int
smtp_write( int sock, char const *fmt, ... )
{
	va_list 	ap;
	char		buff[512];
	int			sval;

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	sval = write( sock, buff, strlen(buff) );
	smtp_log( buff );
	if ( sval != strlen(buff) )
		return -1;
	return sval;

}								// smtp_write


static int
send_email( char *host, char *from, char *subject, char *msg )
{
	int					sock;
	struct sockaddr_in	server;
	struct hostent		*hp;
	char				*buff, *p, *q;
	const int			buffsz = 8192;
	int					rval;

	clear_smtp_log();

	/*--- Create socket ---*/
	sock = socket( AF_INET, SOCK_STREAM, 0 );
	if ( sock < 0 )
	{
		smtp_log( "*** Unable to open a socket! ***" );
		return -1;
	}

	/*--- Connect socket using name specified ---*/
	server.sin_family = AF_INET;
	hp = gethostbyname( MAIL_HOST );
	if (hp == 0) 
	{
		smtp_log( "*** %s : unknown host!*** ", MAIL_HOST );
		return close( sock ), -1;
	}

	/*--- Connect to the mail server ---*/
	memcpy( &server.sin_addr, hp->h_addr, hp->h_length );
	server.sin_port = htons( SMTP_PORT );
	if ( connect( sock, (const struct sockaddr *)&server, sizeof(server) ) < 0 ) 
	{ 
		smtp_log( "Unable to connect to host %s", MAIL_HOST );
		return close( sock ), -1;
	}

	/*--- Greating message: just get rid of it ---*/
	buff = malloc( buffsz + 1 );
	if ( (rval = smtp_read( sock, buff, buffsz, TRUE )) == -1 )
		return free(buff), close( sock ), -1;

	/*---  HELO (only if host is given) ---*/
	if ( strcmp( host, "" ) )
	{
		smtp_write( sock, "HELO %s\015\012", host );
		rval = smtp_read( sock, buff, buffsz, TRUE );
	}

	/*---  Mail from... ---*/
	smtp_write( sock, "MAIL FROM:<%s>\015\012", from );
	rval = smtp_read( sock, buff, buffsz, TRUE );

	/*---  Mail to... ---*/
	smtp_write( sock, "RCPT TO:<%s>\015\012", OS_EMAIL );
	rval = smtp_read( sock, buff, buffsz, TRUE );

	/*---  Data... ---*/
	smtp_write( sock, "DATA\015\012");
	rval = smtp_read( sock, buff, buffsz, TRUE );
	
	/*--- Subject ---*/
	smtp_write( sock, "Subject: %s\015\012", subject );

	/*--- Message ---*/
	for ( p = msg; ; )
	{
		q = strchr( p, '\n' );
		if (q)
			*q = 0;
		smtp_write( sock, "%s\015\012", p );
		if (!q)
			break;
		p = q+1;		
	}

	smtp_write( sock, ".\015\012" );
	rval = smtp_read( sock, buff, buffsz, TRUE );

	/*---  OK ---*/
	smtp_write( sock, "QUIT\015\012" );
	rval = smtp_read( sock, buff, buffsz, TRUE );

	close( sock );
	return 0;

}								// send_email


/* Callback "clicked" for GtkButton widget wgt_dlgemail_btn_send */
void
cb_dlgemail_send(GtkButton *button, gpointer user_data)
{
	char	*host, *from, *subject, *msg;
	
	host	=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_host), 0, -1 );
	from	=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_from), 0, -1 );
	subject	=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_subject), 0, -1 );
	msg		=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_msg), 0, -1 );

	send_email( host, from, subject, msg );
	gtk_widget_set_sensitive( GTK_WIDGET(wgt_dlgemail_btn_send), FALSE );

	if (host)
		free(host);	
	if (from)
		free(from);
	if (subject)
		free(subject);
	if (msg)
		free(msg);

}								// cb_dlgemail_send

/* Callback "clicked" for GtkButton widget wgt_dlgemail_btn_cancel */
void
cb_dlgemail_cancel(GtkButton *button, gpointer user_data)
{
		
	/*--- Close the dialog ---*/
	gtk_widget_destroy(wgt_dlg_email);
	wgt_dlg_email = NULL;
	
	/*--- Close also the smtp log dialog ---*/
	gtk_widget_destroy(wgt_dlg_smtp);
	wgt_dlg_smtp = NULL;
	
}								// cb_dlgemail_cancel


static void
cb_string_changed( GtkEditable *wgt_editable, gpointer user_data )
{
	char	*from, *subject, *msg;
	int		ok;
	
	from	=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_from), 0, -1 );
	subject	=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_subject), 0, -1 );
	msg		=  gtk_editable_get_chars( GTK_EDITABLE(wgt_dlgemail_msg), 0, -1 );

	if ( from && strcmp(from, "") &&
		 subject && strcmp(subject, "") &&
		 msg && strcmp(msg, "") )
		ok = TRUE;
	else
		ok = FALSE;
	gtk_widget_set_sensitive( GTK_WIDGET(wgt_dlgemail_btn_send), ok );

	if (from)
		free(from);
	if (subject)
		free(subject);
	if (msg)
		free(msg);

}								// cb_string_changed


static GtkWidget *
create_wgt_dlg_email( GtkWidget *parent, const int realize )
{
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_2_4, *wgt_2_6, *wgt_2_8, 
				*wgt_2_10, *wgt_2_11;
	char		tmp[200];
	GtkTooltips	*tooltips;

	tooltips = gtk_tooltips_new ();

	wgt_dlg_email = gtk_widget_new( gtk_window_get_type(),
		"width", 500,
		"height", 400,
		"visible", 1,
		"resize_mode", GTK_RESIZE_QUEUE,
		"window_position", GTK_WIN_POS_MOUSE,
		"title", "Send email to GLG author",
		"allow_grow", 1,
		"modal", 1,
		"default_width", 500,
		"default_height", 400,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"parent", wgt_dlg_email,
		"visible", 1,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_1_2,
		"visible", 1,
		"label", "Send an email to the author\n<lo_gafet@yahoo.com>\nBug report, suggestions, etc.",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_3,
		"expand", 0,
		NULL );

	wgt_2_4 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_1_2,
		"height", 55,
		"visible", 1,
		"border_width", 4,
		"label", "Host",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_4,
		"expand", 0,
		"position", 1,
		NULL );

	wgt_dlgemail_host = gtk_widget_new( gtk_entry_get_type(),
		"parent", wgt_2_4,
		"height", 26,
		"visible", 1,
		"can_focus", 1,
		"max_length", 64,
		NULL );
#ifdef __QNXNTO__
	gtk_entry_set_text( GTK_ENTRY(wgt_dlgemail_host), "" );
#else
	getdomainname( tmp, sizeof(tmp) );
	gtk_entry_set_text( GTK_ENTRY(wgt_dlgemail_host), tmp );
#endif
	gtk_signal_connect( GTK_OBJECT(wgt_dlgemail_host), "changed",
		GTK_SIGNAL_FUNC(cb_string_changed), NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgemail_host, 
		"SMTP: 'HELO .. '\nThe HOST field is optional.", 
		NULL );

	wgt_2_4 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_1_2,
		"height", 55,
		"visible", 1,
		"border_width", 4,
		"label", "From",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_4,
		"expand", 0,
		"position", 2,
		NULL );

	wgt_dlgemail_from = gtk_widget_new( gtk_entry_get_type(),
		"parent", wgt_2_4,
		"height", 26,
		"visible", 1,
		"can_focus", 1,
		"max_length", 64,
		NULL );
	gtk_entry_set_text( GTK_ENTRY(wgt_dlgemail_from), "" );
	gtk_signal_connect( GTK_OBJECT(wgt_dlgemail_from), "changed",
		GTK_SIGNAL_FUNC(cb_string_changed), NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgemail_from, 
		"SMTP: 'MAIL FROM:<..>'\nEnter here your email address.", 
		NULL );

	wgt_2_6 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_1_2,
		"height", 55,
		"visible", 1,
		"border_width", 4,
		"label", "Subject",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_6,
		"expand", 0,
		"position", 3,
		NULL );

	wgt_dlgemail_subject = gtk_widget_new( gtk_entry_get_type(),
		"parent", wgt_2_6,
		"height", 26,
		"visible", 1,
		"can_focus", 1,
		"max_length", 127,
		NULL );
	sprintf( tmp, "GLG v%d.%d.%c (%s): ",
		FLG_APPVERSION_MAJOR, FLG_APPVERSION_MINOR, FLG_APPVERSION_SEQ+'a',
		FLG_APPVERSION_DATE );
	gtk_entry_set_text( GTK_ENTRY(wgt_dlgemail_subject), tmp );
	gtk_signal_connect( GTK_OBJECT(wgt_dlgemail_subject), "changed",
		GTK_SIGNAL_FUNC(cb_string_changed), NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgemail_subject, 
		"Enter here the subject of your email\n(bug report, suggestion, question, etc.)", 
		NULL );

	wgt_2_8 = gtk_widget_new( gtk_frame_get_type(),
		"parent", wgt_1_2,
		"visible", 1,
		"border_width", 4,
		"label", "Message",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_8,
		"position", 4,
		NULL );

	wgt_dlgemail_msg = gtk_widget_new( gtk_text_get_type(),
		"parent", wgt_2_8,
		"visible", 1,
		"can_focus", 1,
		"editable", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT(wgt_dlgemail_msg), "changed",
		GTK_SIGNAL_FUNC(cb_string_changed), NULL );
	gtk_tooltips_set_tip( tooltips, wgt_dlgemail_msg, 
		"Enter here your message.", 
		NULL );

	wgt_2_10 = gtk_widget_new( gtk_hseparator_get_type(),
		"parent", wgt_1_2,
		"visible", 1,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_10,
		"expand", 0,
		"position", 5,
		NULL );

	wgt_2_11 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_1_2,
		"height", 40,
		"visible", 1,
		"spacing", 8,
		"homogeneous", 1,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_11,
		"expand", 0,
		"padding", 4,
		"position", 6,
		NULL );

	wgt_dlgemail_btn_send = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_11,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Send the mail",
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgemail_btn_send ), "clicked",
		GTK_SIGNAL_FUNC( cb_dlgemail_send ), NULL );

	wgt_dlgemail_btn_cancel = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_11,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Cancel",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_11 ), wgt_dlgemail_btn_cancel,
		"position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_dlgemail_btn_cancel ), "clicked",
		GTK_SIGNAL_FUNC( cb_dlgemail_cancel ), NULL );

	cb_string_changed( GTK_EDITABLE(wgt_dlgemail_msg), NULL );

	if ( realize )
		gtk_widget_show_all( wgt_dlg_email );

	return wgt_dlg_email;

}								// create_wgt_dlg_email


PUBLIC void
dlg_email( void )
{

	create_wgt_dlg_email( NULL, TRUE );
	create_wgt_dlg_smtp( wgt_dlg_email, TRUE );

	push_block_all_windows();
	while ( wgt_dlg_email != NULL )
		gtk_main_iteration();
	pop_block_all_windows();

}									// dlg_email

#endif
