#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <errno.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

static GtkWidget	*wgt_res_edit_string	INIT( NULL );
static GtkWidget	*wgt_string;
static GtkWidget	*wgt_btn_default;
static GtkWidget	*wgt_btn_cancel;
static GtkWidget	*wgt_btn_apply;
static GtkWidget	*wgt_btn_ok;


typedef struct
{
	GtkWidget		*wgt_entry;
	GtkWidget		*wgt_parent;
	defclass_t		*defclass;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	text_data_t;


typedef struct
{
	defclass_t		*defclass;
	char			*string;
	GtkWidget		*wgt_parent;
	int				first;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;

	/*--- Set the resource of the widget ---*/
	set_res_string( wgt, user->string,
		user->wgtdef, user->wgtdef_depth, user->defclass, user->wgt_parent );

}								// foreach_selection


/*--- This CB is used to avoid the close of window when user clicks X - top/right corner ---*/
static gint
cb_close_window( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_close_window


/* Callback "clicked" for GtkButton widget wgt_3_6 */
static void
cb_btn_default(GtkButton *button, gpointer user_data)
{
	
}								// cb_btn_default


/* Callback "clicked" for GtkButton widget wgt_3_7 */
static void
cb_btn_cancel(GtkButton *button, gpointer user_data)
{
	
	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_res_edit_string );
	wgt_res_edit_string = NULL;
	
}								// cb_btn_cancel


/* Callback "clicked" for GtkButton widget wgt_3_8 */
static void
cb_btn_apply(GtkButton *button, gpointer user_data)
{
	text_data_t		*text_data;
	char			*str;

	/*--- Informations attached to this widget ---*/
	text_data = (text_data_t *)get_object_data( wgt_string, "text_data" );
	assert( text_data != NULL );

	/*--- Copy the current text entry into the widget ---*/
	str = gtk_editable_get_chars( GTK_EDITABLE(wgt_string), 0, -1 );
	if ( text_data->wgt_entry != NULL )
	{
		gtk_entry_set_text( GTK_ENTRY(text_data->wgt_entry), str );
	}
	else
	{
		foreach_selection_data	data;
		
		data.string   	  = str;
		data.defclass 	  = text_data->defclass;
		data.wgt_parent   = text_data->wgt_parent;
		data.first		  = TRUE;
		data.wgtdef		  = text_data->wgtdef;
		data.wgtdef_depth = text_data->wgtdef_depth;
		g_slist_foreach( list_selection, foreach_selection, &data );
	}
	free( str );

	/*--- Reselect current selected widget ---*/
	refresh_properties_selected_widgets();

	/*--- Enable 'Apply' only a next modification ---*/	
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), FALSE );

}								// cb_btn_apply


/* Callback "clicked" for GtkButton widget wgt_3_9 */
static void
cb_btn_ok(GtkButton *button, gpointer user_data)
{

	cb_btn_apply( button, user_data );

	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_res_edit_string );
	wgt_res_edit_string = NULL;
	
}								// cb_btn_ok


static void
cb_string_changed( GtkEditable *wgt_editable, gpointer user_data )
{

	gtk_widget_set_sensitive( GTK_WIDGET(wgt_btn_apply), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(wgt_btn_ok), TRUE );
	
}								// cb_string_changed


static void
create_window( GtkWidget *parent, char *title )
{
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_2_5;

	wgt_res_edit_string = gtk_widget_new( gtk_window_get_type(),
		"width", 400,
		"height", 300,
		"visible", 1,
		"resize_mode", GTK_RESIZE_QUEUE,
		"window_position", GTK_WIN_POS_NONE,
		"title", title,
		"allow_grow", 1,
		"default_width", 400,
		"default_height", 300,
		"modal", 1,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"parent", wgt_res_edit_string,
		"visible", 1,
		"border_width", 8,
		"spacing", 4,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_scrolled_window_get_type(),
		"parent", wgt_1_2,
		"visible", 1,
		"resize_mode", GTK_RESIZE_QUEUE,
		"hscrollbar_policy", GTK_POLICY_AUTOMATIC,
		"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
		NULL );

	wgt_string = gtk_widget_new( gtk_text_get_type(),
		"parent", wgt_2_3,
		"visible", 1,
		"can_focus", 1,
		"line_wrap", 0,
		"editable", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT(wgt_string), "changed",
		GTK_SIGNAL_FUNC(cb_string_changed), NULL );

	wgt_2_5 = gtk_widget_new( gtk_hbox_get_type(),
		"parent", wgt_1_2,
		"height", 32,
		"visible", 1,
		"spacing", 8,
		"homogeneous", 1,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_5,
		"expand", 0,
		"position", 1,
		NULL );

	wgt_btn_default = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_5,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Default",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_default,
		"expand", 0,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_default ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_default ), NULL );

	wgt_btn_cancel = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_5,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Cancel",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_cancel,
		"expand", 0,
		"position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_cancel ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_cancel ), NULL );

	wgt_btn_apply = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_5,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Apply",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_apply,
		"expand", 0,
		"position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_apply ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_apply ), NULL );

	wgt_btn_ok = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_2_5,
		"visible", 1,
		"can_focus", 1,
		"receives_default", 1,
		"label", "Ok",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_ok,
		"expand", 0,
		"position", 3,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_ok ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_ok ), NULL );

}								// create_window


PUBLIC void
dlg_res_editor_string( GtkWidget *wgt_btn, GtkWidget *wgt_entry, 
	defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth )
{
	char					*str;
	text_data_t				*text_data;
	char					title[ 200 ];
	GtkWidget				*wgt_selected;
	GtkArg					arg;

	/*--- Create the dialog ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	assert( wgt_selected != NULL );
	if ( get_nametype_wgt( wgt_selected ) == NAMETYPE_GLOBAL )
	{
		char *widget_name = get_name_wgt( wgt_selected );
		sprintf( title, "%s::%s\n",
			widget_name, defclass->resname );
	}
	else
	{
		sprintf( title, "Resource %s\n",
			defclass->resname );
	}
	create_window( NULL, title );

	/*--- First read current value (stored inside the widget or not) ---*/
	if ( get_res_string( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg ) == -1 )
		return;

	/*--- Update data ---*/
	if ( wgt_entry != NULL )		
		str = gtk_editable_get_chars( GTK_EDITABLE(wgt_entry), 0, -1 );
	else
		str = GTK_VALUE_STRING (arg );
	gtk_text_insert( GTK_TEXT(wgt_string), NULL, NULL, NULL, str, -1 );
	text_data = malloc( sizeof( text_data_t ) );
	memset( text_data, 0, sizeof( text_data_t ) );
	text_data->wgt_entry	 = wgt_entry;
	text_data->wgt_parent	 = wgt_parent;
	text_data->defclass	 	 = defclass;
	text_data->wgtdef		 = wgtdef;
	text_data->wgtdef_depth  = wgtdef_depth;
	set_object_data( wgt_string, "text_data", text_data, TRUE );

	gtk_widget_set_sensitive( GTK_WIDGET(wgt_btn_default), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET(wgt_btn_apply), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET(wgt_btn_ok), FALSE );

	gtk_signal_connect( GTK_OBJECT(wgt_res_edit_string), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_window), NULL );
	gtk_widget_show_all( wgt_res_edit_string );

	push_block_all_windows();
	while ( wgt_res_edit_string != NULL )
		gtk_main_iteration();
	pop_block_all_windows();

}								// dlg_res_editor_string

