#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <errno.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static	GtkWidget	*wgt_res_edit_strings	INIT( NULL );
static	GtkWidget	*wgt_list_items;
static	GtkWidget	*wgt_btn_add_after;
static	GtkWidget	*wgt_btn_add_before;
static	GtkWidget	*wgt_btn_remove;
static	GtkWidget	*wgt_btn_update;
static	GtkWidget	*wgt_item;
static	GtkWidget	*wgt_btn_default;
static	GtkWidget	*wgt_btn_cancel;
static	GtkWidget	*wgt_btn_apply;
static	GtkWidget	*wgt_btn_done;


typedef struct
{
	GtkWidget		*wgt_combo;
	GtkWidget		*wgt_parent;
	defclass_t		*defclass;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	text_data_t;


typedef struct
{
	defclass_t				*defclass;
	gtk_usertype_strings_t	**strings;
	GtkWidget				*wgt_parent;
	int						first;
	gtk_wgtdef_t			*wgtdef;
	gtk_wgtdef_t			*wgtdef_depth;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;

	/*--- Set the resource of the widget ---*/
	set_res_strings( wgt, user->strings,
		user->wgtdef, user->wgtdef_depth, user->defclass, user->wgt_parent );

}								// foreach_selection


/*--- This CB is used to avoid the close of window when user clicks X - top/right corner ---*/
static gint
cb_close_window( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_close_window


static int 
which_row_selected( void )
{
	GtkCList	*clist = GTK_CLIST( wgt_list_items );

	if ( clist->selection == NULL )
		return -1;
	else
		return (int) ( g_list_first( clist->selection ) -> data );

}								// which_row_selected


static gtk_usertype_strings_t *
create_strings( void )
{
	GtkCList				*clist = GTK_CLIST( wgt_list_items );
	gtk_usertype_strings_t	*strings;
	int						nb, index;
	char					*str;

	nb = clist->rows;
	strings = malloc( sizeof( gtk_usertype_strings_t ) );
	strings->nb_strings = nb;
	strings->strings = malloc( sizeof( char * ) * strings->nb_strings );
	for ( index = 0; index < nb; index++ )
	{
		gtk_clist_get_text( clist, index, 0, &str );
		strings->strings[ index ] = strdup( str );
	}

	return strings;

}								// create_strings


/* Callback "clicked" for GtkButton widget wgt_btn_add_after_clicked */
void
cb_btn_add_after_clicked(GtkButton *button, gpointer user_data)
{
	char			*str;
	gchar			*text[ 1 ];
	int				row;

	str = gtk_editable_get_chars( GTK_EDITABLE( wgt_item ), 0, -1 );

	row = which_row_selected();
	if ( row == -1 )
		row = 1;
	else
		row++;
	
	text[ 0 ] = str;
	gtk_clist_insert( GTK_CLIST( wgt_list_items ), row, text );
	
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_done ), TRUE );

}								// cb_btn_add_after

/* Callback "clicked" for GtkButton widget wgt_btn_add_before_clicked */
void
cb_btn_add_before_clicked(GtkButton *button, gpointer user_data)
{
	
	char			*str;
	gchar			*text[ 1 ];
	int				row;

	str = gtk_editable_get_chars( GTK_EDITABLE( wgt_item ), 0, -1 );

	row = which_row_selected();
	if ( row == -1 )
		row = 0;
	
	text[ 0 ] = str;
	gtk_clist_insert( GTK_CLIST( wgt_list_items ), row, text );
	
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_done ), TRUE );

}								// cb_btn_add_before


/* Callback "clicked" for GtkButton widget wgt_btn_remove_clicked */
void
cb_btn_remove_clicked(GtkButton *button, gpointer user_data)
{
	int		row;

	row = which_row_selected();
	if ( row == -1 )
		return;

	gtk_clist_remove( GTK_CLIST( wgt_list_items ), row );

	create_strings();

	gtk_entry_set_text( GTK_ENTRY( wgt_item ), "" );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_after ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_before ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_remove ),		FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_update ),		FALSE );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_done ), TRUE );

}								// cb_btn_remove


/* Callback "clicked" for GtkButton widget wgt_btn_update_clicked */
void
cb_btn_update_clicked(GtkButton *button, gpointer user_data)
{
	char	*str;
	int		row;

	row = which_row_selected();
	if ( row == -1 )
		return;

	str = gtk_editable_get_chars( GTK_EDITABLE( wgt_item ), 0, -1 );
	gtk_clist_set_text( GTK_CLIST( wgt_list_items ), row, 0, str );
	g_free( str );

	create_strings();

	gtk_entry_set_text( GTK_ENTRY( wgt_item ), "" );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_after ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_before ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_remove ),		FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_update ),		FALSE );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_done ), TRUE );

}								// cb_btn_update


/* Callback "clicked" for GtkButton widget wgt_btn_default_clicked */
void
cb_btn_default_clicked(GtkButton *button, gpointer user_data)
{
	
}								// cb_btn_default_clicked


/* Callback "clicked" for GtkButton widget wgt_btn_cancel */
void
cb_btn_cancel_clicked(GtkButton *button, gpointer user_data)
{
	
	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_res_edit_strings );
	wgt_res_edit_strings = NULL;
	
}								// cb_btn_cancel_clicked


/* Callback "clicked" for GtkButton widget wgt_btn_apply */
void
cb_btn_apply_clicked(GtkButton *button, gpointer user_data)
{
	text_data_t				*text_data;
	gtk_usertype_strings_t	*strings;
	foreach_selection_data	data;

	/*--- Informations attached to this widget ---*/
	text_data = (text_data_t *)get_object_data( wgt_list_items, "text_data" );
	assert( text_data != NULL );

	/*--- Read the list and create a strings data ---*/	
	strings = create_strings();
	
	/*--- Copy the current text entry into the widget ---*/
	data.strings   	  = &strings;
	data.defclass 	  = text_data->defclass;
	data.wgt_parent   = text_data->wgt_parent;
	data.first		  = TRUE;
	data.wgtdef		  = text_data->wgtdef;
	data.wgtdef_depth = text_data->wgtdef_depth;
	g_slist_foreach( list_selection, foreach_selection, &data );

	/*--- Reselect current selected widget ---*/
	refresh_properties_selected_widgets();

	/*--- Enable 'Apply' only a next modification ---*/	
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), FALSE );

}								// cb_btn_apply_clicked


/* Callback "clicked" for GtkButton widget wgt_btn_done */
void
cb_btn_done_clicked(GtkButton *button, gpointer user_data)
{

	cb_btn_apply_clicked( button, user_data );
	
	/*--- Close the dialog ---*/
	gtk_widget_destroy( wgt_res_edit_strings );
	wgt_res_edit_strings = NULL;
	
}								// cb_btn_done_clicked


/* Callback "select_child" for GtkCList widget wgt_list_items */
void
cb_list_items_select_row( GtkCList *list, 
	gint row, gint column,
	GdkEventButton *event, gpointer user_data )
{

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_after ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_before ),	FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_remove ),		TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_update ),		TRUE );

}								// cb_list_items_select_row


void
cb_item_changed(GtkEditable *editable, gpointer user_data)
{

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_after ), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_before ), TRUE );

}								// cb_item_changed


static GtkWidget *
create_window( GtkWidget *parent, char *title )
{
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_2_5, *wgt_2_10, 
				*wgt_3_11, *wgt_2_13, *wgt_2_14;

	wgt_res_edit_strings = gtk_widget_new( gtk_window_get_type(),
		"GtkWidget::width", 425,
		"GtkWidget::height", 363,
		"GtkWidget::events", 0x0000030C,
		"GtkContainer::resize_mode", GTK_RESIZE_QUEUE,
		"GtkWindow::type", GTK_WINDOW_DIALOG,
		"GtkWindow::title", title,
		"GtkWindow::allow_grow", TRUE,
		"GtkWindow::modal", TRUE,
		"GtkWindow::default_width", 400,
		"GtkWindow::default_height", 300,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"GtkWidget::parent", wgt_res_edit_strings,
		"GtkContainer::border_width", 6,
		"GtkBox::spacing", 2,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_frame_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkFrame::label", "List of Items",
		NULL );

	wgt_list_items = gtk_widget_new( gtk_clist_get_type(),
		"n_columns", 1,
		"GtkWidget::parent", wgt_2_3,
		"selection_mode", GTK_SELECTION_BROWSE,
		"titles_active", FALSE,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_list_items ), "select_row",
		GTK_SIGNAL_FUNC( cb_list_items_select_row ), NULL );

	wgt_2_5 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkBox::homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_5,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 1,
		NULL );

	wgt_btn_add_after = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_5,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Add After",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_add_after,
		"GtkBox::expand", FALSE,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_add_after ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_add_after_clicked ), NULL );

	wgt_btn_add_before = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_5,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Add Before",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_add_before,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_add_before ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_add_before_clicked ), NULL );

	wgt_btn_remove = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_5,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Remove",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_remove,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_remove ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_remove_clicked ), NULL );

	wgt_btn_update = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_5,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Update",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_5 ), wgt_btn_update,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 3,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_update ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_update_clicked ), NULL );

	wgt_2_10 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkBox::spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_10,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 2,
		NULL );

	wgt_3_11 = gtk_widget_new( gtk_label_get_type(),
		"GtkWidget::parent", wgt_2_10,
		"GtkLabel::label", "Item:",
		"GtkLabel::justify", GTK_JUSTIFY_LEFT,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_10 ), wgt_3_11,
		"GtkBox::expand", FALSE,
		NULL );

	wgt_item = gtk_widget_new( gtk_entry_get_type(),
		"GtkWidget::parent", wgt_2_10,
		"GtkWidget::can_focus", TRUE,
		NULL );
	gtk_entry_set_text( GTK_ENTRY(wgt_item), "" );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_10 ), wgt_item,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_item ), "changed",
		GTK_SIGNAL_FUNC( cb_item_changed ), NULL );

	wgt_2_13 = gtk_widget_new( gtk_hseparator_get_type(),
		"GtkWidget::parent", wgt_1_2,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_13,
		"GtkBox::expand", FALSE,
		"GtkBox::padding", 4,
		"GtkBox::position", 3,
		NULL );

	wgt_2_14 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkBox::homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_14,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 4,
		NULL );

	wgt_btn_default = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_14,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Default",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_14 ), wgt_btn_default,
		"GtkBox::expand", FALSE,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_default ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_default_clicked ), NULL );

	wgt_btn_cancel = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_14,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Cancel",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_14 ), wgt_btn_cancel,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_cancel ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_cancel_clicked ), NULL );

	wgt_btn_apply = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_14,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Apply",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_14 ), wgt_btn_apply,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_apply ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_apply_clicked ), NULL );

	wgt_btn_done = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_14,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Done",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_14 ), wgt_btn_done,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 3,
		NULL );
	gtk_signal_connect( GTK_OBJECT( wgt_btn_done ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_done_clicked ), NULL );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_after ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_add_before ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_remove ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_update ), FALSE );

	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_default ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_apply ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( wgt_btn_done ), FALSE );

	return wgt_res_edit_strings;

}								// create_window


PUBLIC void
dlg_res_editor_strings( GtkWidget *wgt_btn, GtkWidget *wgt_combo, 
	defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth )
{
	text_data_t				*text_data;
	char					title[ 200 ];
	GtkWidget				*wgt_selected;
	GtkArg					arg;
	gtk_usertype_strings_t	*strings;
	int						n;

	/*--- Create the dialog ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	assert( wgt_selected != NULL );
	if ( get_nametype_wgt( wgt_selected ) == NAMETYPE_GLOBAL )
	{
		char *widget_name = get_name_wgt( wgt_selected );
		sprintf( title, "%s::%s\n",
			widget_name, defclass->resname );
	}
	else
	{
		sprintf( title, "Resource %s\n",
			defclass->resname );
	}
	create_window( NULL, title );

	/*--- First read current value (stored inside the widget or not) ---*/
	if ( get_res_strings( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg ) == -1 )
		return;

	/*--- Update data ---*/
	strings = GTK_VALUE_POINTER( arg );
	assert( strings != NULL );
	for ( n = 0; n < strings->nb_strings; n++ )
	{
		char *text[ 1 ];
		text[ 0 ] = strings->strings[ n ];
		gtk_clist_append( GTK_CLIST( wgt_list_items ), text );
	}

	/*--- Update data ---*/
	text_data = malloc( sizeof( text_data_t ) );
	memset( text_data, 0, sizeof( text_data_t ) );
	text_data->wgt_combo	 = wgt_combo;
	text_data->wgt_parent	 = wgt_parent;
	text_data->defclass	 	 = defclass;
	text_data->wgtdef		 = wgtdef;
	text_data->wgtdef_depth  = wgtdef_depth;
	set_object_data( wgt_item, "text_data", text_data, TRUE );
	set_object_data( wgt_list_items, "text_data", text_data, FALSE );

	gtk_signal_connect( GTK_OBJECT( wgt_res_edit_strings ), "delete_event",
		GTK_SIGNAL_FUNC( cb_close_window ), NULL );
	gtk_widget_show_all( wgt_res_edit_strings );

	push_block_all_windows();
	while ( wgt_res_edit_strings != NULL )
		gtk_main_iteration();
	pop_block_all_windows();

}								// dlg_res_editor_strings

