#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <time.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static int
create_header( FILE *fp, char *label, ... )
{
	va_list ap;
	char    name[ 384 ];
	char	buff[ 80 ];
	time_t	now;

	va_start( ap, label );
	vsprintf( name, label, ap );
	va_end( ap );

	fprintf( fp, "#\n" );
	fprintf( fp, "# %s\n", name );
	fprintf( fp, "#\n" );
	now = time( NULL );
	strftime( buff, sizeof(buff), "%A %d %B %Y %T", localtime( &now ) );
	fprintf( fp, "# Date: %s\n", buff );
	fprintf( fp, "#\n" );
	fprintf( fp, "# Generated with Gtk Lo Gafet (GLG), v%d.%d.%c [%s %s]\n",
		FLG_APPVERSION_MAJOR, FLG_APPVERSION_MINOR, FLG_APPVERSION_SEQ + 'a',
		FLG_APPVERSION_DATE, FLG_APPVERSION_TIME );
	fprintf( fp, "# Check-out the next URL for the latest version:\n" );
	fprintf( fp, "#		http://glg.sourceforget.net\n" );
	fprintf( fp, "#\n" );

	return 0;

}								// create_header


PUBLIC int
save_wnd_infos( FILE *fp, GtkWidget *wnd, char *tag )
{
	gint	x, y;
	gint	w, h;

	/*--- Read pos and size of the window ---*/
	gdk_window_get_position( wnd->window, &x, &y );
	gdk_window_get_size( wnd->window, &w, &h );

	/*--- Write infos to file ---*/
	fprintf( fp, "%s = %d,%d,%d,%d\n",
		tag, 
		x, y, w, h );

	return 0;

}								// save_wnd_infos


static int
create_glg_main( char *glgname, char *title )
{
	FILE			*fp;
	module_info_t	*module;
	char			*name_wgt;
	gint			w, h;

	/*--- Create the main project file ---*/
	fp = fopen( glgname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d", glgname, errno );
		return -1;
	}							// if

	/*--- A kind of header ---*/
	create_header( fp, title );

	/*--- Is-it a GNOME only application ? ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "# Is-it a GNOME only application ? Or a GTK only application ?\n" );
	fprintf( fp, "Gnome = %s\n",
		(config.is_gnome) ? "yes" : "no" );

	/*--- Save the dim/pos of the working windows ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "# Working windows position and dimension\n" );
	save_wnd_infos( fp, wgt_wnd_palette, "Palette window" );
	save_wnd_infos( fp, wgt_wnd_tree, "Tree window" );
	save_wnd_infos( fp, wgt_wnd_resources, "Properties window" );

	/*--- Save the names of the modules ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "# List of modules\n" );
	module = next_module( FIRST );
	while ( module )
	{

		switch ( module->type )
		{
		
			case MODULE_TYPE_WINDOW :
				name_wgt = get_name_wgt( module->wgt_window );
				assert( strcmp( name_wgt, "" ) );
				fprintf( fp, "\t%s %s\n",
					"GtkWindow",
					name_wgt );
				gdk_window_get_size( module->wgt_window->window, &w, &h );
				fprintf( fp, "\t\t%s %s %s %s %d,%d\n",
					(module->u1.window.f_opened_from_main) ? "yes" : "no",
					(module->u1.window.f_fixed_position) ? "yes" : "no",
					module->u1.window.name_function,
					(module->iconified) ? "iconified" : "visible",
					w, h );
				break;

			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				name_wgt = get_name_wgt( module->wgt_window );
				assert( strcmp( name_wgt, "" ) );
				fprintf( fp, "\t%s %s\n",
					"GtkDialog",
					name_wgt );
				gdk_window_get_size( module->wgt_window->window, &w, &h );
				fprintf( fp, "\t\t%s %s %s %s %d,%d\n",
					(module->u1.dialog.f_opened_from_main) ? "yes" : "no",
					(module->u1.dialog.f_fixed_position) ? "yes" : "no",
					module->u1.dialog.name_function,
					(module->iconified) ? "iconified" : "visible",
					w, h );
				break;

			case MODULE_TYPE_MENU :
				name_wgt = get_name_wgt( module->wgt_window );
				assert( strcmp( name_wgt, "" ) );
				fprintf( fp, "\t%s %s\n",
					"GtkMenu",
					name_wgt );
				fprintf( fp, "\t\t%s %s\n",
					module->u1.menu.name_function,
					(module->iconified) ? "iconified" : "visible" );
				break;

			default :
				assert(0);
				break;
			
		}						// switch ( module->type )

		module = next_module( NEXT );

	}							// while (module)

	/*--- Ok ---*/
	fclose( fp );
	return 0;

}								// create_glg_main


static int
save_style_color( FILE *fp, char *what, GdkColor color )
{
	fprintf( fp, "\t\t%s = (%u,%u,%u)\n",
		what,
		color.red, color.green, color.blue );
	return 0;
}								// save_style_color


static int
save_style( FILE *fp, GtkStyle *style, style_info_t *style_info )
{

	fprintf( fp, "\n\tstyle %s {\n",
		style_info->name );

	save_style_color( fp, "fg0", style->fg[ 0 ] );
	save_style_color( fp, "fg1", style->fg[ 1 ] );
	save_style_color( fp, "fg2", style->fg[ 2 ] );
	save_style_color( fp, "fg3", style->fg[ 3 ] );
	save_style_color( fp, "fg4", style->fg[ 4 ] );
	
	save_style_color( fp, "bg0", style->bg[ 0 ] );
	save_style_color( fp, "bg1", style->bg[ 1 ] );
	save_style_color( fp, "bg2", style->bg[ 2 ] );
	save_style_color( fp, "bg3", style->bg[ 3 ] );
	save_style_color( fp, "bg4", style->bg[ 4 ] );
	
	save_style_color( fp, "text0", style->text[ 0 ] );
	save_style_color( fp, "text1", style->text[ 1 ] );
	save_style_color( fp, "text2", style->text[ 2 ] );
	save_style_color( fp, "text3", style->text[ 3 ] );
	save_style_color( fp, "text4", style->text[ 4 ] );
	
	save_style_color( fp, "base0", style->base[ 0 ] );
	save_style_color( fp, "base1", style->base[ 1 ] );
	save_style_color( fp, "base2", style->base[ 2 ] );
	save_style_color( fp, "base3", style->base[ 3 ] );
	save_style_color( fp, "base4", style->base[ 4 ] );

	fprintf( fp, "\t}\n" );
	return 0;

}								// save_style


static int
save_styles( char *glgname, char *title )
{
	FILE			*fp;
	GtkStyle		*style;
	style_info_t	*style_info;
	int				n;

	/*--- Create the main project file ---*/
	fp = fopen( glgname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d", glgname, errno );
		return -1;
	}							// if

	/*--- A kind of header ---*/
	create_header( fp, title );

	/*--- How many styles ---*/
	fprintf( fp, "Number of styles = %d\n", nb_styles );

	/*--- Now save each style ---*/
	for ( n = 0; n < nb_styles; n++ )
	{
		style = styles[ n ];
		style_info = styles_infos[ n ];
		assert( style_info != NULL );
		save_style( fp, style, style_info );
		
	}							// for (n)

	/*--- Ok ---*/
	fclose( fp );
	return 0;

}								// save_styles


static void
outs( FILE *fp, const int level, char *resname, char *fmt, ... )
{
	va_list	ap;

	fprintf( fp, "%*s%s ", (level+1)*4, "", resname );
	va_start( ap, fmt );
	vfprintf( fp, fmt, ap );
	va_end( ap );
}								// outs


static void
outstr( FILE *fp, const int level, char *resname, char *str )
{
	char			*p;
	unsigned char	c;

	assert( str != NULL );
	fprintf( fp, "%*s%s \"", (level+1)*4, "", resname );
	for ( p = str; (c = *p) != 0; p++ )
	{
		if ( (c >= 0x20) && (c <= 0x7E) && (c != '\\') )
			fputc( c, fp );
		else
			fprintf( fp, "\\%03o", c );
	}
	fprintf( fp, "\"\n" );

}								// outstr


static void
srcdump_widget_file( GtkWidget *wgt, GtkWidget *wgt_parent, 
	module_info_t *module,
	const int level, FILE *fp )
{
	char				*wgt_name;
	gtk_wgtdef_t		*wgtdef=NULL;
	gtk_wgtdef_t		*wgtdef_depth;
	int					wgt_nametype;
	defclass_t			*classes=NULL, *p;
	int					depth, n, num;
	int					nb_res=0, _nb_args, total;
	GtkArg				arg, *parg=NULL;
	gtk_wgtdef_t		*wgtdef_parent;

	/*--- Name of the widget ---*/
	wgt_name     = get_name_wgt( wgt );
	wgt_nametype = get_nametype_wgt( wgt );

	if (wgt_parent)
	{
		wgtdef_parent = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
		assert( wgtdef_parent != NULL );
	}
	else
	{
		wgtdef_parent = NULL;
	}
	
	/*--- Dump resources set to non default value ---*/
	for ( total = 0, num = 0; num < 2; num++ )
	{

		if ( num == 0 )
		{
			char	hierarchy[ 80 ];
			wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
			assert( wgtdef != NULL );
			if ( IS_WGT_NOCOMPOSITE( wgt ) )
			{
				strcpy( hierarchy, "" );
			}
			else
			{
				char *s = get_object_data( wgt, WGTDATA_HIERARCHY_CHILD );
				assert( s != NULL );
				sprintf( hierarchy, "*%03d*",atoi( s ) );
			}
			fprintf( fp, "%*s%s%s (%s) %s {\n",
				level * 4, "",
				hierarchy,
				wgtdef->widget_name,
				(IS_WGT_PLACEHOLDER(wgt)) ? "placeholder" :
					((wgt_nametype == NAMETYPE_ANONYMOUS) ? "anonymous" : "global"),
				(wgt_name && wgt_name[0]) ? wgt_name : "?" );
		}
		else if ( num == 1 )
		{
			if ( wgt_parent == NULL )
				break;
			wgtdef = gtk_object_get_data( GTK_OBJECT( wgt_parent ), WGTDATA_WGTDEF );
			if ( wgtdef == NULL )
				continue;
		}

		_nb_args = 0;
		for ( depth = 0; 
			  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
			  depth++ )
		{

			/*--- Dump all resources ---*/
			if ( num == 0 )
			{
				classes = wgtdef->def[depth].classes;
				nb_res  = wgtdef->def[depth].nb_classes;
			}
			else if ( num == 1 )
			{
				classes = wgtdef->def[depth].classes_child;
				nb_res  = wgtdef->def[depth].nb_classes_child;
			}

			wgtdef_depth = wgtdef->def[depth].wgtdef_depth;

			for ( p = classes, n = 0; n < nb_res; n++, p++ )
			{

				if ( !p->f_generate )
					continue;
				if ( is_default_value( wgt, wgt_parent, wgtdef_depth, p ) )
					continue;

#if 0
				if (wgtdef_parent != NULL)
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") || !strcmp(p->resname, "~GtkWidget::y") )
						if ( wgtdef_parent->f_children_autopos )
							continue;
					if ( !strcmp(p->resname, "~GtkWidget::width") || !strcmp(p->resname, "~GtkWidget::height") )
						if ( wgtdef_parent->f_children_autodim )
							continue;
				}
#endif

				if ( get_is_disabled_resource_wgt(wgt, p) )
					continue;

				/*--- Don't save X, Y, Width and Height for a Menu module ---*/
				if ( module->type == MODULE_TYPE_MENU )
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") ||
						 !strcmp(p->resname, "~GtkWidget::y") ||
						 !strcmp(p->resname, "~GtkWidget::width") ||
						 !strcmp(p->resname, "~GtkWidget::height") )
						continue;
				}						// if

				if ( wgtdef->pfct_get_widget_type() == GTK_TYPE_WINDOW )
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") ||
						 !strcmp(p->resname, "~GtkWidget::y") )
					{
						if ( !module->u1.window.f_fixed_position )
						{
							if ( (module->type == MODULE_TYPE_WINDOW) || (module->type == MODULE_TYPE_DIALOG)
#ifdef USE_GNOME
								 || (module->type == MODULE_TYPE_GNOMEDIALOG_DIALOG)
								 || (module->type == MODULE_TYPE_GNOMEDIALOG_ABOUT)
								 || (module->type == MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX)
								 || (module->type == MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX)
#endif
								)
								continue;
						}
					}
				}						// if

				if ( p->saved_where == SAVED_INSIDE )
				{
					arg.name = p->resname;
					if ( num == 0 )
						gtk_widget_get( GTK_WIDGET(wgt), &arg );
					else if ( num == 1 )
						gtk_container_child_getv( GTK_CONTAINER(wgt_parent), GTK_WIDGET(wgt), 1, &arg );
					parg = &arg;
				}
				else if ( p->saved_where == SAVED_OUTSIDE || p->saved_where == SAVED_OUTSIDE_FUNCTION )
				{
					parg = get_outsideres_wgt( wgt, p->resname );
					assert( parg != NULL );
				}
				else if ( p->saved_where == SAVED_FUNCTION )
				{
					memset( &arg, 0, sizeof(GtkArg) );
					arg.type = p->restype;
					if ( (*wgtdef_depth->pfct_getdlg)(p, wgt, &arg) == NULL )
						parg = get_outsideres_wgt( wgt, p->resname );
					else
						parg = &arg;
					assert( parg != NULL );
				}
				else assert(0);

				switch ( p->restype )
				{

					case GTK_TYPE_INT :
						outs( fp, level, p->resname, "%d\n", 
							GTK_VALUE_UINT(parg[0]) );
						break;

					case GTK_TYPE_UINT :
						outs( fp, level, p->resname, "%u\n", 
							GTK_VALUE_UINT(parg[0]) );
						break;

					case GTK_TYPE_LONG :
						outs( fp, level, p->resname, "%ld\n", 
							GTK_VALUE_LONG(parg[0]) );
						break;

					case GTK_TYPE_ULONG :
						outs( fp, level, p->resname, "%lu\n", 
							GTK_VALUE_ULONG(parg[0]) );
						break;

					case GTK_TYPE_FLOAT :
						outs( fp, level, p->resname, "%f\n", 
							GTK_VALUE_FLOAT(parg[0]) );
						break;

					case GTK_TYPE_BOOL :
						outs( fp, level, p->resname, "%d\n", 
							GTK_VALUE_BOOL(parg[0]) );
						break;

					case GTK_TYPE_STRING :
						if ( GTK_VALUE_STRING(parg[0]) )
							outstr( fp, level, p->resname, 
								GTK_VALUE_STRING(parg[0]) );
						break;

					case GTK_TYPE_OBJECT :
						if ( !strcmp( p->object_restype_str, "GTK_TYPE_STYLE" ) )
						{
							GtkObject	*obj;
							obj = GTK_VALUE_OBJECT( parg[0] );
							outs( fp, level, p->resname, "%s\n",
								get_style_name( (GtkStyle *)obj ) );
						}
						else if ( !strcmp( p->resname, "GtkWidget::parent" ) )
						{
						}
						else
						{
							GtkObject	*obj;
							obj = GTK_VALUE_OBJECT( parg[0] );
							outs( fp, level, p->resname, "%s\n",
								get_name_wgt( GTK_WIDGET( obj ) ) );
						}
						break;

					case GTK_TYPE_ENUM :
						outs( fp, level, p->resname, "%s\n", 
							p->enum_list[ search_enum( GTK_VALUE_ENUM(parg[0]), p ) ].value_name );
						break;

					case GTK_TYPE_FLAGS :
						outs( fp, level, p->resname, "%08X\n", 
							GTK_VALUE_FLAGS(parg[0]) );
						break;

					case GTK_TYPE_POINTER :
						if ( !strcmp( p->object_restype_str, "GTK_TYPE_ADJUSTMENT" ) )
						{
							GtkAdjustment	*padj;
							padj = GTK_VALUE_POINTER( parg[0] );
							assert( padj != NULL );
							outs( fp, level, p->resname, "%f %f %f %f %f %f\n", 
								padj->value, padj->lower, padj->upper,
								padj->step_increment, padj->page_increment, padj->page_size );
						}
						else if ( !strcmp( p->object_restype_str, "GTK_USERTYPE_STRINGS" ) )
						{	
							gtk_usertype_strings_t	*strings;		 
							int						n;
							strings = GTK_VALUE_POINTER( parg[ 0 ] );
							outs( fp, level, p->resname, "%d\n", 
								strings->nb_strings );
							for ( n = 0; n < strings->nb_strings; n++ )
							{
								char resname[ 80 ];
								sprintf( resname, "string[%4d]", n );
								outstr( fp, level+1, resname, 
									strings->strings[ n ] );
							}
						}
						else
							assert( 0 );
						break;

					default :
						RES_NOT_YET_HANDLED( wgtdef, p );
						assert(0);
						break;

				}				// switch

				_nb_args++;
				total += _nb_args;
			}					// for(p)
		
		}						// for (depth)
		
	}							// for (num)

	/*--- Dump all signals (Callbacks) ---*/
	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );
	for ( depth = 0;
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL);
		  depth++ )
	{
		defclasssignal_t	*classes_signal, *q;
		int					nb_signals;
		
		classes_signal = wgtdef->def[depth].classes_signal;
		nb_signals	   = wgtdef->def[depth].nb_classes_signal;
		for ( q = classes_signal, n = 0; n < nb_signals; n++, q++ )
		{
			data_cb_t	*datacb;
			int			k;
			def_cb_t	*p;
			
			datacb = get_object_data( wgt, q->resname );
			if (datacb == NULL)
				continue;
			for ( k = 0; k < datacb->nb_cb; k++ )
			{
				p = &datacb->cb[ k ];
				outs( fp, level, q->resname, "%s@%s after=%s accelerator=%s",
					p->cbname,
					p->cbfile,
					(p->f_after) ? "yes" : "no",
					(p->is_accelerator) ? "yes" : "no" );
				if ( p->is_accelerator )
				{
					fprintf( fp, " %04X %s shift=%s ctrl=%s alt=%s visible=%s",
						p->accelerator_key_val, p->accelerator_key_string,
						(p->accelerator_mods & GDK_SHIFT_MASK) ? "yes" : "no",
						(p->accelerator_mods & GDK_CONTROL_MASK) ? "yes" : "no",
						(p->accelerator_mods & GDK_MOD1_MASK) ? "yes" : "no",
						(p->accel_flags & GTK_ACCEL_VISIBLE) ? "yes" : "no" );
				}				// if
				fprintf( fp, "\n" );
			}
		}						// for (q)
	}							// for (depth)		

}								// srcdump_widget_file


typedef struct
{
	module_info_t	*module;
	int				level;
	int				cpt;			// Cpt for anonymous variables, same level
	GtkWidget		*parent;
	FILE			*fp;
}	SAVEMODULEWINDOW_DATA;

static void
forall_save_module_window( GtkWidget *widget, SAVEMODULEWINDOW_DATA *data )
{

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if (!config.show_composite_childs )
#endif
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget))
			return;

	data->level++;
	data->cpt++;

	/*--- Don't save 'placeholder' container ---*/
	srcdump_widget_file( widget, data->parent, data->module,
		data->level,
		data->fp );

	if ( GTK_IS_CONTAINER (widget) )
	{
		SAVEMODULEWINDOW_DATA	new_data;

		new_data.module	  = data->module;
		new_data.level    = data->level;
		new_data.cpt   	  = 0;
		new_data.fp		  = data->fp;
		new_data.parent	  = widget;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_save_module_window, &new_data );
	}

	fprintf( data->fp, "%*s}\n",
		data->level * 4,
		"" );

	data->level--;

}								// forall_save_module_window


static int
save_module_window( char *glgdir, module_info_t *module )
{
	char					glgname[ PATH_MAX+NAME_MAX + 64 ];
	FILE					*fp;
	SAVEMODULEWINDOW_DATA	data;

	/*--- Create the main project file ---*/
	sprintf( glgname, "%s/%s.glg", 
		glgdir, get_name_wgt( module->wgt_window ) );
	fp = fopen( glgname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d", glgname, errno );
		return -1;
	}							// if

	/*--- A kind of header ---*/
	create_header( fp, "Description of the Window Module: %s", 
		get_name_wgt( module->wgt_window ) );

	/*--- Save each widget of this module ---*/
	data.module = module;
	data.level  = -1;
	data.cpt    = 0;
	data.parent = NULL;
	data.fp     = fp;
	forall_save_module_window( module->wgt_window, &data );

	/*--- Ok ---*/
	fclose( fp );
	return 0;

}								// save_module_window


static int
save_module_menu( char *glgdir, module_info_t *module )
{
	char					glgname[ PATH_MAX+NAME_MAX + 64 ];
	FILE					*fp;
	SAVEMODULEWINDOW_DATA	data;

	/*--- Create the main project file ---*/
	sprintf( glgname, "%s/%s.glg", 
		glgdir, get_name_wgt( module->wgt_window ) );
	fp = fopen( glgname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d", glgname, errno );
		return -1;
	}							// if

	/*--- A kind of header ---*/
	create_header( fp, "Description of the Menu Module: %s", 
		get_name_wgt( module->wgt_window ) );

	/*--- Save each widget of this module ---*/
	data.module = module;
	data.level  = -1;
	data.cpt    = 0;
	data.parent = NULL;
	data.fp     = fp;
	forall_save_module_window( module->wgt_window, &data );

	/*--- Ok ---*/
	fclose( fp );
	return 0;

}								// save_module_menu


PUBLIC int
do_save_project( char *dirname )
{
	char			glg_fname[ PATH_MAX+NAME_MAX + 100 ];
	char			styles_fname[ PATH_MAX+NAME_MAX + 100 ];
	char			glgdir[ PATH_MAX+NAME_MAX + 100 ];
	struct stat		sbuf;
	int				status;
	module_info_t	*module;

	if ( !stat( dirname, &sbuf ) && S_ISREG( sbuf.st_mode ) )
	{
		modal_error( "\n%s\nis not a directory.\n", dirname );
		return -1;
	}
	
	/*--- Does the 'lg' directory exists ? ---*/
	if ( stat( dirname, &sbuf ) == -1 )
	{
		status = yesno_dialog( "Save Project", YES,
			"\nThis directory does not exist.\n%s\n\nWant to create it ?\n", dirname );
		if ( status == NO )
			return -1;
		status = mkdir( dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d\n", 
				dirname, errno );
			return -1;
		}
	}

	/*--- Does the 'lg' directory exists ? ---*/
	sprintf( glgdir, "%s/glg", dirname );
	if ( stat( glgdir, &sbuf ) == -1 )
	{
//		status = yesno_dialog( "Save Project", YES,
//			"\nThis directory does not exist.\n%s\n\nWant to create it ?\n", glgdir );
//		if ( status == NO )
//			return -1;
		status = mkdir( glgdir, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d\n", 
				glgdir, errno );
			return -1;
		}
	}
	else
	{
		if ( !S_ISDIR( sbuf.st_mode ) )
		{
			modal_error( "%s\nis not a directory.", glgdir );
			return -1;
		}
	}

	/*--- Create the main file ---*/
	sprintf( glg_fname, "%s/glg", glgdir );
	if ( create_glg_main( glg_fname, "General - List of modules" ) == -1 )
		return -1;

	/*--- Now save the styles, if any ---*/
	sprintf( styles_fname, "%s/styles", glgdir );
	save_styles( styles_fname, "List of styles" );

	/*--- Save each module ---*/
	module = next_module( FIRST );
	while ( module )
	{
		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				if ( module->f_not_saved )
				{
					status = save_module_window( glgdir, module );
					if ( status == 0 )
						project_saved( module );
				}
				break;
			case MODULE_TYPE_MENU :
				if ( module->f_not_saved )
				{
					status = save_module_menu( glgdir, module );
					if ( status == 0 )
						project_saved( module );
				}
				break;
			default :
				assert(0);
				break;
		}
		module = next_module( NEXT );
	}							// while

	/*--- Ok, done ---*/
	strcpy( project_dirname, dirname );

	/*--- Add this project to the list of the last projects ---*/
	add_to_last_projects( project_dirname );
	
	/*--- Clear touch project flag ---*/
	is_project_touched = FALSE;
	
	return 0;
	
}								// do_save_project
