#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <sys/wait.h>
#include <inttypes.h>
#include <signal.h>

#include "glg.h"


static	int		f_child_done;


static int
_locate_cb( def_cb_t *defcb, FILE *fp, int *lineno )
{
	defclasssignal_t	*classes_signal;
	int					sz, z;
	char				model[512];
	char				*buff, *p;

	/*--- Generate the lines to be searched ---*/
	classes_signal = defcb->defclasssignal;
	sprintf( model, "%s\n%s(",
		classes_signal->proto.ret,
		defcb->cbname );
	for ( z = 0; 
		  z < sizeof(classes_signal->proto.params)/sizeof(classes_signal->proto.params[0]);
		  z++ )
	{
		if ( !classes_signal->proto.params[z] )
			break;
		sprintf( &model[strlen(model)], "%s%s",
			(!z) ? "" : ", ",
			classes_signal->proto.params[z] );
	}
	sprintf( &model[strlen(model)], ")\n{\n" );

	/*--- Read the file into a temp buffer ---*/
	fseek( fp, 0, SEEK_END );
	sz = (int)ftell( fp );
	fseek( fp, 0, SEEK_SET );
	buff = malloc(sz+1);
	if (buff == NULL)
		return FALSE;
	fread( buff, sz, 1, fp );
	buff[sz] = 0;

	/*--- Found ? ---*/
	if ( (p = strstr( buff, model )) == NULL )
	{
		for ( p = buff, *lineno = 0; ; )
		{
			if ( (p = strchr( p, '\n' )) == NULL )
				break;
			(*lineno)++;
			p++;
		}		
		free(buff);
		return FALSE;
	}

	/*--- Search for the line number ---*/
	for ( p[0] = 0, p = buff, *lineno = 0; ; )
	{
		if ( (p = strchr( p, '\n' )) == NULL )
			break;
		(*lineno)++;
		p++;
	}		
	free(buff);
	return TRUE;

}								// _locate_cb


static int
add_cb_to_file( GtkWidget *wgt, def_cb_t *defcb, FILE *fp )
{
	defclasssignal_t	*classes_signal;
	char				*label = "";
	gtk_wgtdef_t		*wgtdef;
	char				*name_wgt;
	int					z;
	int					lineno;

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	classes_signal = defcb->defclasssignal;

	name_wgt = get_name_wgt(wgt);
	lineno = 0;
	fprintf( fp, "\n/* Callback \"%s\" for %s widget %s%s */\n",
		classes_signal->resname, 
		wgtdef->widget_name,
		(name_wgt) ? name_wgt : "anonymous",
		label );
	lineno++;

	fprintf( fp, "%s\n%s(",
		classes_signal->proto.ret,
		defcb->cbname );
	lineno++;
	for ( z = 0; 
		  z < sizeof(classes_signal->proto.params)/sizeof(classes_signal->proto.params[0]);
		  z++ )
	{
		if ( !classes_signal->proto.params[z] )
			break;
		fprintf( fp, "%s%s",
			(!z) ? "" : ", ",
			classes_signal->proto.params[z] );
	}
	fprintf( fp, ")\n" ); lineno++;
	fprintf( fp, "{\n" ); lineno++;
	if ( classes_signal->proto.retval != NULL )
	{
		fprintf( fp, "\t%s;\n", classes_signal->proto.retval );
		lineno++;
	}
	fprintf( fp, "\t\n" );
	lineno++;
	fprintf( fp, "}\t\t\t\t\t\t\t\t// %s\n", defcb->cbname );
	lineno++;

	/*--- Ok ---*/
	return lineno;

}								// add_cb_to_file


PUBLIC int
locate_cb_in_file( GtkWidget *wgt, def_cb_t *defcb )
{
	char	cfile[ PATH_MAX+NAME_MAX + 100 ];
	FILE	*fp;
	int		status;
	int		found, lineno;

	/*--- Must create src directorty (ie. generate not yet called) ---*/
	sprintf( cfile, "%s/src", project_dirname );
	if ( chdir( cfile ) == -1 )
	{
		status = mkdir( cfile, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d", 
				cfile, errno );
			return -1;
		}
	}							// if

	sprintf( cfile, "%s/src/%s.c", 
		project_dirname, defcb->cbfile );
	fp = fopen( cfile, "r" );
	if ( fp != NULL )
	{

		found = _locate_cb( defcb, fp, &lineno );
		fclose( fp );
		if ( found )
			return lineno + 4;

		/*--- Add this callback at the end of the file ---*/
		fp = fopen( cfile, "a" );
		if ( fp == NULL )
		{
			modal_error( "Unable to add callback %s\nin file %s\nerrno=%d",
				defcb->cbname, defcb->cbfile, errno );
			return -1;
		}
		lineno += add_cb_to_file( wgt, defcb, fp );
		fclose( fp );
		return lineno;

	}							// if
	
	/*--- Add this callback at the end of the file ---*/
	fp = fopen( cfile, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to add callback %s\nin file %s\nerrno=%d",
			defcb->cbname, defcb->cbfile, errno );
		return -1;
	}

	/*--- Add some headers to this file ---*/
	add_system_includes( fp );
	fprintf( fp, "\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp, "#include <gnome.h>\n" );
	else
#endif
	fprintf( fp, "#include <gtk/gtk.h>\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#include \"glg/main.h\"\n" );
	lineno = 9;

	fprintf( fp, "#include \"glg/modules.h\"\n" );
	lineno++;

	fprintf( fp, "\n" );
	lineno++;

	lineno += add_cb_to_file( wgt, defcb, fp );
	fclose( fp );

	return lineno;

}								// locate_cb_in_file


static void
sigchld_sighandler( int signum )
{
	f_child_done = TRUE;
}								// sigchld_sighandler


PUBLIC int
edit_file( char *cfile, int lineno )
{
	const char	*argv[ 10 ];
	pid_t		pid;
	char		tmp[ 100 ];
	int 		nb;

	nb = 0;
	argv[ nb++ ] = config.editor1;
	if ( strcmp( config.editor2, "" ) )
		argv[ nb++ ] = config.editor2;
	if ( strcmp( config.editor3, "" ) )
		argv[ nb++ ] = config.editor3;
	if ( config.is_glo_after )
	{
		argv[ nb++ ] = cfile;
		if ( config.goto_line_option[ 0 ] )
		{
			sprintf( tmp, "%s%d", config.goto_line_option, lineno );
			argv[ nb++ ] = tmp;
		}
	}
	else
	{
		if ( config.goto_line_option[ 0 ] )
		{
			sprintf( tmp, "%s%d", config.goto_line_option, lineno );
			argv[ nb++ ] = tmp;
		}
		argv[ nb++ ] = cfile;
	}
	argv[ nb++ ] = NULL;

	pid = fork();
	if ( pid == -1 )
	{
		modal_error( "Unable to fork\nerrno=%d", errno );
		return -1;
	}
	if ( pid == 0 )
	{
		execvp( argv[0], (char**)argv );
		fprintf( stderr, "Unable to launch [%s]\nerrno=%d\n",
			argv[0], errno );
		_exit(0);
	}
	else
	{
		f_child_done = FALSE;
		signal( SIGCHLD, sigchld_sighandler );
		push_block_all_windows();
		while ( !f_child_done )
			gtk_main_iteration();
		pop_block_all_windows();
	}
	
	return 0;
}								// edit_file


PUBLIC int
edit_cb( GtkWidget *wgt, def_cb_t *defcb )
{
	char		cfile[ PATH_MAX+NAME_MAX + 100 ];
	int			lineno;

	/*--- Project not yet saved ? ---*/
	if ( !strcmp(project_dirname, "" ) )
	{
		if ( !save_project_as() )
			return 0;
	}
	
	if ( (lineno = locate_cb_in_file( wgt, defcb )) == -1 )
		return -1;

	sprintf( cfile, "%s/src/%s.c", 
		project_dirname, defcb->cbfile );

	return edit_file( cfile, lineno );

}								// edit_cb
