#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static int
skip_widget( GtkWidget *widget )
{

#if defined(GLG_SUPPORT_EDIT_COMPOSITE) 
	if (!config.show_composite_childs )
#endif
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget))
			return TRUE;

	return FALSE;

}								// skip_widget

PUBLIC void
add_system_includes( FILE *fp )
{

	fprintf( fp, "#include <stdio.h>\n" );
	fprintf( fp, "#include <stdlib.h>\n" );
	fprintf( fp, "#include <string.h>\n" );
	fprintf( fp, "#include <stdarg.h>\n" );
	fprintf( fp, "#include <time.h>\n" );
	fprintf( fp, "#include <inttypes.h>\n" );
	fprintf( fp, "#include <unistd.h>\n" );

}								// add_system_includes


static int
generate_main_c( const char * const dirname, const char * const filename )
{
	char			fname[ PATH_MAX+NAME_MAX + 80 ];
	FILE			*fp;
	module_info_t	*module;

	sprintf( fname, "%s/glg/%s", dirname, filename );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create file\n%s\nerrno=%d\n", errno );
		return -1;
	}

	add_system_includes( fp );
	fprintf( fp, "\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp, "#include <gnome.h>\n" );
	else
#endif
	{
		fprintf( fp, "#include <gdk/gdkkeysyms.h>\n" );
		fprintf( fp, "#include <gtk/gtk.h>\n" );
	}

	fprintf( fp, "\n" );
	fprintf( fp, "#include \"main.h\"\n" );

	fprintf( fp, "#include \"modules.h\"\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "typedef struct\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\tGtkWidget\t*initial;\n" );
	fprintf( fp, "\tint\t\t\tcpt;\n" );
	fprintf( fp, "\tint\t\t\tdepth;\n" );
	fprintf( fp, "\tGtkWidget	*result;\n" );
	fprintf( fp, "}\t\t\t\t\t\t_HIERARCHYCOMPOSITE_DATA;\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "static void\n" );
	fprintf( fp, "_forall_compute_hierarchy_composite_childs( GtkWidget *widget, _HIERARCHYCOMPOSITE_DATA *data )\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\tif ( gtk_object_get_data( GTK_OBJECT( widget ), \"__non_delete__\" ) != NULL )\n" );
	fprintf( fp, "\t{\n" );
	fprintf( fp, "\t\tif ( widget != data->initial )\n" );
	fprintf( fp, "\t\t{\n" );
	fprintf( fp, "\t\t\tif (data->cpt++ == data->depth)\n" );
	fprintf( fp, "\t\t\t{\n" );
	fprintf( fp, "\t\t\t\tdata->result = widget;\n" );
	fprintf( fp, "\t\t\t\treturn;\n" );
	fprintf( fp, "\t\t\t}\n" );
	fprintf( fp, "\t\t}\n" );
	fprintf( fp, "\t}\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\tif ( GTK_IS_CONTAINER(widget) )\n" );
	fprintf( fp, "\t{\n" );
	fprintf( fp, "\t\tgtk_container_forall( GTK_CONTAINER(widget), \n" );
	fprintf( fp, "\t\t\t(GtkCallback)_forall_compute_hierarchy_composite_childs, data );\n" );
	fprintf( fp, "\t}\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// _forall_compute_hierarchy_composite_childs\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "PUBLIC GtkWidget *\n" );
	fprintf( fp, "_compute_hierarchy_composite_childs( GtkWidget *wgt_main, int depth )\n");
	fprintf( fp, "{\n" );
	fprintf( fp, "\t_HIERARCHYCOMPOSITE_DATA	data;\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\tdata.initial = wgt_main;\n" );
	fprintf( fp, "\tdata.cpt	 = 0;\n" );
	fprintf( fp, "\tdata.depth	 = depth;\n" );
	fprintf( fp, "\tdata.result  = NULL;\n" );
	fprintf( fp, "\t_forall_compute_hierarchy_composite_childs( wgt_main, &data );\n" );
	fprintf( fp, "\treturn data.result;\n" );
	fprintf( fp, "\t\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// _compute_hierarchy_composite_childs\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "PUBLIC void\n" );
	fprintf( fp, "_forall_flag_nondelete_widgets( GtkWidget *widget, GtkWidget *dlg )\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\tif ( widget != dlg )\n" );
	fprintf( fp, "\t\tgtk_object_set_data( GTK_OBJECT( widget ), \"__non_delete__\", widget );\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\tif ( GTK_IS_CONTAINER(widget) )\n" );
	fprintf( fp, "\t\tgtk_container_forall( GTK_CONTAINER(widget),\n" );
	fprintf( fp, "\t\t\t(GtkCallback)_forall_flag_nondelete_widgets, dlg );\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// _forall_flag_nondelete_widgets\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "PUBLIC GtkWidget *\n" );
	fprintf( fp, "__get_parent__( GtkWidget *wgt )\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\tGtkArg		args[1];\n" );
	fprintf( fp, "\tGtkWidget	*wgt_parent;\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "\targs[0].name = \"GtkWidget::parent\";\n" );
	fprintf( fp, "\targs[0].type = GTK_TYPE_CONTAINER;\n" );
	fprintf( fp, "\tgtk_widget_getv( GTK_WIDGET( wgt ), 1, args );\n" );
	fprintf( fp, "\twgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );\n" );
	fprintf( fp, "\treturn wgt_parent;\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// __get_parent__\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "static void \n" );
	fprintf( fp, "my_glog( const gchar *log_domain,\n" );
	fprintf( fp, "	GLogLevelFlags log_level,\n" );
	fprintf( fp, "	const gchar *message,\n" );
	fprintf( fp, "	gpointer user_data)\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\tg_log_default_handler( log_domain, log_level, message, user_data );\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// my_glog\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "int\nmain( int argc, char *argv[] )%s{\n",
		(config.indentstyle_function == 1) ? "\n" : " " );
	fprintf( fp, "\n" );
	fprintf( fp, "\t/*--- Initialize the GTK library ---*/\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp, "\tgnome_init( \"%s\", \"%s\", argc, argv );\n",
			"version", "0.9" );
	else
		fprintf( fp, "\tgtk_init( &argc, &argv );\n" );
#else
		fprintf( fp, "\tgtk_init( &argc, &argv );\n" );
#endif

	/*--- Custom GTK logger ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "\t/*--- Custom GTK logger ---*/\n" );
	fprintf( fp, "\tg_log_set_handler( \"Gtk\",\n" );
	fprintf( fp, "\t\t(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL),\n" ); 
	fprintf( fp, "\t\tmy_glog, NULL );\n" );
	fprintf( fp, "\tg_log_set_handler( \"Gdk\",\n" );
	fprintf( fp, "\t\t(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL),\n" ); 
	fprintf( fp, "\t\tmy_glog, NULL );\n" );
	fprintf( fp, "\tg_log_set_handler( \"GLib\",\n" );
	fprintf( fp, "\t\t(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL),\n" ); 
	fprintf( fp, "\t\tmy_glog, NULL );\n" );
	fprintf( fp, "\tg_log_set_handler( NULL,\n" );
	fprintf( fp, "\t\t(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL),\n" ); 
	fprintf( fp, "\t\tmy_glog, NULL );\n" );

	/*--- Initialize styles ---*/
	if ( nb_styles > 0 )
	{
		fprintf( fp, "\n\t/*--- Create styles ---*/" );
			fprintf( fp, "\n\tinit_styles();\n" );
	}

	/*--- startup() function is always called from main() ---*/
	fprintf( fp, "\n\t%s( argc, argv );\n",
		"startup" );

	/*--- Windows modules opened from main() ---*/
	module = next_module( FIRST );
	while ( module )
	{

		switch ( module->type )
		{

			case MODULE_TYPE_WINDOW :
				if ( module->u1.window.f_opened_from_main )
				{
					if ( !strcmp( module->u1.window.name_function, "" ) )
					{
						fprintf( fp, "\t%s%s( NULL, TRUE );\n", 		// Parent: NULL; Realize: TRUE
							"module_",
							get_name_wgt(module->wgt_window) );
					}
					else
					{
						fprintf( fp, "\t%s( NULL, TRUE );\n", 			// Parent: NULL; Realize: TRUE
							module->u1.window.name_function );
					}
				}
				break;

			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				if ( module->u1.dialog.f_opened_from_main )
				{
					if ( !strcmp( module->u1.dialog.name_function, "" ) )
					{
						fprintf( fp, "\t%s%s( NULL, TRUE );\n", 		// Parent: NULL; Realize: TRUE
							"module_",
							get_name_wgt(module->wgt_window) );
					}
					else
					{
						fprintf( fp, "\t%s( NULL, TRUE );\n", 			// Parent: NULL; Realize: TRUE
							module->u1.dialog.name_function );
					}
				}
				break;

			case MODULE_TYPE_MENU :
				break;

			default :
				assert(0);
				break;

		}
		module = next_module( NEXT );
	}							// while

	fprintf( fp, "\n" );
	fprintf( fp, "\t/*--- Wait until exit ---*/\n" );
	fprintf( fp, "\tgtk_main();\n" );
	fprintf( fp, "\treturn 0;\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "}\t\t\t\t\t\t\t\t// main\n" );

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// generate_main_c


static int
generate_modules_h( const char * const dirname )
{
	char			fname[ PATH_MAX+NAME_MAX + 80 ];
	FILE			*fp;
	module_info_t	*module;

	sprintf( fname, "%s/glg/modules.h", 
		dirname );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d\n", fname, errno );
		return -1;
	}

	/*--- Add styles #include, if any styles ---*/
	if ( nb_styles > 0 )
		fprintf( fp, "#include \"styles.h\"\n" );

	/*--- Header to include ? ---*/
	if ( config.header_to_include[0] )
	{
		fprintf( fp, "#include \"%s\"\n", config.header_to_include );
		fprintf( fp, "\n" );
	}

	/*--- Add an #include for each existant module ---*/
	module = next_module( FIRST );
	while ( module )
	{
		fprintf( fp, "#include \"%s.h\"\n", 
			get_name_wgt( module->wgt_window ) );
		module = next_module( NEXT );
	}

	/*--- startup() function from main() ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "extern void startup( int argc, char *argv[] );\n" );

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// generate_modules_h


typedef struct
{
	module_info_t	*module;
	int				level;
	int				cpt;			// Cpt for anonymous variables, same level
	int				*nb_xpm;
	char			***xpm;
}	GENADDXPM_DATA;

static void
forall_add_xpm( GtkWidget *widget, GENADDXPM_DATA *data )
{
	GtkArg		*parg;
	char		*name;

	if ( skip_widget( widget ) )
		return;

	data->level++;
	data->cpt++;

	if ( GTK_IS_PIXMAP( widget ) )
	{
		parg = get_outsideres_wgt( widget, "~GtkPixmap::pixmap_file" );
		assert( parg != NULL );
		assert( parg->type == GTK_TYPE_STRING );
		if ( strcmp( name = GTK_VALUE_STRING( parg[0] ), "" ) )
		{
			trace( ">> %d %d %s\n", data->level, data->cpt, name );
			(*data->nb_xpm)++;
			if ( *data->nb_xpm == 1 )
				*data->xpm = malloc( sizeof( char * ) );
			else
				*data->xpm = realloc( *data->xpm, sizeof( char * ) * (*data->nb_xpm) );
			*data->xpm[ *data->nb_xpm - 1 ] = strdup( name );
		}						// if
	}							// if

	if ( GTK_IS_CONTAINER (widget) )
	{
		GENADDXPM_DATA	new_data;
		
		new_data.module	  	= data->module;
		new_data.level    	= data->level;
		new_data.cpt   	  	= 0;
		new_data.nb_xpm		= data->nb_xpm;
		new_data.xpm		= data->xpm;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_add_xpm, &new_data );
	}

	data->level--;

}									// forall_add_xpm

static void
add_xpm( int *nb_xpm, char ***xpm, module_info_t *module )
{
	GENADDXPM_DATA	data;

	data.module 	= module;
	data.level  	= -1;
	data.cpt    	= 0;
	data.nb_xpm		= nb_xpm;
	data.xpm		= xpm;
	forall_add_xpm( module->wgt_window, &data );
}								// add_xpm


static int
generate_main_h( const char * const dirname, const char * const filename )
{
	char			fname[ PATH_MAX+NAME_MAX + 80 ];
	FILE			*fp;
	int				nb_xpm;
	char			**xpm;
	module_info_t	*module;

	sprintf( fname, "%s/glg/%s", dirname, filename );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d\n", fname, errno );
		return -1;
	}

	fprintf( fp, "#ifdef __STARTUP__\n" );
	fprintf( fp, "#\tdefine COMMON\n" );
	fprintf( fp, "#\tdefine INIT(v) = v;\n" );
	fprintf( fp, "#else\n" );
	fprintf( fp, "#\tdefine COMMON extern\n" );
	fprintf( fp, "#\tdefine INIT(v) ;\n" );
	fprintf( fp, "#endif\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "#define PUBLIC\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "GtkWidget *\n" );
	fprintf( fp, "_compute_hierarchy_composite_childs( GtkWidget *wgt_main, int depth );\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "void\n" );
	fprintf( fp, "_forall_flag_nondelete_widgets( GtkWidget *widget, GtkWidget *dlg );\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "GtkWidget *\n" );
	fprintf( fp, "__get_parent__( GtkWidget *wgt );\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "COMMON\tGtkArg\t\targs[ 100 ];\n" );
	fprintf( fp, "COMMON\tint\t\t\tnb_args;\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define INIT_ARGS\\\n" );
	fprintf( fp, "\tnb_args=0\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_INT( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_INT;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_INT(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_UINT( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_UINT;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_UINT(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_LONG( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_LONG;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_LONG(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_ULONG( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_ULONG;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_ULONG(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_ENUM( T, N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=T;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_ENUM(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_FLAGS( T, N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=T;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_FLAGS(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_BOOL( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_BOOL;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_BOOL(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_STRING( N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=GTK_TYPE_STRING;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_STRING(args[nb_args++])=V\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#define SETARG_OBJECT( T, N, V )\\\n" );
	fprintf( fp, "\targs[nb_args].type=T;\\\n" );
	fprintf( fp, "\targs[nb_args].name=N;\\\n" );
	fprintf( fp, "\tGTK_VALUE_OBJECT(args[nb_args++])=GTK_OBJECT(V)\n" );
	fprintf( fp, "\n" );

	/*--- Generate #include to all graphics ---*/
	nb_xpm = 0;
	xpm = NULL;
	module = next_module( FIRST );
	while ( module )
	{
		switch ( module->type )
		{

			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			case MODULE_TYPE_MENU :
				add_xpm( &nb_xpm, &xpm, module );
				break;
			default :
				assert(0);
				break;
		}
		module = next_module( NEXT );
	}							// while
	if ( nb_xpm > 0 )
	{
		int		n;
		fprintf( fp, "\n/*--- xpm stuf ---*/\n" );
		fprintf( fp, "#ifdef __STARTUP__\n" );
		fprintf( fp, "#define static\n" );
		for ( n = 0; n < nb_xpm; n++ )
		{
			fprintf( fp, "#include \"xpm/%s.xpm\"\n", xpm[ n ] );
		}						// for
		fprintf( fp, "#undef static\n" );
		fprintf( fp, "#else\n" );
		for ( n = 0; n < nb_xpm; n++ )
		{
			fprintf( fp, "extern char *%s_xpm[];\n", xpm[ n ] );
			free( xpm[ n ] );
		}						// for
		fprintf( fp, "#endif\n" );
		free( xpm );
	}							// if

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// generate_main_h


static void
dump_var( GtkWidget *wgt, 
	FILE *fp_c,  FILE *fp_h,
	const int level, const int cpt, char *list_var )
{
	char		*name_wgt;
	int			nametype_wgt;
	char		gen_name1[ 128 ], gen_name2[ 128 ];

	name_wgt     = get_name_wgt( wgt );
	nametype_wgt = get_nametype_wgt( wgt );

	switch ( nametype_wgt )
	{

		case NAMETYPE_ANONYMOUS :

			sprintf( gen_name1, "%s", get_name_wgt(wgt) );
			sprintf( gen_name2, "*%s, ", gen_name1 );

			/*--- Declaration into the module creation function ---*/
			if ( 16 + strlen(list_var) + strlen(gen_name2) + 2 >= config.max_source_line )
			{
				fprintf( fp_c, "%s\n", list_var );
				strcpy( list_var, "\t\t\t\t" );
			}
			strcat( list_var, gen_name2 );

			break;

		case NAMETYPE_GLOBAL :

			/*--- Declaration into the module header ---*/
			if (level > 0)
				fprintf( fp_h, "COMMON GtkWidget\t*%s;\n", name_wgt );
			
			break;

		default :
			assert( 0 );

	}							// switch

}								// dump_var


typedef struct
{
	int			level;
	int			cpt;			// Cpt for anonymous variables, same level
	FILE		*fp_c;
	FILE		*fp_h;
	char		*list_var;
}	DUMPVAR_DATA;


static void
forall_dump_var( GtkWidget *widget, DUMPVAR_DATA *data )
{
	char		label[ 80 ];

	if ( skip_widget( widget ) )
		return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	sprintf( label, "%s", gtk_widget_get_name (widget) );
	dump_var( widget, data->fp_c, data->fp_h, data->level, data->cpt, data->list_var );

	if ( GTK_IS_CONTAINER(widget) )
	{
		DUMPVAR_DATA	new_data;
		
		new_data.level    = data->level;
		new_data.cpt   	  = data->cpt;
		new_data.fp_c	  = data->fp_c;
		new_data.fp_h	  = data->fp_h;
		new_data.list_var = data->list_var;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_dump_var, &new_data );
	}

	data->level--;

}								// forall_dump_var


static char *
gener_string( char *str )
{
	char	*p, *q, *r;
	int		len, nb;
	
	/*--- Count how many CR in the string ---*/
	len = strlen( str );
	for ( nb = 0, p = str; ; )
	{
		p = strchr( p, '\n' );
		if ( p == NULL )
			break;
		nb++;
		p++;
	}
	if ( nb == 0 )
		return NULL;

	/*--- Generate the new string ---*/
	r = malloc( len + 1 + nb );
	assert( r != NULL );
	for ( strcpy( r, "" ), p = str; ; )
	{
		p = strchr( q = p, '\n' );
		if ( p == NULL )
		{
			strcat( r, q );
			break;
		}
		*p = 0;
		strcat( r, q );
		strcat( r, "\\n" );
		p++;
	}
	return r;	

}								// gener_string


static void
dump_widget_file( module_info_t *module, GtkWidget *wgt, 
	GtkWidget *wgt_parent, 
	FILE *fp_c,
	const int level, const int cpt )
{
	char				*wgt_name;
	gtk_wgtdef_t		*wgtdef=NULL;
	gtk_wgtdef_t		*wgtdef_depth;
	int					wgt_nametype;
	defclass_t			*classes=NULL, *p;
	int					depth, n, num;
	int					nb_res=0, _nb_args=0;
	GtkArg				arg, *parg;
	char				**opf=NULL;
	int					nb_opf=0;

	/*--- Don't save 'placeholder' container ---*/
	if ( IS_WGT_PLACEHOLDER(wgt) )
		return;

	/*--- Name of the widget ---*/
	wgt_name     = get_name_wgt( wgt );
	wgt_nametype = get_nametype_wgt( wgt );

	/*--- Create the widget ---*/
	fprintf( fp_c, "\n");

	/*--- Dump resources set to non default value ---*/
	for ( num = 0; num < 3; num++ )
	{

		if ( (num == 0) || (num == 1) )
		{
			wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
			assert( wgtdef != NULL );
		}
		else if ( num == 2 )
		{
			if (wgt_parent == NULL)
				wgtdef = NULL;
			else
				wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
			if (wgtdef == NULL)
				continue;
		}

		if ( num != 1 )
		{
			_nb_args = 0;
			nb_opf = 0;
			opf = NULL;
		}

		for ( depth = 0; 
			  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[depth].classnames != NULL); 
			  depth++ )
		{

			/*--- Dump all resources ---*/
			if ( (num == 0) || (num == 1) )
			{
				classes = wgtdef->def[depth].classes;
				nb_res  = wgtdef->def[depth].nb_classes;
			}
			else if ( num == 2 )
			{
				classes = wgtdef->def[depth].classes_child;
				nb_res  = wgtdef->def[depth].nb_classes_child;
			}

			wgtdef_depth = wgtdef->def[depth].wgtdef_depth;

			for ( p = classes, n = 0; n < nb_res; n++, p++ )
			{

				if ( (num == 0) && !(p->resflags & GTK_ARG_CONSTRUCT_ONLY) )
					continue;
				if ( (num == 1) && (p->resflags & GTK_ARG_CONSTRUCT_ONLY) )
					continue;

				if ( !p->f_generate )
					continue;
				if ( is_default_value(wgt, wgt_parent, wgtdef_depth, p) )
					continue;

				if ( get_is_disabled_resource_wgt(wgt, p) )
					continue;

				/*--- Don't save X, Y, Width and Height for a Menu module ---*/
				if ( module->type == MODULE_TYPE_MENU )
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") ||
						 !strcmp(p->resname, "~GtkWidget::y") ||
						 !strcmp(p->resname, "~GtkWidget::width") ||
						 !strcmp(p->resname, "~GtkWidget::height") )
						continue;
				}						// if

				if ( p->saved_where == SAVED_INSIDE )
				{
					arg.name = p->resname;
					if ( (num == 0) || (num == 1) )
						gtk_widget_get( GTK_WIDGET(wgt), &arg );
					else if ( num == 2 )
						gtk_container_child_getv( GTK_CONTAINER(wgt_parent), GTK_WIDGET(wgt), 1, &arg );
					parg = &arg;
				}
				else if ( p->saved_where == SAVED_OUTSIDE || p->saved_where == SAVED_OUTSIDE_FUNCTION )
				{
					parg = get_outsideres_wgt( wgt, p->resname );
					assert( parg != NULL );
				}
				else if ( p->saved_where == SAVED_FUNCTION )
				{
					int		nb_str, n;
					char	 *str[100];
					parg = &arg;
					nb_str = (*wgtdef_depth->pfct_writecode)(p, wgt, &parg, str);
					if (nb_str > 0)
					{
						for ( n = 0; n < nb_str; n++ )
						{
							if (opf == NULL )
								opf = malloc(sizeof(char **));
							else
								opf = realloc(opf, sizeof(char **) * (nb_opf+1));
							opf[nb_opf++] = str[ n ];
						}
					}
				}
				else assert(0);

				if ( (parg != NULL) && (_nb_args == 0) )
				{
					if ( (num == 0) || (num == 1) )
					{
						if (config.setgetres_method == SETGETRES_METHOD_ARGS)
							fprintf( fp_c, "\tINIT_ARGS;" );
						else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
						{
							if ( !IS_WGT_NOCOMPOSITE( wgt ) )
							{
								char *s, *t;
								s = get_object_data( wgt, WGTDATA_HIERARCHY_CHILD );
								assert( s != NULL );
								t = get_object_data( wgt, WGTDATA_HIERARCHY_CHILD_MASTER );
								assert( t != NULL );
								fprintf( fp_c, "\t%s = _compute_hierarchy_composite_childs( %s, %s );\t\t// %s\n",
									wgt_name, get_name_wgt(GTK_WIDGET(t)), s,
									gtk_widget_get_name( wgt ) );
								fprintf( fp_c, "\tgtk_widget_set( %s,",
									wgt_name );
							}
							else
							{
								fprintf( fp_c, "\t%s = gtk_widget_new( %s(),",
									wgt_name, wgtdef->widget_type );
							}
						}
						else assert(0);
					}
					else if (num == 2)
					{
						if (config.setgetres_method == SETGETRES_METHOD_ARGS)
							fprintf( fp_c, "\tINIT_ARGS;" );
						else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
							fprintf( fp_c, "\tgtk_container_child_set( GTK_CONTAINER( %s ), %s,",
								get_name_wgt(wgt_parent), wgt_name );
						else assert(0);
					}
				}

				if ( parg != NULL )
				{
					char	*resname;
					
					if (p->resname[0] == '~' )
						resname = strdup( &p->resname[1] );
					else
						resname = strdup( p->resname );
					
					switch ( p->restype )
					{

						case GTK_TYPE_INT :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_INT( \"%s\", %d );",
									resname, GTK_VALUE_INT(parg[0]) );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %d,",
									resname, GTK_VALUE_INT(parg[0]) );
							break;

						case GTK_TYPE_UINT :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_UINT( \"%s\", %u );",
									resname, GTK_VALUE_UINT(parg[0]) );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %u,",
									resname, GTK_VALUE_UINT(parg[0]) );
							break;

						case GTK_TYPE_LONG :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_LONG( \"%s\", %ld );",
									resname, GTK_VALUE_LONG(parg[0]) );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %ld,",
									resname, GTK_VALUE_LONG(parg[0]) );
							break;

						case GTK_TYPE_ULONG :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_ULONG( \"%s\", %lu );",
									resname, GTK_VALUE_ULONG(parg[0]) );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %lu,",
									resname, GTK_VALUE_ULONG(parg[0]) );
							break;

						case GTK_TYPE_FLOAT :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_FLOAT( \"%s\", %f );",
									resname, GTK_VALUE_FLOAT(parg[0]) );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %f,",
									resname, GTK_VALUE_FLOAT(parg[0]) );
							break;

						case GTK_TYPE_BOOL :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "\n\tSETARG_BOOL( \"%s\", %s );",
									resname, 
									(GTK_VALUE_BOOL(parg[0])) ? "TRUE" : "FALSE" );
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
								fprintf( fp_c, "\n\t\t\"%s\", %s,",
									resname, 
									(GTK_VALUE_BOOL(parg[0])) ? "TRUE" : "FALSE" );
							break;

						case GTK_TYPE_STRING :
							{
								char	*str, *new_str;
								
								str = GTK_VALUE_STRING( parg[0] );
								if ( str != NULL )
								{
									new_str = gener_string( str );
									if (config.setgetres_method == SETGETRES_METHOD_ARGS)
										fprintf( fp_c, "\n\tSETARG_STRING( \"%s\", \"%s\" );",
											resname, (new_str) ? new_str : str );
									else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
										fprintf( fp_c, "\n\t\t\"%s\", \"%s\",",
											resname, (new_str) ? new_str : str );
									free( str );
									if ( new_str )
										free( new_str );
								}
							}
							break;

				    	case GTK_TYPE_POINTER :
							{
								gpointer	*ptr;
								
								ptr = GTK_VALUE_POINTER( parg[0] );
								if ( !strcmp( p->object_restype_str, "GTK_TYPE_ADJUSTMENT" ) )
								{
									GtkAdjustment	*adj = GTK_ADJUSTMENT( ptr );
									if (config.setgetres_method == SETGETRES_METHOD_ARGV)
										fprintf( fp_c, "\n\t\t\"%s\", gtk_adjustment_new( %f, %f, %f, %f, %f, %f ),",
											resname, 
											adj->value, adj->lower, adj->upper,
											adj->step_increment, adj->page_increment, adj->page_size );
									else assert( 0 );
								}
								else assert( 0 );
							}
							break;

				    	case GTK_TYPE_OBJECT :
							{
								char		*object_name;
								GtkObject	*obj;
								
								obj = GTK_VALUE_OBJECT( parg[0] );
								if ( !strcmp( p->object_restype_str, "GTK_TYPE_STYLE" ) )
								{
//									if (config.setgetres_method == SETGETRES_METHOD_ARGS)
//										fprintf( fp_c, "\n\tSETARG_OBJECT( %s, \"%s\", %s );",
//											p->object_restype_str, resname, object_name );
//									else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
										fprintf( fp_c, "\n\t\t\"%s\", %s,",
											resname, get_style_name( (GtkStyle *)obj ) );
								}
								else if ( !strcmp( p->object_restype_str, "GTK_TYPE_CONTAINER" ) )
								{
									if ( !GTK_IS_MENU( wgt) &&	   		/* Dont' check parent for GtkMenu */
										 !IS_WGT_DONTDELETE( wgt ) )	/* Composite widgets have implicite parents */ 
									{
										if ( obj )
										{
											object_name = get_name_wgt( (GtkWidget *)GTK_VALUE_OBJECT(parg[0]) );
											if ( object_name )
											{
												if (config.setgetres_method == SETGETRES_METHOD_ARGS)
													fprintf( fp_c, "\n\tSETARG_OBJECT( %s, \"%s\", %s );",
														p->object_restype_str, resname, object_name );
												else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
													fprintf( fp_c, "\n\t\t\"%s\", %s,",
														resname, object_name );
											}
										}							
									}
								}
								else assert( 0 );
							}
							break;

						case GTK_TYPE_ENUM :
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
							{
								fprintf( fp_c, "\n\tSETARG_ENUM( %s, \"%s\", %s );",
									p->object_restype_str,
									resname, 
									p->enum_list[search_enum( GTK_VALUE_ENUM(parg[0]), p )].value_name );
							}
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
							{
								fprintf( fp_c, "\n\t\t\"%s\", %s,",
									resname, 
									p->enum_list[search_enum( GTK_VALUE_ENUM(parg[0]), p )].value_name );
							}
							break;

						case GTK_TYPE_FLAGS :
						{
							guint				flags;
							int					clen, len, first, bline;
							GtkEnumValue const	*penum;
							char				buff[ 400 ];
								
							flags = GTK_VALUE_FLAGS( parg[0] );

							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
							{
								sprintf( buff, "\n\tSETARG_FLAGS( %s, \"%s\",",
									p->object_restype_str,
									resname );
							}
							else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
							{
								sprintf( buff, "\n\t\t\"%s\",",
									resname );
							}

							clen = 12;
							for ( first = TRUE, bline = FALSE, penum = p->enum_list; penum->value_name; penum++ )
							{
								if ( flags & penum->value )
								{
									len = strlen( penum->value_name );
									if ( !first )
									{
										strcat( buff, " |" );
										clen += 2;
									}
									if ( clen + len + 2 >= config.max_source_line )
									{
										fprintf( fp_c, "%s", buff );
										sprintf( buff, "\n\t\t\t" );
										clen = 12;
										bline = TRUE;
									}
									if ( !bline )
										strcat( buff, " " );
									strcat( buff, penum->value_name );
									clen += len;
									first = FALSE;
									bline = FALSE;
								}
							}
							if (config.setgetres_method == SETGETRES_METHOD_ARGS)
								fprintf( fp_c, "%s );", buff );
							else
								fprintf( fp_c, "%s,", buff );

						}
							break;

						default :
							RES_NOT_YET_HANDLED( wgtdef, p );
							break;

					}				// switch

					_nb_args++;

					free( resname );

				}					// if ( parg != NULL )
				
			}						// for (p)

			if ( num >= 1 )
			{
				int		nb_str, n;
				char	 *str[100];
				parg = &arg;
				nb_str = (*wgtdef_depth->pfct_writecode_after)(p, wgt, wgt_parent, &parg, str);
				if (nb_str > 0)
				{
					for ( n = 0; n < nb_str; n++ )
					{
						if (opf == NULL )
							opf = malloc(sizeof(char **));
						else
							opf = realloc(opf, sizeof(char **) * (nb_opf+1));
						opf[nb_opf++] = str[ n ];
					}
				}
			}

		}							// for (depth)

		if ( num >= 1 )
		{
		
			if ( _nb_args > 0 )
			{
				if (config.setgetres_method == SETGETRES_METHOD_ARGS)
				{	
					if ( (num == 0) || (num == 1) )
					{
						fprintf( fp_c, "\n\t%s = gtk_widget_newv( %s(), nb_args, args );\n",
							wgt_name, wgtdef->widget_type );
					}
					else if ( num == 2 )
					{
						fprintf( fp_c, "\n\tgtk_container_child_setv( GTK_CONTAINER( %s ), %s, nb_args, args );\n",
							get_name_wgt(wgt_parent), wgt_name );
					}
				}
				else if (config.setgetres_method == SETGETRES_METHOD_ARGV)
					fprintf( fp_c, "\n\t\tNULL );\n" );
				else assert(0);
			}					// if

			if (opf)
			{
				int n;
				for ( n = 0; n < nb_opf; n++ )
				{
					fprintf( fp_c, "\t%s\n", opf[n] );
					free(opf[n]);
				}
				free(opf);
			}

		}		

	}							// for (num)

	/*--- Does the window contains composite childs, automatically created ? ---*/
	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );
	if ( level == 0 )
	{
		if ( (module->type != MODULE_TYPE_WINDOW) && 
			 (module->type != MODULE_TYPE_MENU) )
		{
			fprintf( fp_c, "\t_forall_flag_nondelete_widgets( %s, %s );\n",
				wgt_name, wgt_name );
		}
	}							// if

	/*--- Dump all signals (Callbacks) ---*/
	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
		  depth++ )
	{
		defclasssignal_t	*classes_signal, *q;
		int					nb_signals;
		
		classes_signal = wgtdef->def[depth].classes_signal;
		nb_signals	   = wgtdef->def[depth].nb_classes_signal;
		for ( q = classes_signal, n = 0; n < nb_signals; n++, q++ )
		{
			data_cb_t	*datacb;
			int			k;
			
			datacb = get_object_data( wgt, q->resname );
			if (datacb == NULL)
				continue;
			for ( k = 0; k < datacb->nb_cb; k++ )
			{
				fprintf( fp_c, 
					"\tgtk_signal_connect%s( GTK_OBJECT( %s ), \"%s\",\n\t\tGTK_SIGNAL_FUNC( %s ), NULL );\n",
					(datacb->cb[k].f_after) ? "_after" : "",
					wgt_name, q->resname, datacb->cb[k].cbname );
			}
		}						// for (q)
	}							// for (depth)		

	/*--- Any accelerator into this module ? ---*/
	if ( (level == 0) && gen_any_accelerator( module ) )
	{
		fprintf( fp_c, "\t_wgt_accel_group = gtk_accel_group_new();\n" );
		if ( GTK_IS_MENU( wgt ) )
			fprintf( fp_c, "\tgtk_window_add_accel_group( GTK_WINDOW( __get_parent__( %s ) ), _wgt_accel_group );\n",
				wgt_name );
		else
			fprintf( fp_c, "\tgtk_window_add_accel_group( GTK_WINDOW( %s ), _wgt_accel_group );\n",
				wgt_name );
	}							// if

}								// dump_widget_file


typedef struct
{
	module_info_t	*module;
	int				level;
	int				cpt;		// Cpt for anonymous variables, same level
	GtkWidget		*parent;
	FILE			*fp_h;
}	DUMPPROTOSCB_DATA;

static void
forall_dump_protoscb( GtkWidget *widget, DUMPPROTOSCB_DATA *data )
{
	gtk_wgtdef_t	*wgtdef;
	int				depth;

	if ( skip_widget( widget ) )
		return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	wgtdef = gtk_object_get_data( GTK_OBJECT(widget), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );
	for ( depth = 0; (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); depth++ )
	{
		defclasssignal_t	*classes_signal, *q;
		int					nb_signals, n;

		classes_signal = wgtdef->def[depth].classes_signal;
		nb_signals	   = wgtdef->def[depth].nb_classes_signal;
		for ( q = classes_signal, n = 0; n < nb_signals; n++, q++ )
		{
			data_cb_t	*datacb;
			int			k, z;

			datacb = get_object_data( widget, q->resname );
			if (datacb == NULL)
				continue;
			for ( k = 0; k < datacb->nb_cb; k++ )
			{
				fprintf( data->fp_h, "extern %s %s(",
					q->proto.ret,
					datacb->cb[k].cbname );
				for ( z = 0; z < sizeof(q->proto.params)/sizeof(q->proto.params[0]); z++ )
				{
					if ( !q->proto.params[z] )
						break;
					fprintf( data->fp_h, "%s%s",
						(!z) ? "" : ", ",
						q->proto.params[z] );
				}
				fprintf( data->fp_h, ");\t// %s.c\n",
					datacb->cb[k].cbfile );
			}
		}						// for (q)
	}							// for (depth)		

	if ( GTK_IS_CONTAINER (widget) )
	{
		DUMPPROTOSCB_DATA	new_data;
		
		new_data.module	  = data->module;
		new_data.level    = data->level;
		new_data.cpt   	  = 0;
		new_data.fp_h	  = data->fp_h;
		new_data.parent	  = widget;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_dump_protoscb, &new_data );
	}

	data->level--;

}								// forall_dump_protoscb


typedef struct
{
	module_info_t	*module;
	int				level;
	int				cpt;			// Cpt for anonymous variables, same level
	int				*nb_cbfiles;
	char			***cbfiles;
	int				**nb_cb;		// Number of cb in each file
	char			****cb;			// List of cb in each file
}	GENCBFILENAME_DATA;

static void
forall_gen_cbfilesname( GtkWidget *widget, GENCBFILENAME_DATA *data )
{
	gtk_wgtdef_t	*wgtdef;
	int				depth;

	if ( skip_widget( widget ) )
		return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	wgtdef = gtk_object_get_data( GTK_OBJECT(widget), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );
	for ( depth = 0;
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL);
		  depth++ )
	{
		defclasssignal_t	*classes_signal, *q;
		int					nb_signals, n;

		classes_signal = wgtdef->def[depth].classes_signal;
		nb_signals	   = wgtdef->def[depth].nb_classes_signal;
		for ( q = classes_signal, n = 0; n < nb_signals; n++, q++ )
		{
			data_cb_t	*datacb;
			int			k;

			datacb = get_object_data( widget, q->resname );
			if (datacb == NULL)
				continue;
			for ( k = 0; k < datacb->nb_cb; k++ )
			{
				int		z, found;

				/*--- Add the filename ? ---*/
				for ( found = FALSE, z = 0; z < *data->nb_cbfiles; z++ )
				{
					if ( !strcmp( (*data->cbfiles)[z], datacb->cb[k].cbfile ) )
					{
						found = TRUE;
						break;
					}
				}
				if ( !found )
				{
					if ( *data->nb_cbfiles == 0 )
					{
						*data->cbfiles = malloc( sizeof(char *) );
						(*data->cbfiles)[ 0 ] = strdup(datacb->cb[k].cbfile);
						++(*data->nb_cbfiles);
					}					// if
					else
					{
						*data->cbfiles = realloc( *data->cbfiles, (*data->nb_cbfiles + 1) * sizeof( char * ) );
						(*data->cbfiles)[ (*data->nb_cbfiles)++ ] = strdup(datacb->cb[k].cbfile);
					}
				}

				/*--- Create the callback into the file, if needed ---*/
				locate_cb_in_file( widget, &datacb->cb[k] );

			}					// for (k)
		}						// for (q)
	}							// for (depth)		

	if ( GTK_IS_CONTAINER (widget) )
	{
		GENCBFILENAME_DATA	new_data;
		
		new_data.module	  	= data->module;
		new_data.level    	= data->level;
		new_data.cpt   	  	= 0;
		new_data.nb_cbfiles	= data->nb_cbfiles;
		new_data.cbfiles	= data->cbfiles;
		new_data.nb_cb	  	= data->nb_cb;
		new_data.cb	  		= data->cb;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_gen_cbfilesname, &new_data );
	}

	data->level--;

}								// forall_gen_cbfilesname

static void
gen_cb_filesname( module_info_t *module, 
	int *nb_cbfiles, char ***cbfiles )
{
	GENCBFILENAME_DATA	data;

	data.module 	= module;
	data.level  	= -1;
	data.cpt    	= 0;
	data.nb_cbfiles = nb_cbfiles;
	data.cbfiles	= cbfiles;
	forall_gen_cbfilesname( module->wgt_window, &data );
}								// gen_cb_filesname



typedef struct
{
	module_info_t	*module;
	int				level;
	int				cpt;			// Cpt for anonymous variables, same level
	GtkWidget		*parent;
	FILE			*fp_c;
}	DUMPWINDOW_DATA;

static void
forall_dump_window( GtkWidget *widget, DUMPWINDOW_DATA *data )
{

	if ( skip_widget( widget ) )
		return;

	data->level++;
	data->cpt++;

	dump_widget_file( data->module, widget, 
		data->parent, 
		data->fp_c, data->level, data->cpt );

	if ( GTK_IS_CONTAINER (widget) )
	{
		DUMPWINDOW_DATA	new_data;
		
		new_data.module	  = data->module;
		new_data.level    = data->level;
		new_data.cpt   	  = 0;
		new_data.fp_c	  = data->fp_c;
		new_data.parent	  = widget;
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_dump_window, &new_data );
	}

	data->level--;

}								// forall_dump_window


static int
generate_module_window( char * dirname, module_info_t *module,
	void (*output)(char *fmt, ... ) )
{
	char				fname_c[ PATH_MAX+NAME_MAX + 80 ];
	char				fname_h[ PATH_MAX+NAME_MAX + 80 ];
	FILE				*fp_c, *fp_h;
	int					nb_local_var, nb_global_var;
	char				function_name[ 96+1 ];
	int					nb_cbfiles;
	char				**cbfiles;
	int					name_cpt;
	int					n;
	char				list_var[256];
	DUMPWINDOW_DATA		data;
	DUMPPROTOSCB_DATA	data_protoscb;
	DUMPVAR_DATA		data_dumpvar;
	char				*name_function=NULL;

	if ( output )
	{
		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
				output( "    Module window %s\n", get_name_wgt( module->wgt_window ) );
				break;
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				output( "    Module dialog %s\n", get_name_wgt( module->wgt_window ) );
				break;
			case MODULE_TYPE_MENU :
				output( "    Module menu %s\n", get_name_wgt( module->wgt_window ) );
				break;
			default :
				assert( 0 );
		}
	}

	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
			name_function = module->u1.window.name_function;
			break;
		case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			name_function = module->u1.dialog.name_function;
			break;
		case MODULE_TYPE_MENU :
			name_function = module->u1.menu.name_function;
			break;
		default :
			assert( 0 );
	}

	/*--- Create .c and .h files ---*/
	sprintf( fname_c, "%s/glg/%s.c", 
		dirname, get_name_wgt( module->wgt_window ) );
	fp_c = fopen( fname_c, "w" );
	if ( fp_c != NULL )
	{
		sprintf( fname_h, "%s/glg/%s.h", 
			dirname, get_name_wgt( module->wgt_window ) );
		fp_h = fopen( fname_h, "w" );
	}
	if ( !fp_c || !fp_h )
	{
		if ( fp_c )
			fclose( fp_c );
		return -1;
	}

	if ( !strcmp( name_function, "" ) )
		sprintf( function_name, "%s%s",
			"module_",
			get_name_wgt( module->wgt_window ) );
	else
		strcpy( function_name, name_function );

	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
		case MODULE_TYPE_DIALOG	:
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			fprintf( fp_h, "extern GtkWidget *%s( GtkWidget *parent, const int realize );\n", 
				function_name );
			break;
		case MODULE_TYPE_MENU :
			fprintf( fp_h, "extern GtkWidget *%s( void );\n", 
				function_name );
			break;
		default :
			break;
	}							// switch

	fprintf( fp_h, "\n" );
	fprintf( fp_h, "COMMON GtkWidget\t*%s\tINIT( NULL );\n", 
		get_name_wgt( module->wgt_window ) );
	
	add_system_includes( fp_c );
	fprintf( fp_c, "\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp_c, "#include <gnome.h>\n" );
	else
#endif
	{
		fprintf( fp_c, "#include <gdk/gdkkeysyms.h>\n" );
		fprintf( fp_c, "#include <gtk/gtk.h>\n" );
	}
	fprintf( fp_c, "\n" );
	fprintf( fp_c, "#include \"main.h\"\n" );
	fprintf( fp_c, "#include \"modules.h\"\n" );
	fprintf( fp_c, "\n" );

	/*--- #include the .c files which define the cb ---*/
	nb_cbfiles = 0;
	cbfiles = NULL;
	gen_cb_filesname( module, &nb_cbfiles, &cbfiles );

	/*--- Create the static (module) variables for local widgets ---*/
	nb_local_var = nb_global_var = 0;

	/*--- Add prototypes for callbacks defined into this module window ---*/
	name_cpt = 0;
	data_protoscb.module = module;
	data_protoscb.level  = -1;
	data_protoscb.cpt    = 0;
	data_protoscb.parent = module->wgt_window;
	data_protoscb.fp_h	 = fp_h;
	forall_dump_protoscb( module->wgt_window, &data_protoscb );

	if ( !strcmp( name_function, "" ) )
	{
		switch ( module->type )
		{

			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG	:
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				fprintf( fp_c, "GtkWidget *\n%s%s( GtkWidget *parent, const int realize )%s{\n", 
					"module_",
					get_name_wgt( module->wgt_window ),
					(config.indentstyle_function == 1) ? "\n" : " " );
				break;
			case MODULE_TYPE_MENU	:
				fprintf( fp_c, "GtkWidget *\n%s%s( void )%s{\n", 
					"module_",
					get_name_wgt( module->wgt_window ),
					(config.indentstyle_function == 1) ? "\n" : " " );
				break;
			default :
				assert( 0 );
		}						// switch
	}
	else
	{
		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG	:
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				fprintf( fp_c, "GtkWidget *\n%s( GtkWidget *parent, const int realize )%s{\n", 
					name_function,
					(config.indentstyle_function == 1) ? "\n" : " " );
				break;
			case MODULE_TYPE_MENU	:
				fprintf( fp_c, "GtkWidget *\n%s( void )%s{\n", 
					name_function,
					(config.indentstyle_function == 1) ? "\n" : " " );
				break;
			default :
				assert( 0 );
		}						// switch
	}

	/*--- Create the auto variables for anonymous widgets ---*/
	strcpy( list_var, "\tGtkWidget\t" );
	data_dumpvar.level = -1;
	data_dumpvar.cpt = 0;
	data_dumpvar.fp_c = fp_c;
	data_dumpvar.fp_h = fp_h;
	data_dumpvar.list_var = list_var;
	forall_dump_var( module->wgt_window, &data_dumpvar );
	if ( strcmp( data_dumpvar.list_var, "\tGtkWidget\t" ) )
	{
		list_var[ strlen(data_dumpvar.list_var) - 2 ] = 0;
		fprintf( fp_c, "%s;\n", data_dumpvar.list_var );
	}

	/*--- Create a tooltips widget if any widget reference a tooltips ---*/
	if ( gen_any_tooltips( module ) )
		fprintf( fp_c, "\tGtkTooltips\t*_wgt_tooltips = gtk_tooltips_new();\n" );

	/*--- Create a accelerator group if any widget reference an accelerator ---*/
	if ( gen_any_accelerator( module ) )
		fprintf( fp_c, "\tGtkAccelGroup\t*_wgt_accel_group;\n" );

	/*--- Save each widget of this module ---*/
	name_cpt = 0;
	data.module = module;
	data.level  = -1;
	data.cpt    = 0;
	data.parent = NULL;
	data.fp_c	= fp_c;
	forall_dump_window( module->wgt_window, &data );

	/*--- Realize the window, and all widgets inside: this is an optional step ---*/
	fprintf( fp_c, "\n" );
	if ( (module->type == MODULE_TYPE_WINDOW) ||
		 (module->type == MODULE_TYPE_DIALOG)
#ifdef USE_GNOME
		 || (module->type == MODULE_TYPE_GNOMEDIALOG_DIALOG)
		 || (module->type == MODULE_TYPE_GNOMEDIALOG_ABOUT)
		 || (module->type == MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX)
		 || (module->type == MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX)
#endif
		)
		fprintf( fp_c, "\tif ( realize )\n\t" );
	fprintf( fp_c, "\tgtk_widget_show_all( %s );\n", 
		get_name_wgt( module->wgt_window ) );
	if ( module->type == MODULE_TYPE_MENU )
	{
		fprintf( fp_c, "\n\tgtk_menu_popup( GTK_MENU( %s ), NULL, NULL, NULL, NULL,",
			get_name_wgt( module->wgt_window ) );
		fprintf( fp_c, "\n\t\t1, NULL );\n" );
	}							// if

	/*--- Return widget pointer ---*/
	fprintf( fp_c, "\n\treturn %s;\n", 
		get_name_wgt( module->wgt_window ) );

	/*--- End of the function ---*/
	fprintf( fp_c, "\n}\t\t\t\t\t\t\t\t// %s\n",
		function_name );

	/*--- Free cb files names ---*/	
	if ( nb_cbfiles )
	{
		for ( n = 0; n < nb_cbfiles; n++ )
			free( cbfiles[ n ] );
		nb_cbfiles = 0;
		free( cbfiles );
		cbfiles = NULL;
	}

	/*--- Ok, done ---*/
	fclose( fp_c );
	fclose( fp_h );
	return 0;

}								// generate_module_window


static int
generate_makefiles( const char * const dirname,
	void (*output)(char *fmt, ... ) )
{
	char			make_dirname[ PATH_MAX+NAME_MAX + 80 ];
	char			fname[ PATH_MAX+NAME_MAX + 120 ];
	FILE			*fp;
	int				status;
	module_info_t	*module;
	int				n;
	int				nb_cbfiles;
	char			**cbfiles;
	char			line[ 300 ];
	char			buff[ 80 ];
	time_t			now;

	/*--- Create the directory '$(dir)/src/qnx' ---*/
	sprintf( make_dirname, "%s/obj", dirname );
	if ( chdir( make_dirname ) == -1 )
	{
		status = mkdir( make_dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d", make_dirname, errno );
			return -1;
		}
	}			// if

	/*--- Create the file '$(dir)/src/qnx/Makefile' ---*/
	sprintf( fname, "%s/Makefile", make_dirname );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d\n", fname, errno );
		return -1;
	}
	if ( output )
		output( "        %s...   ", fname );

	/*--- Create the makefile header ---*/
	fprintf( fp, "#\n" );
	fprintf( fp, "#\tMakefile - Project: %s\n", dirname );
	fprintf( fp, "#\n" );
	fprintf( fp, "#\n" );
	now = time( NULL );
	strftime( buff, sizeof(buff), "%A %d %B %Y %T", localtime( &now ) );
	fprintf( fp, "#\tDate: %s\n", buff );
	fprintf( fp, "#\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "all : libglg.a %s\n",
		config.exe_name );
	fprintf( fp, "\n" );
	
	switch ( config.language_current )
	{
		case LANGUAGE_C :
			fprintf( fp, "CC = gcc\n" );
			fprintf( fp, "LD = gcc\n" );
			break;
#ifdef GLG_SUPPORT_LANGUAGECPP
		case LANGUAGE_CPP :
			fprintf( fp, "CC = g++\n" );
			fprintf( fp, "LD = g++\n" );
			break;
#endif
		default :
			assert( 0 );
	}
	fprintf( fp, "AR = ar\n" );
	fprintf( fp, "ARFLAGS = -urs\n" );
	fprintf( fp, "\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp, "\nUSE_GNOME = 1\n\n" );
#endif
	fprintf( fp, "# C compiler flags\n" );
	fprintf( fp, "ifdef DEBUG\n" );
	fprintf( fp, "DEBUG_FLAGS = -g2 -O0\n" );
	fprintf( fp, "else\n" );
	fprintf( fp, "DEBUG_FLAGS = -g0 -O9\n" );
	fprintf( fp, "endif\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "ifdef USE_GNOME\n" );
	fprintf( fp, "UI_CFLAGS = `gnome-config --cflags gnome` `gnome-config --cflags gnomeui` \\\n" );
	fprintf( fp, "	-DUSE_GNOME $(DEBUG_FLAGS) -Wall -I. -I../src\n" );
	fprintf( fp, "UI_LIB = `gnome-config --libs gnome gnomeui` \\\n" );
	fprintf( fp, "	$(DEBUG_FLAGS) -Wall \n" );
	fprintf( fp, "else\n" );
	fprintf( fp, "UI_CFLAGS = `gtk-config --cflags` \\\n" );
	fprintf( fp, "	$(DEBUG_FLAGS) -Wall -I. -I../src\n" );
	fprintf( fp, "UI_LIB = `gtk-config --libs` \\\n" );
	fprintf( fp, "	$(DEBUG_FLAGS) -Wall\n" );
	fprintf( fp, "endif\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "#LDFLAGS= $(UI_CFLAGS) $(UI_LIB) -lefence\n" );
	fprintf( fp, "LDFLAGS= $(UI_CFLAGS) $(UI_LIB)\n" );
	fprintf( fp, "CFLAGS= $(UI_CFLAGS)\n" );

	fprintf( fp, "\nvpath %%.c ../src/:../src/glg/" );
	fprintf( fp, "\nvpath %%.h ../src/:../src/glg/" );
	fprintf( fp, "\n" );

	fprintf( fp, "\nSYS_OBJS = startup.o " );

	fprintf( fp, "\nMODULE_OBJS = main.o " );
	if ( nb_styles  > 0 )
		fprintf( fp, "styles.o " );
	module = next_module( FIRST );
	while ( module )
	{
		fprintf( fp, " \\\n\t%s.o", get_name_wgt( module->wgt_window ) );
		module = next_module( NEXT );
	}

	fprintf( fp, "\nADDED_OBJS = " );
	for ( n = 0; n < MAXNB_CFILES_ADDED; n++ )
	{
		if ( strcmp( config.list_cfiles_added[ n ], "" ) )
			fprintf( fp, " \\\n\t%s", config.list_cfiles_added[ n ] );
	}

	/*--- CB files ---*/
	nb_cbfiles = 0;
	cbfiles = NULL;
	strcpy( line, "CB_OBJS = " );
	module = next_module( FIRST );
	while ( module )
	{
		if ( (module->type == MODULE_TYPE_MENU) || (module->type == MODULE_TYPE_WINDOW) )
		{
			gen_cb_filesname( module, &nb_cbfiles, &cbfiles );
		}
		module = next_module( NEXT );
	}

	for ( n = 0; n < nb_cbfiles; n++ )
	{
		if ( strlen( line ) + strlen( cbfiles[ n ] ) + 5 > config.max_source_line )
		{
			if ( strncmp( line, "CB_OBJS = ", 10 ) )
				fprintf( fp, "\\" );
			fprintf( fp, "\n%s", line );
			strcpy( line, "\t" );
		}
		strcat( line, cbfiles[ n ] );
		strcat( line, ".o " );
	}

	if ( nb_cbfiles )
	{
		for ( n = 0; n < nb_cbfiles; n++ )
			free( cbfiles[ n ] );
		nb_cbfiles = 0;
		free( cbfiles );
		cbfiles = NULL;
	}

	if ( strncmp( line, "CB_OBJS = ", 10 ) )
		fprintf( fp, "\\" );
	fprintf( fp, "\n%s", line );

	fprintf( fp, "\n" );
	fprintf( fp, "\nHDR = main.h" );
	module = next_module( FIRST );
	while ( module )
	{
		fprintf( fp, " \\\n\t%s.h", get_name_wgt( module->wgt_window ) );
		module = next_module( NEXT );
	}

	if ( config.header_to_include[ 0 ] )
	{
		fprintf( fp, "\n" );
		fprintf( fp, "\nPRJHDR = %s", config.header_to_include );
	}

	fprintf( fp, "\n" );
	if ( config.header_to_include[ 0 ] )
		fprintf( fp, "\n%s\t:\t$(SYS_OBJS) $(ADDED_OBJS) $(CB_OBJS) $(PRJHDR) libglg.a %s", 
			config.exe_name,
			"" );
	else
		fprintf( fp, "\n%s\t:\t$(SYS_OBJS) $(ADDED_OBJS) $(CB_OBJS) libglg.a %s", 
			config.exe_name,
			"" );
	fprintf( fp, "\n\t$(LD) -o %s $(SYS_OBJS) $(ADDED_OBJS) $(CB_OBJS) ./libglg.a $(LDFLAGS)", 
		config.exe_name );
	fprintf( fp, "\n" );

	fprintf( fp, "\n$(SYS_OBJS) : $(HDR) %s",
		(config.header_to_include[0]) ? "$(PRJHDR)" : "" );
	fprintf( fp, "\n$(ADDED_OBJS) : $(HDR) %s",
		(config.header_to_include[0]) ? "$(PRJHDR)" : "" );
	if ( config.header_to_include[0] )
		fprintf( fp, "\n$(CB_OBJS) : $(HDR) $(PRJHDR)\n" );
	else
		fprintf( fp, "\n$(CB_OBJS) : $(HDR)\n" );

	if ( config.header_to_include[ 0 ] )
		fprintf( fp, "main.o\t:\t%s\n", config.header_to_include );


	/*--- Dependences of .c modules to .h headers ---*/
	fprintf( fp, "\nmain.o\t:\tmain.h" );
	fprintf( fp, "\n" );
	module = next_module( FIRST );
	while ( module )
	{
		fprintf( fp, "%s.o\t:\t%s.h\n", 
			get_name_wgt( module->wgt_window ), get_name_wgt( module->wgt_window ) );
		module = next_module( NEXT );
	}

	/*--- GLG library ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "#\n" );
	fprintf( fp, "# GLG library\n" );
	fprintf( fp, "#\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "libglg.a\t:\t$(MODULE_OBJS)\n" );
	fprintf( fp, "\t$(AR) $(ARFLAGS) libglg.a $(MODULE_OBJS)\n" );
	fprintf( fp, "\n" );


	/*--- Clean up ---*/
	fprintf( fp, "\n" );
	fprintf( fp, "#\n" );
	fprintf( fp, "# Clean-up\n" );
	fprintf( fp, "#\n" );
	fprintf( fp, "\n" );
	fprintf( fp, "clean:\n" );
	fprintf( fp, "\t$(RM) main.o $(SYS_OBJS) $(SYS_FLAGS) $(MODULE_OBJS) $(ADDED_OBJS) $(CB_OBJS) *.err *.map %s libglg.a\n", 
		config.exe_name );

	/*--- Ok, done ---*/
	if ( output )
		output( "ok\n" );
	fclose( fp );
	return 0;

}								// generate_makefiles


static int
generate_styles_h( const char * const dirname, const char * const filename )
{
	char		fname[ PATH_MAX+NAME_MAX + 80 ];
	FILE		*fp;
	int			n;

	sprintf( fname, "%s/glg/%s", dirname, filename );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create the file\n%s\nerrno=%d\n", fname, errno );
		return -1;
	}

	/*--- Dump all styles ---*/
	for ( n = 0; n < nb_styles; n++ )
	{
		fprintf( fp, "COMMON\tGtkStyle\t*%s;\n",
			styles_infos[ n ]->name );
	}
	fprintf( fp, "\nCOMMON\tvoid init_styles( void );\n" );

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// generate_styles_h


static int
generate_styles_c( const char * const dirname, const char * const filename )
{
	char			fname[ PATH_MAX+NAME_MAX + 80 ];
	FILE			*fp;
	int				n, k;
	GtkStyle		*style;
	style_info_t	*style_info;
	GdkColor		*color;

	sprintf( fname, "%s/glg/%s", dirname, filename );
	fp = fopen( fname, "w" );
	if ( fp == NULL )
	{
		modal_error( "Unable to create file\n%s\nerrno=%d\n", errno );
		return -1;
	}

	fprintf( fp, "#include <stdio.h>\n" );
	fprintf( fp, "#include <stdlib.h>\n" );
	fprintf( fp, "#include <string.h>\n" );
	fprintf( fp, "#include <stdarg.h>\n" );
	fprintf( fp, "\n" );
#ifdef USE_GNOME
	if (config.is_gnome)
		fprintf( fp, "#include <gnome.h>\n" );
	else
#endif
	{
		fprintf( fp, "#include <gdk/gdkkeysyms.h>\n" );
		fprintf( fp, "#include <gtk/gtk.h>\n" );
	}

	fprintf( fp, "\n" );
	fprintf( fp, "#include \"main.h\"\n" );

	fprintf( fp, "#include \"modules.h\"\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "static void\n" );
	fprintf( fp, "init_color( GdkColor *color,\n");
	fprintf( fp, "\tgushort red, gushort green, gushort blue )\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\tcolor->red   = red;\n" );
	fprintf( fp, "\tcolor->green = green;\n" );
	fprintf( fp, "\tcolor->blue  = blue;\n" );
	fprintf( fp, "}\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "\n" );
	fprintf( fp, "void\n" );
	fprintf( fp, "init_styles( void )\n" );
	fprintf( fp, "{\n" );
	fprintf( fp, "\tGtkStyle\t*style;\n" );
	fprintf( fp, "\tGdkColor\t*color;\n" );
	fprintf( fp, "\n" );

	for ( n = 0; n < nb_styles; n++ )
	{
		style = styles[ n ];
		style_info = styles_infos[ n ];
		fprintf( fp, "\t/*--- %s ---*/\n", 
			style_info->name );
		fprintf( fp, "\t%s = gtk_style_new();\n",
			style_info->name );
		fprintf( fp, "\tstyle = %s;\n",
			style_info->name );

#define DUMP( A ) \
		color = &style->##A[ 0 ]; \
		fprintf( fp, "\tcolor = &style->%s[ 0 ];\n", #A ); \
		for ( k = 0; k < sizeof( style->##A ) / sizeof( style->##A[ 0 ] ); k++ ) \
		{ \
			fprintf( fp, "\tinit_color( color++, %u, %u, %u );\n", \
				color->red, color->green, color->blue ); \
			color++; \
		} \
		fprintf( fp, "\n" )

		DUMP( fg );
		DUMP( bg );
		DUMP( light );
		DUMP( dark);
		DUMP( mid );
		DUMP( text );
		DUMP( base );

#undef DUMP

	}
	fprintf( fp, "}\n" );

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// generate_styles_c


PUBLIC int
generate_c( const char * const dirname,
	void (*output)(char *fmt, ... ) )
{
	char			src_dirname[ PATH_MAX+NAME_MAX + 40 ];
	char			src_dirname_glg[ PATH_MAX+NAME_MAX + 40 ];
	char			src_dirname_xpm[ PATH_MAX+NAME_MAX + 40 ];
	int				status = 0;
	char			hti[ PATH_MAX+NAME_MAX + 120 ];
	module_info_t	*module;

	/*--- Block all others windows ---*/
	push_block_all_windows();

	if ( output )
		output( "Generate %s\n", dirname );

	/*--- Create the '$(dir)/src' directory ---*/
	sprintf( src_dirname, "%s/src", dirname );
	if ( chdir( src_dirname ) == -1 )
	{
		status = mkdir( src_dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d\n", src_dirname, errno );
			pop_block_all_windows();
			return -1;
		}
		else
		{
			if ( output )
				output( "    created directory %s\n", src_dirname );
		}
	}							// if

	/*--- Create the '$(dir)/src/xpm' directory ---*/
	sprintf( src_dirname_xpm, "%s/xpm", src_dirname );
	if ( chdir( src_dirname_xpm ) == -1 )
	{
		status = mkdir( src_dirname_xpm, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d\n", src_dirname_xpm, errno );
			pop_block_all_windows();
			return -1;
		}
		else
		{
			if ( output )
				output( "    created directory %s\n", src_dirname_xpm );
		}
	}							// if

	/*--- Create the '$(dir)/src/glg' directory ---*/
	sprintf( src_dirname_glg, "%s/glg", src_dirname );
	if ( chdir( src_dirname_glg ) == -1 )
	{
		status = mkdir( src_dirname_glg, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
		if ( status == -1 )
		{
			modal_error( "Unable to create the directory\n%s\nerrno=%d\n", src_dirname_glg, errno );
			pop_block_all_windows();
			return -1;
		}
		else
		{
			if ( output )
				output( "    created directory %s\n", src_dirname_glg );
		}
	}							// if

	/*--- Does the header to include exists ? ---*/
	if ( config.header_to_include[0] )
	{
		sprintf( hti, "%s/%s", 
			src_dirname, config.header_to_include );
		if ( access( hti, F_OK ) == -1 )
		{
			FILE * fp;
			fp = fopen( hti, "w" );
			if ( fp != NULL )
			{
				if ( output )
					output( "    created header %s\n", hti );
				fclose( fp );
			}
		}
	}							// if

	/*--- Create the startup.c file, but only if not exist ---*/
	sprintf( hti, "%s/startup.c", 
		src_dirname );
	if ( access( hti, F_OK ) == -1 )
	{
		FILE * fp;
		fp = fopen( hti, "w" );
		if ( fp != NULL )
		{
			if ( output )
				output( "    created startup.c %s\n", hti );

			add_system_includes( fp );
			fprintf( fp, "\n" );
#ifdef USE_GNOME
			if (config.is_gnome)
				fprintf( fp, "#include <gnome.h>\n" );
			else
#endif
			{
				fprintf( fp, "#include <gdk/gdkkeysyms.h>\n" );
				fprintf( fp, "#include <gtk/gtk.h>\n" );
			}
			fprintf( fp, "\n" );
			fprintf( fp, "#define __STARTUP__\n" );
			fprintf( fp, "#include \"glg/main.h\"\n" );
			fprintf( fp, "\n" );

			fprintf( fp, "#include \"glg/modules.h\"\n" );
			fprintf( fp, "\n" );

			fprintf( fp, "void\n" );
			fprintf( fp, "startup( int argc, char *argv[] )\n" );
			fprintf( fp, "{\n" );
			fprintf( fp, "}\n" );

			fclose( fp );
		}
	}

	/*--- Generate '$(dir)/src/main.[ch]' ---*/
	if ( (generate_main_c( src_dirname, "main.c" ) == -1) ||
		 (generate_main_h( src_dirname, "main.h" ) == -1) )
	{
		pop_block_all_windows();
		return -1;
	}

	/*--- Generate '$(dir)/src/styles.[ch]' ---*/
	if ( nb_styles > 0 )
	{
		if ( (generate_styles_c( src_dirname, "styles.c" ) == -1) ||
			 (generate_styles_h( src_dirname, "styles.h" ) == -1) )
		{
			pop_block_all_windows();
			return -1;
		}
	}

	/*--- Generate Makefiles ---*/
	if ( output )
		output( "    Makefile\n" );
	if ( generate_makefiles( dirname, output ) == -1 )
	{
		pop_block_all_windows();
		return -1;
	}

	/*--- Create a header 'modules.h' which includes the headers for all modules ---*/
	if ( output )
		output( "    created modules.h...  " );
	generate_modules_h( src_dirname );
	if ( output )
		output( "done\n" );

	/*--- Generate '$(dir)/src/$(MODULE).c' ---*/
	module = next_module( FIRST );
	while ( module )
	{
		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			case MODULE_TYPE_MENU :
//				if ( module->f_not_generated )
				{
					status = generate_module_window( src_dirname, module, output );
					project_generated( module );
				}
				break;
			default :
				assert(0);
				break;
		}
		if ( status == -1 )
		{
			pop_block_all_windows();
			return -1;
		}
		module = next_module( NEXT );
	}							// while

	/*--- Ok, done ---*/
	if ( output )
		output( "Done... \n" );
	pop_block_all_windows();
	return 0;

}								// generate_c

