#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


PUBLIC char *
get_style_name( GtkStyle *style )
{
	int		n;
	
	for ( n = 0; n < nb_styles; n++ )
		if ( styles[ n ] == style )
			return styles_infos[ n ]->name;

	assert( 0 );
	return NULL;

}								// get_style_name


/*
	gen_any_tooltips
*/

typedef struct
{
	int			level;
	int			cpt;
	int			found;
}	FORALL_ANYTOOLTIPS;


static void
forall_anytooltips( GtkWidget *widget, FORALL_ANYTOOLTIPS *data )
{
	GtkArg	*parg;

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if ( !config.show_composite_childs )
#endif
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget))
			return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	parg = get_outsideres_wgt( widget, "~GtkWidget::tip_text" );
	assert( parg != NULL );
	assert( parg->type == GTK_TYPE_STRING );
	if ( strcmp( GTK_VALUE_STRING( parg[0] ), "" ) )
	{
		data->found = TRUE;
	}

	if ( !data->found && GTK_IS_CONTAINER( widget ) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_anytooltips, data );
	}

	data->level--;

}								// forall_anytooltips


PUBLIC int
gen_any_tooltips( module_info_t *module )
{
	FORALL_ANYTOOLTIPS	data;

	data.level	= -1;
	data.cpt	= 0;
	data.found	= FALSE;
	forall_anytooltips( module->wgt_window, &data );
	return data.found;

}								// gen_any_tooltips


PUBLIC int
is_accelerator( GtkWidget *wgt, char **signal_name, guint *keyval,
	GdkModifierType *accelerator_mods, GtkAccelFlags *accel_flags )
{
	gtk_wgtdef_t		*wgtdef;
	int					depth, n;

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );
	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
		  depth++ )
	{
		defclasssignal_t	*classes_signal, *q;
		int					nb_signals;
		
		classes_signal = wgtdef->def[depth].classes_signal;
		nb_signals	   = wgtdef->def[depth].nb_classes_signal;
		for ( q = classes_signal, n = 0; n < nb_signals; n++, q++ )
		{
			data_cb_t	*datacb;
			int			k;
			
			datacb = get_object_data( wgt, q->resname );
			if (datacb == NULL)
				continue;
			for ( k = 0; k < datacb->nb_cb; k++ )
			{
				if ( datacb->cb[k].is_accelerator )
				{
					if ( signal_name )
						*signal_name = strdup( q->resname );
					if ( keyval )
						*keyval = datacb->cb[k].accelerator_key_val;
					if ( accelerator_mods )
						*accelerator_mods = datacb->cb[k].accelerator_mods;
					if ( accel_flags )
						*accel_flags = datacb->cb[k].accel_flags;
					return TRUE;
				}				// if
			}
		}						// for (q)
	}							// for (depth)
	
	return FALSE;
		
}								// is_accelerator


/*
	gen_any_accelerator
*/

typedef struct
{
	int			level;
	int			cpt;
	int			found;
}	FORALL_ANYACCELERATOR;


static void
forall_anyaccelerator( GtkWidget *widget, FORALL_ANYACCELERATOR *data )
{

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if ( !config.show_composite_childs )
#endif
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget))
			return;

	data->level++;
	data->cpt++;

	/*--- Don't save blocked container ---*/
	if ( is_accelerator( widget, NULL, NULL, NULL, NULL ) )
	{
		data->found = TRUE;
	}

	if ( !data->found && GTK_IS_CONTAINER( widget ) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_anyaccelerator, data );
	}

	data->level--;

}								// forall_anytooltips


PUBLIC int
gen_any_accelerator( module_info_t *module )
{
	FORALL_ANYACCELERATOR	data;

	data.level	= -1;
	data.cpt	= 0;
	data.found	= FALSE;
	forall_anyaccelerator( module->wgt_window, &data );
	return data.found;
}								// gen_any_accelerator


PUBLIC int
generate( const char * const dirname,
	void (*output)(char *fmt, ... ) )
{
	int		status=0;

	switch ( config.language_current )
	{
		case LANGUAGE_C :
			status = generate_c( dirname, output );
			break;
#ifdef GLG_SUPPORT_LANGUAGECPP
		case LANGUAGE_CPP :
			status = generate_c( dirname, output );
			break;
#endif
		default :
			assert( 0 );
	}

	return status;

}								// generate

