#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <strings.h>
#include <inttypes.h>
#include <errno.h>
#include <assert.h>

#define __MAIN__
#include "glg.h"


static void
init_enums_infos( void )
{

#define ___( X ) \
	enum_infos[ _token_##X ].type = ##X;
#include "resources/enum_infos.h"
#undef ___X

}								// init_enums_infos

static void
cb_warning_dialog_cancel( GtkWidget *widget, gpointer data )
{
	GtkWidget	**wgt = (GtkWidget **)data;

	/*--- Close the dialog ---*/
	gtk_widget_destroy( *wgt );
	*wgt = NULL;
	
}								// cb_warning_dialog_cancel


static gint
cb_close_window( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_close_window


static void
initial_warning( void )
{
	GtkWidget	*window1;
	GtkWidget	*label;
	GtkWidget	*wgt_1_2, *wgt_2_3, *wgt_3_4, *wgt_4_5, *wgt_5_6, 
				*wgt_3_5, *wgt_4_6, *wgt_5_7, *wgt_2_4, *wgt_2_5;
	static char *french =
		"Bonjour!\n\n"
		"'Lo Gafet' est un 'application builder' pour le GTK.\n"
		"L'etat actuel est plutot du type Alpha. Beaucoup de\n"
		"choses sont certainemeny a ameliorer et a ajouter!\n\n"
		"Je compte vraiment sur vos nombreux emails pour non seulement\n"
		"ameliorer les choses, me donner des idees, mais aussi pour\n"
		"me motiver!\n\n"
		"Olivier Singla\n"
		"lo_gafet@users.sourceforge.net"
		;
	static char *english =
		"Hi!\n\n"
		"'Lo Gafet' is an application builder for the GTK.\n"
		"Current state is Alpha. Many things have to be \n"
		"either added or much improved!\n\n"
		"I need your email and support to improve things, to implement\n"
		"new ideas, and also to motivate me!\n\n"
		"Olivier Singla\n"
		"lo_gafet@users.sourceforge.net"
		;

	window1 = gtk_widget_new( gtk_window_get_type(),
		"width", 450,
		"height", 350,
		"title", "Initial Warning...",
		"allow_grow", 1,
		"modal", 1,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"parent", window1,
		NULL );

	label = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_1_2,
		"label", "This appears the first time that you run Gtk Lo Gafet...",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), label,
		"expand", 0,
		"padding", 4,
		"position", 0,
		NULL );

	wgt_2_4 = gtk_widget_new( gtk_hseparator_get_type(),
		"parent", wgt_1_2,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_4,
		"expand", 0,
		"padding", 4,
		"position", 1,
		NULL );

	wgt_2_3 = gtk_widget_new( gtk_notebook_get_type(),
		"parent", wgt_1_2,
		"can_focus", 1,
		"page", 0,
		NULL );

	wgt_3_4 = gtk_widget_new( gtk_scrolled_window_get_type(),
		"parent", wgt_2_3,
		"hscrollbar_policy", GTK_POLICY_AUTOMATIC,
		"vscrollbar_policy", GTK_POLICY_AUTOMATIC,
		NULL );

	wgt_4_5 = gtk_widget_new( gtk_viewport_get_type(),
		"parent", wgt_3_4,
		NULL );

	wgt_5_6 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_4_5,
		"xpad", 4,
		"ypad", 4,
		"label", english,
		NULL );

  label = gtk_label_new ("English");
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_2_3), 
  	gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_2_3), 0), label );

	wgt_3_5 = gtk_widget_new( gtk_scrolled_window_get_type(),
		"parent", wgt_2_3,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_3 ), wgt_3_5,
		"position", 1,
		NULL );

	wgt_4_6 = gtk_widget_new( gtk_viewport_get_type(),
		"parent", wgt_3_5,
		NULL );

  label = gtk_label_new ("French");
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_2_3), 
  	gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_2_3), 1), label );

	wgt_5_7 = gtk_widget_new( gtk_label_get_type(),
		"parent", wgt_4_6,
		"xpad", 4,
		"ypad", 4,
		"label", french,
		NULL );

	wgt_2_4 = gtk_widget_new( gtk_hseparator_get_type(),
		"parent", wgt_1_2,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_4,
		"expand", 0,
		"padding", 4,
		"position", 3,
		NULL );

	wgt_2_5 = gtk_widget_new( gtk_button_get_type(),
		"parent", wgt_1_2,
		"can_focus", 1,
		"receives_default", 1,
		"border_width", 4,
		"label", "Ok",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_5,
		"expand", 0,
		"position", 4,
		NULL );
	gtk_signal_connect( GTK_OBJECT(wgt_2_5), "clicked",
		GTK_SIGNAL_FUNC(cb_warning_dialog_cancel), &window1 );

	gtk_signal_connect( GTK_OBJECT(window1), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_window), NULL );
	gtk_widget_show_all( window1 );

	while ( window1 != NULL )
		gtk_main_iteration();

}								// initial_warning


static void
create_pixmaps( GdkWindow *window )
{
	gtk_wgtdef_t	**wgtdef, *w;
	GdkColor		color_transparent = { 0 };

	for ( wgtdef = (gtk_wgtdef_t **)gtk_wgtdef; (w = *wgtdef) != NULL; wgtdef++ )
	{
		w->gdkpixmap = NULL;
		w->gdkbitmap = NULL;
		if ( w->pixmap )
		{
			w->gdkpixmap = gdk_pixmap_create_from_xpm_d( window, &w->gdkbitmap, 
				&color_transparent, w->pixmap );
			assert( w->gdkpixmap != NULL );
		}						// if
	}							// for

}								// create_pixmaps


static void
load_fonts( void )
{
	GtkStyle 	*style;
	GdkFont		*font;

	style = gtk_widget_get_default_style();
	
	properties_style_normal = gtk_style_copy( style );
	font = gdk_font_load( "-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-iso8859-1");
	properties_style_normal->font = font;
	
	properties_style_bold = gtk_style_copy( style );
	font = gdk_font_load( "-*-helvetica-bold-r-normal-*-*-120-*-*-p-*-iso8859-1");
	properties_style_bold->font = font;

	properties_style_oblique = gtk_style_copy( style );
	font = gdk_font_load( "-*-helvetica-medium-o-normal-*-*-120-*-*-p-*-iso8859-1");
	properties_style_oblique->font = font;

}								// load_fonts


static void 
my_glog( const gchar *log_domain,
	GLogLevelFlags log_level,
	const gchar *message,
	gpointer user_data)
{
//	g_log_default_handler( log_domain, log_level, message, user_data );
}								// my_glog



static void
which_system( void )
{
	struct utsname	u;

	uname( &u );
	if ( !strcasecmp( u.sysname, "freebsd" ) )
		syst = SYST_FREEBSD;
	else
		syst = SYST_UNKNOW;

}								// which_system


int
main( int argc, char *argv[] )
{
	char		*p;
	int			do_initial_warning = FALSE;
	GtkWidget	*wgt_fake_wnd;
	int			n;

#ifdef USE_GNOME
	gnome_init( "Gtk Lo Gafet", "0.0", argc, argv );
#else
	gtk_init( &argc, &argv );
#endif
	
	g_log_set_handler( "Gtk", 
		(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL), 
		my_glog, NULL );
	g_log_set_handler( "Gdk", 
		(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL), 
		my_glog, NULL );
	g_log_set_handler( "GLib", 
		(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL), 
		my_glog, NULL );
	g_log_set_handler( NULL, 
		(GLogLevelFlags)(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL), 
		my_glog, NULL );

	which_system();

	init_enums_infos();

    /*--- Current project name ---*/
    strcpy( project_dirname, "" );

    /*--- Last opened projects ---*/
    for ( n = 0; n < MAXNB_LAST_PROJECTS; n++ )
        last_projects[ n ] = NULL;

	/*--- Load fonts used for internal use ---*/
	load_fonts();

	/*--- Check default value for each resource ---*/
	check_default_values();

	create_default_config();

	/*--- Restore Windows Positions ? ---*/
	if ( getenv( ENVVAR_GLG_RESTOREWNDOWSPOSITION ) )
		is_restore_windows_position = TRUE;
	else
		is_restore_windows_position = FALSE;

	/*--- Restore Windows Positions ? ---*/
	if ( getenv( ENVVAR_GLG_EDITOR1 ) )
		strcpy( config.editor1, getenv( ENVVAR_GLG_EDITOR1 ) );
	if ( getenv( ENVVAR_GLG_EDITOR2 ) )
		strcpy( config.editor2, getenv( ENVVAR_GLG_EDITOR2 ) );
	if ( getenv( ENVVAR_GLG_EDITOR3 ) )
		strcpy( config.editor3, getenv( ENVVAR_GLG_EDITOR3 ) );

	wgt_fake_wnd = gtk_widget_new( gtk_window_get_type(),
		"GtkWindow::type", GTK_WINDOW_DIALOG,
		"GtkWindow::window_position", GTK_WIN_POS_NONE,
		"GtkWidget::x", 0,
		"GtkWidget::y", 0,
		"GtkWidget::width", 1,
		"GtkWidget::height", 1,
		"GtkWindow::default_width", 1,
		"GtkWindow::default_height", 1,
		"GtkWindow::title", "Fake Window",
		"GtkWindow::allow_grow", 1,
		"GtkWidget::visible", 0,
		NULL );
	gtk_widget_show_now( wgt_fake_wnd );
	gdk_window_get_position( wgt_fake_wnd->window,  &wm_x_offset, &wm_y_offset );
	gtk_widget_destroy( wgt_fake_wnd );
	
    clear_project( NULL );

	/*--- Read global config name ---*/
    if ( (p = getenv( "HOME" )) != NULL )
    {
        char    _dir_name[ PATH_MAX+NAME_MAX + 100 ];
        char    _config_name[ PATH_MAX+NAME_MAX + 100 ];
        int     len = strlen( p );
		char	cwd_buff[ PATH_MAX + 1 ], *cwd;
        sprintf( _dir_name, "%s%s.glgcfg",
            p,
            (p[ len-1 ] == '/') ? "" : "/" );
		cwd = getcwd( cwd_buff, PATH_MAX + 1 );
        if ( chdir( _dir_name ) == -1 )
		{
            if ( mkdir( _dir_name, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) == -1 )
				fprintf( stderr, "Error %d creating directory %s\n",
					errno, _dir_name );
		}
        sprintf( _config_name, "%s/lo_gafet.cfg",
            _dir_name );
		if (access(_config_name, F_OK) == -1 )
			do_initial_warning = TRUE;
		read_config( _config_name, TRUE );
        config_name = strdup( _config_name );
		if ( cwd != NULL )
			chdir( cwd );
    }

	create_palette_window();
	create_resources_window();
	create_tree_window();

#if 0
	gdk_window_set_decorations( wgt_wnd_project->window, 
		GDK_DECOR_BORDER | GDK_DECOR_RESIZEH | GDK_DECOR_TITLE | GDK_DECOR_MENU );
	gdk_window_set_functions( wgt_wnd_project->window, 
		GDK_FUNC_RESIZE | GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE );
#endif

	/*--- Make the windows visible ---*/
	gtk_widget_show_all( wgt_wnd_palette );
	gtk_widget_show_all( wgt_wnd_resources );
	gtk_widget_show_all( wgt_wnd_tree );

	clear_warning();

	if ( do_initial_warning )
	{
		cb_windows_arrange_right( NULL, NULL );
	}
	else
	{
		if ( is_restore_windows_position )
		{
			gtk_widget_set_uposition( wgt_wnd_palette,
				config.palette_wnd.x, config.palette_wnd.y );
			gtk_widget_set_uposition( wgt_wnd_resources,
				config.resources_wnd.x, config.resources_wnd.y );
			gtk_widget_set_uposition( wgt_wnd_tree,
				config.tree_wnd.x, config.tree_wnd.y );
		}
	}

	if ( do_initial_warning )
		initial_warning();

	/*--- Create pixmap, used for tree ---*/
	create_pixmaps( wgt_wnd_tree->window );

  	gtk_main();
	
	return (0);
}								// main

