#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static	GtkWidget	*wgt_filesel;
static	int			f_ok;


static void
cb_filesel_ok( GtkWidget *widget, void *data )
{
	gchar	*filename;
	gint	len;
	int		status;

	filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION(wgt_filesel) );
	len = strlen( filename );
	if (len == 0)
		return;
	if ( (len >= 1) && (filename[len - 1] == '/') )
		filename[len - 1] = 0;

	status = do_load_project( wgt_filesel, filename );

	gtk_widget_destroy( widget );
	wgt_filesel = NULL;
	f_ok = TRUE;

}								// cb_filesel_ok

static void
cb_filesel_cancel( GtkWidget *widget, void *data )
{
	gtk_widget_destroy( widget );
	wgt_filesel = NULL;
	f_ok = FALSE;
}								// cb_filesel_cancel


static int 
cb_filesel_button_release_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkWidget	*wgt_filesel = GTK_WIDGET( data );
	char		fname[ PATH_MAX+NAME_MAX + 80 ];
	int			ok;
	FILE		*fp;

	ok = TRUE;
	sprintf( fname, "%sglg/glg", 
		gtk_file_selection_get_filename( GTK_FILE_SELECTION( wgt_filesel ) ) );
	fp = fopen( fname, "r" );
	if ( access( fname, F_OK ) == -1 )
		ok = FALSE;
	
	if ( ok )
	{
		sprintf( fname, "%sglg/styles", 
			gtk_file_selection_get_filename( GTK_FILE_SELECTION( wgt_filesel ) ) );
		if ( access( fname, F_OK ) == -1 )
			ok = FALSE;
	}
	
	gtk_widget_set( GTK_WIDGET( GTK_FILE_SELECTION(wgt_filesel)->ok_button ), 
		"sensitive", ok, NULL );

	return FALSE;
}								// cb_filesel_button_release_event


PUBLIC void
load_project( void )
{
	GtkWidget	*parent;

	/*--- Create the dialog 'Load Project...' ---*/
	wgt_filesel = gtk_file_selection_new( "Load Project" );
	gtk_window_set_position( GTK_WINDOW(wgt_filesel), GTK_WIN_POS_MOUSE );
	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(wgt_filesel) );
	gtk_window_set_modal( GTK_WINDOW( wgt_filesel ), TRUE );

	/*--- Hide file list ---*/
	parent = gtk_widget_get_ancestor( GTK_WIDGET(GTK_FILE_SELECTION(wgt_filesel)->file_list), 
		GTK_TYPE_SCROLLED_WINDOW );
	gtk_widget_hide( parent ),

	/*--- 'Ok' button signal ---*/
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION(wgt_filesel)->ok_button ),
		"clicked", GTK_SIGNAL_FUNC( cb_filesel_ok ), (gpointer)wgt_filesel );

	/*--- 'Cancel' button signal ---*/
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION(wgt_filesel)->cancel_button ),
		"clicked", GTK_SIGNAL_FUNC( cb_filesel_cancel ), (gpointer)wgt_filesel );

	/*--- Ensure that the dialog box is destroyed when the user clicks a button. ---*/
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION(wgt_filesel)->cancel_button ),
		"clicked", GTK_SIGNAL_FUNC( gtk_widget_destroy ), (gpointer)wgt_filesel );

	gtk_signal_connect_after( GTK_OBJECT( GTK_FILE_SELECTION(wgt_filesel)->dir_list ), "button_release_event",
		GTK_SIGNAL_FUNC( cb_filesel_button_release_event ), (gpointer)wgt_filesel );
	gtk_widget_set( GTK_WIDGET( GTK_FILE_SELECTION(wgt_filesel)->ok_button ), 
		"sensitive", FALSE, NULL );

	/*--- Current project directory ---*/
	if ( strcmp( project_dirname, "" ) )
		gtk_file_selection_set_filename( GTK_FILE_SELECTION(wgt_filesel), project_dirname );

	/*--- Show the file selection dialog ---*/
	gtk_clist_set_selection_mode( GTK_CLIST( GTK_FILE_SELECTION(wgt_filesel)->dir_list ), 
		GTK_SELECTION_BROWSE );
	gtk_widget_show( wgt_filesel );
	gdk_window_show( wgt_filesel->window );

	/*--- Wait until user selects either 'yes' or 'no' ---*/
	push_block_all_windows();
	while ( wgt_filesel )
		gtk_main_iteration();
	pop_block_all_windows();

	/*--- Add this project to the list of the last projects ---*/
	add_to_last_projects( project_dirname );
	
}								// load_project


PUBLIC void
add_to_last_projects( char *new_prj )
{
	int		n;
	int		found;

	/*--- Already in there ? ---*/
	for ( found = FALSE, n = 0; n < MAXNB_LAST_PROJECTS; n++ )
	{
		if ( !last_projects[ n ] )
			break;
		if ( !strcmp( last_projects[ n ], new_prj ) )
		{
			found = TRUE;
			break;
		}
	}
	if ( found )
		return;

	/*--- Enought space ? ---*/
	if ( n < MAXNB_LAST_PROJECTS )
	{
		last_projects[ n ] = strdup( new_prj );
		return;
	}

}								// add_to_last_projects

