#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


enum
{
	TYPE_MENU_ITEM = 1,
	TYPE_TOGGLE_ITEM,
	TYPE_RADIO_ITEM,
	TYPE_SEPARATOR_ITEM,
	TYPE_SUBMENU_ITEM,
};

typedef struct
{
	int			type;
	union 
	{
		struct 
		{
			GtkWidget		*wgt_menu_item;
			GtkWidget		*wgt_accel_label;
		}		menu_item;
		struct 
		{
			GtkWidget		*wgt_check_menu_item;
			GtkWidget		*wgt_accel_label;
		}		toggle_item;
		struct 
		{
			GtkWidget		*wgt_radio_menu_item;
			GtkWidget		*wgt_accel_label;
		}		radio_item;
		struct 
		{
			GtkWidget		*wgt_menu_item;
		}		separator_item;
		struct 
		{
			GtkWidget		*wgt_menu_item;
			GtkWidget		*wgt_accel_label;
			GtkWidget		*wgt_menu;
		}		submenu_item;
	}			u;
}	data_item_t;

/*--- Data attached to widgets ---*/
#define _WGTDATA_TREELINK			"tree_link"


static int
which_type( module_menu_editor_t *dlg )
{

	if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_menuitem ) ) )
		return TYPE_MENU_ITEM;
	else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_toggleitem ) ) )
		return TYPE_TOGGLE_ITEM;
	else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_radioitem ) ) )
		return TYPE_RADIO_ITEM;
	else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_separator ) ) )
		return TYPE_SEPARATOR_ITEM;
	else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_submenu ) ) )
		return TYPE_SUBMENU_ITEM;
	else assert( 0 );

	return -1;

}								// which_type


static GtkCTreeNode * 
which_selected( module_menu_editor_t *dlg )
{
	GtkCList	*clist = GTK_CLIST( dlg->wgt_menued_list_items );
	GList		*first;

	first =  g_list_first( clist->selection );
	if ( first == NULL )
		return NULL;
	else
		return (GtkCTreeNode *) first->data;

}								// which_selected


typedef struct
{
	int				level;
	int				*cpt;
	module_info_t	*module;
}	TREE_DATA;

static void
forall_walk_menu( GtkWidget *widget, TREE_DATA *data )
{
	GtkCTreeNode			*wgt_node;
	GtkCTreeNode			*wgt_node_parent;
	GtkCTreeNode			*wgt_node_sibling;
	char					*text[3];
	module_menu_editor_t	*dlg = &data->module->u1.menu.menued;
	GList					*children;
	GtkWidget				*child=NULL;
	GtkArg					arg;
	char					*label;
	data_item_t				*dataitem=NULL;
	GtkArg					args[1];
	GtkWidget				*wgt_parent;
	GtkWidget				*wgt_father;

	data->level++;
	(*data->cpt)++;

	args[0].name = "GtkWidget::parent";
	args[0].type = GTK_TYPE_CONTAINER;	
	gtk_widget_getv( GTK_WIDGET(widget), 1, args );
	wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );

#if 0
	trace( "%2d: %2d %p=%-18s parent=%p\n", 
		*data->cpt, data->level, 
		widget, gtk_widget_get_name( widget ),
		wgt_parent );
#endif

	text[2] = "None"; 
	if ( GTK_IS_MENU_ITEM( widget ) )
	{

		dataitem = malloc( sizeof( data_item_t ) );
		memset( dataitem, 0, sizeof( data_item_t ) );
		dataitem->type = TYPE_MENU_ITEM;
		dataitem->u.menu_item.wgt_menu_item = widget;

		children = gtk_container_children( GTK_CONTAINER (widget ) );
		text[1] = "Menu Item"; 
		if ( children && children->data )
		{
			child = children->data;
			arg.name = "GtkLabel::label";
			gtk_widget_get( GTK_WIDGET( child ), &arg );
			label = GTK_VALUE_STRING( arg );
			dataitem->u.toggle_item.wgt_accel_label = child;
		}
		else
		{
			label = "";
		}

		/*--- Search the father ---*/
		assert( GTK_IS_MENU( wgt_parent ) );
		wgt_father = GTK_WIDGET( gtk_menu_get_attach_widget( GTK_MENU( wgt_parent ) ) );
		wgt_node_parent = get_object_data( wgt_father, _WGTDATA_TREELINK );

		wgt_node_sibling = NULL;

		text[0] = label;
		wgt_node = gtk_ctree_insert_node( GTK_CTREE( dlg->wgt_menued_list_items ), 
			wgt_node_parent, 
			wgt_node_sibling,
			text, 5,
			NULL, NULL,
			NULL, NULL,
			FALSE, FALSE );

		/*--- Attach some data to this object ---*/
		gtk_ctree_node_set_row_data_full( GTK_CTREE( dlg->wgt_menued_list_items ), 
			GTK_CTREE_NODE( wgt_node ), dataitem,
			(GtkDestroyNotify) destroy_notify );

		/*--- Link the widget to the tree ---*/
		set_object_data( widget, _WGTDATA_TREELINK, wgt_node, FALSE );
		if ( child )
			set_object_data( child, _WGTDATA_TREELINK, wgt_node, FALSE );

	}							// if ( GTK_IS_MENU_ITEM( widget ) )

	else if ( GTK_IS_SEPARATOR( widget ) )
	{
	
		dataitem->type = TYPE_SEPARATOR_ITEM;

		wgt_node_parent  = NULL;
		wgt_node_sibling = NULL;

		text[0] = "---";
		text[1] = "Separator Item"; 
		wgt_node = gtk_ctree_insert_node( GTK_CTREE( dlg->wgt_menued_list_items ), 
			wgt_node_parent, 
			wgt_node_sibling,
			text, 5,
			NULL, NULL,
			NULL, NULL,
			FALSE, FALSE );
			
	}							// if ( GTK_IS_SEPARATOR( widget ) )

	else if ( GTK_IS_MENU( widget ) )
	{
	
		wgt_father = GTK_WIDGET( gtk_menu_get_attach_widget( GTK_MENU( widget ) ) );

		if ( wgt_father == NULL )
		{
			wgt_node_parent = NULL;
		}
		else
		{
			wgt_node_parent = get_object_data( wgt_father, _WGTDATA_TREELINK );
			assert( wgt_node_parent != NULL );
		}

		if ( data->level > 0 )
		{
		}						// if
			
	}							// if ( GTK_IS_SEPARATOR( widget ) )

	if ( GTK_IS_CONTAINER(widget) )
	{
		TREE_DATA	new_data;

		new_data.level  = data->level;
		new_data.cpt	= data->cpt;
		new_data.module	= data->module;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_walk_menu, &new_data );

	}

	data->level--;

}								// forall_walk_menu


static void
walk( module_info_t *module )
{
	TREE_DATA				data;
	int						cpt;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;

	/*--- First clear the tree ---*/
	gtk_clist_freeze( GTK_CLIST( dlg->wgt_menued_list_items ) );
	gtk_clist_clear( GTK_CLIST( dlg->wgt_menued_list_items ) );

	/*--- Then rebuild the whole tree ---*/
	data.level   = -1;
	cpt = 0;
	data.cpt     = &cpt;
	data.module  = module;
	forall_walk_menu( module->wgt_window, &data );
	gtk_clist_thaw( GTK_CLIST( dlg->wgt_menued_list_items ) );
	
}								// walk


PUBLIC void
menu_touched( module_info_t *module )
{
	if ( module->u1.menu.menued.wgt_menued_window != NULL )
		walk( module );
}								// menu_touched


static void
update_btn_add( module_menu_editor_t *dlg )
{

	if ( which_type( dlg ) == TYPE_SEPARATOR_ITEM )
	{
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_btn_add_item ), TRUE );
	}
	else
	{
		char *str = gtk_editable_get_chars( GTK_EDITABLE( dlg->wgt_menued_entry_item ), 0, -1 );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_btn_add_item ), 
			strcmp( str, "" ) ? TRUE : FALSE );
		g_free( str );
	}

}								// update_btn_add


/* Callback "clicked" for GtkButton widget wgt_menued_close */
void
cb_menued_close_clicked(GtkButton *button, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
	/*--- Close the dialog ---*/
	gtk_widget_destroy( dlg->wgt_menued_window );
	dlg->wgt_menued_window = NULL;
	
}								// cb_menued_close_clicked


/* Callback "clicked" for GtkButton widget wgt_menued_btn_testmenu */
void
cb_menued_btn_testmenu(GtkButton *button, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	GdkEvent				*event;

	event = gtk_get_current_event();
	gtk_menu_popup( GTK_MENU( module->wgt_window ), NULL, NULL, NULL, NULL,
		1, NULL );

}								// cb_menued_btn_testmenu


/* Callback "tree_select_row" for GtkCTree widget wgt_menued_list_items */
void
cb_menued_list_items_select_row(GtkCTree *ctree, GList *node, gint column, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	data_item_t				*dataitem;

	dataitem = gtk_ctree_node_get_row_data( GTK_CTREE( ctree ), GTK_CTREE_NODE( node ) );
	assert( dataitem != NULL );
	
	switch ( dataitem->type )
	{
		case TYPE_MENU_ITEM :
			select_widget( dataitem->u.menu_item.wgt_menu_item, TRUE, FALSE ); 
			break;
		case TYPE_TOGGLE_ITEM :
			select_widget( dataitem->u.toggle_item.wgt_check_menu_item, TRUE, FALSE ); 
			break;
		case TYPE_RADIO_ITEM :
			select_widget( dataitem->u.radio_item.wgt_radio_menu_item, TRUE, FALSE ); 
			break;
		case TYPE_SEPARATOR_ITEM :
			select_widget( dataitem->u.separator_item.wgt_menu_item, TRUE, FALSE ); 
			break;
		case TYPE_SUBMENU_ITEM :
			select_widget( dataitem->u.submenu_item.wgt_menu_item, TRUE, FALSE ); 
			break;
	}							// switch

}								// cb_menued_list_items_select_row

/* Callback "clicked" for GtkButton widget wgt_menued_btn_move_up */
void
cb_menued_btn_move_up(GtkButton *button, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
}								// cb_menued_btn_move_up

/* Callback "clicked" for GtkButton widget wgt_menued_btn_move_down */
void
cb_menued_btn_move_down(GtkButton *button, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
}								// cb_menued_btn_move_down

/* Callback "clicked" for GtkButton widget wgt_menued_btn_move_left */
void
cb_menued_btn_move_left(GtkButton *button, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
}								// cb_menued_btn_move_left

/* Callback "clicked" for GtkButton widget wgt_menued_btn_move_right */
void
cb_menued_btn_move_right(GtkButton *button, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
}								// cb_menued_btn_move_right

/* Callback "changed" for GtkEntry widget wgt_menued_entry_item */
void
cb_menued_entry_item_changed(GtkEditable *editable, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;

	update_btn_add( dlg );
	
}								// cb_menued_entry_item_changed


/* Callback "toggled" for GtkCheckButton widget wgt_menued_checkbtn_type_menuitem */
void
cb_menued_checkbtn_type_menuitem_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;

	if ( gtk_toggle_button_get_active( togglebutton ) )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_toggleitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_radioitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_separator ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_submenu ), FALSE );
		
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_label_item ), TRUE );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_entry_item ), TRUE );
		
		update_btn_add( dlg );
	}							// if		
	
}								// cb_menued_checkbtn_type_menuitem_toggled


/* Callback "toggled" for GtkCheckButton widget wgt_menued_checkbtn_type_toggleitem */
void
cb_menued_checkbtn_type_toggleitem_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;

	if ( gtk_toggle_button_get_active( togglebutton ) )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_menuitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_radioitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_separator ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_submenu ), FALSE );
		
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_label_item ), TRUE );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_entry_item ), TRUE );
		
		update_btn_add( dlg );
	}							// if		
	
}								// cb_menued_checkbtn_type_toggleitem_toggled

/* Callback "toggled" for GtkCheckButton widget wgt_menued_checkbtn_type_radioitem */
void
cb_menued_checkbtn_type_radiobtn_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
	if ( gtk_toggle_button_get_active( togglebutton ) )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_menuitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_toggleitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_separator ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_submenu ), FALSE );
		
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_label_item ), TRUE );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_entry_item ), TRUE );
		
		update_btn_add( dlg );
	}							// if		
	
}								// cb_menued_checkbtn_type_radiobtn_toggled


/* Callback "toggled" for GtkCheckButton widget wgt_menued_checkbtn_type_separator */
void
cb_menued_checkbtn_type_separator_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
	if ( gtk_toggle_button_get_active( togglebutton ) )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_menuitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_toggleitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_radioitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_submenu ), FALSE );
		
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_label_item ), FALSE );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_entry_item ), FALSE );
		
		update_btn_add( dlg );
	}							// if		
	
}								// cb_menued_checkbtn_type_separator_toggled


/* Callback "toggled" for GtkCheckButton widget wgt_menued_checkbtn_type_submenu */
void
cb_menued_checkbtn_type_submenu_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
	if ( gtk_toggle_button_get_active( togglebutton ) )
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_menuitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_toggleitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_radioitem ), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dlg->wgt_menued_checkbtn_type_separator ), FALSE );
		
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_label_item ), TRUE );
		gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_entry_item ), TRUE );
		
		update_btn_add( dlg );
	}							// if		
	
}								// cb_menued_checkbtn_type_submenu_toggled


static void
build_menu_item( module_info_t *module, GtkWidget *menu, data_item_t *dataitem, char *str )
{
	gtk_wgtdef_t			*wgtdef;

	wgtdef = search_wgtdef( "GtkMenuItem" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", menu );
	dataitem->u.menu_item.wgt_menu_item = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_widget_show( dataitem->u.menu_item.wgt_menu_item );

	wgtdef = search_wgtdef( "GtkAccelLabel" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", 
		dataitem->u.menu_item.wgt_menu_item );
	SETARG_STRING( "GtkLabel::label", str );
	dataitem->u.menu_item.wgt_accel_label  = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	wgt_put_flag( dataitem->u.menu_item.wgt_accel_label, WGTFLAG_NOCOMPOSITE );
	INIT_ARGS;
	SETARG_STRING( "GtkLabel::label", str );
	gtk_widget_setv( GTK_WIDGET(dataitem->u.menu_item.wgt_accel_label), nb_args, args );
	gtk_widget_show( dataitem->u.menu_item.wgt_accel_label );

	update_tree();

	select_widget( dataitem->u.menu_item.wgt_menu_item, FALSE, FALSE ); 

}								// build_menu_item


static void
build_toggle_item( module_info_t *module, GtkWidget *menu, data_item_t *dataitem, char *str )
{
	gtk_wgtdef_t			*wgtdef;

	wgtdef = search_wgtdef( "GtkCheckMenuItem" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", menu );
	dataitem->u.toggle_item.wgt_check_menu_item = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_widget_show( dataitem->u.toggle_item.wgt_check_menu_item );

	wgtdef = search_wgtdef( "GtkAccelLabel" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent",
		dataitem->u.toggle_item.wgt_check_menu_item );
	SETARG_STRING( "GtkLabel::label", str );
	dataitem->u.toggle_item.wgt_accel_label  = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
			module );
	wgt_put_flag( dataitem->u.toggle_item.wgt_accel_label, WGTFLAG_NOCOMPOSITE );
	INIT_ARGS;
	SETARG_STRING( "GtkLabel::label", str );
	gtk_widget_setv( GTK_WIDGET(dataitem->u.toggle_item.wgt_accel_label), nb_args, args );
	gtk_widget_show( dataitem->u.toggle_item.wgt_accel_label );

	update_tree();

	select_widget( dataitem->u.toggle_item.wgt_check_menu_item, FALSE, FALSE ); 

}								// build_toggle_item


static void
build_radio_item( module_info_t *module, GtkWidget *menu, data_item_t *dataitem, char *str )
{
	gtk_wgtdef_t			*wgtdef;

	wgtdef = search_wgtdef( "GtkRadioMenuItem" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", menu );
	dataitem->u.radio_item.wgt_radio_menu_item = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_widget_show( dataitem->u.radio_item.wgt_radio_menu_item );

	wgtdef = search_wgtdef( "GtkAccelLabel" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent",
		dataitem->u.radio_item.wgt_radio_menu_item );
	SETARG_STRING( "GtkLabel::label", str );
	dataitem->u.radio_item.wgt_accel_label  = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
			module );
	wgt_put_flag( dataitem->u.radio_item.wgt_accel_label, WGTFLAG_NOCOMPOSITE );
	INIT_ARGS;
	SETARG_STRING( "GtkLabel::label", str );
	gtk_widget_setv( GTK_WIDGET(dataitem->u.radio_item.wgt_accel_label), nb_args, args );
	gtk_widget_show( dataitem->u.radio_item.wgt_accel_label );

	update_tree();

	select_widget( dataitem->u.radio_item.wgt_radio_menu_item, FALSE, FALSE ); 

}								// build_radio_item


static void
build_separator_item( module_info_t *module, GtkWidget *menu, data_item_t *dataitem )
{
	gtk_wgtdef_t			*wgtdef;

	wgtdef = search_wgtdef( "GtkMenuItem" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", menu );
	dataitem->u.separator_item.wgt_menu_item = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_widget_show( dataitem->u.separator_item.wgt_menu_item );

	update_tree();

	select_widget( dataitem->u.separator_item.wgt_menu_item, FALSE, FALSE ); 

}								// build_separator_item


static void
build_submenu_item( module_info_t *module, GtkWidget *menu, data_item_t *dataitem, char *str )
{
	gtk_wgtdef_t			*wgtdef;

	wgtdef = search_wgtdef( "GtkMenuItem" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", menu );
	dataitem->u.submenu_item.wgt_menu_item = insert_new_widget( 
		NULL, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_widget_show( dataitem->u.submenu_item.wgt_menu_item );

	wgtdef = search_wgtdef( "GtkAccelLabel" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", 
		dataitem->u.submenu_item.wgt_menu_item );
	SETARG_STRING( "GtkLabel::label", str );
	dataitem->u.submenu_item.wgt_accel_label  = insert_new_widget( 
		dataitem->u.submenu_item.wgt_menu_item, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	wgt_put_flag( dataitem->u.submenu_item.wgt_accel_label, WGTFLAG_NOCOMPOSITE );
	gtk_widget_show( dataitem->u.submenu_item.wgt_accel_label );

	wgtdef = search_wgtdef( "GtkMenu" );
	assert( wgtdef != NULL );
	INIT_ARGS;
	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", 
		dataitem->u.submenu_item.wgt_menu_item );
	dataitem->u.submenu_item.wgt_menu = insert_new_widget( 
		dataitem->u.submenu_item.wgt_menu_item, wgtdef, 
		FALSE, 			// f_select
		TRUE,			// is_disabled
		TRUE,			// Default values
		module );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( dataitem->u.submenu_item.wgt_menu_item),
		dataitem->u.submenu_item.wgt_menu );
	wgt_put_flag( dataitem->u.submenu_item.wgt_menu, WGTFLAG_NOCOMPOSITE );
	gtk_widget_show( dataitem->u.submenu_item.wgt_menu );

	update_tree();

	select_widget( dataitem->u.menu_item.wgt_menu_item, FALSE, FALSE ); 

}								// build_submenu_item


/* Callback "clicked" for GtkButton widget wgt_menued_btn_add_item */
void
cb_menued_btn_add_item_clicked(GtkButton *button, gpointer user_data)
{
	module_info_t			*module = (module_info_t *)user_data;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	char					*str;
	GtkCTreeNode			*wgt_node;
	GtkCTreeNode			*wgt_node_parent;
	GtkCTreeNode			*wgt_node_sibling;
	char					*text[3];
	GtkCTreeNode			*curr, *brother;
	GtkCTreeRow				*row_curr, *row_brother;
	int						type;
	data_item_t				*dataitem=NULL;
	data_item_t				*dataitem_curr;
	GtkWidget				*menu=NULL;
	
	str = gtk_editable_get_chars( GTK_EDITABLE( dlg->wgt_menued_entry_item ), 0, -1 );
	type = which_type( dlg );

	curr = which_selected( dlg );
	if ( curr )
	{
		dataitem_curr = gtk_ctree_node_get_row_data( GTK_CTREE( dlg->wgt_menued_list_items ),
			GTK_CTREE_NODE( curr ) );
		assert( dataitem_curr != NULL );
		switch ( dataitem_curr->type )
		{
			case TYPE_MENU_ITEM 		:	
				menu = gtk_widget_get_ancestor( dataitem_curr->u.menu_item.wgt_menu_item, 
					GTK_TYPE_MENU );
				break;
			case TYPE_TOGGLE_ITEM 		:
				break;
			case TYPE_RADIO_ITEM 		:
				break;
			case TYPE_SEPARATOR_ITEM 	:
				break;
			case TYPE_SUBMENU_ITEM 		:
				menu = dataitem_curr->u.submenu_item.wgt_menu;
				break;
		}							// switch
	}
	else
	{
		menu = module->wgt_window;
	}

	if ( type == TYPE_SUBMENU_ITEM )
	{
		wgt_node_parent = curr;
		wgt_node_sibling = NULL;
	}
	else
	{
		if ( curr == NULL )
		{
			wgt_node_parent  = NULL;
			wgt_node_sibling = NULL;
		}
		else
		{
			row_curr = GTK_CTREE_ROW( curr );
			wgt_node_parent = row_curr->parent;
			brother = GTK_CTREE_NODE_NEXT( curr );
			if ( brother == NULL )
			{
				wgt_node_sibling = NULL;
			}
			else
			{
				row_brother = GTK_CTREE_ROW( brother );
				if ( row_curr->level == row_brother->level )
					wgt_node_sibling = brother;
				else
					wgt_node_sibling = NULL;
			}
		}
	}

	switch ( type )
	{
		case TYPE_MENU_ITEM 		:	
			text[0] = str;
			text[1] = "Menu Item"; 
			break;
		case TYPE_TOGGLE_ITEM 		:
			text[0] = str;
			text[1] = "Toggle Item"; 
			break;
		case TYPE_RADIO_ITEM 		:
			text[0] = str;
			text[1] = "Radio Item"; 
			break;
		case TYPE_SEPARATOR_ITEM 	:
			text[0] = "---";
			text[1] = "Separator"; 
			break;
		case TYPE_SUBMENU_ITEM 		:
			text[0] = str;
			text[1] = "Sub Menu";
			break;
	}							// switch
	text[2] = "none";
	wgt_node = gtk_ctree_insert_node( GTK_CTREE( dlg->wgt_menued_list_items ), 
		wgt_node_parent, 
		wgt_node_sibling,
		text, 5,
		NULL, NULL,
		NULL, NULL,
		FALSE, FALSE );

	/*--- Update now the menu ---*/
	dataitem = malloc( sizeof( data_item_t ) );
	memset( dataitem, 0, sizeof( data_item_t ) );
	dataitem->type = type;
	switch ( type )
	{
		case TYPE_MENU_ITEM :
			build_menu_item( module, menu, dataitem, str );
			break;
		case TYPE_TOGGLE_ITEM :
			build_toggle_item( module, menu, dataitem, str );
			break;
		case TYPE_RADIO_ITEM :
			build_radio_item( module, menu, dataitem, str );
			break;
		case TYPE_SEPARATOR_ITEM :
			build_separator_item( module, menu, dataitem );
			break;
		case TYPE_SUBMENU_ITEM :
			build_submenu_item( module, menu, dataitem, str );
			break;
	}							// switch

	/*--- Attach some data to this object ---*/
	gtk_ctree_node_set_row_data_full( GTK_CTREE( dlg->wgt_menued_list_items ), 
		GTK_CTREE_NODE( wgt_node ), dataitem,
		(GtkDestroyNotify) destroy_notify );

	g_free( str );
	gtk_widget_set_sensitive( GTK_WIDGET( dlg->wgt_menued_btn_add_item ), FALSE );

}								// cb_menued_btn_add_item_clicked


/* Callback "clicked" for GtkButton widget wgt_menued_btn_delete_item */
void
cb_menuued_btn_delete_item_clicked(GtkButton *button, gpointer user_data)
{
//	module_info_t			*module = (module_info_t *)user_data;
//	module_menu_editor_t	*dlg = &module->u1.menu.menued;
	
}								// cb_menuued_btn_delete_item_clicked


static GtkWidget *
create_window( char *title, module_info_t *module )
{
	GtkWidget				*wgt_1_2, *wgt_2_4, *wgt_2_9, *wgt_3_10, 
							*wgt_4_11, *wgt_4_14, *wgt_5_15, *wgt_5_16, *wgt_4_22, 
							*wgt_2_25, *wgt_2_26;
	module_menu_editor_t	*dlg = &module->u1.menu.menued;

	dlg->wgt_menued_window = gtk_widget_new( gtk_window_get_type(),
		"GtkWidget::width", 570,
		"GtkWidget::height", 509,
		"GtkWidget::events", 0x0000030C,
		"GtkContainer::resize_mode", GTK_RESIZE_QUEUE,
		"GtkWindow::title", title,
		"GtkWindow::allow_grow", TRUE,
		"GtkWindow::default_width", 400,
		"GtkWindow::default_height", 300,
		NULL );

	wgt_1_2 = gtk_widget_new( gtk_vbox_get_type(),
		"GtkWidget::parent", dlg->wgt_menued_window,
		NULL );

	dlg->wgt_menued_btn_testmenu = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Test Menu",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), dlg->wgt_menued_btn_testmenu,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 0,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_testmenu ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_testmenu ), module );

#if 0
	dlg->wgt_menued_list_items = gtk_widget_new( gtk_ctree_get_type(),
		"GtkCTree::n_columns", 3,
		"GtkCTree::tree_column", 0,
		"GtkWidget::parent", wgt_1_2,
		"GtkWidget::can_focus", TRUE,
		NULL );
#else
	dlg->wgt_menued_list_items = gtk_ctree_new_with_titles( 3, 0, NULL );
	gtk_widget_set( dlg->wgt_menued_list_items, "GtkWidget::parent", wgt_1_2, NULL );
#endif
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), dlg->wgt_menued_list_items,
		"GtkBox::position", 1,
		NULL );
	gtk_clist_column_titles_show(GTK_CLIST(dlg->wgt_menued_list_items));
	gtk_clist_set_column_title( GTK_CLIST( dlg->wgt_menued_list_items ), 0, "Label" );
	gtk_clist_set_column_title( GTK_CLIST( dlg->wgt_menued_list_items ), 1, "Type" );
	gtk_clist_set_column_title( GTK_CLIST( dlg->wgt_menued_list_items ), 2, "Accelerator" );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_list_items ), "tree_select_row",
		GTK_SIGNAL_FUNC( cb_menued_list_items_select_row ), module );

	wgt_2_4 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkContainer::border_width", 2,
		"GtkBox::spacing", 4,
		"GtkBox::homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_4,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 2,
		NULL );

	dlg->wgt_menued_btn_move_up = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_4,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Move Up",
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_move_up ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_move_up ), module );

	dlg->wgt_menued_btn_move_down = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_4,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Move Down",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_4 ), dlg->wgt_menued_btn_move_down,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_move_down ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_move_down ), module );

	dlg->wgt_menued_btn_move_left = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_4,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Move Left",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_4 ), dlg->wgt_menued_btn_move_left,
		"GtkBox::position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_move_left ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_move_left ), module );

	dlg->wgt_menued_btn_move_right = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_4,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Move Right",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_4 ), dlg->wgt_menued_btn_move_right,
		"GtkBox::position", 3,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_move_right ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_move_right ), module );

	wgt_2_9 = gtk_widget_new( gtk_frame_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkContainer::border_width", 8,
		"GtkFrame::label", "Item",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_9,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 3,
		NULL );

	wgt_3_10 = gtk_widget_new( gtk_vbox_get_type(),
		"GtkWidget::parent", wgt_2_9,
		NULL );

	wgt_4_11 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_3_10,
		"GtkContainer::border_width", 2,
		NULL );

	dlg->wgt_menued_label_item = gtk_widget_new( gtk_label_get_type(),
		"GtkWidget::parent", wgt_4_11,
		"GtkWidget::width", 42,
		"GtkLabel::label", "Label:",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_11 ), dlg->wgt_menued_label_item,
		"GtkBox::expand", FALSE,
		NULL );

	dlg->wgt_menued_entry_item = gtk_widget_new( gtk_entry_get_type(),
		"GtkWidget::parent", wgt_4_11,
		"GtkWidget::can_focus", TRUE,
		NULL );
	gtk_entry_set_text( GTK_ENTRY( dlg->wgt_menued_entry_item ), "" );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_11 ), dlg->wgt_menued_entry_item,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_entry_item ), "changed",
		GTK_SIGNAL_FUNC( cb_menued_entry_item_changed ), module );

	wgt_4_14 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_3_10,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_3_10 ), wgt_4_14,
		"GtkBox::position", 1,
		NULL );

	wgt_5_15 = gtk_widget_new( gtk_label_get_type(),
		"GtkWidget::parent", wgt_4_14,
		"GtkWidget::width", 42,
		"GtkLabel::label", "Type:",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_14 ), wgt_5_15,
		"GtkBox::expand", FALSE,
		NULL );

	wgt_5_16 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_4_14,
		"GtkBox::spacing", 4,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_14 ), wgt_5_16,
		"GtkBox::position", 1,
		NULL );

	dlg->wgt_menued_checkbtn_type_menuitem = gtk_widget_new( gtk_check_button_get_type(),
		"GtkWidget::parent", wgt_5_16,
		"GtkWidget::can_focus", TRUE,
		"GtkButton::label", "Menu Item",
		"GtkToggleButton::active", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_16 ), dlg->wgt_menued_checkbtn_type_menuitem,
		"GtkBox::expand", FALSE,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_checkbtn_type_menuitem ), "toggled",
		GTK_SIGNAL_FUNC( cb_menued_checkbtn_type_menuitem_toggled ), module );

	dlg->wgt_menued_checkbtn_type_toggleitem = gtk_widget_new( gtk_check_button_get_type(),
		"GtkWidget::parent", wgt_5_16,
		"GtkWidget::can_focus", TRUE,
		"GtkButton::label", "Toggle Item",
		"GtkToggleButton::active", FALSE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_16 ), dlg->wgt_menued_checkbtn_type_toggleitem,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_checkbtn_type_toggleitem ), "toggled",
		GTK_SIGNAL_FUNC( cb_menued_checkbtn_type_toggleitem_toggled ), module );

	dlg->wgt_menued_checkbtn_type_radioitem = gtk_widget_new( gtk_check_button_get_type(),
		"GtkWidget::parent", wgt_5_16,
		"GtkWidget::can_focus", TRUE,
		"GtkButton::label", "Radio Item",
		"GtkToggleButton::active", FALSE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_16 ), dlg->wgt_menued_checkbtn_type_radioitem,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 2,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_checkbtn_type_radioitem ), "toggled",
		GTK_SIGNAL_FUNC( cb_menued_checkbtn_type_radiobtn_toggled ), module );

	dlg->wgt_menued_checkbtn_type_separator = gtk_widget_new( gtk_check_button_get_type(),
		"GtkWidget::parent", wgt_5_16,
		"GtkWidget::can_focus", TRUE,
		"GtkButton::label", "Separator",
		"GtkToggleButton::active", FALSE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_16 ), dlg->wgt_menued_checkbtn_type_separator,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 3,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_checkbtn_type_separator ), "toggled",
		GTK_SIGNAL_FUNC( cb_menued_checkbtn_type_separator_toggled ), module );

	dlg->wgt_menued_checkbtn_type_submenu = gtk_widget_new( gtk_check_button_get_type(),
		"GtkWidget::parent", wgt_5_16,
		"GtkWidget::can_focus", TRUE,
		"GtkButton::label", "Sub Menu",
		"GtkToggleButton::active", FALSE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_5_16 ), dlg->wgt_menued_checkbtn_type_submenu,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 4,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_checkbtn_type_submenu ), "toggled",
		GTK_SIGNAL_FUNC( cb_menued_checkbtn_type_submenu_toggled ), module );

	wgt_4_22 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_3_10,
		"GtkContainer::border_width", 4,
		"GtkBox::homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_3_10 ), wgt_4_22,
		"GtkBox::position", 2,
		NULL );

	dlg->wgt_menued_btn_add_item = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_4_22,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Add",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_22 ), dlg->wgt_menued_btn_add_item,
		"GtkBox::padding", 12,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_add_item ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_btn_add_item_clicked ), module );

	dlg->wgt_menued_btn_delete_item = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_4_22,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Delete",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_4_22 ), dlg->wgt_menued_btn_delete_item,
		"GtkBox::padding", 12,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_btn_delete_item ), "clicked",
		GTK_SIGNAL_FUNC( cb_menuued_btn_delete_item_clicked ), module );

	wgt_2_25 = gtk_widget_new( gtk_hseparator_get_type(),
		"GtkWidget::parent", wgt_1_2,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_25,
		"GtkBox::expand", FALSE,
		"GtkBox::padding", 4,
		"GtkBox::position", 4,
		NULL );

	wgt_2_26 = gtk_widget_new( gtk_hbox_get_type(),
		"GtkWidget::parent", wgt_1_2,
		"GtkContainer::border_width", 4,
		"GtkBox::spacing", 8,
		"GtkBox::homogeneous", TRUE,
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_1_2 ), wgt_2_26,
		"GtkBox::expand", FALSE,
		"GtkBox::position", 5,
		NULL );

	dlg->wgt_menued_close = gtk_widget_new( gtk_button_get_type(),
		"GtkWidget::parent", wgt_2_26,
		"GtkWidget::can_focus", TRUE,
		"GtkWidget::receives_default", TRUE,
		"GtkButton::label", "Close",
		NULL );
	gtk_container_child_set( GTK_CONTAINER( wgt_2_26 ), dlg->wgt_menued_close,
		"GtkBox::position", 1,
		NULL );
	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_close ), "clicked",
		GTK_SIGNAL_FUNC( cb_menued_close_clicked ), module );

	gtk_widget_show_all( dlg->wgt_menued_window );

	gtk_widget_set_sensitive( dlg->wgt_menued_btn_move_up,		FALSE );
	gtk_widget_set_sensitive( dlg->wgt_menued_btn_move_down,	FALSE );
	gtk_widget_set_sensitive( dlg->wgt_menued_btn_move_left, 	FALSE );
	gtk_widget_set_sensitive( dlg->wgt_menued_btn_move_right, 	FALSE );

	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_window ), "focus_in_event",
		GTK_SIGNAL_FUNC( cb_window_focus_in ), module );

	gtk_signal_connect( GTK_OBJECT( dlg->wgt_menued_window ), "destroy",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );
	gtk_signal_connect(GTK_OBJECT( dlg->wgt_menued_window ), "delete_event",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );

	return dlg->wgt_menued_window;

}								// create_window


PUBLIC void
menu_editor( module_info_t *module )
{

	/*--- Create the dialog ---*/
	if ( module->u1.menu.menued.wgt_menued_window == NULL )
	{
		create_window( "", module );
	}
	else
	{
		gdk_window_raise( module->u1.menu.menued.wgt_menued_window->window );
		gtk_window_set_focus( GTK_WINDOW( module->u1.menu.menued.wgt_menued_window ),
			module->u1.menu.menued.wgt_menued_btn_testmenu );
	}
	
	/*--- Populate the dialog with all menu items ---*/
	walk( module );

}								// menu_editor
