#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


static gint
cb_module_button_press_event( GtkWidget *window, GdkEventButton *event, gpointer user_data );


PUBLIC int
nb_modules( void )
{
	GtkCList	*clist;
	guint		nb_children;

	if ( !wgt_tree_modules )
		return 0;

	clist = GTK_CLIST( wgt_tree_modules );
	nb_children = clist->rows;
	return nb_children;
	
}								// nb_modules


PUBLIC module_info_t *
next_module( int op )
{
	static int		row = 0;
	GtkCTreeNode	*node;
	module_info_t	*module;
	
	if ( op == FIRST )
		row = 0;
	node = gtk_ctree_node_nth( GTK_CTREE(wgt_tree_modules), row++ );

	if ( node )
	{
		module = gtk_ctree_node_get_row_data( GTK_CTREE(wgt_tree_modules), 
			GTK_CTREE_NODE(node) );
		assert( module != NULL );
		return module;
	}
	else
	{
		return NULL;
	}
	
}								// next_module


PUBLIC void
set_data_wgtdef( GtkWidget *widget, GtkWidget *wgt_parent,
	module_info_t *module, 
	int is_placeholder, 
	int is_composite,
	int is_disabled )
{
	gtk_wgtdef_t	*wgtdef;
	char			*widget_name;
	int				*pflag;
	int				depth;
	defclass_t		*classes=NULL, *p;
	int				nb_res=0, n, num;

	widget_name = gtk_widget_get_name( GTK_WIDGET( widget ) );	
	wgtdef = search_wgtdef( widget_name );
	assert( wgtdef != NULL );
	
	set_object_data( widget, WGTDATA_WGTDEF, wgtdef, FALSE );
	set_object_data( widget, WGTDATA_MODULE, module, FALSE );

	if ( !GTK_WIDGET_NO_WINDOW(widget) )
	{
		if ( !GTK_WIDGET_REALIZED(widget) )
		{
			gtk_widget_set_events( widget, 
				gtk_widget_get_events(widget) | GDK_BUTTON_PRESS_MASK  | GDK_POINTER_MOTION_MASK);
		}
		else
		{
			GdkEventMask event_mask = gdk_window_get_events (widget->window);
			gdk_window_set_events( widget->window,
				event_mask | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK );
		}
	}

	if ( module->type != MODULE_TYPE_MENU )
	{
		gtk_signal_connect( GTK_OBJECT( widget ), "button_press_event",
			GTK_SIGNAL_FUNC( cb_module_button_press_event ), widget );
	}

	set_nametype_wgt( widget, NAMETYPE_ANONYMOUS );
	set_name_wgt( widget, "" );

	for ( num = 0; num < 2; num++ )
	{
		gtk_wgtdef_t	*wgtdef = NULL;
		int				f_disabled;
		gtk_wgtdef_t	*wgtdef_parent;

		if ( num == 0 )
		{
			wgtdef = gtk_object_get_data( GTK_OBJECT(widget), WGTDATA_WGTDEF );
			assert( wgtdef != NULL );
		}
		else if ( num == 1 )
		{
			if ( wgt_parent == NULL )
				break;
			wgtdef = gtk_object_get_data( GTK_OBJECT( wgt_parent ), WGTDATA_WGTDEF );
			if ( wgtdef == NULL )
				continue;
#if 0
			if (wgt_parent == NULL)
				wgtdef = NULL;
			else
				wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
			if (wgtdef == NULL)
				continue;
#endif
		}

		if (wgt_parent)
		{
			wgtdef_parent = gtk_object_get_data( GTK_OBJECT(wgt_parent), WGTDATA_WGTDEF );
			assert( wgtdef_parent != NULL );
		}
		else
		{
			wgtdef_parent = NULL;
		}

		for ( depth = 0; 
			  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[depth].classnames != NULL); 
			  depth++ )
		{

			/*--- Dump all resources ---*/
			if ( num == 0 )
			{
				classes = wgtdef->def[depth].classes;
				nb_res  = wgtdef->def[depth].nb_classes;
			}
			else if ( num == 1 )
			{
				classes = wgtdef->def[depth].classes_child;
				nb_res  = wgtdef->def[depth].nb_classes_child;
			}

			for ( p = classes, n = 0; n < nb_res; n++, p++ )
			{
				if ( !p->f_generate )
					continue;

				f_disabled = FALSE;
				if ( wgtdef_parent != NULL )
				{
					if ( !strcmp(p->resname, "~GtkWidget::x") || !strcmp(p->resname, "~GtkWidget::y") )
						if ( wgtdef_parent->f_children_autopos )
							f_disabled = TRUE;
					if ( !strcmp(p->resname, "~GtkWidget::width") || !strcmp(p->resname, "~GtkWidget::height") )
						if ( wgtdef_parent->f_children_autodim )
							f_disabled = TRUE;
				}
				set_is_disabled_resource_wgt( widget, p, f_disabled );
			}

		}
	}							// for (num)

	/*--- Widget flag ---*/
	pflag = get_object_data( widget, WGTDATA_FLAGS );
	if (pflag == NULL)
	{
		pflag = malloc(sizeof(int));
		*pflag = 0;
		set_object_data( widget, WGTDATA_FLAGS, pflag, TRUE );
	}
	if (!is_composite)
		*pflag |= WGTFLAG_NOCOMPOSITE;
	if ( is_placeholder )
		*pflag |= WGTFLAG_PLACEHOLDER;

	/*--- Update the tooltip for this widget ---*/	
	update_tooltip_widget( widget );
	
}								// set_data_wgtdef


PUBLIC GtkWidget *
create_new_wgt( struct gtk_wgtdef_t *wgtdef_newwgt, module_info_t *module,
	GtkWidget *wgt_parent, GtkWidget *wgt_container,
	GtkWidget * (*pfct_create_wgt)(void) )
{
	GtkArg			arg_container[ 50 ];
	int				nb=0;
	GtkWidget		*wgt_new;
	gtk_wgtdef_t	*wgtdef=NULL;
	int				depth;
	defclass_t		*classes=NULL, *p;
	int				n, nb_res=0;

	if ( wgt_container && IS_WGT_PLACEHOLDER( wgt_container ) )
	{

		nb = 0;

		wgtdef = get_object_data( GTK_WIDGET( wgt_parent ), WGTDATA_WGTDEF );
		assert( wgtdef != NULL );

		for ( depth = 0; 
			  (depth < MAXNB_HIERARCHIES) && (wgtdef->def[ depth ].classnames != NULL); 
			  depth++ )
		{
			classes = wgtdef->def[ depth ].classes_child;
			nb_res  = wgtdef->def[ depth ].nb_classes_child;
			for ( p = classes, n = 0; n < nb_res; n++, p++ )
			{
//				if ( !p->f_generate )
//					continue;
				assert( nb < sizeof( arg_container )/ sizeof(arg_container[0]) );
				arg_container[ nb++ ].name = p->resname;
			}
		}

		if ( nb )
		{
			gtk_container_child_getv( GTK_CONTAINER( wgt_parent ), 
				wgt_container, nb, arg_container );
		}

	}

	if ( pfct_create_wgt == NULL )
		wgt_new = gtk_widget_newv( wgtdef_newwgt->pfct_get_widget_type(), nb_args, args );
	else
		wgt_new = pfct_create_wgt();

	if ( wgt_container && IS_WGT_PLACEHOLDER( wgt_container ) )
	{
		if ( nb > 0 )
			gtk_container_child_setv( GTK_CONTAINER( wgt_parent ), wgt_new, 
				nb, arg_container );
	}

	set_data_wgtdef( wgt_new, wgt_parent, module, 
		FALSE, 			// Placeholder
		FALSE, 			// Composite
		FALSE );		// Disabled
	return wgt_new;

}								// create_new_wgt


static void
get_pos_wgt( GtkWidget *wgt, gint *px, gint *py )
{
	GtkArg		args[2];
	GtkWidget	*curr;
	gint		x, y;
	int			cx, cy;
	int			depth;
	int			type;

	cx = cy = 0;
	for ( curr = wgt, depth = 0; curr != NULL; depth++ )
	{	

		type = GTK_WIDGET_TYPE( curr );
		if ( (type == gtk_window_get_type()) || (type == gtk_dialog_get_type()) )
		{
			gdk_window_get_root_origin( curr->window, &x, &y );
//			gdk_window_get_position( curr->window, &x, &y );
		}
		else
		{
			x = curr->allocation.x;
			y = curr->allocation.y;
		}
		if ( (curr == wgt) || !GTK_WIDGET_NO_WINDOW( curr ) )
		{
			if ( x != -1 )
				cx += x;
			if ( y != -1 )
				cy += y;
		}
		
		args[0].name = "GtkWidget::parent";
		args[0].type = GTK_TYPE_CONTAINER;	
		gtk_widget_getv( GTK_WIDGET( curr ), 1, args );
		curr = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );
	
	}							// for (depth)
	
	*px = cx;
	*py = cy;
	
}								// get_pos_wgt


static gint
cb_module_enter_notify_event( GtkWidget *window, GdkEventCrossing *event,
	gpointer user_data )
{
	GtkWidget	*wgt_window = (GtkWidget *)user_data;
	GdkCursor	*cursor;

	cursor = gdk_cursor_new( (wgt_selected_wgt_palette == NULL ) ?
		GDK_ARROW : GDK_PLUS );
	gdk_window_set_cursor( wgt_window->window, cursor );
	gdk_cursor_destroy( cursor );

	return FALSE;				
}								// cb_module_enter_notify_event
	

typedef struct
{
	int				level;
	int				cpt;			// Cpt for anonymous variables, same level
	gint			mx, my;
	module_info_t	*module;
	GtkWidget		**wgt_hit;
}	SEARCHWGTHIT_DATA;


static void
forall_search_wgt_hit( GtkWidget *widget, SEARCHWGTHIT_DATA *data )
{
	gint			x=0, y=0, w, h;
	gint			mx, my;
	gtk_wgtdef_t	*wgtdef=NULL;
	GtkArg			args[ 1 ];
	GtkWidget		*wgt_parent;

	if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget) )
		return;
//	if ( get_object_data(widget, WGTDATA_HIERARCHY_CHILD) )
//		return;

	data->level++;
	data->cpt++;

	wgtdef = gtk_object_get_data( GTK_OBJECT( widget ), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	if ( !wgtdef->is_window )
	{
		args[ 0 ].name = "GtkWidget::parent";
		args[ 0 ].type = GTK_TYPE_CONTAINER;	
		gtk_widget_getv( GTK_WIDGET( widget ), 1, args );
		wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT( args[ 0 ] ) );
		assert( wgt_parent != NULL );

		wgtdef = gtk_object_get_data( GTK_OBJECT( wgt_parent ), WGTDATA_WGTDEF );
		assert( wgtdef != NULL );
	}
	else
	{
		wgtdef = NULL;
	}
		

	/*--- Don't save blocked container ---*/
	if ( data->level >= 0 )
	{

		get_pos_wgt( widget, &x, &y);
		w = widget->allocation.width;
		h = widget->allocation.height;

		mx = data->mx;
		my = data->my;
#if 0
	{
		static int cpt = 0;
		trace( "%3d: {%d} [%d%d%d%d] %-18s] xywh=(%4d,%-4d-%3d,%-3d) mouse=(%4d,%-4d)\n",
			++cpt, 
			((mx >= x) && (my >= y) && (mx < x+w) && (my < y+h)) ? 1 : 0,
			(mx >= x) ? 1 : 0, (my >= y) ? 1 : 0, (mx < x+w) ? 1 : 0, (my < y+h) ? 1 : 0, 
			(get_nametype_wgt( widget ) == NAMETYPE_GLOBAL) ? get_name_wgt( widget ) : gtk_widget_get_name (widget),
			x, y, w, h, 
			data->mx, data->my );
	}
#endif
		if ( (mx >= x) && (my >= y) && (mx < x+w) && (my < y+h) )
			*data->wgt_hit = widget;

	}

	if ( GTK_IS_CONTAINER (widget) )
	{
		SEARCHWGTHIT_DATA	new_data;
		
		new_data.level    = data->level;
		new_data.cpt   	  = 0;
		new_data.mx		  = data->mx;
		new_data.my		  = data->my;
		new_data.module	  = data->module;
		new_data.wgt_hit  = data->wgt_hit;

		gtk_container_forall( GTK_CONTAINER( widget ), 
			(GtkCallback) forall_search_wgt_hit, &new_data );

	}

	data->level--;

}								// forall_search_wgt_hit


static GtkWidget *
search_wgt_hit( module_info_t *module, GdkEventButton *event )
{
	gint				mx, my;
	SEARCHWGTHIT_DATA	data;
	GtkWidget			*wgt_hit;

	/*--- Mouse absolute position ---*/
	mx = (gint)event->x_root;
	my = (gint)event->y_root;

	/*--- Search for a widget sitting at position (mx, my) ---*/
	wgt_hit = NULL;
	data.level 	 = -1;
	data.cpt 	 = 0;
	data.mx 	 = mx;
	data.my 	 = my;
	data.module  = module;
	data.wgt_hit = &wgt_hit;
	forall_search_wgt_hit( module->wgt_window, &data );

	if ( wgt_hit == NULL )
	{
		fprintf( stderr, "WARNING!!! hit=NULL %s %d\n",
			__FILE__, __LINE__ );
			return NULL;
	}							// if

	assert( wgt_hit != NULL );
	select_widget( wgt_hit, 
		TRUE,
		TRUE );					// Select 1st CB, if any

	return wgt_hit;

}								// search_wgt_hit


PUBLIC void
new_widget_inserted( GtkWidget *wgt_new, GtkWidget *wgt_container, 
	gtk_wgtdef_t *wgtdef_newwgt )
{
	defclass_t		*classes, *p;
	int				depth, n;
	int				nb_res;
	gtk_wgtdef_t	*wgtdef_depth;
	GtkArg			*arg_saved_outside;

	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef_newwgt->def[ depth ].classnames != NULL); 
		  depth++ )
	{

		classes = wgtdef_newwgt->def[ depth ].classes;
		nb_res  = wgtdef_newwgt->def[ depth ].nb_classes;
		wgtdef_depth = wgtdef_newwgt->def[depth].wgtdef_depth;
		for ( p = classes, n = 0; n < nb_res; n++, p++ )
		{
			if ( p->saved_where == SAVED_OUTSIDE || p->saved_where == SAVED_OUTSIDE_FUNCTION )
			{
				arg_saved_outside = gtk_arg_new(p->restype);
				arg_saved_outside->name = p->resname;
				gtk_widget_get( GTK_WIDGET(wgt_new), arg_saved_outside );
				set_outsideres_wgt( wgt_new, p->resname, arg_saved_outside );
			}					// if
		}						// for (p)

		if (wgt_container != NULL)
		{
			classes = wgtdef_newwgt->def[ depth ].classes_child;
			nb_res  = wgtdef_newwgt->def[ depth ].nb_classes_child;
			for ( p = classes, n = 0; n < nb_res; n++, p++ )
			{
				if ( (p->saved_where == SAVED_OUTSIDE) || 
					 (p->saved_where == SAVED_OUTSIDE_FUNCTION) ||
					 (p->saved_where == SAVED_FUNCTION) )
				{
					arg_saved_outside = gtk_arg_new(p->restype);
					arg_saved_outside->name = p->resname;
					gtk_container_child_getv( GTK_CONTAINER(wgt_container), 
						GTK_WIDGET(wgt_new), 1, arg_saved_outside );
					set_outsideres_wgt( wgt_new, p->resname, arg_saved_outside );
				}				// if
			}					// for (p)
		}

	}							//

}								// new_widget_inserted


PUBLIC void
set_outsideres_function( GtkWidget *wgt_new, gtk_wgtdef_t *wgtdef_newwgt )
{
	init_GtkArg		*init_arg;
	GtkArg			arg;
	int				depth, n;
	gtk_wgtdef_t	*wgtdef_depth;
	defclass_t		*classes, *p=NULL;
	int				nb_res, k, found=FALSE;
	int				xdepth;

	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef_newwgt->def[ depth ].classnames != NULL); 
		  depth++ )
	{

		wgtdef_depth = wgtdef_newwgt->def[depth].wgtdef_depth;

		if (wgtdef_depth->nb_deft_values > 0)
		{

			for ( init_arg = wgtdef_depth->deft_values, n = 0; 
				  n < wgtdef_depth->nb_deft_values;
				  n++, init_arg++ )
			{

				for ( xdepth = 0; 
					  (xdepth < MAXNB_HIERARCHIES) && (wgtdef_newwgt->def[ xdepth ].classnames != NULL); 
					  xdepth++ )
				{
					classes = wgtdef_newwgt->def[ xdepth ].classes;
					nb_res  = wgtdef_newwgt->def[ xdepth ].nb_classes;
					for ( found = FALSE, p = classes, k = 0; k < nb_res; k++, p++ )
					{
						if ( !strcmp( p->resname, init_arg->name ) )
						{
							found = TRUE;
							break;
						}
					}			// for
					if ( found )
						break;
				}				// for
				if ( !found )
					continue;
				if ( (p->saved_where != SAVED_FUNCTION) )
					continue;

				arg.type = init_arg->type;
				arg.name = init_arg->name;
				switch ( arg.type )
				{
					case GTK_TYPE_STRING :
						GTK_VALUE_STRING( arg ) = init_arg->u.string_value;
						break;
					case GTK_TYPE_INT :
						GTK_VALUE_INT( arg ) = init_arg->u.int_value;
						break;
					case GTK_TYPE_FLOAT :
						GTK_VALUE_FLOAT( arg ) = init_arg->u.float_value;
						break;
					case GTK_TYPE_BOOL :
						GTK_VALUE_BOOL( arg ) = init_arg->u.bool_value;
						break;
					case GTK_TYPE_ENUM :
						GTK_VALUE_ENUM( arg ) = init_arg->u.int_value;
						break;
					case GTK_TYPE_FLAGS :
						GTK_VALUE_FLAGS( arg ) = init_arg->u.int_value;
						break;
					default :
						assert( 0 );
				}

				if ( (*wgtdef_depth->pfct_getdlg)(p, wgt_new, &arg) == NULL )
				{
					GtkArg	*arg_saved_outside;
					arg_saved_outside = gtk_arg_new( arg.type );
					gtk_arg_copy( &arg, arg_saved_outside );
					set_outsideres_wgt( wgt_new, arg.name, arg_saved_outside );
				}				// if

			}
		}						// if
		
	}							// for

}								// set_outsideres_function


static void
set_deft_values( GtkWidget *wgt_new, gtk_wgtdef_t *wgtdef_newwgt,
	int f_deft_values )
{
	init_GtkArg		*init_arg;
	GtkArg			arg;
	int				depth, n;
	gtk_wgtdef_t	*wgtdef_depth;
	defclass_t		*classes, *p=NULL;
	int				nb_res, k, found=FALSE;
	int				xdepth;

	for ( depth = 0; 
		  (depth < MAXNB_HIERARCHIES) && (wgtdef_newwgt->def[ depth ].classnames != NULL); 
		  depth++ )
	{

		wgtdef_depth = wgtdef_newwgt->def[depth].wgtdef_depth;

		if (wgtdef_depth->nb_deft_values > 0)
		{

			for ( init_arg = wgtdef_depth->deft_values, n = 0; 
				  n < wgtdef_depth->nb_deft_values;
				  n++, init_arg++ )
			{

				for ( xdepth = 0; 
					  (xdepth < MAXNB_HIERARCHIES) && (wgtdef_newwgt->def[ xdepth ].classnames != NULL); 
					  xdepth++ )
				{
					classes = wgtdef_newwgt->def[ xdepth ].classes;
					nb_res  = wgtdef_newwgt->def[ xdepth ].nb_classes;
					for ( found = FALSE, p = classes, k = 0; k < nb_res; k++, p++ )
					{
						if ( !strcmp( p->resname, init_arg->name ) )
						{
							found = TRUE;
							break;
						}
					}			// for
					if ( found )
						break;
				}				// for

				arg.type = init_arg->type;
				arg.name = init_arg->name;
				switch ( arg.type )
				{
					case GTK_TYPE_STRING :
						if ( f_deft_values )
							GTK_VALUE_STRING( arg ) = init_arg->u.string_value;
						else
							GTK_VALUE_STRING( arg ) = strdup( "" );
						break;
					case GTK_TYPE_INT :
						if ( f_deft_values )
							GTK_VALUE_INT( arg ) = init_arg->u.int_value;
						else
							GTK_VALUE_INT( arg ) = 0;
						break;
					case GTK_TYPE_FLOAT :
						if ( f_deft_values )
							GTK_VALUE_FLOAT( arg ) = init_arg->u.float_value;
						else
							GTK_VALUE_FLOAT( arg ) = 0;
						break;
					case GTK_TYPE_BOOL :
						if ( f_deft_values )
							GTK_VALUE_BOOL( arg ) = init_arg->u.bool_value;
						else
							GTK_VALUE_BOOL( arg ) = FALSE;
						break;
					case GTK_TYPE_ENUM :
						if ( f_deft_values )
							GTK_VALUE_ENUM( arg ) = init_arg->u.int_value;
						else
							GTK_VALUE_ENUM( arg ) = 0;
						break;
					case GTK_TYPE_FLAGS :
						if ( f_deft_values )
							GTK_VALUE_FLAGS( arg ) = init_arg->u.int_value;
						else
							GTK_VALUE_FLAGS( arg ) = 0;
						break;
					default :
						assert( 0 );
				}

				if ( !found || (p->saved_where == SAVED_INSIDE) )
				{
					gtk_widget_setv( GTK_WIDGET( wgt_new ), 1, &arg );
				}				// if

			}
		}						// if
		
	}							// for

}								// set_deft_values


PUBLIC GtkWidget *
insert_new_widget( GtkWidget *wgt_container, gtk_wgtdef_t *wgtdef_newwgt,
	int f_select, 
	int is_disabled,
	int f_deft_values,
	module_info_t *_module )
{
	GtkWidget		*wgt_new;
	module_info_t	*module;
	GtkWidget		*parent=NULL;
	int				signal_id;
	gtk_wgtdef_t	*wgtdef_parent=NULL;
	GtkWidget		*wgt_temp_window=NULL;

	/*--- A new widget can be added only in a container ---*/
	if ( wgt_container != NULL )
	{
		if ( !IS_WGT_PLACEHOLDER(wgt_container) && !GTK_IS_CONTAINER(wgt_container) )
		{
			warning( "%s must be a container to add a new widget!",
				gtk_widget_get_name( wgt_container) );
			return NULL;
		}						// if
	}							// if

	/*--- Get the module_info stored into the line ---*/
	if ( _module == NULL )
	{
		module = get_object_data( wgt_container, WGTDATA_MODULE );
		assert( module != NULL );
	}
	else
	{
		module = _module;
	}

	/*--- Now create the new widget ---*/
	if ( wgt_container != NULL )
	{
		if ( !IS_WGT_PLACEHOLDER( wgt_container ) )
		{	
			parent = wgt_container;
		}
		else
		{
			GtkArg	arg;

			arg.name = "GtkWidget::parent";
			arg.type = GTK_TYPE_CONTAINER;	
			gtk_widget_getv( GTK_WIDGET( wgt_container ), 1, &arg );
			parent = GTK_WIDGET( GTK_VALUE_OBJECT(arg) );
			wgtdef_parent = get_object_data( GTK_WIDGET(parent), WGTDATA_WGTDEF );
		}
	}							// if

	SETARG_OBJECT( GTK_TYPE_CONTAINER, "GtkWidget::parent", parent );
	if ( f_select )
	{
		if ( IS_WGT_PLACEHOLDER(wgt_container) && wgtdef_parent->insnewwgt_insinwgt )
		{
			wgt_temp_window = gtk_type_new( gtk_window_get_type() );
			gtk_widget_reparent( GTK_WIDGET( wgt_container ), wgt_temp_window );
		}
		wgt_new = wgtdef_newwgt->pfct_addwgt( wgtdef_newwgt, module, parent, wgt_container );
		if (wgt_new == NULL)
		{
			if ( parent == NULL )
				warning( "Unable to create widget %s",
					wgtdef_newwgt->widget_name );
			else
				warning( "Unable to create widget %s inside %s",
					wgtdef_newwgt->widget_name, gtk_widget_get_name( parent ) );
			return NULL;
		}
		if ( wgtdef_newwgt->pfct_afteraddwgt != NULL )
			wgtdef_newwgt->pfct_afteraddwgt( wgt_new );

		if ( IS_WGT_PLACEHOLDER(wgt_container) && !wgtdef_parent->insnewwgt_insinwgt )
		{
			wgt_temp_window = gtk_type_new( gtk_window_get_type() );
			gtk_widget_reparent( GTK_WIDGET( wgt_container ), wgt_temp_window );
		}

		args[0].name = "GtkWidget::parent";
		args[0].type = GTK_TYPE_CONTAINER;	
		gtk_widget_getv( GTK_WIDGET(wgt_new), 1, args );
		if ( (GtkWidget *)GTK_VALUE_OBJECT(args[0]) == NULL )
		{
			if ( parent == NULL )
				warning( "Unable to place widget %s",
					wgtdef_newwgt->widget_name );
			else
				warning( "Unable to place widget %s inside %s",
					wgtdef_newwgt->widget_name, gtk_widget_get_name( parent ) );
			gtk_widget_destroy( wgt_new );
			return NULL;
		}

		/*--- For a GtkMenu ---*/
		if ( GTK_IS_MENU( wgt_new ) )
			gtk_menu_item_set_submenu( GTK_MENU_ITEM( parent ), wgt_new );

		set_data_wgtdef( wgt_new, parent, module, 
			FALSE, 			// Placeholder
			FALSE, 			// Composite
			is_disabled );
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
		if (config.show_composite_childs)
			check_for_composite_childs( wgt_new );
#endif

		if ( IS_WGT_PLACEHOLDER(wgt_container) )
		{
			gtk_widget_destroy( wgt_temp_window );
			reset_selection();
		}
	
	}
	else
	{
		GtkArg		arg_container;
		GtkWidget	*wgt_parent = parent;

		if ( wgt_container != NULL )
		{
			ADDWGT_PLACEHOLDER_BEFORE
		}
		wgt_new = gtk_widget_newv( wgtdef_newwgt->pfct_get_widget_type(), nb_args, args );
		if ( wgt_container != NULL )
		{
			ADDWGT_PLACEHOLDER_AFTER
		}
		if ( wgtdef_newwgt->pfct_afteraddwgt != NULL )
			wgtdef_newwgt->pfct_afteraddwgt( wgt_new );
		set_data_wgtdef( wgt_new, 
//			(module == NULL) ? wgt_parent : NULL,
			wgt_parent,
			module, 
			FALSE, 			// Placeholder
			FALSE, 			// Composite
			is_disabled );
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
		if ( config.show_composite_childs )
			check_for_composite_childs( wgt_new );
#endif
			
	}

	/*--- Set the default values for the newly created widget ---*/
	if ( f_deft_values )
		set_deft_values( wgt_new, wgtdef_newwgt, f_deft_values );
	set_outsideres_function( wgt_new, wgtdef_newwgt );

	/*--- Show now the newly created widget ---*/	
	gtk_widget_show( wgt_new );

	/*--- Resources stored outside the widget ---*/
	new_widget_inserted( wgt_new, wgt_container, wgtdef_newwgt );
	
	/*--- Select the new added widget ---*/
	check_for_composite_childs( module->wgt_window );
	if (f_select)
	{
		update_tree();
		select_widget( wgt_new, 
			TRUE,
			TRUE );				// Select 1st CB, if any
	}

#if defined(GLG_SUPPORT_EDIT_COMPOSITE) 
	/*--- Recompute the hierarchy of all the composite widgets ---*/
	if (config.show_composite_childs)
		compute_hierarchy_composite_childs( wgt_new );
#endif

	/*--- Toggle off the selected widget into the palette widgets ---*/
	if ( f_select )
	{
		signal_id = (int)gtk_object_get_data( GTK_OBJECT( wgt_selected_wgt_palette ), "signal_id" );
		gtk_signal_handler_block( GTK_OBJECT( wgt_selected_wgt_palette ), signal_id );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_selected_wgt_palette), FALSE );
		gtk_signal_handler_unblock( GTK_OBJECT( wgt_selected_wgt_palette ), signal_id );
		wgt_selected_wgt_palette = NULL;
		cb_module_enter_notify_event( NULL, NULL, module->wgt_window );
	}

	/*--- This module has been modified ---*/
	project_touched( module );

	return wgt_new;

}								// insert_new_widget


PUBLIC gboolean 
cb_window_focus_in( GtkWidget *widget, GdkEventFocus *event, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;

	if ( curr_module != module )
	{

		/*--- Update the tree of widgets for this module ---*/
		curr_module = module;
		update_tree();
		select_widget( module->wgt_window, 
			TRUE,
			TRUE );				// Select 1st CB, if any

		/*--- Make the line selection (list of modules) ---*/
		gtk_ctree_select( GTK_CTREE(wgt_tree_modules),
			GTK_CTREE_NODE(module->wgt_tree_node) );

	}

	return FALSE;
}								// cb_window_focus_in
	

static gint
cb_module_button_press_event( GtkWidget *window, GdkEventButton *event, gpointer user_data )
{
	GtkWidget		*wgt_hit;	
	module_info_t	*module;
	GtkWidget		*wgt = GTK_WIDGET( user_data );
	gtk_wgtdef_t	*wgtdef_container;
	gtk_wgtdef_t	*wgtdef_newwgt;

	if ( wgt == NULL )
		return 0;

	/*--- Get the module_info stored into the line ---*/	
	module = get_object_data( wgt, WGTDATA_MODULE );
	assert( module != NULL );

	if( module != curr_module )
		cb_window_focus_in( module->wgt_window, NULL, module );

	if ( event->type == GDK_BUTTON_PRESS )
	{

		wgt_hit = search_wgt_hit( module, event );
		if ( wgt_hit == NULL )
			return 0;
		wgtdef_container = get_object_data( GTK_WIDGET(wgt_hit), WGTDATA_WGTDEF );
		assert( wgtdef_container != NULL );
		if ( wgtdef_container->stop_emit_btnpressevt )
			gtk_signal_emit_stop_by_name( GTK_OBJECT( wgt ), "button_press_event" );
		if ( event->button == 3 )
		{
			GtkCTreeNode	*wgt_node;
			wgt_node = get_object_data(wgt_hit, TREENODE_WGTDATA);
			assert( wgt_node != NULL);
			tree_select_popup_menu( GTK_WIDGET(wgt_hit), event, GTK_CTREE_NODE(wgt_node), FALSE );
			return 1;
		}

		if ( wgt_selected_wgt_palette == NULL )
			return 1;
		if ( wgtdef_container->max_nb_children != -1 )
		{
			GtkWidget	*wgt_container;
			GList		*list_children;
			wgt_container = wgt;
			assert( GTK_IS_CONTAINER(wgt_container) );
			list_children = gtk_container_children(GTK_CONTAINER(wgt_container));
			if ( list_children && (g_list_length(list_children) >= wgtdef_container->max_nb_children))
				return 0;
		}
		wgtdef_newwgt = 
			get_object_data( GTK_WIDGET(wgt_selected_wgt_palette), WGTDATA_WGTDEF );
		assert( wgtdef_newwgt != NULL );
		INIT_ARGS;
		insert_new_widget( wgt_hit, wgtdef_newwgt, 
			TRUE, 			// f_select
			TRUE,			// is_disabled
			TRUE,			// Default values
			NULL );
		return 1;
	}

	return 0;
}								// cb_module_button_press_event


typedef struct
{
	int			level;
	int			blocked;
}	BLOCKMODULE_DATA;


static void
forall_block_module( GtkWidget *widget, BLOCKMODULE_DATA *data )
{
	int		nb_signals, n;

	if ( !IS_WGT_NOCOMPOSITE(widget) )
		return;
	if ( get_object_data(widget, WGTDATA_HIERARCHY_CHILD) )
		return;

	data->level++;

	nb_signals = GTK_OBJECT_NSIGNALS( widget );
	if ( nb_signals )
	{
		guint	*signals;

		signals = GTK_OBJECT_SIGNALS( widget );
		if ( data->blocked )
			for ( n = 0; n < nb_signals; n++ )
{
GtkSignalQuery *info;
int signal_id;
int handler_id;
info = gtk_signal_query( signals[ n ] );
handler_id = gtk_signal_handler_pending( GTK_OBJECT( widget ), info->signal_id, TRUE );
signal_id = gtk_signal_lookup (info->signal_name, GTK_OBJECT_TYPE (widget));
}
		else
			for ( n = 0; n < nb_signals; n++ )
				gtk_signal_handler_unblock( GTK_OBJECT( widget ), signals[ n ] );
	}

	if ( GTK_IS_CONTAINER (widget) )
	{
		BLOCKMODULE_DATA	new_data;
		
		new_data.level   = data->level;
		new_data.blocked = data->blocked;

		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_block_module, &new_data );
	}

	data->level--;

}								// forall_block_module


static void
block_module( module_info_t *module, int blocked )
{
	BLOCKMODULE_DATA	data;

	data.level = -1;
	data.blocked = blocked;
	forall_block_module( module->wgt_window, &data );

}								// block_module


PUBLIC gint
cb_module_window_close( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_module_window_close


static GtkWidget*
create_window( module_info_t *module, char *title, int w, int h )
{
	GtkWidget *wgt_window;

	INIT_ARGS;
	wgt_window = gtk_type_new( gtk_window_get_type() );
	if ( w == -1 )
		gtk_window_set_default_size( GTK_WINDOW(wgt_window), 
			config.initial_w_module_window, config.initial_h_module_window );
	else
		gtk_window_set_default_size( GTK_WINDOW(wgt_window), 
			w, h );
	gtk_window_set_policy( GTK_WINDOW(wgt_window), 
		FALSE, TRUE, FALSE );
	gtk_window_set_title( GTK_WINDOW (wgt_window), title );
	set_data_wgtdef( wgt_window, NULL, module, 
		FALSE, 			// Placeholder
		FALSE, 			// Composite
		FALSE );		// Disabled
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if (config.show_composite_childs)
		check_for_composite_childs( wgt_window );
#endif
	gtk_signal_connect( GTK_OBJECT( wgt_window ), "focus_in_event",
		GTK_SIGNAL_FUNC(cb_window_focus_in), module );
	gtk_widget_set_events(wgt_window, 
		gtk_widget_get_events( wgt_window ) | GDK_FOCUS_CHANGE );

	gtk_signal_connect( GTK_OBJECT(wgt_window), "destroy",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );
	gtk_signal_connect(GTK_OBJECT(wgt_window), "delete_event",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );

	module->wgt_window = wgt_window;

	return wgt_window;
}								// create_window


PUBLIC module_info_t *
module_create_window( char *title, int w, int h, int iconified )
{
	module_info_t	*module;
	char			title_window[ 200 ];
	GtkWidget		*wgt_window;
	gtk_wgtdef_t	*wgtdef_window=NULL;
	GtkCTreeNode	*wgt_node;
	char			*text[1];

	/*--- Create a new module info ---*/
	module = g_malloc( sizeof( module_info_t ) );
	cpt_module_window++;
	if (title == NULL)
		sprintf( title_window, "window%d", cpt_module_window );
	else
		sprintf( title_window, title );
	memset( module, 0, sizeof( module_info_t ) );
	module->type = MODULE_TYPE_WINDOW;
	module->u1.window.f_opened_from_main = TRUE;
	module->u1.window.f_fixed_position = FALSE;
	sprintf( module->u1.window.name_function, "create_%s", title_window );
	module->iconified = FALSE;
	module->f_not_saved = FALSE;
	module->u1.window.tooltips = gtk_tooltips_new();

	/*--- Create the window ---*/
	wgtdef_window = search_wgtdef( "GtkWindow" );
	assert( wgtdef_window != NULL );
	wgt_window = create_window( module, title_window, w, h );
	gtk_widget_set_events( wgt_window,
		gtk_widget_get_events( wgt_window ) |
		GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK  );
	gtk_signal_connect( GTK_OBJECT( wgt_window ), "enter_notify_event",
		GTK_SIGNAL_FUNC( cb_module_enter_notify_event ), wgt_window );
	gtk_signal_connect( GTK_OBJECT(wgt_window), "destroy",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );
	gtk_signal_connect(GTK_OBJECT(wgt_window), "delete_event",
		GTK_SIGNAL_FUNC( cb_module_window_close ), NULL );

	/*--- Add this new module into the tree ---*/
	text[ 0 ] = title_window;	
	wgt_node = gtk_ctree_insert_node( GTK_CTREE(wgt_tree_modules), 
		NULL, NULL,
		text, 5,
		wgtdef_window->gdkpixmap, wgtdef_window->gdkbitmap,
		wgtdef_window->gdkpixmap, wgtdef_window->gdkbitmap,
		FALSE, FALSE );
	module->wgt_tree_node = wgt_node;
	module->wgt_window = wgt_window;
	gtk_ctree_node_set_row_data( GTK_CTREE(wgt_tree_modules), GTK_CTREE_NODE(wgt_node), 
		module );

	/*--- Attach some data to the window widget ---*/
 	set_name_wgt( module->wgt_window, title_window );
	set_nametype_wgt( module->wgt_window, NAMETYPE_GLOBAL );
	if ( iconified )
	{
		gtk_widget_show_all( module->wgt_window );
		gdk_window_lower( module->wgt_window->window );
		module->iconified = TRUE;
	}
	else
	{
		gtk_widget_show_all( module->wgt_window );
		gdk_window_raise( module->wgt_window->window );
		module->iconified = FALSE;
	}

	/*--- Set the default values for the newly created widget ---*/
	set_deft_values( wgt_window, wgtdef_window, TRUE );
	set_outsideres_function( wgt_window, wgtdef_window );

	/*--- Resources stored outside the widget ? ---*/
	new_widget_inserted( wgt_window, NULL, wgtdef_window );

	/*--- Project modified ---*/
	is_project_touched = TRUE;

	block_module( module, TRUE );

	/*--- Make the new window the selected widget ---*/
	curr_module = module;
	update_tree();
	select_widget( module->wgt_window, 
		TRUE,
		TRUE );					// Select 1st CB, if any

	/*--- Make the line selection (list of modules) ---*/
	gtk_ctree_select( GTK_CTREE(wgt_tree_modules),
		GTK_CTREE_NODE(module->wgt_tree_node) );

	/*--- Make the palette widget sensitive ---*/
	set_palette_sensitive( TRUE );

	/*--- This module has been modified ---*/
	project_touched( module );

	return module;
	
}								// module_create_window


typedef struct
{
	GtkWidget		*initial;
	module_info_t	*module;
}	SETWGT_CREATEDIALOG;


static void
forall_createdialog( GtkWidget *widget, SETWGT_CREATEDIALOG *data )
{
	gtk_wgtdef_t	*wgtdef;

	if ( widget != data->initial )
	{
		int		*curr = get_object_data( widget, WGTDATA_FLAGS );
		if (curr == NULL)
		{
			curr = malloc(sizeof(int));
			*curr = 0;
			set_object_data( widget, WGTDATA_FLAGS, curr, TRUE );
		}
		if ( !IS_WGT_NOCOMPOSITE(widget) )
		{
			GtkArg		args[1];
			GtkWidget	*wgt_parent;

			args[0].name = "parent";
			args[0].type = GTK_TYPE_CONTAINER;	
			gtk_widget_getv( GTK_WIDGET(widget), 1, args );
			wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );

			wgt_put_flag( widget, WGTFLAG_DONTDELETE );
			set_data_wgtdef( widget, wgt_parent, data->module, 
				FALSE, 			// Placeholder
				TRUE, 			// Composite
				FALSE );		// Disabled
			wgtdef = get_object_data( GTK_WIDGET(widget), WGTDATA_WGTDEF );
			assert( widget != NULL );
			new_widget_inserted( widget, NULL, wgtdef );
			set_outsideres_function( widget, wgtdef );
		}
	}
	if ( GTK_IS_CONTAINER(widget) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_createdialog, data );
	}

}								// forall_createdialog


PUBLIC void
recreate_tree( module_info_t *module, GtkWidget *wgt )
{
	SETWGT_CREATEDIALOG		data;

	data.initial = wgt;
	data.module  = module;
	gtk_container_forall( GTK_CONTAINER( wgt ), 
		(GtkCallback)forall_createdialog, &data );
}								// recreate_tree

#ifdef USE_GNOME

static GtkWidget *
create_gnome_dialog( void )
{
	GtkWidget	*wgt;

	wgt = gnome_dialog_new (
		"Titre",
		GNOME_STOCK_BUTTON_YES,
		GNOME_STOCK_BUTTON_NO,
		NULL );
	
	return wgt;

}								// create_gnome_dialog


static GtkWidget *
create_gnome_dialog_about( void )
{
	GtkWidget	*wgt;
	const gchar *authors[] = {
		"Authors",
   		NULL
  	};

	wgt = gnome_about_new (
		"Titre",
		"Version",
		"Copyright",
		authors,
		"Comment",
		"Logo" );
	
	return wgt;

}								// create_gnome_dialog_about


static GtkWidget *
create_gnome_dialog_propertybox( void )
{
	GtkWidget	*wgt;

	wgt = gnome_property_box_new ( );
	
	return wgt;

}								// create_gnome_dialog_propertybox


static GtkWidget *
create_gnome_dialog_messagebox( void )
{
	GtkWidget	*wgt;

	wgt = gnome_message_box_new (
		"Mon Message",
		" blah",
		GNOME_STOCK_BUTTON_OK,
		NULL );
	
	return wgt;

}								// create_gnome_dialog_messagebox

#endif

static GtkWidget*
create_dialog( module_info_t *module, char *title, int w, int h,
	gtk_wgtdef_t *wgtdef_dialog )
{
	GtkWidget	*wgt_dialog;

	INIT_ARGS;
	switch ( module->type )
	{
		case MODULE_TYPE_DIALOG :
			wgt_dialog = gtk_type_new( wgtdef_dialog->pfct_get_widget_type() );
			break;
#ifdef USE_GNOME
		case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			wgt_dialog = create_gnome_dialog();
			break;
		case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			wgt_dialog = create_gnome_dialog_about();
			break;
		case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			wgt_dialog = create_gnome_dialog_propertybox();
			break;
		case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
			wgt_dialog = create_gnome_dialog_messagebox();
			break;
#endif
		default :
			assert( 0 );
	}
	gtk_widget_set_usize( wgt_dialog, 
		-1, -1 );
	if (w == -1)
		gtk_widget_set_usize( wgt_dialog, 
			config.initial_w_module_dialog, config.initial_h_module_dialog );
	else
		gtk_widget_set_usize( wgt_dialog, 
			w, h );
	gtk_object_set_data( GTK_OBJECT(wgt_dialog), "wgt_dialog", wgt_dialog );;
	gtk_window_set_title( GTK_WINDOW(wgt_dialog), title );
	set_data_wgtdef( wgt_dialog, NULL, module, 
		FALSE, 			// Placeholder
		FALSE, 			// Composite
		FALSE );		// Disabled

	recreate_tree( module, wgt_dialog );

	gtk_signal_connect( GTK_OBJECT(wgt_dialog), "focus_in_event",
		GTK_SIGNAL_FUNC(cb_window_focus_in), module );
	gtk_widget_set_events( wgt_dialog, 
		gtk_widget_get_events(wgt_dialog) | GDK_FOCUS_CHANGE );

	module->wgt_window = wgt_dialog;

	return wgt_dialog;
}								// create_dialog


PUBLIC gint
cb_module_dialog_close( GtkWidget * widget, gpointer data )
{
	return TRUE;
}								// cb_module_dialog_close


PUBLIC module_info_t *
module_create_dialog( char *title, int w, int h, int iconified,
	int type, char *which_dlg )
{
	module_info_t	*module;
	char			title_dialog[200];
	GtkWidget		*wgt_dialog;
	gtk_wgtdef_t	*wgtdef_dialog=NULL;
	GtkCTreeNode	*wgt_node;
	char			*text[1];

	/*--- Create a new module info ---*/
	module = g_malloc( sizeof( module_info_t ) );
	cpt_module_dialog++;
	if (title == NULL)
		sprintf( title_dialog, "dialog%d", cpt_module_dialog );
	else
		sprintf( title_dialog, title );
	memset( module, 0, sizeof( module_info_t ) );
	module->type = type;
	module->u1.dialog.f_opened_from_main = TRUE;
	module->u1.dialog.f_fixed_position = FALSE;
	sprintf( module->u1.dialog.name_function, "create_%s", title_dialog );
	module->iconified = FALSE;
	module->f_not_saved = FALSE;

	wgtdef_dialog = search_wgtdef( which_dlg );
	assert( wgtdef_dialog != NULL );
	
	wgt_dialog = create_dialog( module, title_dialog, w, h, wgtdef_dialog );
	gtk_signal_connect( GTK_OBJECT(wgt_dialog), "enter_notify_event",
		GTK_SIGNAL_FUNC( cb_module_enter_notify_event ), wgt_dialog );

	gtk_signal_connect( GTK_OBJECT( wgt_dialog ), "destroy",
		GTK_SIGNAL_FUNC( cb_module_dialog_close ), NULL );
	gtk_signal_connect(GTK_OBJECT( wgt_dialog ), "delete_event",
		GTK_SIGNAL_FUNC( cb_module_dialog_close ), NULL );

	compute_hierarchy_composite_childs( wgt_dialog );

	/*--- Add this new module into the tree ---*/
	text[ 0 ] = title_dialog;	
	wgt_node = gtk_ctree_insert_node( GTK_CTREE(wgt_tree_modules), 
		NULL, NULL,
		text, 5,
		wgtdef_dialog->gdkpixmap, wgtdef_dialog->gdkbitmap,
		wgtdef_dialog->gdkpixmap, wgtdef_dialog->gdkbitmap,
		FALSE, FALSE );
	module->wgt_tree_node = wgt_node;
	module->wgt_window = wgt_dialog;
	gtk_ctree_node_set_row_data( GTK_CTREE(wgt_tree_modules), GTK_CTREE_NODE( wgt_node ), 
		module );

	/*--- Attach some data to the window widget ---*/
 	set_name_wgt( module->wgt_window, title_dialog );
	set_nametype_wgt( module->wgt_window, NAMETYPE_GLOBAL );
	gtk_widget_show_all( module->wgt_window );

	/*--- Set the default values for the newly created widget ---*/
	set_deft_values( module->wgt_window, wgtdef_dialog, TRUE );
	set_outsideres_function( module->wgt_window, wgtdef_dialog );

	/*--- Resources stored outside the widget ? ---*/
	new_widget_inserted( wgt_dialog, NULL, wgtdef_dialog );

	/*--- Project modified ---*/
	is_project_touched = TRUE;

	block_module( module, TRUE );

	/*--- Make the new window the selected widget ---*/
	curr_module = module;
	update_tree();
	select_widget( module->wgt_window, 
		TRUE,
		TRUE );					// Select 1st CB, if any

	/*--- Make the line selection (list of modules) ---*/
	gtk_ctree_select( GTK_CTREE(wgt_tree_modules),
		GTK_CTREE_NODE(module->wgt_tree_node) );

	/*--- Make the palette widget sensitive ---*/
	set_palette_sensitive( TRUE );

	/*--- This module has been modified ---*/
	project_touched( module );

	return module;
	
}								// module_create_dialog

