#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


#define ICON_WIDTH	21
#define ICON_HEIGHT 21
#define ICON_INTER	3

static	GtkWidget	*wgt_fixed_basic = NULL;
static	GtkWidget	*wgt_fixed_more;
static	GtkWidget	*wgt_fixed_menu;
#ifdef USE_GNOME
static	GtkWidget	*wgt_fixed_gnome;
#endif
#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
static	GtkWidget	*wgt_fixed_templates;
#endif


void toto() {}


/* This is a dummy pixmap we use when a pixmap can't be found. */
static char *dummy_pixmap_xpm[] = {
/* columns rows colors chars-per-pixel */
"1 1 1 1",
"  c None",
/* pixels */
" "
};

/* This is an internally used function to create pixmaps. */
static GtkWidget*
create_dummy_pixmap                    (GtkWidget       *widget)
{
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
                                                     NULL, dummy_pixmap_xpm);
  if (gdkpixmap == NULL)
    g_error ("Couldn't create replacement pixmap.");
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

static GtkWidget *
create_pixmap( GtkWidget *wgt_window, char **pixmap, char *label )
{
	GdkColormap		*colormap;
	GdkPixmap		*gdkpixmap;
	GdkBitmap		*mask;
	GtkWidget		*wgt_pixmap;

	if ( pixmap == NULL )
	{
		return create_dummy_pixmap( wgt_window );
	}

	colormap = gtk_widget_get_colormap (wgt_window);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d( NULL, colormap, &mask,
		NULL, pixmap);
	if ( gdkpixmap == NULL )
    {
		g_warning( "Error loading pixmap file: %s", label );
		return create_dummy_pixmap( wgt_window );
    }

	wgt_pixmap = gtk_pixmap_new( gdkpixmap, mask );
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
	return wgt_pixmap;

}								// create_pixmap


static void
forall_button_group( GtkWidget *widget, GtkWidget *wgt_me )
{
	int	id;

	if ( GTK_IS_CHECK_BUTTON(widget) )
	{
		if ( widget != wgt_me )
		{
			id = (int)gtk_object_get_data( GTK_OBJECT( widget ), "signal_id" );
			gtk_signal_handler_block( GTK_OBJECT( widget ), id );
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(widget), FALSE );
			gtk_signal_handler_unblock( GTK_OBJECT( widget ), id );
		}						// if
	}							// if

	if ( GTK_IS_CONTAINER(widget) )
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_button_group, wgt_me );

}								// forall_button_group


static void
cb_palette_button_clicked( GtkWidget *button, void *nothing )
{

	if ( GTK_TOGGLE_BUTTON (button)->active )
	{
		wgt_selected_wgt_palette = GTK_WIDGET(button);
		assert( wgt_selected_wgt_palette != NULL );
	}
	else
	{
		wgt_selected_wgt_palette = NULL;
	}
		
	gtk_container_forall( GTK_CONTAINER(wgt_wnd_palette), 
		(GtkCallback) forall_button_group, button );
}								// cb_palette_button_clicked


static void
display_widgets( int which_palette, GtkWidget *wgt_window, GtkWidget *wgt_fixed,
	int first )
{
	gtk_wgtdef_t	**p, *q;
	int				cnt, row, col;
	GtkWidget		*wgt_pixmap;
	GtkTooltips		*wgt_tooltips=NULL;
	GtkWidget		*button;
	int				id;
	char			name[80];
	gint			w, h;
	int				rows, cols;
	int				x, y;
	int				offset;

	if ( !GTK_WIDGET_REALIZED(wgt_fixed) )
		return;

	gdk_window_get_size( wgt_fixed->window, &w, &h );
	rows = h / (ICON_INTER + ICON_HEIGHT + ICON_INTER);
	cols = w / (ICON_INTER + ICON_WIDTH + ICON_INTER);
	offset = (w - cols * (ICON_INTER + ICON_WIDTH + ICON_INTER)) / 2;

	for ( cnt = row = col = 0, p = (gtk_wgtdef_t **)gtk_wgtdef; (*p); p++ )
	{
		sprintf( name, "index_%d\n", ++cnt );
		q = *p;
		if ( q->which_palette == which_palette )
		{

			x = offset + (ICON_INTER + ICON_WIDTH + ICON_INTER) * col;
			y = (ICON_INTER + ICON_HEIGHT + ICON_INTER) * row;

			if (first)
			{
				if (wgt_tooltips == NULL)
					wgt_tooltips = gtk_tooltips_new();
				button = gtk_check_button_new();
				gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE );
				gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
				id = gtk_signal_connect (GTK_OBJECT (button), "clicked",
					(GtkSignalFunc) cb_palette_button_clicked, NULL );
				set_object_data( GTK_WIDGET( button ), "signal_id", (void *)id, FALSE );
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), FALSE );
				set_object_data( button, WGTDATA_WGTDEF, q, FALSE );
				set_object_data( wgt_fixed, name, button, FALSE );

				assert(wgt_tooltips != NULL);
 				wgt_pixmap = create_pixmap( wgt_window, q->pixmap, q->widget_name );
				gtk_container_add( GTK_CONTAINER(button), wgt_pixmap );
				gtk_widget_show( wgt_pixmap );
				gtk_widget_show( button );
				gtk_tooltips_set_tip( wgt_tooltips, button,
					q->widget_name, NULL);

				gtk_container_add( GTK_CONTAINER(wgt_fixed), button );
					
			}
			else
			{

				button = get_object_data( wgt_fixed, name );
				assert( button != NULL );

			}

			INIT_ARGS;
			SETARG_INT( "x", x );
			SETARG_INT( "y", y );
			gtk_widget_setv( GTK_WIDGET(button), nb_args, args );

			if (++col == cols )
			{
				col=0;
				row++;
			}

		}						// if
	}							// for

}								// display_widgets


static void
cb_table_check_allocate( GtkWidget *widget, GtkAllocation *allocation, gpointer user_data )
{

	display_widgets( PALETTE_BASIC, wgt_wnd_palette, wgt_fixed_basic, FALSE );
	display_widgets( PALETTE_MORE, wgt_wnd_palette, wgt_fixed_more, FALSE );
#ifdef USE_GNOME
	display_widgets( PALETTE_GNOME, wgt_wnd_palette, wgt_fixed_gnome, FALSE );
#endif
#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
	display_widgets( PALETTE_TEMPLATES, wgt_wnd_palette, wgt_fixed_templates, FALSE );
#endif

}								// cb_table_check_allocate


PUBLIC void
create_palette_window( void )
{
	GtkWidget	*wgt_label;
	GtkWidget	*wgt_vbox;
	GtkWidget	*wgt_notebook;
	int			tab;

	/*--- Create the base window ---*/
	wgt_wnd_palette = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title( GTK_WINDOW( wgt_wnd_palette ), 
		"Palette - GTk Lo Gafet" );
	gtk_widget_set_usize( GTK_WIDGET( wgt_wnd_palette ), -1, -1 );
	gtk_widget_set_usize( GTK_WIDGET( wgt_wnd_palette ), 200, 100 );
	gtk_window_set_default_size( GTK_WINDOW( wgt_wnd_palette ),
		config.palette_wnd.w, config.palette_wnd.h );
	gtk_signal_connect( GTK_OBJECT(wgt_wnd_palette), "destroy",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );
	gtk_signal_connect(GTK_OBJECT(wgt_wnd_palette), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );

	wgt_vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (wgt_wnd_palette), wgt_vbox);
	INIT_ARGS;
	SETARG_ULONG( "border_width", 0 );
	SETARG_INT( "spacing", 0 );
	gtk_widget_setv( GTK_WIDGET(wgt_vbox), nb_args, args );

	wgt_notebook = gtk_notebook_new ();
	gtk_container_add (GTK_CONTAINER (wgt_vbox), wgt_notebook);

	/*--- Create the icons for 'Basic' Gtk objects ---*/
	tab = 0;
	wgt_fixed_basic = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER(wgt_notebook), wgt_fixed_basic );
	wgt_label = gtk_label_new ("Basic");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), tab++), wgt_label);

	/*--- Create the icons for 'More' Gtk objects ---*/
	wgt_fixed_more = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER(wgt_notebook), wgt_fixed_more );
	wgt_label = gtk_label_new ("More");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), tab++), wgt_label);

	/*--- Create the icons for 'Menu' Gtk objects ---*/
	wgt_fixed_menu = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER(wgt_notebook), wgt_fixed_menu );
	wgt_label = gtk_label_new ("Menu");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), tab++), wgt_label);

#ifdef USE_GNOME
	/*--- Create the icons for 'Gnome' Gtk objects ---*/
	wgt_fixed_gnome = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER(wgt_notebook), wgt_fixed_gnome );
	wgt_label = gtk_label_new ("Gnome");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), tab++), wgt_label);
#endif

#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
	/*--- Create the icons for 'Templates' Gtk objects ---*/
	wgt_fixed_templates = gtk_fixed_new();
	gtk_container_add( GTK_CONTAINER(wgt_notebook), wgt_fixed_templates );
	wgt_label = gtk_label_new ("Templates");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), tab++), wgt_label);
#endif

	gtk_widget_realize( wgt_fixed_basic );
	display_widgets( PALETTE_BASIC, wgt_wnd_palette, wgt_fixed_basic, TRUE );

	gtk_widget_realize( wgt_fixed_more );
	display_widgets( PALETTE_MORE, wgt_wnd_palette, wgt_fixed_more, TRUE );
	
	gtk_widget_realize( wgt_fixed_menu );
	display_widgets( PALETTE_MENU, wgt_wnd_palette, wgt_fixed_menu, TRUE );
	
#ifdef USE_GNOME
	gtk_widget_realize( wgt_fixed_gnome );
	display_widgets( PALETTE_GNOME, wgt_wnd_palette, wgt_fixed_gnome, TRUE );
#endif

#ifdef GLG_SUPPORT_WIDGET_TEMPLATES
	gtk_widget_realize( wgt_fixed_templates );
	display_widgets( PALETTE_TEMPLATES, wgt_wnd_palette, wgt_fixed_templates, TRUE );
#endif
	
	gtk_widget_realize( wgt_wnd_palette );
	
	gtk_signal_connect_after( GTK_OBJECT(wgt_wnd_palette), "size_allocate",
		GTK_SIGNAL_FUNC(cb_table_check_allocate), NULL );

}								// create_palette_window


static void
forall_set_palette_sensitive( GtkWidget *widget, void *pok )
{

	if ( GTK_IS_CHECK_BUTTON(widget) )
		gtk_widget_set_sensitive( GTK_WIDGET(widget), (pok) ? TRUE : FALSE );

	if ( GTK_IS_CONTAINER(widget) )
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback) forall_set_palette_sensitive, pok );

}								// forall_set_palette_sensitive

PUBLIC void
set_palette_sensitive( int ok )
{
	gtk_container_forall( GTK_CONTAINER(wgt_wnd_palette), 
		(GtkCallback) forall_set_palette_sensitive, (ok) ? wgt_wnd_palette : NULL );
}								// set_palette_sensitive
