#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

/* The pixmap to use for placeholders */
#include "graphics/placeholder.xpm"

static GdkPixmap *placeholder_pixmap = NULL;


static gint
cb_placeholder_draw( GtkWidget *widget, GdkRectangle *area, gpointer data )
{
	GdkGC		*light_gc;
	GdkGC		*dark_gc;
	gint		w, h;

	/* Don't try to draw anything if the width or height of the widget is 0. */
	gdk_window_get_size( widget->window, &w, &h );
	if ( w==0 && h==0 )
		return FALSE;

	/* Draw the placeholder pixmap in it and a 3D border around it. */
	light_gc = widget->style->light_gc[ GTK_STATE_NORMAL ];
	dark_gc  = widget->style->dark_gc[ GTK_STATE_NORMAL ];
	gdk_window_get_size( widget->window, &w, &h );
	gdk_draw_line( widget->window, light_gc, 0, 0, w - 1, 0 );
	gdk_draw_line( widget->window, light_gc, 0, 0, 0, h - 1 );
	gdk_draw_line( widget->window, dark_gc, 0, h - 1, w - 1, h - 1 );
	gdk_draw_line( widget->window, dark_gc, w - 1, 0, w - 1, h - 1 );

	return FALSE;
	
}								// cb_placeholder_draw


PUBLIC void
install_draw_placeholder( GtkWidget *wgt_placeholder )
{

	if (placeholder_pixmap == NULL)
	{
		placeholder_pixmap = gdk_pixmap_create_from_xpm_d (wgt_placeholder->window,
			NULL, NULL, gtk_placeholder_xpm );
		assert( placeholder_pixmap != NULL );
	}
								 
	gtk_drawing_area_size( GTK_DRAWING_AREA( wgt_placeholder ), 100, 20);
	gtk_widget_set_events( wgt_placeholder, 
		gtk_widget_get_events( wgt_placeholder )| GDK_EXPOSURE_MASK );
	gdk_window_set_back_pixmap( wgt_placeholder->window, placeholder_pixmap, FALSE );
	gtk_signal_connect( GTK_OBJECT( wgt_placeholder ), "expose_event",
		GTK_SIGNAL_FUNC( cb_placeholder_draw ), NULL );

}								// install_draw_placeholder


PUBLIC GtkWidget *
create_placeholder( int type, GtkWidget *wgt_parent, int index, module_info_t *module )
{
	GtkWidget		*wgt_placeholder;
	gtk_wgtdef_t	*wgtdef_placeholder;

	wgt_placeholder = gtk_type_new( type );
	gtk_container_add( GTK_CONTAINER(wgt_parent), wgt_placeholder );
	gtk_widget_show( wgt_placeholder );
	set_data_wgtdef( wgt_placeholder, wgt_parent, module, TRUE, FALSE, TRUE );
	check_for_composite_childs( wgt_placeholder );

	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER(wgt_parent), wgt_placeholder, nb_args, args );

	install_draw_placeholder( wgt_placeholder );

	/*--- Resources stored outside the widget ---*/
	wgtdef_placeholder = get_object_data( GTK_WIDGET(wgt_placeholder), WGTDATA_WGTDEF );
	assert( wgtdef_placeholder != NULL );
	new_widget_inserted( wgt_placeholder, wgt_parent, wgtdef_placeholder );
	set_outsideres_function( wgt_placeholder, wgtdef_placeholder );

 	return wgt_placeholder;
}								// create_placeholder
