#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

#define OFS_X	6
#define OFS_Y	22

PUBLIC void
create_default_config( void )
{
	struct stat		sbuf;

	memset( &config, 0, sizeof( config_t ) );
	
	config.tree_wnd.w = 200;
	config.tree_wnd.h = 350;

	config.resources_wnd.w = 400;
	config.resources_wnd.h = 300;

	config.palette_wnd.w = 200;
	config.palette_wnd.h = 100;

	config.initial_w_module_window = 400;
	config.initial_h_module_window = 320;
	config.initial_w_module_dialog = 400;
	config.initial_h_module_dialog = 320;
	config.selection_mark_type = 2;						// 1: rect, 2: round
	config.flag_list_as = 2;							// 1: constant, 2: manifest
	config.max_source_line = 80;						// Default is 80
	if ( stat( "/usr/X11R6/bin/nedit", &sbuf ) == 0 )
	{
		strcpy( config.editor1, "/usr/X11R6/bin/nedit" );
		strcpy( config.editor2, "" );
		strcpy( config.editor3, "" );
		config.is_glo_after = FALSE;
		strcpy( config.goto_line_option, "+" );
	}
	else
	{
		strcpy( config.editor1, "xterm" );
		strcpy( config.editor2, "vi" );
		strcpy( config.editor3, "-e" );
		config.is_glo_after = FALSE;
		strcpy( config.goto_line_option, "+:" );
	}
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	config.show_composite_childs = FALSE;				// Show and Edit composite childs ?
#endif

	config.is_expand_container_blank = TRUE;

	config.indentstyle_function = 1;					// 1, 2
	config.indentstyle_block = INDENTSTYLE_BLOCK1;
	config.setgetres_method = SETGETRES_METHOD_ARGV;	// SETGETRES_METHOD_[ARGS|ARGV]

	config.tree_is_display_icon = TRUE;
	config.tree_is_display_gtktype = TRUE;
	config.callbacks_is_display_icon = TRUE;

	/*--- Project specific ---*/
	strcpy( config.exe_name, "my_exe" );
	strcpy( config.header_to_include, "" );
	memset( config.list_cfiles_added, 0, sizeof( config.list_cfiles_added ) );
#ifdef USE_GNOME
	config.is_gnome = TRUE;
#else
	config.is_gnome = FALSE;
#endif
	config.language_newprj = LANGUAGE_C;
	config.language_current = config.language_newprj;
	
}								// create_default_config

PUBLIC int 
write_config( const char * const filename, const int save_project_specific )
{
	FILE		*fp;
	char		buff[ 80 ];
	time_t		now;
	int			n;

	/*--- Create config file ---*/	
	fp = fopen( filename, "w" );
	if ( fp == NULL )
		return -1;

	fprintf( fp, "#\n" );
	now = time( NULL );
	strftime( buff, sizeof(buff), "%A %d %B %Y %T", localtime( &now ) );
	fprintf( fp, "# Date: %s\n", buff );
	fprintf( fp, "#\n" );
	fprintf( fp, "\n" );

	/*--- Save the dim/pos of the working windows ---*/
	fprintf( fp, "# Working windows position and dimension\n" );
	save_wnd_infos( fp, wgt_wnd_palette, "Palette window" );
	save_wnd_infos( fp, wgt_wnd_tree, "Tree window" );
	save_wnd_infos( fp, wgt_wnd_resources, "Resources window" );

	fprintf( fp, "\n" );
	fprintf( fp, "Expand container (blank) = %s\n", 	
		(config.is_expand_container_blank) ? "yes" : "no" );

	fprintf( fp, "Tree display icon = %s\n", 	
		(config.tree_is_display_icon) ? "yes" : "no" );
	fprintf( fp, "Tree display GtkType = %s\n", 	
		(config.tree_is_display_gtktype) ? "yes" : "no" );
	fprintf( fp, "Callbacks display icon = %s\n", 	
		(config.callbacks_is_display_icon) ? "yes" : "no" );

	fprintf( fp, "\n" );
	fprintf( fp, "Initial width for module window = %d\n", config.initial_w_module_window );
	fprintf( fp, "Initial height for module window = %d\n", config.initial_h_module_window );
	fprintf( fp, "Initial width for module dialog = %d\n", config.initial_w_module_dialog );
	fprintf( fp, "Initial height for module dialog = %d\n", config.initial_h_module_dialog );
	if ( !save_project_specific )
	{
		fprintf( fp, "Editor[1] = %s\n", config.editor1 );
		fprintf( fp, "Editor[2] = %s\n", config.editor2 );
		fprintf( fp, "Editor[3] = %s\n", config.editor3 );
		fprintf( fp, "Goto line option after = %s\n", (config.is_glo_after) ? "yes" : "no" );
		fprintf( fp, "Goto line option = %s\n", config.goto_line_option );
	}
	fprintf( fp, "Selection mark type = %d\n", config.selection_mark_type );
	fprintf( fp, "Flag and list code generation = %d\n", config.flag_list_as );
	fprintf( fp, "Max source line = %d\n", config.max_source_line );

	fprintf( fp, "\n" );
	fprintf( fp, "-m lib = %s\n", (config.platform_is_lib_m) ? "yes" : "no" );

	fprintf( fp, "\n" );
	fprintf( fp, "Function indentation style = %d\n", config.indentstyle_function );
	fprintf( fp, "Block indentation style = %d\n", config.indentstyle_block );

	fprintf( fp, "\n" );
	fprintf( fp, "Set/Get resources method = %d\n", config.setgetres_method );
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	fprintf( fp, "Show composite childs = %s\n",
		(config.show_composite_childs) ? "yes" : "no" );
#endif

	/*--- Project specific ---*/
	if ( save_project_specific )
	{
		fprintf( fp, "Exe name = %s\n", config.exe_name );
		fprintf( fp, "Header to include = %s\n", config.header_to_include );
		fprintf( fp, "Is gnome = %s\n", (config.is_gnome) ? "yes" : "no" );
		switch (config.language_current)
		{
			case LANGUAGE_C		: fprintf( fp, "Language (current) = C\n" ); break;
#ifdef GLG_SUPPORT_LANGUAGECPP
			case LANGUAGE_CPP	: fprintf( fp, "Language (current) = C++\n" ); break;
#endif
			default				: assert(0);
		}
		for ( n = 0; n < MAXNB_CFILES_ADDED; n++ )
			fprintf( fp, "cfiles added [%d] = %s\n", n, config.list_cfiles_added[ n ] );
	}							// if
	switch (config.language_newprj)
	{
		case LANGUAGE_C		: fprintf( fp, "Language (new project) = C\n" ); break;
#ifdef GLG_SUPPORT_LANGUAGECPP
		case LANGUAGE_CPP	: fprintf( fp, "Language (new project) = C++\n" ); break;
#endif
		default				: assert(0);
	}

	/*--- Last opened projects ---*/
	if ( !save_project_specific )
	{
		fprintf( fp, "\n" );
		for ( n = 0; n < MAXNB_LAST_PROJECTS; n++ )
		{
			if ( last_projects[ n ] )
				fprintf( fp, "last projects [%d] = %s\n", n, last_projects[ n ] );
		}
	}							// if

	/*--- Ok, done ---*/
	fclose( fp );
	return 0;

}								// write_config


static int
test( int lineno, int *p, int vmin, int vmax )
{
	int	ok = TRUE;

	if ( *p < vmin )
	{
		fprintf( stderr, "Value out of range at line %d\n", lineno );
		*p = vmin;
		ok = FALSE;
	}

	if ( *p > vmax )
	{
		fprintf( stderr, "Value out of range at line %d\n", lineno );
		*p = vmax;
		ok = FALSE;
	}

	return ok;

}								// test


PUBLIC int 
read_config( const char * const filename, const int position_windows )
{
	char		*line;
	FILE		*fp;
	int			len;
	char		*token, *arg;
	int			w = gdk_screen_width();
	int			h = gdk_screen_height();
	int			lineno;

	/*--- Open config file ---*/	
	fp = fopen( filename, "r" );
	if ( fp == NULL )
		return -1;

	line = malloc( 32768 );
	assert( line != NULL );

	/*--- Initialize config ---*/
	create_default_config();

	/*--- Each line ---*/	
	lineno = 0;								   
	while ( fgets( line, 32767, fp ) )
	{

		lineno++;
		
		/*--- Skip line beginning with '#' ---*/
		if ( ((len = strlen(line)) >= 1) && (line[len-1] == '\n') )
			line[len-1] = 0;
		if ( !strcmp( line, "" ) || (line[0] == '#') )
			continue;
	
		token = strtok( line, "=" );
		if ( (len = strlen( token )) > 0 )
			token[ len-1 ] = 0;
		arg = strtok( NULL, "" );
		arg++;			// Skip space

		if ( !strcmp( token, "Palette window" ) )
		{
			if ( position_windows )
			{
				sscanf( arg, "%d,%d,%d,%d",
					&config.palette_wnd.x, &config.palette_wnd.y, 
					&config.palette_wnd.w, &config.palette_wnd.h );
				if ( test( lineno, &config.palette_wnd.x, 0, w - 1 ) &&
					 test( lineno, &config.palette_wnd.y, 0, h - 1 ) &&
					 test( lineno, &config.palette_wnd.w, 0, w - 1 ) &&
					 test( lineno, &config.palette_wnd.h, 0, h - 1 ) )
				{
					if ( wgt_wnd_palette  )
					{
						if ( is_restore_windows_position )
							gtk_widget_set_uposition( wgt_wnd_palette,
								config.palette_wnd.x-OFS_X, config.palette_wnd.y-OFS_Y );
						gtk_widget_set_usize( wgt_wnd_palette,
							-1, -1 );
						gtk_widget_set_usize( wgt_wnd_palette,
							config.palette_wnd.w, config.palette_wnd.h );
						continue;
					}
				}
			}
		}
		else if ( !strcmp( token, "Tree window" ) )
		{
			if ( position_windows )
			{
				sscanf( arg, "%d,%d,%d,%d",
					&config.tree_wnd.x, &config.tree_wnd.y,
					&config.tree_wnd.w, &config.tree_wnd.h );
				if ( test( lineno, &config.tree_wnd.x, 0, w - 1 ) &&
					 test( lineno, &config.tree_wnd.y, 0, h - 1 ) &&
					 test( lineno, &config.tree_wnd.w, 0, w - 1 ) &&
					 test( lineno, &config.tree_wnd.h, 0, h - 1 ) )
				{
					if ( wgt_wnd_tree  )
					{
						if ( is_restore_windows_position )
							gtk_widget_set_uposition( wgt_wnd_tree,
								config.tree_wnd.x-OFS_X, config.tree_wnd.y-OFS_Y );
						gtk_widget_set_usize( wgt_wnd_tree,
							-1, -1 );
						gtk_widget_set_usize( wgt_wnd_tree,
							config.tree_wnd.w, config.tree_wnd.h );
						continue;
					}
				}
			}
		}
		else if ( !strcmp( token, "Resources window" ) )
		{
			if ( position_windows )
			{
				sscanf( arg, "%d,%d,%d,%d",
					&config.resources_wnd.x, &config.resources_wnd.y,
					&config.resources_wnd.w, &config.resources_wnd.h );
				if ( test( lineno, &config.resources_wnd.x, 0, w - 1 ) &&
					 test( lineno, &config.resources_wnd.y, 0, h - 1 ) &&
					 test( lineno, &config.resources_wnd.w, 0, w - 1 ) &&
					 test( lineno, &config.resources_wnd.h, 0, h - 1 ) )
				{
					if ( wgt_wnd_resources  )
					{
						if ( is_restore_windows_position )
							gtk_widget_set_uposition( wgt_wnd_resources,
								config.resources_wnd.x-OFS_X, config.resources_wnd.y-OFS_Y );
						gtk_widget_set_usize( wgt_wnd_resources,
							-1, -1 );
						gtk_widget_set_usize( wgt_wnd_resources,
							config.resources_wnd.w, config.resources_wnd.h );
						continue;
					}
				}
			}
		}

		else if ( !strcmp( token, "Expand container (blank)" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.is_expand_container_blank = TRUE;
			else							config.is_expand_container_blank = FALSE;
			continue;
		}

		else if ( !strcmp( token, "Tree display icon" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.tree_is_display_icon = TRUE;
			else							config.tree_is_display_icon = FALSE;
			continue;
		}
		else if ( !strcmp( token, "Tree display GtkType" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.tree_is_display_gtktype = TRUE;
			else							config.tree_is_display_gtktype = FALSE;
			continue;
		}
		else if ( !strcmp( token, "Callbacks display icon" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.callbacks_is_display_icon = TRUE;
			else							config.callbacks_is_display_icon = FALSE;
			continue;
		}

		else if ( !strcmp( token, "Initial width for module window" ) )
		{
			config.initial_w_module_window = atoi( arg );
			continue;
		}
		else if ( !strcmp( token, "Initial height for module window" ) )
		{
			config.initial_h_module_window = atoi( arg );
			continue;
		}

		else if ( !strcmp( token, "Initial width for module dialog" ) )
		{
			config.initial_w_module_dialog = atoi( arg );
			continue;
		}
		else if ( !strcmp( token, "Initial height for module dialog" ) )
		{
			config.initial_h_module_dialog = atoi( arg );
			continue;
		}

		else if ( !strcmp( token, "Selection mark type" ) )
		{
			config.selection_mark_type = atoi( arg );
			continue;
		}
		else if ( !strcmp( token, "Editor[1]" ) )
		{
			strcpy( config.editor1, arg );
			continue;
		}
		else if ( !strcmp( token, "Editor[2]" ) )
		{
			strcpy( config.editor2, arg );
			continue;
		}
		else if ( !strcmp( token, "Editor[3]" ) )
		{
			strcpy( config.editor3, arg );
			continue;
		}
		else if ( !strcmp( token, "Goto line option after" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.is_glo_after = TRUE;
			else							config.is_glo_after = FALSE;
			continue;
		}
		else if ( !strcmp( token, "Goto line option" ) )
		{
			strcpy( config.goto_line_option, arg );
			continue;
		}
		else if ( !strcmp( token, "Flag and list code generation" ) )
		{
			config.flag_list_as = atoi( arg );
			continue;
		}
		else if ( !strcmp( token, "Max source line" ) )
		{
			config.max_source_line = atoi( arg );
			continue;
		}

		else if ( !strcmp( token, "-m lib" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.platform_is_lib_m = TRUE;
			else							config.platform_is_lib_m = FALSE;
			continue;
		}

		else if ( !strcmp( token, "Function indentation style" ) )
		{
			switch ( atoi(arg) )
			{
				case 1 : config.indentstyle_function = 1; break;
				case 2 : config.indentstyle_function = 2; break;
			}
			continue;
		}

		else if ( !strcmp( token, "Block indentation style" ) )
		{
			switch ( atoi(arg) )
			{
				case 1 : config.indentstyle_block = INDENTSTYLE_BLOCK1; break;
				case 2 : config.indentstyle_block = INDENTSTYLE_BLOCK2; break;
				case 3 : config.indentstyle_block = INDENTSTYLE_BLOCK3; break;
			}
			continue;
		}

		else if ( !strcmp( token, "Set/Get resources method" ) )
		{
			config.setgetres_method = atoi( arg );
			continue;
		}

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
		else if ( !strcmp( token, "Show composite childs" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.show_composite_childs = TRUE;
			else							config.show_composite_childs = FALSE;
			continue;
		}
#endif

		/*--- Project specific ---*/
		else if ( !strcmp( token, "Exe name" ) )
		{
			strcpy( config.exe_name, arg );
			continue;
		}
		else if ( !strcmp( token, "Header to include" ) )
		{
			strcpy( config.header_to_include, arg );
			continue;
		}
		else if ( !strcmp( token, "Is gnome" ) )
		{
			if ( !strcmp( arg, "yes" ) )	config.is_gnome = TRUE;
			else							config.is_gnome = FALSE;
			continue;
		}
		else if ( !strcmp( token, "Language (current)" ) )
		{
			if ( !strcmp( arg, "C" ) )			config.language_current = LANGUAGE_C;
#ifdef GLG_SUPPORT_LANGUAGECPP
			else if ( !strcmp( arg, "C++" ) )	config.language_current = LANGUAGE_CPP;
#endif
			else								assert(0);
			continue;
		}
		else if ( !strcmp( token, "Language (new project)" ) )
		{
			if ( !strcmp( arg, "C" ) )			config.language_newprj = LANGUAGE_C;
#ifdef GLG_SUPPORT_LANGUAGECPP
			else if ( !strcmp( arg, "C++" ) )	config.language_newprj = LANGUAGE_CPP;
#endif
			else								assert(0);
			continue;
		}
		else if ( !strncmp( token, "cfiles added ", 13 ) )
		{
			int		found, n;
			char	tmp[ 80 ];
			for ( found = FALSE, n = 0; n < MAXNB_CFILES_ADDED; n++ )
			{
				sprintf( tmp, "cfiles added [%d]", n );
				if ( !strcmp( token, tmp ) )
				{
					strcpy( config.list_cfiles_added[ n ], arg );
					found = TRUE;
				}
			}
			if ( found )
				continue;
		}
		else if ( !strncmp( token, "last projects ", 14 ) )
		{
			int		found, n;
			char	tmp[ 80 ];
			for ( found = FALSE, n = 0; n < MAXNB_LAST_PROJECTS; n++ )
			{
				sprintf( tmp, "last projects [%d]", n );
				if ( !strcmp( token, tmp ) )
				{
					last_projects[ n ] = strdup( arg );
					found = TRUE;
				}
			}
			if ( found )
				continue;
		}

	}							// while

	/*--- Ok, done ---*/
	fclose( fp );
	free( line );
	return 0;

}								// read_config
