#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


PUBLIC int
give_a_chance_to_save( void )
{

	if ( is_project_touched && (nb_modules() > 0) )
	{
		int answer = dialog3( "Exit", 1,
			"Yes", "No", "Cancel",
			"Save current project before exiting ?" );
		if (answer == 3)		// "Cancel"
		{
//			is_project_touched = FALSE;
			return -1;
		}
		else if (answer == 1)	// "Yes"
		{
			save_project();
		}
		else
		{
			is_project_touched = FALSE;
		}
	}
	
	write_config( config_name, FALSE );
	
	return 0;

}								 // give_a_chance_to_save



PUBLIC gint
cb_close_app( GtkWidget * widget, gpointer data )
{

//	if ( give_a_chance_to_save() == 0 )
//		gtk_main_quit();

	return FALSE;
}								// cb_close_app


PUBLIC void
reset_styles( void )
{
	int		n;

	if ( nb_styles > 0 )
	{
		for ( n = 0; n < nb_styles; n++ )
			free( styles[ n ] );
		nb_styles = 0;
		free( styles );
		styles = NULL;
		free( styles_infos );
		styles_infos = NULL;
	}

}								// reset_styles


PUBLIC int
clear_project(char *title)
{
	module_info_t	*module;

	if ( is_project_touched && (nb_modules() > 0) )
	{
		int answer = dialog3( title, 1,
			"Yes", "No", "Cancel",
			"Save current project before reset ?" );
		if (answer == 3)		// "Cancel"
			return -1;
		else if (answer == 1)	// "Yes"
		{
			save_project();
		}
	}

	/*--- Reset any messages displayed into the warning area ---*/
	clear_warning();

	/*--- Clear styles ---*/
	reset_styles();
	
	/*--- Initialize config ---*/
	create_default_config();

	/*--- No more widget selected ---*/
	reset_selection();

	/*--- Clear properties windows ---*/
	clear_resources( TRUE );

	/*--- Current project name ---*/
	strcpy( project_dirname, "" );

	/*--- Counters used when creating new modules ---*/
	cpt_module_window = 0;
	cpt_module_dialog = 0;
	cpt_module_menu = 0;

	/*--- Now project is not to save ---*/
	is_project_touched = FALSE;

	/*--- Cycle for each defined module ---*/
	if (wgt_tree_modules)
	{
		module = next_module( FIRST );
		while (module)
		{
			switch ( module->type )
			{
				case MODULE_TYPE_WINDOW :
				case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
				case MODULE_TYPE_GNOMEDIALOG_DIALOG :
				case MODULE_TYPE_GNOMEDIALOG_ABOUT :
				case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
				case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
					gtk_widget_destroy( module->wgt_window );
					break;
				case MODULE_TYPE_MENU :
					gtk_widget_destroy( module->u1.menu.menudlg.wgt_showmenu );
					if ( module->u1.menu.menued.wgt_menued_window != NULL )
						gtk_widget_destroy( module->u1.menu.menued.wgt_menued_window );
					break;
				default :
					assert( 0 );
			}					// switch
			module = next_module(NEXT);
		}						// while
		gtk_clist_clear( GTK_CLIST(wgt_tree_modules) );
	}							// if

	/*--- Clear now the tree window ---*/
	gtk_clist_clear( GTK_CLIST( wgt_tree_widgets ) );
	gtk_clist_clear( GTK_CLIST( wgt_tree_callbacks ) );
	gtk_clist_clear( GTK_CLIST( wgt_tree_styles ) );

	/*--- Make the palette widget insensitive ---*/
	set_palette_sensitive( FALSE );

	/*--- Update main menu ---*/
	update_main_menu();
	
	return 0;

}								// clear_project


static void
cb_modulemenu_raise( GtkMenuItem *menuitem, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;

	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
		case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
		case MODULE_TYPE_GNOMEDIALOG_DIALOG :
		case MODULE_TYPE_GNOMEDIALOG_ABOUT :
		case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
		case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			gdk_window_raise( module->wgt_window->window );
			break;
		case MODULE_TYPE_MENU :
			gdk_window_raise( module->u1.menu.menudlg.wgt_showmenu->window );
			break;
		default :
			assert( 0 );
	}							// switch

	module->iconified = FALSE;
	project_touched( module );
	
}								// cb_modulemenu_raise


static void
cb_modulemenu_lower( GtkMenuItem *menuitem, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;

	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
		case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
		case MODULE_TYPE_GNOMEDIALOG_DIALOG :
		case MODULE_TYPE_GNOMEDIALOG_ABOUT :
		case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
		case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			gdk_window_lower( module->wgt_window->window );
			break;
		case MODULE_TYPE_MENU :
			gdk_window_lower( module->u1.menu.menudlg.wgt_showmenu->window );
			break;
		default :
			assert( 0 );
	}							// switch

	module->iconified = TRUE;
	project_touched( module );
	
}								// cb_modulemenu_lower


static void
cb_modulemenu_touch( GtkMenuItem *menuitem, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;

	project_touched( module );
	
}								// cb_modulemenu_touch


PUBLIC void
touch_all_modules( void )
{
	module_info_t	*module;
	
	module = next_module( FIRST );
	while ( module )
	{
		project_touched( module );
		module = next_module( NEXT );
	}							// while (module)

	is_project_touched = TRUE;

}								// touch_all_modules


static void
cb_modulemenu_touchall( GtkMenuItem *menuitem, gpointer user_data )
{
	touch_all_modules();
}								// cb_modulemenu_touchall


static void
cb_modulemenu_delete( GtkMenuItem *menuitem, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;
	int				status;

	status = yesno_dialog( "Delete Module", YES,
		"\nDelete module %s ?\n", get_name_wgt( module->wgt_window ) );
	if ( status == NO )
		return;

	gtk_ctree_remove_node( GTK_CTREE(wgt_tree_modules),
		GTK_CTREE_NODE( module->wgt_tree_node ) );
	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
		case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
		case MODULE_TYPE_GNOMEDIALOG_DIALOG :
		case MODULE_TYPE_GNOMEDIALOG_ABOUT :
		case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
		case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			gtk_widget_destroy( module->wgt_window );
			break;
		case MODULE_TYPE_MENU :
			gtk_widget_destroy( module->u1.menu.menudlg.wgt_showmenu );
			if ( module->u1.menu.menued.wgt_menued_window != NULL )
				gtk_widget_destroy( module->u1.menu.menued.wgt_menued_window );
			break;
		default :
			assert( 0 );
	}							// switch

	touch_all_modules();
	module = next_module( FIRST );

	/*--- Give focus the selected module window ---*/
	if ( module != NULL )
	{
		gtk_window_set_focus( GTK_WINDOW( module->wgt_window ), NULL );
		gtk_widget_grab_focus( GTK_WIDGET(module->wgt_window) );
	}
		
	if ( curr_module != module )
	{

		/*--- Update the tree of widgets for this module ---*/
		curr_module = module;
		update_tree();
		if ( module != NULL )
		{
			select_widget( module->wgt_window, 
				TRUE, 
				TRUE );	   		// Select 1st CB, if any
		}						// if

	}

}								// cb_modulemenu_delete


static void
cb_modulemenu_properties( GtkMenuItem *menuitem, gpointer user_data )
{
	module_info_t	*module = (module_info_t *)user_data;

	switch ( module->type )
	{
		case MODULE_TYPE_WINDOW :
		case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
		case MODULE_TYPE_GNOMEDIALOG_DIALOG :
		case MODULE_TYPE_GNOMEDIALOG_ABOUT :
		case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
		case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
			module_window_properties( module );
			break;
		case MODULE_TYPE_MENU :
			module_menu_properties( module );
			break;
		default :
			assert( 0 );
	}							// switch

}								// cb_modulemenu_properties



static void
module_select_popup_menu( module_info_t *module, GdkEventButton *event )
{
	GtkWidget		*wgt_modulemenu_raise, *wgt_modulemenu_lower;
	GtkWidget		*wgt_modulemenu_touch; 
	GtkWidget		*wgt_modulemenu_touchall; 
	GtkWidget		*wgt_modulemenu_delete; 
	GtkWidget		*wgt_modulemenu_properties; 
	module_info_t	*m;
	int				f_all_touched;

	if ( wgt_popup_menu )
		gtk_widget_destroy( wgt_popup_menu );	
	wgt_popup_menu = gtk_menu_new();

	/*--- Are all modules touched ? ---*/	
	f_all_touched = TRUE;
	m = next_module( FIRST );
	while ( m )
	{
		if ( m->f_not_saved )
		{
			f_all_touched = FALSE;
			break;
		}
		m = next_module( NEXT );
	}							// while (module)

	wgt_modulemenu_raise = create_menu_item( wgt_popup_menu, "Raise", 
		(module->iconified) ? TRUE : FALSE, 
		0, 0, cb_modulemenu_raise, module );
	wgt_modulemenu_lower = create_menu_item( wgt_popup_menu, "Lower", 
		((module->u1.menu.menued.wgt_menued_window == NULL) && (!module->iconified)) ? TRUE : FALSE,
		0, 0, cb_modulemenu_lower, module );
	create_menu_sep( wgt_popup_menu );
	
	wgt_modulemenu_touch = create_menu_item( wgt_popup_menu, "Touch", 
		(module->f_not_saved) ? FALSE : TRUE, 
		0, 0, cb_modulemenu_touch, module );
	wgt_modulemenu_touchall = create_menu_item( wgt_popup_menu, "Touch all modules", 
		(f_all_touched) ? FALSE : TRUE, 
		0, 0, cb_modulemenu_touchall, NULL );
	create_menu_sep( wgt_popup_menu );
	wgt_modulemenu_delete = create_menu_item( wgt_popup_menu, "Delete this module", 
		TRUE, 0, 0, cb_modulemenu_delete, module );
	create_menu_sep( wgt_popup_menu );
	wgt_modulemenu_properties = create_menu_item( wgt_popup_menu, "Properties", 
		TRUE, 0, 0, cb_modulemenu_properties, module );

	gtk_menu_popup( GTK_MENU(wgt_popup_menu), NULL, NULL, NULL, NULL,
		event->button, event->time );

}								// module_select_popup_menu


PUBLIC int 
cb_project_module_button_press_event( GtkCTree *ctree, GdkEventButton *event, gpointer data )
{
	GtkCTreeNode	*node;
	gint			row, col;
	module_info_t	*module;

	gtk_clist_get_selection_info (GTK_CLIST (ctree), event->x, event->y,
		&row, &col);
	node = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (ctree)->row_list, row));
	if (node == NULL)
		return FALSE;

	if (event->button != 1)
		gtk_ctree_select( GTK_CTREE(ctree), GTK_CTREE_NODE(node) );
	module = gtk_ctree_node_get_row_data( ctree, node );
	assert( module != NULL );

	/*--- Give focus the selected module window ---*/
	gtk_window_set_focus( GTK_WINDOW( module->wgt_window ), NULL );
	gtk_widget_grab_focus( GTK_WIDGET(module->wgt_window) );
		
	if ( curr_module != module )
	{

		curr_module = module;
		update_tree();

		select_widget( module->wgt_window, 
			TRUE, 
			TRUE );				// Select 1st CB, if any

		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				gdk_window_raise( module->wgt_window->window );
				break;
			case MODULE_TYPE_MENU :
				gdk_window_raise( module->u1.menu.menudlg.wgt_showmenu->window );
				if ( module->u1.menu.menued.wgt_menued_window != NULL )
					gdk_window_raise( module->u1.menu.menued.wgt_menued_window->window );
				break;
			default :
				assert( 0 );
		}					// switch
		module->iconified = FALSE;
		
	}

	if (event->button == 3)
	{
		module_select_popup_menu( module, event );
		return TRUE;
	}

	return FALSE;
  	
}								// cb_project_module_button_press_event


PUBLIC void
project_touched( module_info_t *module )
{
	char			str[1024];

	is_project_touched = TRUE;

	if ( module->wgt_tree_node )
	{
		gtk_wgtdef_t	*wgtdef;

		wgtdef = get_object_data( GTK_WIDGET(module->wgt_window), WGTDATA_WGTDEF );
		assert( wgtdef != NULL );

		if ( !module->f_not_saved )
		{
			sprintf( str, "*%s", get_name_wgt(module->wgt_window) );
			gtk_ctree_set_node_info( GTK_CTREE(wgt_tree_modules),
				GTK_CTREE_NODE(module->wgt_tree_node), 
				str, 5,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				FALSE, FALSE );
		}
		else if ( !module->f_not_generated )
		{
			sprintf( str, "!%s", get_name_wgt(module->wgt_window) );
			gtk_ctree_set_node_info( GTK_CTREE(wgt_tree_modules),
				GTK_CTREE_NODE(module->wgt_tree_node),
				str, 5,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				FALSE, FALSE );
		}
	}							// if

	module->f_not_saved = TRUE;
	module->f_not_generated = TRUE;

}								// project_touched


PUBLIC void
project_saved( module_info_t *module )
{

	if ( module->f_not_saved )
	{
		if ( module->wgt_tree_node )
		{
			char			str[1024];
			gtk_wgtdef_t	*wgtdef;

			wgtdef = get_object_data( GTK_WIDGET(module->wgt_window), WGTDATA_WGTDEF );
			assert( wgtdef != NULL );

			sprintf( str, "%s%s", 
				(module->f_not_generated) ? "!" : "",
				get_name_wgt( module->wgt_window ) );
			gtk_ctree_set_node_info( GTK_CTREE(wgt_tree_modules),
				GTK_CTREE_NODE(module->wgt_tree_node),
				str, 5,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				FALSE, FALSE );
		}
		module->f_not_saved = FALSE;

	}							// if

}								// project_saved


PUBLIC void
project_generated( module_info_t *module )
{

	if ( module->f_not_generated )
	{

		if ( module->wgt_tree_node )
		{
			gtk_wgtdef_t	*wgtdef;

			wgtdef = get_object_data( GTK_WIDGET(module->wgt_window), WGTDATA_WGTDEF );
			assert( wgtdef != NULL );

			gtk_ctree_set_node_info( GTK_CTREE(wgt_tree_modules),
				GTK_CTREE_NODE(module->wgt_tree_node),
				get_name_wgt( module->wgt_window ), 5,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				wgtdef->gdkpixmap, wgtdef->gdkbitmap,
				FALSE, FALSE );
		}
		module->f_not_saved = FALSE;

	}							// if

}								// project_generated

