#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

#define DO_VALUE	"<value>"
#define DO_LOWER	"<lower>"
#define DO_UPPER	"<upper>"


typedef struct
{
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
	GtkWidget		*wgt_spin_value;
	GtkWidget		*wgt_spin_lower;
	GtkWidget		*wgt_spin_upper;
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	gfloat			value;
	GtkWidget		*wgt_parent;
	int				first;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
	char			*which;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;
	GtkArg					arg;
	GtkAdjustment			*padj=NULL;

	/*--- Set the resource of the widget ---*/
	if ( user->defclass->saved_where == SAVED_INSIDE )
	{
		arg.type = user->defclass->restype;
		arg.name = user->defclass->resname;
		gtk_widget_get( GTK_WIDGET( wgt ), &arg );
		padj = GTK_VALUE_POINTER( arg );
	}
	else if ( (user->defclass->saved_where == SAVED_OUTSIDE) ||
			 (user->defclass->saved_where == SAVED_OUTSIDE_FUNCTION) )
	{
		assert( 0 );
	}
	else if (user->defclass->saved_where == SAVED_FUNCTION)
	{
		(*user->wgtdef_depth->pfct_getdlg)(user->defclass, wgt, &arg);
		padj = GTK_VALUE_POINTER( arg );
	}
	else assert(0);

	if ( !strcmp( user->which, DO_VALUE ) )
		padj->value = user->value;
	else if ( !strcmp( user->which, DO_LOWER ) )
		padj->lower = user->value;
	else if ( !strcmp( user->which, DO_UPPER ) )
		padj->upper = user->value;
	else
		assert( 0 );
	gtk_adjustment_value_changed( padj );

}								// foreach_selection


static void
value_changed( GtkWidget *wgt_spin, defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	char *which )
{
	foreach_selection_data	data;
	GtkWidget				*wgt_selected;
	module_info_t			*module;
	char					*str;

	str = gtk_editable_get_chars( GTK_EDITABLE( wgt_spin ), 0, -1 );
	data.value   	  = atof( str );
	data.defclass 	  = defclass;
	data.wgt_parent   = wgt_parent;
	data.first		  = TRUE;
	data.wgtdef		  = wgtdef;
	data.wgtdef_depth = wgtdef_depth;
	data.which		  = which;
	g_slist_foreach( list_selection, foreach_selection, &data );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	/*--- Update the menu editor, if displayed ---*/
	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
	check_for_composite_childs( module->wgt_window );

}								// value_changed


static void
cb_spin_value_changed( GtkEditable *wgt_editable, gpointer user_data )
{
	char			*which = (char *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( GTK_WIDGET( wgt_editable ), "entry_data" );
	assert( entry_data != NULL );
	value_changed( GTK_WIDGET( wgt_editable ), entry_data->defclass,
		entry_data->wgt_parent, 
		entry_data->wgtdef, entry_data->wgtdef_depth,
		which );

}								// cb_spin_value_changed


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
//	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_adjustment( gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget		*wgt_selected;
	GtkArg			arg, *parg;
	GtkWidget		*wgt_hbox0;
	GtkWidget		*wgt_hbox;
	GtkWidget		*wgt_label_value, *wgt_spin_value;
	GtkAdjustment	*adj_value;
	GtkWidget		*wgt_label_lower, *wgt_spin_lower;
	GtkAdjustment	*adj_lower;
	GtkWidget		*wgt_label_upper, *wgt_spin_upper;
	GtkAdjustment	*adj_upper;
	entry_data_t	*entry_data;
	GtkAdjustment	*padj;
	GtkWidget		*wgt_alignment;
	GtkWidget		*wgt_btn_dialog;
	GtkTooltips 	*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	if ( defclass->saved_where == SAVED_INSIDE )
	{
		arg.name = defclass->resname;
		if ( !wgt_parent )
			gtk_widget_get( GTK_WIDGET(wgt_selected), &arg );
		else
			gtk_container_child_getv( GTK_CONTAINER( wgt_parent ), 
				GTK_WIDGET(wgt_selected), 1, &arg );
		parg = &arg;
	}
	else if ( defclass->saved_where == SAVED_OUTSIDE ||
			 defclass->saved_where == SAVED_OUTSIDE_FUNCTION )
	{
		parg = get_outsideres_wgt( wgt_selected, defclass->resname );
		assert( parg != NULL );
	}
	else if ( defclass->saved_where == SAVED_FUNCTION )
	{
		if ( wgt_parent != NULL )
			assert(0);
		if ( (*wgtdef_depth->pfct_getdlg)(defclass, wgt_selected, &arg) == NULL )
			parg = get_outsideres_wgt( wgt_selected, defclass->resname );
		else
			parg = &arg;
		assert( parg != NULL );
	}
	else assert(0);

	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX(wgt_container), wgt_hbox0, FALSE, FALSE, 0 );

	/*--- Create an horizontal box to store the value, lower, upper ---*/
	wgt_hbox = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX(wgt_hbox0), wgt_hbox, FALSE, FALSE, 0 );

	padj = GTK_VALUE_POINTER( arg );

#define CREATE( wgt_label, label, adj, wgt_spin, which, value ) \
	wgt_label = gtk_label_new( label ); \
	gtk_box_pack_start( GTK_BOX( wgt_hbox ), wgt_label, FALSE, FALSE, 2 ); \
\
    adj = (GtkAdjustment *) gtk_adjustment_new( value, -MAXFLOAT, MAXFLOAT, 0, 0, 0 ); \
	wgt_spin = gtk_spin_button_new( adj, 1, 0 ); \
	INIT_ARGS; \
	SETARG_INT( "width", 48 ); \
	gtk_widget_setv( GTK_WIDGET(wgt_spin), nb_args, args ); \
	gtk_box_pack_start( GTK_BOX( wgt_hbox ), wgt_spin, FALSE, FALSE, 4 ); \
\
	gtk_signal_connect_after( GTK_OBJECT( wgt_spin ), "changed", \
		GTK_SIGNAL_FUNC( cb_spin_value_changed ), which )

	CREATE( wgt_label_value, "Value", adj_value, wgt_spin_value, DO_VALUE, padj->value );
	CREATE( wgt_label_lower, "Lower", adj_lower, wgt_spin_lower, DO_LOWER, padj->lower );
	CREATE( wgt_label_upper, "Upper", adj_upper, wgt_spin_upper, DO_UPPER, padj->upper );
	
#undef CREATE

	/*--- Button to bring a dialog ---*/
	wgt_alignment = gtk_alignment_new( 1, 0.5, 0, 1 );
	gtk_container_add (GTK_CONTAINER( wgt_hbox0 ), wgt_alignment );
	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_alignment, nb_args, args );
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add (GTK_CONTAINER( wgt_alignment ), wgt_btn_dialog );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the adjustment", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );
	
	entry_data = malloc( sizeof( entry_data_t ) );
	memset( entry_data, 0, sizeof( entry_data_t ) );
	entry_data->defclass	 = defclass;
	entry_data->wgt_parent	 = wgt_parent;
	entry_data->wgtdef		 = wgtdef;
	entry_data->wgtdef_depth = wgtdef_depth;
	set_object_data( wgt_spin_value, "entry_data", entry_data, TRUE );
	set_object_data( wgt_spin_lower, "entry_data", entry_data, FALSE );
	set_object_data( wgt_spin_upper, "entry_data", entry_data, FALSE );

	return wgt_hbox0;

}								// res_editor_adjustment
