#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


typedef struct
{
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	GtkWidget		*wgt_radiobtn_yes, *wgt_radiobtn_no;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	gboolean		state;
	GtkWidget		*wgt_parent;
	int				first;
	GtkWidget		*wgt_radiobtn_yes, *wgt_radiobtn_no;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;
	GtkArg					arg2;
	int						v2;

	/*--- Set the resource of the widget ---*/
	set_res_bool( wgt, user->state, 
		user->wgtdef, user->wgtdef_depth,
		user->defclass, user->wgt_parent );

	/*--- Reread value from the widget, change ? ---*/
	if ( user->first && (user->defclass->saved_where == SAVED_INSIDE) )
	{
		arg2.name = user->defclass->resname;
		if ( !user->wgt_parent )
			gtk_widget_get( GTK_WIDGET(wgt), &arg2 );
		else
			gtk_container_child_getv( GTK_CONTAINER(user->wgt_parent), GTK_WIDGET( wgt ),
				1, &arg2 );
		v2 = GTK_VALUE_BOOL(arg2);
		if ( user->state != v2 )
		{
			if ( v2 )
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(user->wgt_radiobtn_yes), TRUE );
			else
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(user->wgt_radiobtn_no ), TRUE );
		}
		user->first = FALSE;
	}
	else if ( user->first && 
		   ( (user->defclass->saved_where == SAVED_OUTSIDE) || (user->defclass->saved_where == SAVED_OUTSIDE_FUNCTION) ) )
	{
		GtkArg	*parg;
		parg = get_outsideres_wgt( wgt, user->defclass->resname );
		assert( parg != NULL );
		GTK_VALUE_BOOL(parg[0]) = user->state;
	}
	else
	{
		GtkArg	arg;

		GTK_VALUE_BOOL( arg ) = user->state;
		(*user->wgtdef_depth->pfct_putdlg)( user->defclass, wgt, &arg );
	}

}								// foreach_selection


static void
bool_changed( GtkWidget *wgt_radiobtn_yes, GtkWidget *wgt_radiobtn_no,
	GtkWidget *wgt, defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth )
{
	foreach_selection_data	data;
	GtkWidget				*wgt_selected;
	module_info_t			*module;

	data.state 			  = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( wgt ) );
	data.defclass 		  = defclass;
	data.wgt_parent 	  = wgt_parent;
	data.first			  = TRUE;
	data.wgt_radiobtn_yes = wgt_radiobtn_yes;
	data.wgt_radiobtn_no  = wgt_radiobtn_no;
	data.wgtdef		  	  = wgtdef;
	data.wgtdef_depth 	  = wgtdef_depth;
	g_slist_foreach( list_selection, foreach_selection, &data );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	/*--- Update the menu editor, if displayed ---*/
	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
	check_for_composite_childs( module->wgt_window );

}								// bool_changed


static void
cb_editor_radiobutton_toggled( GtkToggleButton *wgt_radiobtn, gpointer user_data )
{
	GtkWidget		*wgt_entry = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_entry, "entry_data" );
	assert( entry_data != NULL );
	bool_changed( entry_data->wgt_radiobtn_yes, entry_data->wgt_radiobtn_no,
		GTK_WIDGET(wgt_radiobtn), entry_data->defclass,
		entry_data->wgt_parent,
		entry_data->wgtdef, entry_data->wgtdef_depth );
		
}								// cb_editor_radiobutton_toggled


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_btn_dialog, "entry_data" );
	assert( entry_data != NULL );
	dlg_res_editor_bool( wgt_btn_dialog,
		entry_data->defclass,
		entry_data->wgt_parent,
		entry_data->wgtdef, entry_data->wgtdef_depth );

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_bool( gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget		*wgt_selected;
	GtkArg			arg, *parg;
	GtkWidget		*wgt_hbox0;
	GtkWidget		*wgt_hbox;
	GSList			*hbox1_group = NULL;
	entry_data_t	*entry_data;
	GtkWidget		*wgt_radiobtn_yes, *wgt_radiobtn_no;
	GtkWidget		*wgt_alignment;
	GtkWidget		*wgt_btn_dialog;
	GtkTooltips 	*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	assert( wgt_selected != NULL );
	if ( get_res_bool( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg, &parg ) == -1 )
		return NULL;

	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_container ), wgt_hbox0, FALSE, FALSE, 0 );

	/*--- Create an horizontal box to store the 2 toggle buttons 'yes' 'no' ---*/
	wgt_hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start( GTK_BOX( wgt_hbox0 ), wgt_hbox, FALSE, FALSE, 0 );

	wgt_radiobtn_yes = gtk_radio_button_new_with_label (hbox1_group, "Yes");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (wgt_radiobtn_yes));
	gtk_box_pack_start( GTK_BOX( wgt_hbox ), wgt_radiobtn_yes, FALSE, FALSE, 8 );

	wgt_radiobtn_no = gtk_radio_button_new_with_label (hbox1_group, "No");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (wgt_radiobtn_no));
	gtk_box_pack_start (GTK_BOX (wgt_hbox), wgt_radiobtn_no, FALSE, FALSE, 0);

	/*--- Button to bring a dialog ---*/
	wgt_alignment = gtk_alignment_new( 1, 0.5, 0, 1 );
	gtk_container_add (GTK_CONTAINER( wgt_hbox0 ), wgt_alignment );
	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_alignment, nb_args, args );
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add (GTK_CONTAINER( wgt_alignment ), wgt_btn_dialog );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the value", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );
	
	/*--- Data attached to the dialog ---*/	
	entry_data = malloc( sizeof( entry_data_t ) );
	memset( entry_data, 0, sizeof( entry_data_t ) );
	entry_data->defclass   		 = defclass;
	entry_data->wgt_parent 	     = wgt_parent;
	entry_data->wgt_radiobtn_yes = wgt_radiobtn_yes;
	entry_data->wgt_radiobtn_no  = wgt_radiobtn_no;
	entry_data->wgtdef		 	 = wgtdef;
	entry_data->wgtdef_depth 	 = wgtdef_depth;
	set_object_data( wgt_radiobtn_yes, "entry_data", entry_data, TRUE );
	set_object_data( wgt_radiobtn_no, "entry_data", entry_data, FALSE );
	set_object_data( wgt_btn_dialog, "entry_data", entry_data, FALSE );

	/*--- Show current value: Yes or No ---*/
	if ( GTK_VALUE_BOOL(parg[0]) )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_radiobtn_yes), TRUE );
	else
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_radiobtn_no ), TRUE );

	gtk_signal_connect( GTK_OBJECT( wgt_radiobtn_yes ), "toggled",
		GTK_SIGNAL_FUNC( cb_editor_radiobutton_toggled ), wgt_radiobtn_yes );
	
	return wgt_radiobtn_yes;
}								// res_editor_bool
