#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


typedef struct
{
	defclass_t			*defclass;
	GtkWidget			*wgt_parent;
	GtkEnumValue const	*penum;
	gtk_wgtdef_t		*wgtdef;
	gtk_wgtdef_t		*wgtdef_depth;
	GtkCheckMenuItem	**list_items;
	GtkEnumValue const	*enums;
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	int				first;
	GtkWidget		*wgt_optionmenu;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
	gint			flags;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;
	gint					flags = user->flags;

	/*--- Set the resource of the widget ---*/
	set_res_enum( wgt, flags, user->wgtdef, user->wgtdef_depth, user->defclass,
		user->wgt_parent );

}								// foreach_selection


static void
menuitem_changed( GtkWidget *wgt_optionmenu, defclass_t *defclass, 
	GtkWidget *wgt_parent, 
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	gint flags )
{
	foreach_selection_data	data;
	GtkArg					arg;
	GtkWidget				*wgt_selected;
	module_info_t			*module;

	GTK_VALUE_FLAGS( arg ) = flags;
	data.defclass 		= defclass;
	data.wgt_parent 	= wgt_parent;
	data.first			= TRUE;
	data.wgt_optionmenu	= wgt_optionmenu;
	data.wgtdef		  	= wgtdef;
	data.wgtdef_depth 	= wgtdef_depth;
	data.flags			= flags;
	g_slist_foreach( list_selection, foreach_selection, &data );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	/*--- Update the menu editor, if displayed ---*/
	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
	check_for_composite_childs( module->wgt_window );

}								// menuitem_changed


static void
cb_editor_menuitem_toggled( GtkWidget *widget, gpointer user_data )
{
	GtkWidget			*wgt_entry = (GtkWidget *)user_data;
	entry_data_t		*entry_data;
	GtkCheckMenuItem	**p;
	gint				flags;
	int					n;
	GtkEnumValue const	*penum;

	entry_data = (entry_data_t *)get_object_data( wgt_entry, "entry_data" );
	assert( entry_data != NULL );

	/*--- Recompute the whole flags value ---*/
	for ( flags = 0, n = 0, p = entry_data->list_items, penum = entry_data->enums;
		  penum->value_name; 
		  penum++, p++, n++ )
	{
		if ( (*p)->active )
			flags |= penum->value;
	}

	menuitem_changed( wgt_entry, entry_data->defclass, entry_data->wgt_parent, 
		entry_data->wgtdef, entry_data->wgtdef_depth,
		flags );

}								// cb_editor_menuitem_toggled


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;

	printf( "%s %d - Dialog not yet implemented\n", __FILE__, __LINE__ );

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_flags( gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget			*wgt_selected;
	GtkArg				arg, *parg=NULL;
	GtkWidget			*wgt_optionmenu;
	entry_data_t		*entry_data;
	GtkWidget			*wgt_menu;
	GtkWidget			*wgt_menuitem;
	GtkEnumValue const	*penum;
	guint				flags;
	int					nb, first;
	GtkCheckMenuItem	**list_items;
	GtkWidget			*wgt_hbox0;
	GtkWidget			*wgt_btn_dialog;
	GtkTooltips 		*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	get_res_flags( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent,
		&arg, &parg );

	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_container ), wgt_hbox0, FALSE, FALSE, 0 );

	wgt_optionmenu = gtk_option_menu_new();
	wgt_menu = gtk_menu_new();

	for ( nb = 0, penum = defclass->enum_list; penum->value_name; penum++ )
		nb++;
	list_items = malloc( sizeof( GtkCheckMenuItem * ) * (nb + 1) );
	list_items[ nb ] = NULL;
 
	/*--- Create the menu (choice) to be attached to the option menu ---*/
	flags = GTK_VALUE_FLAGS( parg[0] );
	for ( nb = 0, first = TRUE, penum = defclass->enum_list; penum->value_name; penum++ )
	{
		entry_data = malloc( sizeof( entry_data_t ) );
		memset( entry_data, 0, sizeof( entry_data_t ) );
		entry_data->defclass   	 = defclass;
		entry_data->wgt_parent 	 = wgt_parent;
		entry_data->wgtdef		 = wgtdef;
		entry_data->wgtdef_depth = wgtdef_depth;
		entry_data->penum      	 = penum;
		entry_data->list_items	 = list_items;
		entry_data->enums      	 = defclass->enum_list;

		wgt_menuitem = gtk_check_menu_item_new_with_label( penum->value_name  );
		list_items[ nb++ ] = (GtkCheckMenuItem *)wgt_menuitem;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( wgt_menuitem ),
			((flags & penum->value) == penum->value ) ? TRUE : FALSE );		
		gtk_menu_append( GTK_MENU(wgt_menu), wgt_menuitem );
		gtk_signal_connect( GTK_OBJECT(wgt_menuitem), "toggled",
			GTK_SIGNAL_FUNC( cb_editor_menuitem_toggled ), wgt_menuitem );
		set_object_data( wgt_menuitem, "entry_data", entry_data, TRUE );
		if ( first )
		{
			set_object_data( wgt_menuitem, "list_items", list_items, TRUE );
			first = FALSE;
		}
		
	}
	
	gtk_option_menu_set_menu( GTK_OPTION_MENU(wgt_optionmenu), GTK_WIDGET(wgt_menu) );
	gtk_box_pack_start( GTK_BOX( wgt_hbox0 ), wgt_optionmenu, FALSE, FALSE, 0 );
	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_optionmenu, nb_args, args );

	/*--- Button to bring a dialog ---*/
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog );
	INIT_ARGS;
	SETARG_BOOL( "expand", FALSE );
	SETARG_BOOL( "fill", FALSE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog, nb_args, args );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the flag", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );
	
	return wgt_optionmenu;
}								// res_editor_flags

