#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


typedef struct
{
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	GtkArg			*arg;
	GtkWidget		*wgt_parent;
	int				first;
	GtkSpinButton	*wgt_num;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	foreach_selection_changed_data;


static GtkWidget *
_create_spin_button_int( GtkObject **wgt_adj, int vmin, int vmax )
{
	GtkWidget	*wgt;

	*wgt_adj = gtk_adjustment_new( vmin, vmin, vmax, 1, 1, 1 );
	wgt = gtk_spin_button_new( GTK_ADJUSTMENT( *wgt_adj ), 1, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( wgt ), TRUE );
	gtk_entry_set_max_length( GTK_ENTRY( wgt ), 6 );

	return wgt;
}								// _create_spin_button_int


static GtkWidget *
_create_spin_button_uint( GtkObject **wgt_adj, int vmin, int vmax )
{
	GtkWidget	*wgt;

	*wgt_adj = gtk_adjustment_new( vmin, vmin, vmax, 1, 1, 1 );
	wgt = gtk_spin_button_new( GTK_ADJUSTMENT( *wgt_adj ), 1, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( wgt ), TRUE );
	gtk_entry_set_max_length( GTK_ENTRY( wgt ), 6 );

	return wgt;
}								// _create_spin_button_uint


static GtkWidget *
_create_spin_button_long( GtkObject **wgt_adj, int vmin, int vmax )
{
	GtkWidget	*wgt;

	*wgt_adj = gtk_adjustment_new( vmin, vmin, vmax, 1, 1, 1 );
	wgt = gtk_spin_button_new( GTK_ADJUSTMENT( *wgt_adj ), 1, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( wgt ), TRUE );
	gtk_entry_set_max_length( GTK_ENTRY( wgt ), 6 );

	return wgt;
}								// _create_spin_button_long


static GtkWidget *
_create_spin_button_ulong( GtkObject **wgt_adj, int vmin, int vmax )
{
	GtkWidget	*wgt;

	*wgt_adj = gtk_adjustment_new( vmin, vmin, vmax, 1, 1, 1 );
	wgt = gtk_spin_button_new( GTK_ADJUSTMENT( *wgt_adj ), 1, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( wgt ), TRUE );
	gtk_entry_set_max_length( GTK_ENTRY( wgt ), 6 );

	return wgt;
}								// _create_spin_button_ulong


static GtkWidget *
_create_spin_button_float( GtkObject **wgt_adj, float vmin, float vmax )
{
	GtkWidget	*wgt;

	*wgt_adj = gtk_adjustment_new( vmin, vmin, vmax, 0.05, 0.05, 0.05 );
	wgt = gtk_spin_button_new( GTK_ADJUSTMENT( *wgt_adj ), 0.05, 2 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( wgt ), TRUE );
	gtk_entry_set_max_length( GTK_ENTRY( wgt ), 6 );

	return wgt;
}								// _create_spin_button_float


static void
foreach_selection_changed( gpointer data, gpointer user_data )
{
	GtkWidget						*wgt = GTK_WIDGET( data );
	foreach_selection_changed_data	*user = (foreach_selection_changed_data *)user_data;

	/*--- Set the resource of the widget ---*/
	set_res_int( wgt, user->arg,
		user->wgtdef, user->wgtdef_depth, user->defclass, user->wgt_parent,
		&user->first, user->wgt_num );

}								// foreach_selection_changed


static void
int_changed( GtkSpinButton *wgt_num, defclass_t *defclass, GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth )
{
	GtkArg							arg_sel;
	foreach_selection_changed_data	data;
	GtkWidget						*wgt_selected;
	module_info_t					*module;
	char							*str=NULL;

	str = gtk_editable_get_chars( GTK_EDITABLE(wgt_num), 0, -1 );
	arg_sel.name = defclass->resname;
	data.arg 		  = &arg_sel;
	data.defclass 	  = defclass;
	data.wgt_parent   = wgt_parent;
	data.first		  = TRUE;
	data.wgt_num	  = wgt_num;
	data.wgtdef		  = wgtdef;
	data.wgtdef_depth = wgtdef_depth;

	switch ( defclass->restype )
	{
		case GTK_TYPE_INT :
			GTK_VALUE_INT(arg_sel) = (gint)atol(str);
			break;
		case GTK_TYPE_UINT :
			GTK_VALUE_UINT(arg_sel) = (guint)atol(str);
			break;
		case GTK_TYPE_LONG :
			GTK_VALUE_LONG(arg_sel) = (glong)atol(str);
			break;
		case GTK_TYPE_ULONG :
			GTK_VALUE_ULONG(arg_sel) = (gulong)strtoul(str, NULL, 10);
			break;
		case GTK_TYPE_FLOAT :
			GTK_VALUE_FLOAT(arg_sel) = (gfloat)atof(str);
			break;
		default :
			assert(0);
	}							// switch
	g_slist_foreach( list_selection, foreach_selection_changed, &data );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	/*--- Update the menu editor, if displayed ---*/
	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );
	
	check_for_composite_childs( module->wgt_window );

}								// int_changed


static void
cb_editor_num_changed( GtkEditable *wgt, gpointer user_data )
{
	GtkWidget		*wgt_num = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_num, "entry_data" );
	assert( entry_data != NULL );
	int_changed( GTK_SPIN_BUTTON(wgt_num), entry_data->defclass, entry_data->wgt_parent,
		entry_data->wgtdef, entry_data->wgtdef_depth );

}								// cb_editor_num_changed


static void
cb_editor_num_value_changed( GtkEditable *wgt, gpointer user_data )
{
	GtkWidget		*wgt_num = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_num, "entry_data" );
	assert( entry_data != NULL );
	int_changed( GTK_SPIN_BUTTON( wgt_num ), entry_data->defclass, entry_data->wgt_parent,
		entry_data->wgtdef, entry_data->wgtdef_depth );

}								// cb_editor_num_value_changed


#define WGTDATA_ENABLE	"__enable__"
typedef struct
{
	GtkWidget		*wgt_num;
	defclass_t		*defclass;
}	toggled_data_t;


static void
cb_optional_toggled( GtkToggleButton *togglebutton, gpointer user_data )
{
	int				active;
	toggled_data_t	*toggled_data;
	GtkWidget		*wgt_selected;
	module_info_t	*module;

	toggled_data = get_object_data( GTK_WIDGET(togglebutton), WGTDATA_ENABLE );
	assert( toggled_data != NULL);
	
	active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton));
	gtk_widget_set_sensitive( toggled_data->wgt_num, active );

	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	set_is_disabled_resource_wgt( wgt_selected, toggled_data->defclass, 
		(active) ? FALSE : TRUE );

	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );
	
}								// cb_optional_toggled


static void
foreach_selection_focus_out( gpointer data, gpointer user_data )
{
	GtkWidget						*wgt = GTK_WIDGET( data );
	foreach_selection_changed_data	*user = (foreach_selection_changed_data *)user_data;
	GtkArg							*arg = user->arg, arg2;

	/*--- Reread value from the widget, change ? ---*/ \
#define SET( V ) \
	if ( user->first ) \
	{ \
		arg2.name = user->defclass->resname; \
		if ( user->defclass->saved_where == SAVED_INSIDE) \
		{ \
			if ( !user->wgt_parent ) \
				gtk_widget_get( GTK_WIDGET(wgt), &arg2 ); \
			else \
				gtk_container_child_getv( GTK_CONTAINER(user->wgt_parent), GTK_WIDGET( wgt ), \
					1, &arg2 ); \
			if ( V(arg[0]) != V(arg2) ) \
				gtk_spin_button_set_value( GTK_SPIN_BUTTON(user->wgt_num), V(arg2) ); \
		} \
		user->first = FALSE; \
	}

	switch ( user->defclass->restype )
	{
		case GTK_TYPE_INT :
			SET( GTK_VALUE_INT );
			break;
		case GTK_TYPE_UINT :
			SET( GTK_VALUE_UINT );
			break;
		case GTK_TYPE_LONG :
			SET( GTK_VALUE_LONG );
			break;
		case GTK_TYPE_ULONG :
			SET( GTK_VALUE_ULONG );
			break;
		case GTK_TYPE_FLOAT :
			SET( GTK_VALUE_FLOAT );
			break;
	}							// switch
	
#undef SET

}								// foreach_selection_focus_out


PUBLIC gboolean
cb_editor_num_focus_out( GtkWidget *widget, GdkEventFocus *event, gpointer user_data )
{
	GtkWidget						*wgt_num = (GtkWidget *)user_data;
	entry_data_t					*entry_data;
	GtkArg							arg_sel;
	foreach_selection_changed_data	data;
	char							*str=NULL;

	entry_data = (entry_data_t *)get_object_data( wgt_num, "entry_data" );
	assert( entry_data != NULL );

	str = gtk_editable_get_chars( GTK_EDITABLE(wgt_num), 0, -1 );
	arg_sel.name = entry_data->defclass->resname;
	data.arg 		  = &arg_sel;
	data.defclass 	  = entry_data->defclass;
	data.wgt_parent   = entry_data->wgt_parent;
	data.first		  = TRUE;
	data.wgt_num	  = GTK_SPIN_BUTTON(wgt_num);
	data.wgtdef		  = entry_data->wgtdef;
	data.wgtdef_depth = entry_data->wgtdef_depth;

	switch ( entry_data->defclass->restype )
	{
		case GTK_TYPE_INT :
			GTK_VALUE_INT(arg_sel) = (gint)atol(str);
			break;
		case GTK_TYPE_UINT :
			GTK_VALUE_UINT(arg_sel) = (guint)atol(str);
			break;
		case GTK_TYPE_LONG :
			GTK_VALUE_LONG(arg_sel) = (glong)atol(str);
			break;
		case GTK_TYPE_ULONG :
			GTK_VALUE_ULONG(arg_sel) = (gulong)strtoul(str, NULL, 10);
			break;
		case GTK_TYPE_FLOAT :
			GTK_VALUE_FLOAT(arg_sel) = (gfloat)atof(str);
			break;
		default :
			assert(0);
	}							// switch
	g_slist_foreach( list_selection, foreach_selection_focus_out, &data );

	return TRUE;
}								// cb_editor_num_focus_out


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_btn_dialog, "entry_data" );
	assert( entry_data != NULL );
	dlg_res_editor_int( wgt_btn_dialog,
		entry_data->defclass,
		entry_data->wgt_parent, 
		entry_data->wgtdef, entry_data->wgtdef_depth );

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_int( int f_optional, 
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget			*wgt_selected;
	GtkArg				arg;
	GtkWidget			*wgt_hbox;
	GtkWidget			*wgt_enable;
	GtkWidget			*wgt_num=NULL;
	GtkObject			*wgt_adj;
	entry_data_t		*entry_data;
	toggled_data_t		*toggled_data;
	int					f_disabled;
	GtkWidget			*wgt_hbox0;
	GtkWidget			*wgt_btn_dialog;
	GtkTooltips 		*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	if ( get_res_int( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg ) == -1 )
		return NULL;

	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_container ), wgt_hbox0, FALSE, FALSE, 0 );

	/*--- Create an horizontal box ---*/
	wgt_hbox = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_hbox0 ), wgt_hbox, TRUE, TRUE, 0 );

	switch ( defclass->restype )
	{
		case GTK_TYPE_INT :
			wgt_num = _create_spin_button_int( &wgt_adj, 0, 10000 );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON(wgt_num), GTK_VALUE_INT(arg) );
			break;
		case GTK_TYPE_UINT :
			wgt_num = _create_spin_button_uint( &wgt_adj, 0, 10000 );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON(wgt_num), GTK_VALUE_UINT(arg) );
			break;
		case GTK_TYPE_LONG :
			wgt_num = _create_spin_button_long( &wgt_adj, 0, 10000 );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON(wgt_num), GTK_VALUE_LONG(arg) );
			break;
		case GTK_TYPE_ULONG :
			wgt_num = _create_spin_button_ulong( &wgt_adj, 0, 10000 );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON(wgt_num), GTK_VALUE_ULONG(arg) );
			break;
		case GTK_TYPE_FLOAT :
			wgt_num = _create_spin_button_float( &wgt_adj, 0, 10000 );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON(wgt_num), GTK_VALUE_FLOAT(arg) );
			break;
		default :
			RES_NOT_YET_HANDLED( wgtdef, defclass );
			assert( 0 );
	}
	gtk_container_add (GTK_CONTAINER(wgt_hbox), wgt_num );
	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER(wgt_hbox), wgt_num, nb_args, args );

	/*--- Create a check button 'Optional' ---*/
	f_disabled = FALSE;
	if (f_optional)
	{
		INIT_ARGS;
		SETARG_OBJECT( GTK_TYPE_CONTAINER, "parent", wgt_hbox );
		wgt_enable = gtk_widget_newv( gtk_check_button_get_type(), nb_args, args );
		INIT_ARGS;
		SETARG_LONG( "position", 0 );
		SETARG_BOOL( "expand", FALSE );
		SETARG_BOOL( "fill", FALSE );
		gtk_container_child_setv( GTK_CONTAINER(wgt_hbox), wgt_enable, nb_args, args );
		toggled_data = malloc(sizeof(toggled_data_t));
		toggled_data->wgt_num  = wgt_num;
		toggled_data->defclass = defclass;
		set_object_data( wgt_enable, WGTDATA_ENABLE, toggled_data, TRUE );
		f_disabled = get_is_disabled_resource_wgt( wgt_selected, defclass );
		gtk_signal_connect( GTK_OBJECT(wgt_enable), "toggled",
			GTK_SIGNAL_FUNC(cb_optional_toggled), NULL );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(wgt_enable), 
			(f_disabled) ? FALSE : TRUE );
		cb_optional_toggled( GTK_TOGGLE_BUTTON(wgt_enable), (gpointer)wgt_num );
	}							// if
	gtk_widget_set_sensitive( wgt_num, (f_disabled) ? FALSE : TRUE );

	/*--- Button to bring a dialog ---*/
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog );
	INIT_ARGS;
	SETARG_BOOL( "expand", FALSE );
	SETARG_BOOL( "fill", FALSE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog, nb_args, args );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the value", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );
	
	entry_data = malloc( sizeof( entry_data_t ) );
	memset( entry_data, 0, sizeof( entry_data_t ) );
	entry_data->defclass     = defclass;
	entry_data->wgt_parent   = wgt_parent;
	entry_data->wgtdef		 = wgtdef;
	entry_data->wgtdef_depth = wgtdef_depth;
	set_object_data( wgt_num, "entry_data", entry_data, TRUE );
	set_object_data( wgt_btn_dialog, "entry_data", entry_data, FALSE );

	gtk_signal_connect_after( GTK_OBJECT( wgt_num ), "changed",
		GTK_SIGNAL_FUNC( cb_editor_num_changed ), wgt_num );
	gtk_signal_connect_after( GTK_OBJECT( wgt_adj ), "value_changed",
		GTK_SIGNAL_FUNC( cb_editor_num_value_changed ), wgt_num );
	gtk_signal_connect( GTK_OBJECT(wgt_num), "focus_in_event",
		GTK_SIGNAL_FUNC(cb_string_focus_in), wgt_num );
	gtk_signal_connect( GTK_OBJECT(wgt_num), "focus_out_event",
		GTK_SIGNAL_FUNC(cb_editor_num_focus_out), wgt_num );
	
	return wgt_num;
}								// res_editor_int
