#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


typedef struct
{
	GtkWidget		*wgt_entry;
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	char			*string;
	GtkWidget		*wgt_parent;
	int				first;
	GtkWidget		*wgt_entry;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;
	GtkArg					arg2;
	char					*v1=NULL, *v2=NULL;
	GtkWidget				*wgt_label;

	/*--- Set the resource of the widget ---*/
	set_res_string( wgt, user->string,
		user->wgtdef, user->wgtdef_depth, user->defclass, user->wgt_parent );

	/*--- Reread value from the widget, change ? ---*/
	if ( user->first && (user->defclass->saved_where == SAVED_INSIDE) )
	{
		arg2.type = user->defclass->restype;
		arg2.name = user->defclass->resname;
		if ( !user->wgt_parent )
			gtk_widget_get( GTK_WIDGET(wgt), &arg2 );
		else
			gtk_container_child_getv( GTK_CONTAINER(user->wgt_parent), GTK_WIDGET( wgt ),
				1, &arg2 );
		if ( arg2.type != user->defclass->restype )
			return;
		v2 = GTK_VALUE_STRING(arg2);
		if ( !v2 || strcmp( user->string, v2 ) )
			gtk_entry_set_text( GTK_ENTRY(user->wgt_entry), v2 );
		wgt_label = get_object_data( user->wgt_entry, PROPERTIES_WGTDATA_LABEL );
		v1 = GTK_VALUE_STRING(user->defclass->arg_default);
//		if ( ( (!v1 || !v2) && (v1 != v2) ) || strcmp( v1, v2 ) )
//			gtk_widget_set_style( wgt_label, properties_style_bold );
//		else
//			gtk_widget_set_style( wgt_label, properties_style_normal );
		if (v2) g_free(v2);
		user->first = FALSE;
	}
	else
	{
		g_print( "%s %d\n", __FILE__, __LINE__ );
	}

}								// foreach_selection


static void
string_changed( GtkWidget *wgt_entry, GtkWidget *wgt, defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth )
{
	foreach_selection_data	data;
	GtkWidget				*wgt_selected;
	module_info_t			*module;

	data.string   	  = gtk_editable_get_chars( GTK_EDITABLE( wgt ), 0, -1 );
	data.defclass 	  = defclass;
	data.wgt_parent   = wgt_parent;
	data.first		  = TRUE;
	data.wgt_entry	  = wgt_entry;
	data.wgtdef		  = wgtdef;
	data.wgtdef_depth = wgtdef_depth;
	g_slist_foreach( list_selection, foreach_selection, &data );
	g_free( data.string );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	if ( module->type == MODULE_TYPE_MENU )
		menu_touched( module );

	check_for_composite_childs( module->wgt_window );

}								// string_changed


static void
cb_editor_string_changed( GtkEditable *wgt_editable, gpointer user_data )
{
	GtkWidget		*wgt_entry = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_entry, "entry_data" );
	assert( entry_data != NULL );
	string_changed( wgt_entry, GTK_WIDGET(wgt_editable), entry_data->defclass,
		entry_data->wgt_parent, 
		entry_data->wgtdef, entry_data->wgtdef_depth );
}								// cb_editor_string_changed


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_btn_dialog, "entry_data" );
	assert( entry_data != NULL );
	dlg_res_editor_string( wgt_btn_dialog, entry_data->wgt_entry,
		entry_data->defclass,
		entry_data->wgt_parent,
		entry_data->wgtdef, entry_data->wgtdef_depth );

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_string( gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget		*wgt_selected;
	GtkArg			arg;
	GtkWidget		*wgt_entry;
	entry_data_t	*entry_data;
	GtkWidget		*wgt_hbox0;
	GtkWidget		*wgt_hbox;
	GtkWidget		*wgt_btn_dialog;
	GtkTooltips 	*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	assert( wgt_selected != NULL );
	if ( get_res_string( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg ) == -1 )
		return NULL;
		
	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_container ), wgt_hbox0, FALSE, FALSE, 0 );

	/*--- Create the display area ---*/		
	wgt_hbox = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_hbox0 ), wgt_hbox, TRUE, TRUE, 0 );

	/*--- Field to display/edit the string ---*/
	wgt_entry = gtk_entry_new();
	if ( GTK_VALUE_STRING(arg) )
	{
		gtk_entry_set_text( GTK_ENTRY(wgt_entry), GTK_VALUE_STRING(arg) );
		g_free( GTK_VALUE_STRING(arg) );
	}
	else
	{
		gtk_entry_set_text( GTK_ENTRY(wgt_entry), "" );
	}
	gtk_box_pack_start( GTK_BOX( wgt_hbox ), wgt_entry, TRUE, TRUE, 0 );

	/*--- Button to bring a dialog ---*/
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog );
	INIT_ARGS;
	SETARG_BOOL( "expand", FALSE );
	SETARG_BOOL( "fill", FALSE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog, nb_args, args );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the string", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );

	/*--- Data attached to the dialog ---*/	
	entry_data = malloc( sizeof( entry_data_t ) );
	memset( entry_data, 0, sizeof( entry_data_t ) );
	entry_data->wgt_entry	 = wgt_entry;
	entry_data->defclass	 = defclass;
	entry_data->wgt_parent	 = wgt_parent;
	entry_data->wgtdef		 = wgtdef;
	entry_data->wgtdef_depth = wgtdef_depth;
	set_object_data( wgt_entry, "entry_data", entry_data, TRUE );
	set_object_data( wgt_btn_dialog, "entry_data", entry_data, FALSE );

	gtk_signal_connect( GTK_OBJECT( wgt_entry ), "changed",
		GTK_SIGNAL_FUNC( cb_editor_string_changed ), wgt_entry );
	gtk_signal_connect( GTK_OBJECT( wgt_entry ), "focus_in_event",
		GTK_SIGNAL_FUNC( cb_string_focus_in ), wgt_entry );

	return wgt_hbox0;
}								// res_editor_string
