#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

extern void
dlg_res_editor_style( GtkWidget *wgt_btn, GtkWidget *wgt_entry, 
	defclass_t *defclass, 
	GtkWidget *wgt_parent,
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth );


GtkStyle *xstyle;

typedef struct
{
	GtkWidget		*wgt_entry;
	defclass_t		*defclass;
	GtkWidget		*wgt_parent;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
	int				index;				// -1 (none) or 0..nb_styles-1
}	entry_data_t;


typedef struct
{
	defclass_t		*defclass;
	GtkArg			*arg;
	GtkWidget		*wgt_parent;
	int				first;
	GtkWidget		*wgt_optionmenu;
	gtk_wgtdef_t	*wgtdef;
	gtk_wgtdef_t	*wgtdef_depth;
}	foreach_selection_data;


static void
foreach_selection( gpointer data, gpointer user_data )
{
	GtkWidget				*wgt = GTK_WIDGET( data );
	foreach_selection_data	*user = (foreach_selection_data *)user_data;
	GtkArg					*arg = user->arg;

	/*--- Set the resource of the widget ---*/
	if ( user->defclass->saved_where == SAVED_INSIDE )
	{
		if ( !user->wgt_parent )
			gtk_widget_set( GTK_WIDGET(wgt),
				user->defclass->resname, GTK_VALUE_OBJECT( arg[0] ), NULL );
		else
			assert( 0 );
	}
	else assert(0);

}								// foreach_selection


static void
menuitem_changed( GtkWidget *wgt_optionmenu, defclass_t *defclass, 
	GtkWidget *wgt_parent, 
	gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	int index )
{
	foreach_selection_data	data;
	GtkArg					arg;
	GtkWidget				*wgt_selected;
	module_info_t			*module;

	if ( index == -1 )
		GTK_VALUE_OBJECT( arg ) = (GtkObject *)gtk_widget_get_default_style();
	else
		GTK_VALUE_OBJECT( arg ) = (GtkObject *)styles[ index ];
	data.defclass 		= defclass;
	data.wgt_parent 	= wgt_parent;
	data.arg			= &arg;
	data.first			= TRUE;
	data.wgt_optionmenu	= wgt_optionmenu;
	data.wgtdef		  	= wgtdef;
	data.wgtdef_depth 	= wgtdef_depth;
	g_slist_foreach( list_selection, foreach_selection, &data );

	/*--- This module has been modified ---*/
	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	assert( wgt_selected != NULL );
	module = get_object_data( wgt_selected, WGTDATA_MODULE );
	assert( module != NULL );
	project_touched( module );

	check_for_composite_childs( module->wgt_window );

}								// menuitem_changed


static void
cb_editor_menuitem_activate( GtkWidget *widget, gpointer user_data )
{
	GtkWidget		*wgt_entry = (GtkWidget *)user_data;
	entry_data_t	*entry_data;

	entry_data = (entry_data_t *)get_object_data( wgt_entry, "entry_data" );
	assert( entry_data != NULL );
	menuitem_changed( wgt_entry, entry_data->defclass, entry_data->wgt_parent, 
		entry_data->wgtdef, entry_data->wgtdef_depth,
		entry_data->index );

}								// cb_editor_menuitem_activate


static void
cb_btn_dialog_clicked( GtkWidget *wgt, gpointer user_data )
{
	GtkWidget		*wgt_btn_dialog = (GtkWidget *)user_data;

	printf( "%s %d - Dialog not yet implemented\n", __FILE__, __LINE__ );

}								// cb_btn_dialog_clicked


PUBLIC GtkWidget *
res_editor_style( gtk_wgtdef_t *wgtdef, gtk_wgtdef_t *wgtdef_depth,
	GtkWidget *wgt_container, defclass_t *defclass,
	GtkWidget *wgt_parent )
{
	GtkWidget			*wgt_selected;
	GtkArg				arg, *parg;
	GtkWidget			*wgt_optionmenu;
	GtkWidget			*wgt_menu;
	GtkWidget			*wgt_menuitem;
	entry_data_t		*entry_data;
	int					n, index;
	GtkWidget 			*sep;
	GtkWidget			*wgt_hbox0;
	GtkWidget			*wgt_btn_dialog;
	GtkTooltips 		*tooltips;

	/*--- First read current value (stored inside the widget or not) ---*/
	wgt_selected = GTK_WIDGET( g_slist_last( list_selection )->data );
	assert( wgt_selected != NULL );
	if ( get_res_pointer( wgt_selected, wgtdef, wgtdef_depth, defclass, wgt_parent, &arg, &parg ) == -1 )
		return NULL;

	/*--- Create an horizontal box ---*/
	wgt_hbox0 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( wgt_container ), wgt_hbox0, FALSE, FALSE, 0 );

	wgt_optionmenu = gtk_option_menu_new();
	wgt_menu = gtk_menu_new();

	/*--- No style ---*/
 	entry_data = malloc( sizeof( entry_data_t ) );
	memset( entry_data, 0, sizeof( entry_data_t ) );
	entry_data->defclass   	 = defclass;
	entry_data->wgt_parent 	 = wgt_parent;
	entry_data->wgtdef		 = wgtdef;
	entry_data->wgtdef_depth = wgtdef_depth;
	entry_data->index		 = -1;
	wgt_menuitem = gtk_menu_item_new_with_label( "None" );
	gtk_menu_append( GTK_MENU(wgt_menu), wgt_menuitem );
	gtk_signal_connect( GTK_OBJECT(wgt_menuitem), "activate",
		GTK_SIGNAL_FUNC( cb_editor_menuitem_activate ), wgt_menuitem );
	set_object_data( wgt_menuitem, "entry_data", entry_data, TRUE );

	sep = gtk_menu_item_new();
	gtk_container_add( GTK_CONTAINER( wgt_menu ), sep );

	/*--- Create the menu (choice) to be attached to the option menu ---*/
	for ( n = 0; n < nb_styles; n++ )
	{
		entry_data = malloc( sizeof( entry_data_t ) );
		memset( entry_data, 0, sizeof( entry_data_t ) );
		entry_data->defclass   	 = defclass;
		entry_data->wgt_parent 	 = wgt_parent;
		entry_data->wgtdef		 = wgtdef;
		entry_data->wgtdef_depth = wgtdef_depth;
		entry_data->index		 = n;

		wgt_menuitem = gtk_menu_item_new_with_label( styles_infos[ n ]->name  );
		gtk_menu_append( GTK_MENU(wgt_menu), wgt_menuitem );
		gtk_signal_connect( GTK_OBJECT(wgt_menuitem), "activate",
			GTK_SIGNAL_FUNC( cb_editor_menuitem_activate ), wgt_menuitem );
		set_object_data( wgt_menuitem, "entry_data", entry_data, TRUE );
	}							// for (n)

	/*--- Which style ? ---*/
	for ( index = -1, n = 0; n < nb_styles; n++ )
	{
		if ( styles[ n ] == (GtkStyle *)GTK_VALUE_OBJECT( arg ) )
		{
			index = n;
			break;
		}
	}
	
	gtk_option_menu_set_menu( GTK_OPTION_MENU(wgt_optionmenu), GTK_WIDGET(wgt_menu) );
	gtk_option_menu_set_history( GTK_OPTION_MENU(wgt_optionmenu),
		(index == -1) ? 0 : index + 2 );
	gtk_box_pack_start( GTK_BOX( wgt_hbox0 ), wgt_optionmenu, FALSE, FALSE, 0 );
	INIT_ARGS;
	SETARG_BOOL( "expand", TRUE );
	SETARG_BOOL( "fill", TRUE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_optionmenu, nb_args, args );

	/*--- Button to bring a dialog ---*/
	wgt_btn_dialog = gtk_button_new_with_label("...");
	gtk_container_add( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog );
	INIT_ARGS;
	SETARG_BOOL( "expand", FALSE );
	SETARG_BOOL( "fill", FALSE );
	gtk_container_child_setv( GTK_CONTAINER( wgt_hbox0 ), wgt_btn_dialog, nb_args, args );
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip( tooltips, wgt_btn_dialog, 
		"Click here to edit the style", NULL );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_dialog ), "clicked",
		GTK_SIGNAL_FUNC( cb_btn_dialog_clicked ), wgt_btn_dialog );
	
	return wgt_optionmenu;
}								// res_editor_style
