#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


PUBLIC void
update_tooltip_widget( GtkWidget *wgt )
{
	module_info_t 			*module;
	gtk_wgtdef_t			*wgtdef;
	char					tip1[ 1000 ], *name;
	int						type;
	int						n, nb_cbget;
	get_first_callback_t	*cbget, *p;
	char					label[ 200 ];

	if ( GTK_WIDGET_NO_WINDOW( wgt ) )
		return;

	module = get_object_data( GTK_WIDGET( wgt ), WGTDATA_MODULE );
	assert( module != NULL );

	wgtdef = get_object_data( GTK_WIDGET( wgt ), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	/*--- Name of the widget ---*/
	name = get_name_wgt( wgt );
	type = get_nametype_wgt( wgt );
	sprintf( tip1, "%s", gtk_widget_get_name( wgt ) );
	if ( (type == NAMETYPE_GLOBAL) && (name  != NULL) )
		sprintf( &tip1[ strlen( tip1 ) ], ": %s", name );

	/*--- First 2 callbacks ---*/
	cbget = malloc( sizeof( get_first_callback_t ) * 2 );	
	nb_cbget = get_first_callbacks( 2, cbget, wgt, wgtdef );
	for ( p = cbget, n = 0; n < nb_cbget; n++, p++ )
	{
		if ( !strcmp( p->defcb->cbname, p->defcb->cbfile ) )
			sprintf( label, "%s::%s - %s",
				wgtdef->widget_name, p->resname, 
				p->defcb->cbname );
		else
			sprintf( label, "%s::%s - %s@%s",
				wgtdef->widget_name, p->resname, 
				p->defcb->cbname, p->defcb->cbfile );
		strcat( tip1, "\n" );
		strcat( tip1, label );
	}							// for (n)

	/*--- Update the tooltip for this widget ---*/
	if ( !GTK_WIDGET_NO_WINDOW( wgt ) )
	{
		gtk_tooltips_set_tip( module->u1.window.tooltips, wgt,
			tip1,
			tip1 );
	}							// else

}								// update_tooltip_widget


static void
cb_name_changed( GtkEditable *wgt_editable, gpointer user_data )
{
	gchar			*name;
	GtkWidget		*wgt_selected;
	module_info_t	*module;
	char			label[200];
	gtk_wgtdef_t	*wgtdef;
	GtkCTreeNode	*wgt_node;

	if ( nb_selection() != 1 )
		return;

	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );

	module = get_object_data( GTK_WIDGET(wgt_selected), WGTDATA_MODULE );
	assert( module != NULL );

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_selected), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	wgt_node = get_object_data(wgt_selected, TREENODE_WGTDATA);
	assert( wgt_node != NULL);

	name = gtk_editable_get_chars( GTK_EDITABLE(wgt_resources_entry_name), 0, -1 );
	set_name_wgt( wgt_selected, name );

	sprintf( label, "%s: %s",
		gtk_widget_get_name( wgt_selected ),
		name );
	gtk_ctree_set_node_info( GTK_CTREE( wgt_tree_widgets ),
		GTK_CTREE_NODE(wgt_node), 
		label, 5,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		FALSE, FALSE );

	g_free( name );

	/*--- Update the tooltip for this widget ---*/	
	update_tooltip_widget( wgt_selected );
	
	project_touched( module );

}								// cb_name_changed


PUBLIC gboolean
cb_string_focus_in( GtkWidget *widget, GdkEventFocus *event, gpointer user_data )
{

	gtk_editable_select_region( GTK_EDITABLE(widget), 0, -1 );
	return TRUE;
	
}								// cb_string_focus_in


static void
cb_btn_anonymous_clicked( GtkWidget *widget, void *data )
{
	GtkWidget		*wgt_selected;
	module_info_t	*module;
	char			label[200];
	gtk_wgtdef_t	*wgtdef;
	GtkCTreeNode	*wgt_node;

	if ( nb_selection() != 1 )
		return;

	if ( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) )
		return;

	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	set_nametype_wgt( wgt_selected, NAMETYPE_ANONYMOUS );

	module = get_object_data( GTK_WIDGET(wgt_selected), WGTDATA_MODULE );
	assert( module != NULL );

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_selected), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	wgt_node = get_object_data(wgt_selected, TREENODE_WGTDATA);
	assert( wgt_node != NULL);

	gtk_widget_set( GTK_WIDGET(wgt_resources_label_name), "sensitive", FALSE, NULL );
	gtk_widget_set( GTK_WIDGET(wgt_resources_entry_name), "sensitive", FALSE, NULL );
	gtk_signal_handler_block( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );
	gtk_entry_set_text( GTK_ENTRY(wgt_resources_entry_name), "" );
	gtk_signal_handler_unblock( GTK_OBJECT(wgt_resources_entry_name), cb_name_changed_id );

	sprintf( label, "%s",
		gtk_widget_get_name(wgt_selected) );
	gtk_ctree_set_node_info( GTK_CTREE( wgt_tree_widgets ),
		GTK_CTREE_NODE(wgt_node), 
		label, 5,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		FALSE, FALSE );

	/*--- Update the tooltip for this widget ---*/	
	update_tooltip_widget( wgt_selected );
	
	project_touched( module );

}								// cb_btn_anonymous_clicked


static void
cb_btn_global_clicked( GtkWidget *widget, void *data )
{
	GtkWidget	*wgt_selected;
	module_info_t	*module;
	char			label[200];
	gtk_wgtdef_t	*wgtdef;
	GtkCTreeNode	*wgt_node;

	if ( nb_selection() != 1 )
		return;

	if ( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) )
		return;

	wgt_selected = GTK_WIDGET( g_slist_last(list_selection)->data );
	set_nametype_wgt( wgt_selected, NAMETYPE_GLOBAL );

	module = get_object_data( GTK_WIDGET(wgt_selected), WGTDATA_MODULE );
	assert( module != NULL );

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt_selected), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	wgt_node = get_object_data(wgt_selected, TREENODE_WGTDATA);
	assert( wgt_node != NULL);

	gtk_widget_set( GTK_WIDGET(wgt_resources_label_name), "sensitive", TRUE, NULL );
	gtk_widget_set( GTK_WIDGET(wgt_resources_entry_name), "sensitive", TRUE, NULL );
	gtk_widget_grab_focus( GTK_WIDGET(wgt_resources_entry_name) );

	sprintf( label, "%s: %s",
		gtk_widget_get_name(wgt_selected),
		get_name_wgt(wgt_selected) );
	gtk_ctree_set_node_info( GTK_CTREE( wgt_tree_widgets ),
		GTK_CTREE_NODE(wgt_node), 
		label, 5,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		wgtdef->gdkpixmap, wgtdef->gdkbitmap,
		FALSE, FALSE );

	/*--- Update the tooltip for this widget ---*/	
	update_tooltip_widget( wgt_selected );
	
	project_touched( module );

}								// cb_btn_global_clicked


static void
create_window( void )
{
	GtkWidget *vbox1;
	GtkWidget *scrolledwgt_wnd_resources;
	GtkWidget *viewport1;
	GtkWidget *hpaned2;
	GtkWidget *vbox3;
	GtkWidget *vbox4;
	GtkWidget *hbox1;
	GtkWidget *hbox3;
	GtkWidget *vseparator1;
	GtkWidget *hbox4;
	GtkWidget *hbox2;
	GtkWidget *table1;
	GSList *fixed1_group = NULL;

	wgt_wnd_resources = gtk_widget_new( gtk_window_get_type(),
		"GtkWindow::type", GTK_WINDOW_DIALOG,
		"GtkWindow::window_position", GTK_WIN_POS_NONE,
		"GtkWidget::x", config.resources_wnd.x,
		"GtkWidget::y", config.resources_wnd.y,
		"GtkWidget::width", 200,
		"GtkWidget::height", 150,
		"GtkWindow::default_width", config.resources_wnd.w,
		"GtkWindow::default_height", config.resources_wnd.h,
		"GtkWindow::title", "Resources - GTk Lo Gafet",
		"GtkWindow::allow_grow", 1,
		"GtkWidget::visible", 0,
		NULL );

#if 0
	wgt_wnd_resources = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set( GTK_WINDOW(wgt_wnd_resources), 
		"Resources - GTk Lo Gafet" );
	gtk_widget_set_usize( GTK_WIDGET(wgt_wnd_resources), -1, -1 );
	gtk_widget_set_usize( GTK_WIDGET(wgt_wnd_resources), 200, 150 );
	gtk_window_set_default_size(GTK_WINDOW(wgt_wnd_resources),
		config.resources_wnd.w, config.resources_wnd.h );
#endif
	gtk_signal_connect( GTK_OBJECT(wgt_wnd_resources), "destroy",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );
	gtk_signal_connect(GTK_OBJECT(wgt_wnd_resources), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (wgt_wnd_resources), vbox1);

	wgt_res_notebook = gtk_notebook_new ();
	gtk_box_pack_start (GTK_BOX (vbox1), wgt_res_notebook, TRUE, TRUE, 0);

	scrolledwgt_wnd_resources = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (wgt_res_notebook), scrolledwgt_wnd_resources);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwgt_wnd_resources), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	viewport1 = gtk_viewport_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (scrolledwgt_wnd_resources), viewport1);

	hpaned2 = gtk_hpaned_new ();
	gtk_container_add (GTK_CONTAINER (viewport1), hpaned2);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (hpaned2), vbox3);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (hpaned2), vbox4);

	hbox1 = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), hbox3, FALSE, FALSE, 0);

	wgt_resources_btn_anonymous = gtk_radio_button_new_with_label (fixed1_group, "Anonymous" );
	fixed1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (wgt_resources_btn_anonymous));
	gtk_box_pack_start (GTK_BOX (hbox3), wgt_resources_btn_anonymous, FALSE, FALSE, 0);
	gtk_signal_connect( GTK_OBJECT(wgt_resources_btn_anonymous), "clicked",
		GTK_SIGNAL_FUNC(cb_btn_anonymous_clicked), NULL );

	wgt_resources_btn_global = gtk_radio_button_new_with_label (fixed1_group, "Global" );
	fixed1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (wgt_resources_btn_global));
	gtk_box_pack_start (GTK_BOX (hbox3), wgt_resources_btn_global, FALSE, FALSE, 0);
	gtk_signal_connect( GTK_OBJECT(wgt_resources_btn_global), "clicked",
		GTK_SIGNAL_FUNC(cb_btn_global_clicked), NULL );

	vseparator1 = gtk_vseparator_new ();
	gtk_box_pack_start (GTK_BOX (hbox1), vseparator1, FALSE, FALSE, 0);

	hbox4 = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_end (GTK_BOX (hbox1), hbox4, TRUE, TRUE, 0);

	wgt_resources_label_name = gtk_label_new ("Name:");
	gtk_box_pack_start (GTK_BOX (hbox4), wgt_resources_label_name, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (wgt_resources_label_name), GTK_JUSTIFY_LEFT);

	wgt_resources_entry_name = gtk_entry_new_with_max_length (48);
	gtk_entry_set_text( GTK_ENTRY(wgt_resources_entry_name), "" );
	gtk_box_pack_start (GTK_BOX (hbox4), wgt_resources_entry_name, TRUE, TRUE, 0);
	cb_name_changed_id = 
		gtk_signal_connect( GTK_OBJECT(wgt_resources_entry_name), "changed",
			GTK_SIGNAL_FUNC( cb_name_changed ), NULL );
	cb_name_focus_in_id=
		gtk_signal_connect( GTK_OBJECT(wgt_resources_entry_name), "focus_in_event",
			GTK_SIGNAL_FUNC( cb_string_focus_in ), NULL );

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);

	table1 = gtk_table_new (2, 5, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox2), table1, FALSE, FALSE, 0);

}								// create_window


PUBLIC void
create_resources_window( void )
{

	/*--- Create the base window ---*/
	create_window();

}								// create_resources_window
