#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "../glg.h"

void
clist_create_columns( GtkWidget *wgt_new, int nb_columns );


PUBLIC GtkArg * 
getdlg_clist( defclass_t *defclass, GtkWidget *wgt, GtkArg *parg )
{
	if ( !strcmp( defclass->resname, "~GtkCList::show_titles" ) )
	{
		parg->type = GTK_TYPE_BOOL;
		GTK_VALUE_BOOL(parg[0]) = (GTK_CLIST_SHOW_TITLES(wgt)) ? TRUE : FALSE;
		return parg;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_passive" ) )
	{
		return NULL;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_columns" ) )
	{
		gtk_usertype_strings_t	*strings;
		strings = gtk_object_get_data( GTK_OBJECT( wgt ), defclass->resname );
		GTK_VALUE_POINTER( parg[ 0 ] ) = strings;
		return parg;
	} else assert(0);
	return NULL;
}								// getdlg_clist


PUBLIC int 
putdlg_clist( defclass_t *defclass, GtkWidget *wgt, GtkArg *parg )
{
	if ( !strcmp( defclass->resname, "~GtkCList::show_titles" ) )
	{
		if (GTK_VALUE_BOOL(parg[0]))
			gtk_clist_column_titles_show(GTK_CLIST(wgt));
		else
			gtk_clist_column_titles_hide(GTK_CLIST(wgt));
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_passive" ) )
	{
		return 1;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_columns" ) )
	{
		gtk_usertype_strings_t	*strings;
		int						nb_columns, nb, n;
		GtkArg					args[ 1 ];

		args[ 0 ].name = "GtkCList::n_columns";
		args[ 0 ].type = GTK_TYPE_UINT;	
		gtk_widget_getv( GTK_WIDGET( wgt ), 1, args );
		nb_columns = GTK_VALUE_UINT( args[ 0 ] );

		strings = GTK_VALUE_POINTER( parg[ 0 ] );
		nb = MIN( nb_columns, strings->nb_strings );
		for ( n = 0; n < nb; n++ )
			gtk_clist_set_column_title( GTK_CLIST( wgt ), n, strings->strings[ n ] );

		gtk_object_set_data_full( GTK_OBJECT( wgt ), defclass->resname, strings, 
			(GtkDestroyNotify) cb_destroy_notify_strings );

		return 0;
	} else assert(0);
	return -1;
}								// putdlg_clist


PUBLIC int
writecode_clist( defclass_t *defclass, GtkWidget *wgt, GtkArg **parg, char **str )
{
	GtkArg		arg;

	*parg = NULL;
	if ( !strcmp( defclass->resname, "~GtkCList::show_titles" ) )
	{
		str[0] = malloc(200);
		assert(str[0] != NULL);
		if (GTK_CLIST_SHOW_TITLES(wgt))
			sprintf( str[0], "gtk_clist_column_titles_show(GTK_CLIST(%s));",
				get_name_wgt(wgt) );
		else
			sprintf( str[0], "gtk_clist_column_titles_hide(GTK_CLIST(%s));",
				get_name_wgt(wgt) );
		return 1;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_passive" ) )
	{
		GtkArg	arg, *xarg;
		if ( (*defclass->wgtdef_depth->pfct_getdlg)( defclass, wgt, &arg ) == NULL )
			xarg = get_outsideres_wgt( wgt, defclass->resname );
		else
			xarg = &arg;
		assert( xarg != NULL );
		if ( GTK_VALUE_BOOL( xarg[ 0 ] ) )
		{
			str[0] = malloc( 200 );
			assert(str[0] != NULL);
			sprintf( str[0], "gtk_clist_column_titles_passive( GTK_CLIST( %s ) );",
				get_name_wgt( wgt ) );
			return 1;
		}
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkCList::titles_columns" ) )
	{
		gtk_usertype_strings_t	*strings;
		int						n;
		char					*tmp;

		memset( &arg, 0, sizeof(GtkArg) );
		arg.type = defclass->restype;
		(*defclass->wgtdef_depth->pfct_getdlg)( defclass, wgt, &arg );

		strings = GTK_VALUE_POINTER( arg );
		assert( strings->nb_strings < 100 );		// See generate.c for max strings
		for ( n = 0; n < strings->nb_strings; n++ )
		{
			tmp = malloc( 100 + strlen( strings->strings[ n ] ) );
			sprintf( tmp, "gtk_clist_set_column_title( GTK_CLIST( %s ), %d, \"%s\" );",
				get_name_wgt( wgt ), 
				n,
				strings->strings[ n ] );
			str[ n ] = strdup( tmp );
			free( tmp );
		}
		return strings->nb_strings;
	}
	else assert(0);

	return 0;
}								// writecode_clist


PUBLIC int
writecode_after_clist( defclass_t *defclass, GtkWidget *wgt, GtkArg **parg, char **str )
{
	return 0;
}								// writecode_after_clist


PUBLIC void
afteraddwgt_clist( GtkWidget *wgt )
{
}								// afteraddwgt_clist


PUBLIC GtkWidget *
addwgt_clist( struct gtk_wgtdef_t *wgtdef_newwgt, module_info_t *module,
	GtkWidget *wgt_parent, GtkWidget *wgt_container )
{
	int			nb, n;
	GtkWidget	*wgt_new;
	
	nb = 3;
	if ( !infos_addwgt_dialog( "New GtkCList", &nb, 1, 100, "Number of columns:" ) )
		return NULL;

	for ( n = nb_args; n > 0; n-- )
		memmove( &args[n], &args[n-1], sizeof(args[0]) );
	n = nb_args; nb_args = 0;
	SETARG_UINT( "n_columns", nb );
	nb_args = n+1;
	wgt_new = create_new_wgt( wgtdef_newwgt, module, wgt_parent, wgt_container, NULL );

	gtk_clist_column_titles_show( GTK_CLIST( wgt_new ) );
	clist_create_columns( wgt_new, nb );

	return wgt_new;

}								// addwgt_clist


PUBLIC void
clist_create_columns( GtkWidget *wgt_new, int nb_columns )
{
	gtk_usertype_strings_t	*strings;
	int						n;
	char					label[ 80 ];
	GtkWidget				*wgt_label;

	strings = malloc( sizeof( gtk_usertype_strings_t ) );
	strings->nb_strings = nb_columns;
	strings->strings = malloc( sizeof( char * ) * strings->nb_strings );
	for ( n = 0; n < nb_columns; n++ )
	{

		sprintf( label, "column%d", n+1 );
		gtk_clist_set_column_widget( GTK_CLIST( wgt_new ), n,
			wgt_label = gtk_label_new(label) );
		strings->strings[ n ] = strdup( label );
#if 0
		set_data_wgtdef( wgt_label, wgt_new, module,
			FALSE,			// Placeholder
			FALSE,			// Composite
			FALSE );		// Disabled
#endif
	}

	gtk_object_set_data_full( GTK_OBJECT( wgt_new ), "~GtkCList::titles_columns", strings, 
		(GtkDestroyNotify) cb_destroy_notify_strings );

}								// clist_create_columns

