#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "../glg.h"


PUBLIC GtkArg * 
getdlg_widget( defclass_t *defclass, GtkWidget *wgt, GtkArg *parg )
{

	if ( !strcmp( defclass->resname, "~GtkWidget::x" ) )
	{
		parg->type = defclass->restype;
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
		{
			GTK_VALUE_INT(parg[0]) = wgt->allocation.x;
		}
		else
		{
			gint	x, y;
			gdk_window_get_position( wgt->window, &x, &y );
			GTK_VALUE_INT(parg[0]) = x;
		}
		return parg;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::y" ) )
	{
		parg->type = defclass->restype;
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
		{
			GTK_VALUE_INT(parg[0]) = wgt->allocation.y;
		}
		else
		{
			gint	x, y;
			gdk_window_get_position( wgt->window, &x, &y );
			GTK_VALUE_INT(parg[0]) = y;
		}
		return parg;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::width" ) )
	{
		parg->type = defclass->restype;
		GTK_VALUE_INT(parg[0]) = wgt->allocation.width;
		return parg;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::height" ) )
	{
		parg->type = defclass->restype;
		GTK_VALUE_INT(parg[0]) = wgt->allocation.height;
		return parg;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::visible" ) )
	{
		return NULL;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::tip_text" ) )
	{
		return NULL;
	}
	else assert(0);

	return NULL;

}								// getdlg_widget


PUBLIC int 
putdlg_widget( defclass_t *defclass, GtkWidget *wgt, GtkArg *parg )
{
	gint			x, y, w, h;
	gtk_wgtdef_t	*wgtdef=NULL;

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	if ( !strcmp( defclass->resname, "~GtkWidget::x" ) )
	{
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
			y = wgt->allocation.y;
		else
			gdk_window_get_position( wgt->window, &x, &y );
		gtk_widget_set_uposition( wgt, GTK_VALUE_INT(parg[0]), y );
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::y" ) )
	{
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
			x = wgt->allocation.x;
		else
			gdk_window_get_position( wgt->window, &x, &y );
		gtk_widget_set_uposition( wgt, x, GTK_VALUE_INT(parg[0]) );
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::width" ) )
	{
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
			h = wgt->allocation.height;
		else
			gdk_window_get_size( wgt->window, &w, &h );
		if ( wgtdef->pfct_get_widget_type() == GTK_TYPE_WINDOW )
		{
			gdk_window_get_size( wgt->window, &w, &h );
			gdk_window_resize( wgt->window, GTK_VALUE_INT(parg[0]), h );
		}
		else
		{
			if ( GTK_WIDGET_NO_WINDOW(wgt) )
				gtk_widget_set_usize( wgt, GTK_VALUE_INT(parg[0]), -2 );
			else
				gdk_window_resize( wgt->window, GTK_VALUE_INT(parg[0]), h );
		}
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::height" ) )
	{
		if ( GTK_WIDGET_NO_WINDOW(wgt) )
			w = wgt->allocation.width;
		else
			gdk_window_get_size( wgt->window, &w, &h );
		if ( wgtdef->pfct_get_widget_type() == GTK_TYPE_WINDOW )
		{
			gdk_window_get_size( wgt->window, &w, &h );
			gdk_window_resize( wgt->window, w, GTK_VALUE_INT(parg[0]) );
		}
		else
		{
			if ( GTK_WIDGET_NO_WINDOW(wgt) )
				gtk_widget_set_usize( wgt, -2, GTK_VALUE_INT(parg[0]) );
			else
				gdk_window_resize( wgt->window, w, GTK_VALUE_INT(parg[0]) );
		}
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::visible" ) )
	{
		return 1;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::tip_text" ) )
	{
		return 1;
	}
	else assert(0);

	return -1;
}								// putdlg_widget


PUBLIC int
writecode_widget( defclass_t *defclass, GtkWidget *wgt, GtkArg **parg, char **str )
{
	module_info_t	*module;
	gtk_wgtdef_t	*wgtdef=NULL;

	module = get_object_data( wgt, WGTDATA_MODULE );
	assert( module != NULL );

	wgtdef = gtk_object_get_data( GTK_OBJECT(wgt), WGTDATA_WGTDEF );
	assert( wgtdef != NULL );

	memset( *parg, 0, sizeof(GtkArg) );
	(*parg)->type = defclass->restype;
	if ( !strcmp(defclass->resname, "~GtkWidget::x") ||
		 !strcmp(defclass->resname, "~GtkWidget::y") ||
		 !strcmp(defclass->resname, "~GtkWidget::width") ||
		 !strcmp(defclass->resname, "~GtkWidget::height") )
	{
		if ( wgtdef->pfct_get_widget_type() == GTK_TYPE_WINDOW )
		{
			if ( !strcmp(defclass->resname, "~GtkWidget::x") ||
				 !strcmp(defclass->resname, "~GtkWidget::y") )
			{
				if ( (module->type == MODULE_TYPE_WINDOW) && !module->u1.window.f_fixed_position )
				{
					*parg = NULL;
					return 0;
				}
				if ( (module->type == MODULE_TYPE_DIALOG) && !module->u1.dialog.f_fixed_position )
				{
					*parg = NULL;
					return 0;
				}
			}
		}						// if
		(*defclass->wgtdef_depth->pfct_getdlg)( defclass, wgt, *parg );
		return 0;
	} 
	else if ( !strcmp( defclass->resname, "~GtkWidget::visible" ) )
	{
		return 0;
	}
	else if ( !strcmp( defclass->resname, "~GtkWidget::tip_text" ) )
	{
		GtkArg	arg, *xarg;
		if ( (*defclass->wgtdef_depth->pfct_getdlg)( defclass, wgt, &arg ) == NULL )
			xarg = get_outsideres_wgt( wgt, defclass->resname );
		else
			xarg = &arg;
		str[ 0 ] = malloc( 200 );
		str[ 1 ] = malloc( 200 + strlen( GTK_VALUE_STRING( xarg[0] ) ) );
		str[ 2 ] = malloc( 200 + strlen( GTK_VALUE_STRING( xarg[0] ) ) );
		assert(str[0] != NULL);
		sprintf( str[ 0 ], "gtk_tooltips_set_tip( GTK_TOOLTIPS( _wgt_tooltips ), %s,",
			get_name_wgt(wgt) );
		sprintf( str[ 1 ], "\t\"%s\",",
			GTK_VALUE_STRING( xarg[0] ) );
		sprintf( str[ 2 ], "\t\"%s\" );",
			GTK_VALUE_STRING( xarg[0] ) );
		*parg = NULL;
		return 3;
	}
	else if ( !strcmp( defclass->resname, "GtkWidget::events" ) )
	{
		GtkArg	arg, *xarg;
		if ( (*defclass->wgtdef_depth->pfct_getdlg)( defclass, wgt, &arg ) == NULL )
			xarg = get_outsideres_wgt( wgt, defclass->resname );
		else
			xarg = &arg;
		str[ 0 ] = malloc( 200 );
		sprintf( str[ 0 ], "gtk_widget_set_events( %s, 0x%04X );",
			get_name_wgt(wgt), GTK_VALUE_FLAGS( xarg[0] ) );
		*parg = NULL;
		return 1;
	}
	else assert(0);
	return 0;
}								// writecode_widget


PUBLIC int
writecode_after_widget( defclass_t *defclass, GtkWidget *wgt, GtkArg **parg, char **str )
{
	return 0;
}								// writecode_after_widget


PUBLIC void
afteraddwgt_widget( GtkWidget *wgt )
{
}								// afteraddwgt_widget


PUBLIC GtkWidget *
addwgt_widget( struct gtk_wgtdef_t *wgtdef_newwgt, module_info_t *module,
    GtkWidget *wgt_parent, GtkWidget *wgt_container )
{
	assert(0);
	return NULL;				
}                               // addwgt_widget
