#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"

static	GtkWidget	*wgt_filemenu_newproject;
static	GtkWidget	*wgt_filemenu_loadproject;
static	GtkWidget	*wgt_filemenu_loadlastprojects;
static	GtkWidget	*wgt_filemenu_saveproject;
static	GtkWidget	*wgt_filemenu_saveprojectas;
static	GtkWidget	*wgt_filemenu_generate;
static	GtkWidget	*wgt_filemenu_compilerun;
static	GtkWidget	*wgt_filemenu_projectoptions;
static	GtkWidget	*wgt_filemenu_quit;

#ifdef GLG_SUPPORT_UNDO
static	GtkWidget	*wgt_editmenu_undo;
static	GtkWidget	*wgt_editmenu_redo;
#endif
static	GtkWidget	*wgt_editmenu_editheader = NULL;
static	GtkWidget	*wgt_editmenu_editstartup;
static	GtkWidget	*wgt_editmenu_editstyles;

static	GtkWidget	*wgt_viewmenu_treewindow;
static	GtkWidget	*wgt_viewmenu_clipboardwindow;

static	GtkWidget	*wgt_modulemenu_newwindow;
static	GtkWidget	*wgt_modulemenu_newdialog;
static	GtkWidget	*wgt_modulemenu_newmenu;
#ifdef USE_GNOME
static	GtkWidget	*wgt_modulemenu_new_gnome_dialog;
static	GtkWidget	*wgt_modulemenu_new_gnome_about;
static	GtkWidget	*wgt_modulemenu_new_gnome_propertybox;
static	GtkWidget	*wgt_modulemenu_new_gnome_messagebox;
#endif

static	GtkWidget	*wgt_windowsmenu_arrange_left;
static	GtkWidget	*wgt_windowsmenu_arrange_right;

static	GtkWidget	*wgt_helpmenu_about;
#if defined(DEBUG) || defined(__QNXNTO__)
static	GtkWidget	*wgt_debugmenu_walk;
#endif


#ifdef GLG_SUPPORT_EDIT_COMPOSITE

typedef struct
{
	int				level;
	int				*cpt;
	module_info_t	*module;
}	CLEARTREE_DATA;

static void
forall_clearflag_tree( GtkWidget *widget, CLEARTREE_DATA *data )
{

	data->level++;
	(*data->cpt)++;

	wgt_clear_flag( widget, WGTFLAG_DONTDELETE );

	if ( GTK_IS_CONTAINER( widget ) )
	{
		CLEARTREE_DATA	new_data;
		
		new_data.level  = data->level;
		new_data.cpt	= data->cpt;
		new_data.module	= data->module;

		gtk_container_forall( GTK_CONTAINER( widget ), 
			(GtkCallback) forall_clearflag_tree, &new_data );
	}

	data->level--;

}								// forall_clearflag_tree

static void
clear_dontdelete_flag( module_info_t *module )
{
	CLEARTREE_DATA	data;
	int				cpt;

	cpt = 0;
	data.level   = -1;
	data.cpt     = &cpt;
	data.module	 = module;
	forall_clearflag_tree( module->wgt_window, &data );
	
}								// clear_dontdelete_flag

#endif


/*
	File
*/

static void
cb_activate_menubtn_file( GtkMenuItem *menuitem, gpointer user_data )
{
	update_main_menu();
}
								// cb_activate_menubtn_file

static void
cb_file_new_project(GtkWidget *w, gpointer data)
{
	clear_project( "New project" );	
}								// cb_file_new_project


static void
cb_file_load_project(GtkWidget * w, gpointer data)
{
	if ( clear_project( "Load project" ) == -1 )
		return;
	load_project();
}								// cb_file_load_project


static void
cb_file_load_last_projects(GtkWidget * w, gpointer data)
{
	int		index = (int)data;

	if ( clear_project( "Load last project" ) == -1 )
		return;

	assert( (index >= 0) && (index < MAXNB_LAST_PROJECTS) );
	assert( last_projects[ index ] );
	do_load_project( NULL, last_projects[ index ] );

	is_project_touched = FALSE;
		
}								// cb_file_load_last_projects


static void
cb_file_save_project(GtkWidget * w, gpointer data)
{
	save_project();
}								// cb_file_save_project


static void
cb_file_save_project_as(GtkWidget * w, gpointer data)
{
	save_project_as();
}								// cb_file_save_project_as


static void
cb_file_generate(GtkWidget * w, gpointer data)
{

	/*--- Not yet a project name ? ---*/
	if ( !strcmp( project_dirname, "" ) )
		cb_file_save_project_as( w, data );

	/*--- Already a project name ---*/
	generate( project_dirname, NULL );

}								// cb_file_generate


static void
cb_file_compile_run(GtkWidget * w, gpointer data)
{
	comprun_dlg();
}								// cb_file_compile_run


static void
cb_file_project_options(GtkWidget * w, gpointer data)
{

	/*--- Already a project name ? ---*/
	if ( !strcmp( project_dirname, "" ) )
		dialog_project_options( FALSE );
	else
		dialog_project_options( TRUE );

}								// cb_file_project_options


static void
cb_file_quit(GtkWidget * w, gpointer data)
{

	if ( give_a_chance_to_save() == 0 )
		gtk_main_quit();

}								// cb_file_quit


/*
	Edit
*/

void
cb_activate_menubtn_edit( GtkMenuItem *menuitem, gpointer user_data )
{
#ifdef GLG_SUPPORT_UNDO
	gtk_widget_set_sensitive( wgt_editmenu_undo, FALSE );
	gtk_widget_set_sensitive( wgt_editmenu_redo, FALSE );
#endif
	update_main_menu();
}
								// cb_activate_menubtn_edit

#ifdef GLG_SUPPORT_UNDO

static void
cb_edit_undo(GtkWidget * w, gpointer data)
{
	printf( "Undo...\n" );
}								// cb_edit_undo


static void
cb_edit_redo(GtkWidget * w, gpointer data)
{
	printf( "Redo...\n" );
}								// cb_edit_redo

#endif

static void
cb_edit_header(GtkWidget * w, gpointer data)
{
	edit_header();;
}								// cb_edit_header


static void
cb_edit_startup(GtkWidget * w, gpointer data)
{
	edit_startup();
}								// cb_edit_startup


static void
cb_view_window_tree(GtkWidget * w, gpointer data)
{
	printf( "View Tree Window...\n" );
}								// cb_view_window_tree


static void
cb_view_window_clipboard(GtkWidget * w, gpointer data)
{
	printf( "View Clipboard Window...\n" );
}								// cb_view_window_clipboard


static void
cb_module_new_window( GtkWidget * w, gpointer data )
{
	module_create_window( NULL, -1, -1, FALSE );
}								// cb_module_new_window


static void
cb_module_new_dialog( GtkWidget * w, gpointer data )
{
	module_create_dialog( NULL, -1, -1, FALSE,
		MODULE_TYPE_DIALOG, "GtkDialog" );
}								// cb_module_new_dialog


#ifdef USE_GNOME

static void
cb_windows_module_gnome_dialog( GtkWidget * w, gpointer data )
{
	module_create_dialog( NULL, -1, -1, FALSE,
		MODULE_TYPE_GNOMEDIALOG_DIALOG, "GnomeDialog" );
}								// cb_windows_module_gnome_dialog


static void
cb_windows_module_gnome_about( GtkWidget * w, gpointer data )
{
	module_create_dialog( NULL, -1, -1, FALSE,
		MODULE_TYPE_GNOMEDIALOG_ABOUT, "GnomeAbout" );
}								// cb_windows_module_gnome_about


static void
cb_windows_module_gnome_propertybox( GtkWidget * w, gpointer data )
{
	module_create_dialog( NULL, -1, -1, FALSE,
		MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX, "GnomePropertyBox" );
}								// cb_windows_module_gnome_propertybox


static void
cb_windows_module_gnome_messagebox( GtkWidget * w, gpointer data )
{
	module_create_dialog( NULL, -1, -1, FALSE,
		MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX, "GnomeMessageBox" );
}								// cb_windows_module_gnome_messagebox


#endif

static void
cb_module_new_menu( GtkWidget * w, gpointer data )
{
	module_create_menu( NULL, FALSE );
}								// cb_module_new_menu


static void
cb_module_edit_styles( GtkWidget widget, gpointer data )
{
	edit_styles( NULL );
}								// cb_module_edit_styles


static void
set_move( int width, int height, 
	GtkWidget *wgt,
	int hstart, int hsize,
	int vstart, int vsize )
{
#ifdef __QNXNTO__

#else
	gint		x, y, w, h;

	x = (hstart * width) / 100;
	w = (hsize * width) / 100;
	y = (vstart * height) / 100;
	h = (vsize * height ) / 100;
//	printf( " a) x=%d y=%d  w=%d h=%d \n", x, y, w, h );
	x += wm_x_offset;
	y += wm_y_offset;
	w -= wm_x_offset;
	h -= wm_y_offset;
	gtk_widget_set_uposition( wgt, x, y );
	gtk_widget_set_usize( wgt, -1, -1 );
	gtk_widget_set_usize( wgt, w, h );
	gtk_window_set_default_size(GTK_WINDOW( wgt ), w, h );

#endif

}								// set_move


PUBLIC void
cb_windows_arrange_right( GtkWidget *widget, gpointer data )
{
	int		w = gdk_screen_width();
	int		h = gdk_screen_height();
	
	set_move( w, h, wgt_wnd_resources,	67, 33,	  0, 35 );
	set_move( w, h, wgt_wnd_palette,	67, 33,	 35, 15 );
	set_move( w, h, wgt_wnd_tree,		67, 33,	 50, 50 );	

	/*--- Mark all modules to be saved and generated ---*/
	touch_all_modules();

}								// cb_windows_arrange_right


PUBLIC void
cb_windows_arrange_left( GtkWidget *widget, gpointer data )
{
	int		w = gdk_screen_width();
	int		h = gdk_screen_height();
	
	set_move( w, h, wgt_wnd_resources,	0, 33,	  0, 35 );
	set_move( w, h, wgt_wnd_palette,	0, 33,	 35, 15 );
	set_move( w, h, wgt_wnd_tree,		0, 33,	 50, 50 );	

	/*--- Mark all modules to be saved and generated ---*/
	touch_all_modules();

}								// cb_windows_arrange_left


static void
cb_activate_menubtn_help( GtkMenuItem *menuitem, gpointer user_data )
{
	update_main_menu();
}								// cb_activate_menubtn_help

static void
cb_help_about( GtkWidget * w, gpointer data )
{
	dlg_about();
}								// cb_help_about


#if defined(DEBUG) || defined(__QNXNTO__)

typedef struct
{
	int				level;
	int				*cpt;
	module_info_t	*module;
}	WALKTREE_DATA;

static void
forall_walk_tree( GtkWidget *widget, WALKTREE_DATA *data )
{
	gtk_wgtdef_t	*wgtdef;
	int				dont;
	int				*pflag;
	GtkArg			args[1];
	GtkWidget		*wgt_parent;


	data->level++;
	(*data->cpt)++;

	dont = FALSE;
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	if (!config.show_composite_childs)
#endif
	{
		if ( !IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget) )
			dont = TRUE;;
	}

	pflag = get_object_data( widget, WGTDATA_FLAGS );

	args[0].name = "parent";
	args[0].type = GTK_TYPE_CONTAINER;	
	gtk_widget_getv( GTK_WIDGET(widget), 1, args );
	wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );

	trace( "level=%-3dcpt=%-3d (%d) %d %d %d %p=%-18s x,y=%4d,%-4d parent=%p %s\n",
		data->level, *data->cpt, 
		(!IS_WGT_NOCOMPOSITE(widget) && !IS_WGT_DONTDELETE(widget)) ? 1 : 0,
		IS_WGT_NOCOMPOSITE(widget), IS_WGT_DONTDELETE(widget),
		(GTK_WIDGET_FLAGS( widget ) & GTK_COMPOSITE_CHILD) ? 1 : 0,
		widget, gtk_widget_get_name( widget ), 
		widget->allocation.x, widget->allocation.y,
		wgt_parent, gtk_widget_get_name( wgt_parent ) );

{
	char *s;
	if ( (s = get_object_data( widget, WGTDATA_HIERARCHY_CHILD )) )
	{
		char *t;
		t = get_object_data( widget, WGTDATA_HIERARCHY_CHILD_MASTER );
		assert( t != NULL );
		printf( "\t_compute_hierarchy_composite_childs( %s, %s );\n",
			get_name_wgt(GTK_WIDGET(t)), s );
	}
}

	if ( !dont )
	{
	 	wgtdef = gtk_object_get_data( GTK_OBJECT( widget ), WGTDATA_WGTDEF );
		assert ( wgtdef != NULL );
	}

	if ( GTK_IS_CONTAINER( widget ) )
	{
		WALKTREE_DATA	new_data;
		
		new_data.level  = data->level;
		new_data.cpt	= data->cpt;
		new_data.module	= data->module;

		gtk_container_forall( GTK_CONTAINER( widget ), 
			(GtkCallback) forall_walk_tree, &new_data );
	}

	data->level--;

}								// forall_walk_tree

PUBLIC void
cb_debug_walk( GtkWidget * w, gpointer _data )
{
	module_info_t	*module = (module_info_t *)_data;
	WALKTREE_DATA	data;
	int				cpt;

	if ( module == NULL )
		module = curr_module;

	if ( module )
	{
		cpt = 0;
		data.level   = -1;
		data.cpt     = &cpt;
		data.module	 = module;
		forall_walk_tree( curr_module->wgt_window, &data );
	}							// if
	
}								// cb_debug_walk

#endif

								
static void
create_menu_btn( GtkWidget *wgt_menubar, GtkWidget **wgt_menubtn, GtkWidget **wgt_menu,
	char *label,
	void (*pcb_activate)( GtkMenuItem *, gpointer ) )
{

	*wgt_menubtn = gtk_menu_item_new_with_label( label );
	gtk_widget_show( *wgt_menubtn );
	gtk_container_add( GTK_CONTAINER(wgt_menubar), *wgt_menubtn );
	if ( pcb_activate != NULL )
		gtk_signal_connect( GTK_OBJECT( *wgt_menubtn ), "activate",
			GTK_SIGNAL_FUNC( pcb_activate ), NULL);

	*wgt_menu = gtk_menu_new();
	gtk_menu_item_set_submenu( GTK_MENU_ITEM(*wgt_menubtn), *wgt_menu );

}								// create_menu_btn


PUBLIC GtkWidget *
create_toggle_item( GtkWidget *wgt_menu, char *label,
	int sensitive, int state,
	int acc_key, int acc_control, GtkSignalFunc cb, gpointer user_data )
{
	GtkWidget		*wgt_menu_item;
	GtkAccelGroup	*wgt_accel_group;
	
	wgt_menu_item = gtk_check_menu_item_new_with_label( label );
	gtk_widget_set_sensitive( wgt_menu_item, sensitive );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(wgt_menu_item), state );
	gtk_widget_show( wgt_menu_item );
	gtk_container_add( GTK_CONTAINER(wgt_menu), wgt_menu_item );
	gtk_signal_connect(GTK_OBJECT(wgt_menu_item), "toggled",
		GTK_SIGNAL_FUNC( cb ), user_data );
		
	if ( acc_key != 0 )
	{
		wgt_accel_group = gtk_accel_group_new();
		gtk_window_add_accel_group( GTK_WINDOW(wgt_wnd_tree), wgt_accel_group );
		gtk_widget_add_accelerator( wgt_menu_item, "toggled", wgt_accel_group,
			acc_key, acc_control, GTK_ACCEL_VISIBLE );
	}

	return wgt_menu_item;

}								// create_toggle_item


PUBLIC GtkWidget *
create_menu_sep( GtkWidget *menu )
{
	GtkWidget *sep;

	sep = gtk_menu_item_new();
	gtk_widget_show( sep );
	gtk_container_add( GTK_CONTAINER(menu), sep );
	return sep;

}								// create_menu_sep


PUBLIC GtkWidget *
create_menu_item( GtkWidget *wgt_menu, char *label,
	int sensitive,
	int acc_key, int acc_control, GtkSignalFunc cb, gpointer user_data )
{
	GtkWidget		*wgt_menu_item;
	GtkAccelGroup	*wgt_accel_group;

	wgt_menu_item = gtk_menu_item_new_with_label( label );
	gtk_widget_set_sensitive( wgt_menu_item, sensitive );
	gtk_widget_show( wgt_menu_item );
	gtk_container_add( GTK_CONTAINER(wgt_menu), wgt_menu_item );
	gtk_signal_connect(GTK_OBJECT(wgt_menu_item), "activate",
		GTK_SIGNAL_FUNC( cb ), user_data );
		
	if ( acc_key != 0 )
	{
		wgt_accel_group = gtk_accel_group_new();
		gtk_window_add_accel_group( GTK_WINDOW(wgt_wnd_tree), wgt_accel_group );
		gtk_widget_add_accelerator( wgt_menu_item, "activate", wgt_accel_group,
			acc_key, acc_control, GTK_ACCEL_VISIBLE );
	}

	return wgt_menu_item;

}								// create_menu_item


PUBLIC void
update_main_menu( void )
{
	char		tmp[ 200 ];
	GtkWidget	*child;

	/*--- File Menu ---*/
	gtk_widget_set_sensitive( wgt_filemenu_newproject, 
		(nb_modules() == 0) ? FALSE : TRUE );
	gtk_widget_set_sensitive( wgt_filemenu_saveproject, 
		((nb_modules() > 0) && (is_project_touched)) );
	gtk_widget_set_sensitive( wgt_filemenu_saveprojectas, 
		(nb_modules() == 0) ? FALSE : TRUE );
	gtk_widget_set_sensitive( wgt_filemenu_generate, project_dirname[0] );
	gtk_widget_set_sensitive( wgt_filemenu_compilerun, project_dirname[0] );

	/*--- Edit Menu ---*/
	sprintf( tmp, "Edit header %s", 
		!strcmp( config.header_to_include, "" ) ? config.header_to_include : "" );
	if ( wgt_editmenu_editheader == NULL )
	{
		GtkWidget	*menu_item;
		GtkWidget	*accel_label;

		menu_item = gtk_menu_item_new();
		accel_label = gtk_accel_label_new( tmp );
		gtk_accel_label_set_accel_widget( GTK_ACCEL_LABEL( wgt_editmenu_editheader ), 
			menu_item );
	}
	else
	{
		child = GTK_BIN( wgt_editmenu_editheader )->child;
		gtk_label_set_text( GTK_LABEL( child ), tmp );
	}
	if ( !strcmp( config.header_to_include, "" ) )
		gtk_widget_set_sensitive( wgt_editmenu_editheader, FALSE );
	else
		gtk_widget_set_sensitive( wgt_editmenu_editheader, TRUE );

	if ( !strcmp( project_dirname, "" ) )
		gtk_widget_set_sensitive( wgt_editmenu_editstartup, FALSE );
	else
		gtk_widget_set_sensitive( wgt_editmenu_editstartup, TRUE );

}								// update_main_menu


static void
create_main_menu( GtkWidget *vbox )
{
	GtkWidget	*wgt_menubar;
	GtkWidget	*wgt_menubtn_file, *wgt_menu_file;
	GtkWidget	*wgt_menubtn_loadlastprojects, *wgt_menu_loadlastprojects;
	GtkWidget	*wgt_menubtn_edit, *wgt_menu_edit;
	GtkWidget	*wgt_menubtn_view, *wgt_menu_view;
	GtkWidget	*wgt_menubtn_module, *wgt_menu_module;
#ifdef USE_GNOME
	GtkWidget	*wgt_menubtn_gnomedlg, *wgt_menu_gnomedlg;
#endif
	GtkWidget	*wgt_menubtn_windows, *wgt_menu_windows;
	GtkWidget	*wgt_menubtn_arrange, *wgt_menu_arrange;
	GtkWidget	*wgt_menubtn_help, *wgt_menu_help;
	char		tmp[ 200 ];
	int			n;	

	wgt_menubar = gtk_menu_bar_new();
	gtk_widget_show( wgt_menubar );
	gtk_box_pack_start( GTK_BOX(vbox), wgt_menubar, FALSE, TRUE, 0 );

	/*--- File ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_file, &wgt_menu_file, "File", 
		cb_activate_menubtn_file );
	wgt_filemenu_newproject = create_menu_item( wgt_menu_file, "New Project", 
		TRUE, GDK_N, GDK_CONTROL_MASK, cb_file_new_project, NULL );
	wgt_filemenu_loadproject = create_menu_item( wgt_menu_file, "Load Project", 
		TRUE, GDK_L, GDK_CONTROL_MASK, cb_file_load_project, NULL );
	create_menu_btn( wgt_menu_file, &wgt_menubtn_loadlastprojects, &wgt_menu_loadlastprojects, "Load Last Project",
		NULL );
	for ( n = 0; n < MAXNB_LAST_PROJECTS; n++ )
	{
		if ( last_projects[ n ] )
			wgt_filemenu_loadlastprojects = create_menu_item( wgt_menu_loadlastprojects, 
				last_projects[ n ], 
				TRUE, 0, 0, cb_file_load_last_projects, (void *)n );
	}
	wgt_filemenu_saveproject = create_menu_item( wgt_menu_file, "Save Project", 
		TRUE, GDK_S, GDK_CONTROL_MASK, cb_file_save_project, NULL );
	wgt_filemenu_saveprojectas = create_menu_item( wgt_menu_file, "Save Project As", 
		TRUE, 0, 0, cb_file_save_project_as, NULL );
	create_menu_sep( wgt_menu_file );
	wgt_filemenu_generate = create_menu_item( wgt_menu_file, "Generate", 
		TRUE, GDK_F5, GDK_SHIFT_MASK, cb_file_generate, NULL );
	wgt_filemenu_compilerun = create_menu_item( wgt_menu_file, "Compile/Run", 
		TRUE, GDK_F5, 0, cb_file_compile_run, NULL );
	create_menu_sep( wgt_menu_file );
	wgt_filemenu_projectoptions = create_menu_item( wgt_menu_file, "Project Options", 
		TRUE, GDK_O, GDK_CONTROL_MASK, cb_file_project_options, NULL );
	create_menu_sep( wgt_menu_file );
	wgt_filemenu_quit = create_menu_item( wgt_menu_file, "Quit", 
		TRUE, GDK_Q, GDK_CONTROL_MASK, cb_file_quit, NULL );

	/*--- Edit ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_edit, &wgt_menu_edit, "Edit",
		cb_activate_menubtn_edit );
#ifdef GLG_SUPPORT_UNDO
	wgt_editmenu_undo = create_menu_item( wgt_menu_edit, "Undo", 
		TRUE, GDK_Z, GDK_CONTROL_MASK, cb_edit_undo, NULL );
	wgt_editmenu_redo = create_menu_item( wgt_menu_edit, "Redo", 
		TRUE, GDK_Z, GDK_CONTROL_MASK | GDK_SHIFT_MASK, cb_edit_redo, NULL );
	create_menu_sep( wgt_menu_edit );
#endif
	sprintf( tmp, "Edit header %s", 
		strcmp( config.header_to_include, "" ) ? config.header_to_include : "" );
	wgt_editmenu_editheader = create_menu_item( wgt_menu_edit, tmp, 
		strcmp( config.header_to_include, "" ) ? TRUE : FALSE,
		0, 0, cb_edit_header, NULL );
	wgt_editmenu_editstartup = create_menu_item( wgt_menu_edit, "Edit Startup function", 
		strcmp( project_dirname, "" ) ? TRUE : FALSE, 
		0, 0, cb_edit_startup, NULL );
	create_menu_sep( wgt_menu_edit );
	wgt_editmenu_editstyles = create_menu_item( wgt_menu_edit, "Edit Styles", 
		TRUE, GDK_S, GDK_SHIFT_MASK | GDK_CONTROL_MASK, cb_module_edit_styles, NULL );

	/*--- View ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_view, &wgt_menu_view, "View",
		NULL );
	wgt_viewmenu_treewindow = create_menu_item( wgt_menu_view, "Tree Window", 
		TRUE, 0, 0, cb_view_window_tree, NULL );
	wgt_viewmenu_clipboardwindow = create_menu_item( wgt_menu_view, "Clipboard Window", 
		TRUE, 0, 0, cb_view_window_clipboard, NULL );

	/*--- Module ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_module, &wgt_menu_module, "Module",
		NULL );
	wgt_modulemenu_newwindow = create_menu_item( wgt_menu_module, "New Window", 
		TRUE, GDK_W, GDK_SHIFT_MASK | GDK_CONTROL_MASK, cb_module_new_window, NULL );
	wgt_modulemenu_newdialog = create_menu_item( wgt_menu_module, "New Dialog", 
		TRUE, GDK_D, GDK_SHIFT_MASK | GDK_CONTROL_MASK, cb_module_new_dialog, NULL );
	create_menu_sep( wgt_menu_module );
	wgt_modulemenu_newmenu = create_menu_item( wgt_menu_module, "New Menu", 
		TRUE, GDK_M, GDK_SHIFT_MASK | GDK_CONTROL_MASK, cb_module_new_menu, NULL );
#ifdef USE_GNOME
	create_menu_sep( wgt_menu_module );
	create_menu_btn( wgt_menu_module, &wgt_menubtn_gnomedlg, &wgt_menu_gnomedlg, "New Gnome Dialogs",
		NULL );
	wgt_modulemenu_new_gnome_dialog = create_menu_item( wgt_menu_gnomedlg, "Dialog", 
		TRUE, 0, 0, cb_windows_module_gnome_dialog, NULL );
	wgt_modulemenu_new_gnome_about = create_menu_item( wgt_menu_gnomedlg, "About", 
		TRUE, 0, 0, cb_windows_module_gnome_about, NULL );
	wgt_modulemenu_new_gnome_propertybox = create_menu_item( wgt_menu_gnomedlg, "Property Box", 
		TRUE, 0, 0, cb_windows_module_gnome_propertybox, NULL );
	wgt_modulemenu_new_gnome_messagebox = create_menu_item( wgt_menu_gnomedlg, "Message Box", 
		TRUE, 0, 0, cb_windows_module_gnome_messagebox, NULL );
#endif
	
	/*--- Windows ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_windows, &wgt_menu_windows, "Windows",
		NULL );
	create_menu_btn( wgt_menu_windows, &wgt_menubtn_arrange, &wgt_menu_arrange, "Arrange",
		NULL );
	wgt_windowsmenu_arrange_left = create_menu_item( wgt_menu_arrange, "Left", 
		TRUE, GDK_L, GDK_CONTROL_MASK | GDK_SHIFT_MASK, cb_windows_arrange_left, NULL );
	wgt_windowsmenu_arrange_right = create_menu_item( wgt_menu_arrange, "Right", 
		TRUE, GDK_R, GDK_CONTROL_MASK | GDK_SHIFT_MASK, cb_windows_arrange_right, NULL );
		
	/*--- Help ---*/
	create_menu_btn( wgt_menubar, &wgt_menubtn_help, &wgt_menu_help, "Help",
		cb_activate_menubtn_help );
	wgt_helpmenu_about = create_menu_item( wgt_menu_help, "About", 
		TRUE, 0, 0, cb_help_about, NULL );

#if defined(DEBUG) || defined(__QNXNTO__)
	create_menu_sep( wgt_menu_help );
	wgt_debugmenu_walk = create_menu_item( wgt_menu_help, "Walk tree", 
		TRUE, 0, 0, cb_debug_walk, NULL );
#endif

}								// create_main_menu


#ifdef GLG_SUPPORT_EDIT_COMPOSITE
static void
cb_opt_editcompositechilds_toggled( GtkToggleButton *togglebutton, gpointer user_data )
{
	char			filename[PATH_MAX+NAME_MAX + 100];
	module_info_t	*module;

	/*--- Update value ---*/
	if ( gtk_toggle_button_get_active( togglebutton ) )
		config.show_composite_childs = TRUE;
	else
		config.show_composite_childs = FALSE;

	/*--- If there is current project, save the config for this project ---*/
	if ( !strcmp( project_dirname, "" ) )
	{
		write_config( config_name, FALSE );
	}
	else
	{
		sprintf( filename, "%s/lo_gafet.cfg",
			project_dirname );
		write_config( filename, TRUE );
		touch_all_modules();
	}

	/*--- Recompute and display the widget tree ---*/
	if ( nb_modules() > 0 )
	{
		module = next_module( FIRST );
		while ( module )
		{
			if ( (module->type == MODULE_TYPE_WINDOW) ||
				 (module->type == MODULE_TYPE_DIALOG)
#ifdef USE_GNOME
				 || (module->type == MODULE_TYPE_GNOMEDIALOG_DIALOG)
				 || (module->type == MODULE_TYPE_GNOMEDIALOG_ABOUT)
				 || (module->type == MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX)
				 || (module->type == MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX)
#endif
				)
			{
				recreate_tree( module, module->wgt_window );
				clear_dontdelete_flag( module );
			}					// if
			module = next_module( NEXT );
		}						// while
		update_tree();
	}							// if

}								// cb_opt_editcompositechilds_toggled
#endif


PUBLIC void
create_tree_window( void )
{
	GtkWidget	*wgt_vbox;
	GtkWidget	*wgt_notebook;
	GtkWidget	*wgt_scrolledwindow1;
	GtkWidget	*wgt_scrolledwindow2;
	GtkWidget	*wgt_scrolledwindow3;
	GtkWidget	*wgt_label1;
	GtkWidget	*wgt_label2;
	GtkWidget	*wgt_label3;
	GtkWidget	*wgt_hbox;
#if defined(DEBUG) || defined(GLG_SUPPORT_EDIT_COMPOSITE)
	GtkWidget	*wgt_hbox2=NULL;
#endif
	GtkTooltips *tooltips;

	/*--- Create the base window ---*/
	wgt_wnd_tree = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title( GTK_WINDOW(wgt_wnd_tree), 
		"Widgets/CB Tree - GTk Lo Gafet" );
	gtk_widget_set_usize( GTK_WIDGET(wgt_wnd_tree), -1, -1 );
	gtk_widget_set_usize( GTK_WIDGET(wgt_wnd_tree), 200, 150 );	// Minimal size
	gtk_window_set_default_size(GTK_WINDOW(wgt_wnd_tree),
		config.tree_wnd.w, config.tree_wnd.h );
	gtk_signal_connect( GTK_OBJECT(wgt_wnd_tree), "destroy",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );
	gtk_signal_connect(GTK_OBJECT(wgt_wnd_tree), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_app), NULL );
	tooltips = gtk_tooltips_new ();

	wgt_vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (wgt_wnd_tree), wgt_vbox);

	/*--- Create the main menu ---*/
	create_main_menu( wgt_vbox );

	wgt_notebook = gtk_notebook_new ();
	gtk_container_add (GTK_CONTAINER (wgt_vbox), wgt_notebook);

	/*--- 'Modules' tree ---*/

	wgt_scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (wgt_notebook), wgt_scrolledwindow1);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(wgt_scrolledwindow1), 
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	wgt_tree_modules = gtk_ctree_new (1, 0);
	gtk_container_add (GTK_CONTAINER (wgt_scrolledwindow1), wgt_tree_modules);
	gtk_clist_set_selection_mode( GTK_CLIST( wgt_tree_modules ), GTK_SELECTION_BROWSE );
	gtk_clist_set_column_width (GTK_CLIST (wgt_tree_modules), 0, 80);
	gtk_signal_connect_after( GTK_OBJECT( wgt_tree_modules ), "button_press_event",
		GTK_SIGNAL_FUNC( cb_project_module_button_press_event ),  wgt_tree_modules );

	wgt_label1 = gtk_label_new ("Modules");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), 0), wgt_label1);

	/*--- 'Widgets' tree ---*/

	wgt_hbox = gtk_vbox_new( FALSE, 0 );
	gtk_container_add (GTK_CONTAINER (wgt_notebook), wgt_hbox);

	wgt_scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_box_pack_start (GTK_BOX (wgt_hbox), wgt_scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(wgt_scrolledwindow1), 
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	wgt_tree_widgets = gtk_ctree_new (1, 0);
	gtk_container_add (GTK_CONTAINER (wgt_scrolledwindow1), wgt_tree_widgets);
	gtk_clist_set_selection_mode( GTK_CLIST( wgt_tree_widgets ), GTK_SELECTION_BROWSE );
	gtk_clist_set_column_width (GTK_CLIST (wgt_tree_widgets), 0, 80);

#ifdef GLG_SUPPORT_EDIT_COMPOSITE
	wgt_hbox2 = gtk_hbox_new( FALSE, 0 );
	gtk_box_pack_start (GTK_BOX (wgt_hbox), wgt_hbox2, FALSE, FALSE, 0);

	wgt_opt_editcompositechilds = gtk_check_button_new_with_label( "Show and Edit Composite Childs" );
	gtk_misc_set_alignment( GTK_MISC( wgt_opt_editcompositechilds ), 0, 0.5 );
	gtk_box_pack_start (GTK_BOX (wgt_hbox2), wgt_opt_editcompositechilds, FALSE, FALSE, 0);
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( wgt_opt_editcompositechilds ),
		config.show_composite_childs );
	gtk_signal_connect( GTK_OBJECT( wgt_opt_editcompositechilds ), "toggled",
		GTK_SIGNAL_FUNC( cb_opt_editcompositechilds_toggled ), NULL );
	gtk_tooltips_set_tip( tooltips, wgt_opt_editcompositechilds, 
		"This option is still *highly* experimental\nand is not recommended.",
		NULL );
#endif

#ifdef DEBUG
	if ( wgt_hbox2 == NULL )
	{
		wgt_hbox2 = gtk_hbox_new( FALSE, 0 );
		gtk_box_pack_start (GTK_BOX (wgt_hbox), wgt_hbox2, FALSE, FALSE, 0);
	}
	wgt_debug_infos_properties = gtk_label_new( "" );
	gtk_box_pack_start (GTK_BOX (wgt_hbox2), wgt_debug_infos_properties, TRUE, FALSE, 0);
#endif

	wgt_label1 = gtk_label_new ("Widget");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), 1), wgt_label1);

	/*--- 'Callbacks' ---*/

	wgt_scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (wgt_notebook), wgt_scrolledwindow2);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(wgt_scrolledwindow2), 
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	wgt_tree_callbacks = gtk_ctree_new (1, 0);
	gtk_container_add (GTK_CONTAINER (wgt_scrolledwindow2), wgt_tree_callbacks);
	gtk_clist_set_selection_mode( GTK_CLIST( wgt_tree_callbacks ), GTK_SELECTION_BROWSE );
	gtk_clist_set_column_width (GTK_CLIST (wgt_tree_callbacks), 0, 80);

	wgt_label2 = gtk_label_new ("Callbacks");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (wgt_notebook), 2), wgt_label2);

	/*--- 'Styles' ---*/

	wgt_scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (wgt_notebook), wgt_scrolledwindow3 );
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(wgt_scrolledwindow3 ), 
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	wgt_tree_styles = gtk_ctree_new (1, 0);
	gtk_container_add (GTK_CONTAINER (wgt_scrolledwindow3), wgt_tree_styles );
	gtk_clist_set_selection_mode( GTK_CLIST( wgt_tree_styles ), GTK_SELECTION_BROWSE );
	gtk_clist_set_column_width( GTK_CLIST (wgt_tree_styles), 0, 80 );

	wgt_label3 = gtk_label_new( "Styles" );
	gtk_notebook_set_tab_label( GTK_NOTEBOOK (wgt_notebook), 
		gtk_notebook_get_nth_page( GTK_NOTEBOOK(wgt_notebook), 3), wgt_label3 );


	wgt_statusbar = gtk_statusbar_new ();
	gtk_widget_ref (wgt_statusbar);
	gtk_box_pack_start (GTK_BOX (wgt_vbox), wgt_statusbar, FALSE, FALSE, 0);

}								// create_tree_window

