#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <inttypes.h>
#include <assert.h>

#include "glg.h"


typedef struct
{
	int		nb;
}	COUNTBNBWIDGETS_DATA;

static void
forall_count_nb_widgets( GtkWidget *widget, COUNTBNBWIDGETS_DATA *data )
{

	data->nb++;

	if ( GTK_IS_CONTAINER( widget ) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_count_nb_widgets, data );
	}

}								// forall_count_nb_widgets

PUBLIC int
count_nb_widgets( GtkWidget *wgt )
{
	COUNTBNBWIDGETS_DATA	data;

	data.nb = 0;
	gtk_container_forall( GTK_CONTAINER( wgt ), 
		(GtkCallback)forall_count_nb_widgets, &data );
	return data.nb;
}								// count_nb_widgets


PUBLIC GtkWidget *
get_parent( GtkWidget *wgt )
{
	GtkArg		args[1];
	GtkWidget	*wgt_parent;

	args[0].name = "GtkWidget::parent";
	args[0].type = GTK_TYPE_CONTAINER;	
	gtk_widget_getv( GTK_WIDGET( wgt ), 1, args );
	wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );
	return wgt_parent;
}								// get_parent


#ifdef DEBUG

PUBLIC double
now( void )
{
static	struct timeval	_tv;
static	struct timezone	_tz;
static	int				first = TRUE;
		struct timeval	tv;
		struct timezone	tz;

	if ( first )
	{
		gettimeofday( &_tv, &_tz );
		first = FALSE;
	}

	gettimeofday( &tv, &tz );
	return (((double)tv.tv_sec - (double)_tv.tv_sec) * (double)1000000) + ((double)tv.tv_usec);
}								// now

#endif


typedef struct
{
	GtkWidget		*initial;
	module_info_t	*module;
}	SETCOMPOSITE_DATA;

static void
forall_check_for_composite_childs( GtkWidget *widget, SETCOMPOSITE_DATA *data )
{

	if ( widget != data->initial )
	{
		int		*curr = get_object_data( widget, WGTDATA_FLAGS );
		if (curr == NULL)
		{
			curr = malloc(sizeof(int));
			*curr = 0;
			set_object_data( widget, WGTDATA_FLAGS, curr, TRUE );
		}
		if ( !IS_WGT_NOCOMPOSITE(widget) )
		{
#ifdef GLG_SUPPORT_EDIT_COMPOSITE
			if (config.show_composite_childs)
			{
				GtkArg			args[1];
				GtkWidget		*wgt_parent;
				gtk_wgtdef_t	*wgtdef;

				args[0].name = "GtkWidget::parent";
				args[0].type = GTK_TYPE_CONTAINER;	
				gtk_widget_getv( GTK_WIDGET(widget), 1, args );
				wgt_parent = GTK_WIDGET( GTK_VALUE_OBJECT(args[0]) );

				set_data_wgtdef( widget, wgt_parent, data->module, FALSE, TRUE, FALSE );
				wgtdef = get_object_data( GTK_WIDGET(widget), WGTDATA_WGTDEF );
				assert( widget != NULL );
				new_widget_inserted( widget, NULL, wgtdef );
				set_outsideres_function( widget, wgtdef );
			}
#endif
		}
	}
	if ( GTK_IS_CONTAINER(widget) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_check_for_composite_childs, data );
	}

}								// forall_check_for_composite_childs

PUBLIC void
check_for_composite_childs( GtkWidget *widget )
{
	
	if ( GTK_IS_CONTAINER(widget) )
	{
		SETCOMPOSITE_DATA	data;
		module_info_t		*module;

		module = get_object_data( widget, WGTDATA_MODULE );
		assert( module != NULL );
		data.initial = widget;
		data.module  = module;
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_check_for_composite_childs, &data );
	}
}								// check_for_composite_childs


typedef struct
{
	GtkWidget	*initial;
	int			cpt;
}	HIERCOMPOSITE_DATA;

static void
forall_compute_hierarchy_composite_childs( GtkWidget *widget, HIERCOMPOSITE_DATA *data )
{

	if ( IS_WGT_DONTDELETE(widget) )
	{

		if ( widget != data->initial )
		{
			if ( !IS_WGT_PLACEHOLDER(widget) )
			{
				char *p = malloc( 8 );
				sprintf( p, "%d", data->cpt++ );
				set_object_data( widget, WGTDATA_HIERARCHY_CHILD, p, TRUE );
				set_object_data( widget, WGTDATA_HIERARCHY_CHILD_MASTER, data->initial, FALSE );
			}
		}
		
	}							// if

	if ( GTK_IS_CONTAINER(widget) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_compute_hierarchy_composite_childs, data );
	}

}								// forall_compute_hierarchy_composite_childs

/*--- Recompute the hierarchy of all the composite widgets ---*/
PUBLIC void
compute_hierarchy_composite_childs( GtkWidget *wgt_main )
{

	if ( GTK_IS_CONTAINER(wgt_main) )
	{
		HIERCOMPOSITE_DATA	data;

		data.initial = wgt_main;
		data.cpt	 = 0;
		forall_compute_hierarchy_composite_childs( wgt_main, &data );
	}
	
}								// compute_hierarchy_composite_childs


PUBLIC void
destroy_notify( gpointer data )
{
	g_free( data );
}								// destroy_notify


PUBLIC void 
set_object_data( GtkWidget *wgt, char *key, void *ptr, int f_free )
{

	if ( f_free )
		gtk_object_set_data_full( GTK_OBJECT( wgt ), key, ptr, 
			(GtkDestroyNotify) destroy_notify );
	else
		gtk_object_set_data( GTK_OBJECT( wgt ), key, ptr );

}								// set_object_data


PUBLIC void *
get_object_data( GtkWidget *wgt, char *key )
{
	gpointer	ptr;
	
	if (wgt== NULL)
		return NULL;
	ptr = gtk_object_get_data( GTK_OBJECT(wgt), key );
	return ptr;
	
}								// get_object_data


PUBLIC void 
wgt_set_flag( GtkWidget *wgt, const int flag )
{
	int		*curr = get_object_data( wgt, WGTDATA_FLAGS );
	assert( curr != NULL );
	*curr = flag;
}								// wgt_set_flag


PUBLIC void 
wgt_put_flag( GtkWidget *wgt, const int flag )
{
	int		*curr = get_object_data( wgt, WGTDATA_FLAGS );
	assert( curr != NULL );
	*curr |= flag;
}								// wgt_put_flag


PUBLIC void 
wgt_clear_flag( GtkWidget *wgt, const int flag )
{
	int		*curr = get_object_data( wgt, WGTDATA_FLAGS );
	assert( curr != NULL );
	*curr &= ~flag;
}								// wgt_clear_flag


PUBLIC int 
wgt_get_flag( GtkWidget *wgt )
{
	int		*curr = get_object_data( wgt, WGTDATA_FLAGS );
	assert( curr != NULL );
	return *curr;
}								// wgt_get_flag


PUBLIC void 
set_name_wgt( GtkWidget *wgt, char *name )
{
	set_object_data( wgt, WGTDATA_NAME, g_strdup(name), TRUE );
}								// set_name_wgt

PUBLIC char * 
get_name_wgt( GtkWidget *wgt )
{
	char	*name;
	name = get_object_data( wgt, WGTDATA_NAME );
	assert( name != NULL );
	return name;
}								// get_name_wgt


PUBLIC void 
set_outsideres_wgt( GtkWidget *wgt, char *resname, GtkArg *arg )
{
	char	key[ 128 ];

	sprintf( key, "%s_%s", WGTDATA_OUTSIDERES, resname );
	set_object_data( wgt, key, arg, TRUE );
}								// set_outsideres_wgt

PUBLIC GtkArg * 
get_outsideres_wgt( GtkWidget *wgt, char *resname )
{
	char	key[ 128 ];
	
	sprintf( key, "%s_%s", WGTDATA_OUTSIDERES, resname );
	return get_object_data( wgt, key );
}								// get_outsideres_wgt


PUBLIC void 
set_nametype_wgt( GtkWidget *wgt, int type )
{
	set_object_data( wgt, WGTDATA_NAMETYPE, (char *)type, FALSE );
}								// set_nametype_wgt

PUBLIC int 
get_nametype_wgt( GtkWidget *wgt )
{
	char	*p = get_object_data( wgt, WGTDATA_NAMETYPE );
	if ( p == NULL )
		return NAMETYPE_ANONYMOUS;
	else
		return (int)p;
}								// get_nametype_wgt


PUBLIC void 
set_is_disabled_resource_wgt( GtkWidget *wgt, defclass_t *defclass, int is_disabled )
{
	char	*tag;
	int		*value;

	tag = malloc(strlen(defclass->resname) + 12 );
	sprintf( tag, "%s_disabled", defclass->resname );
	value = malloc(sizeof(int));
	*value = is_disabled;
	set_object_data( wgt, tag, value, TRUE );
	
}								// set_is_disabled_resource_wgt

PUBLIC int 
get_is_disabled_resource_wgt( GtkWidget *wgt, defclass_t *defclass )
{
	char	*tag;
	int		*value;

	tag = malloc(strlen(defclass->resname) + 12 );
	sprintf( tag, "%s_disabled", defclass->resname );
	value = get_object_data( wgt, tag );
	assert( value != NULL );
	free(tag);
	return (*value);
	
}								// get_is_disabled_resource_wgt


static void
cb_modaldlg_btn_ok( GtkWidget *widget, void *data )
{
	int		*done = (int *)data;
	*done = TRUE;
}								// cb_modaldlg_btn_ok


PUBLIC void
modal_error( char *fmt, ... ) 
{
	va_list		ap;
	char		buff[ 384 ];
    GtkWidget	*wgt_dialog, *wgt_label, *wgt_btn_ok;
	int			done;

	/*--- Develop message to display ---*/ 
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

    /* Create the widgets */  
    wgt_dialog = gtk_dialog_new();
	gtk_window_set_position( GTK_WINDOW(wgt_dialog), GTK_WIN_POS_CENTER );
    wgt_label  = gtk_label_new( buff );
    wgt_btn_ok = gtk_button_new_with_label( "Ok" );
    
    gtk_signal_connect( GTK_OBJECT(wgt_btn_ok), "clicked",
		GTK_SIGNAL_FUNC(cb_modaldlg_btn_ok), (gpointer) &done );

    /* Ensure that the dialog box is destroyed when the user clicks ok. */   
    gtk_signal_connect_object( GTK_OBJECT( wgt_btn_ok ), "clicked",
		GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) wgt_dialog );
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_dialog)->action_area),
		wgt_btn_ok );

    /* Add the label, and show everything we've added to the dialog. */
	INIT_ARGS;
	SETARG_ULONG( "border_width", 16 );
	SETARG_INT( "spacing", 8 );
	gtk_widget_setv( GTK_WIDGET(GTK_DIALOG(wgt_dialog)->vbox), nb_args, args );
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG( wgt_dialog )->vbox), wgt_label );
	gtk_window_set_modal( GTK_WINDOW( wgt_dialog ), TRUE );
	gtk_window_set_title( GTK_WINDOW( wgt_dialog ), "Error" );
    gtk_widget_show_all( wgt_dialog );

	/*--- Wait until user selects either 'yes' or 'no' ---*/
	done = FALSE;
	while ( !done )
		gtk_main_iteration();
		   	   
}								// modal_error


static GtkWidget	*wgt_yesno_dialog;

static void
cb_yesno_dialog_yes( GtkWidget *widget, void *data )
{
	int		*result = (int *)data;

	*result = TRUE;
	gtk_window_set_modal( GTK_WINDOW(wgt_yesno_dialog), FALSE );
	gtk_widget_destroy(wgt_yesno_dialog);

}								// cb_yesno_dialog_yes


static void
cb_yesno_dialog_no( GtkWidget *widget, void *data )
{
	int		*result = (int *)data;

	*result = FALSE;
	gtk_window_set_modal( GTK_WINDOW(wgt_yesno_dialog), FALSE );
	gtk_widget_destroy(wgt_yesno_dialog);

}								// cb_yesno_dialog_no


PUBLIC int
yesno_dialog( char *title, int default_val, char *fmt, ... )
{
	va_list		ap;
	char		buff[384];
    GtkWidget	*wgt_label, *wgt_btn_yes, *wgt_btn_no;
	int			result;		

	/*--- Develop message to display ---*/ 
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

    /*--- Create the widgets ---*/  
    wgt_yesno_dialog = gtk_dialog_new();
	gtk_window_set_position( GTK_WINDOW(wgt_yesno_dialog), GTK_WIN_POS_MOUSE );
    wgt_label = gtk_label_new( buff );
    wgt_btn_yes = gtk_button_new_with_label( "Yes" );
    wgt_btn_no = gtk_button_new_with_label( "No" );
 
    gtk_signal_connect( GTK_OBJECT( wgt_btn_yes ), "clicked",
		GTK_SIGNAL_FUNC( cb_yesno_dialog_yes ), (gpointer) &result );
    gtk_signal_connect( GTK_OBJECT( wgt_btn_no ), "clicked",
		GTK_SIGNAL_FUNC( cb_yesno_dialog_no ), (gpointer) &result );

    /*--- Ensure that the dialog box is destroyed when the user clicks ok. ---*/   
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_yesno_dialog)->action_area),
		wgt_btn_yes );
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_yesno_dialog)->action_area),
		wgt_btn_no );
	gtk_window_set_focus( GTK_WINDOW( wgt_yesno_dialog ), 
		GTK_WIDGET( (default_val == YES) ? wgt_btn_yes : wgt_btn_no ) );

    /*--- Add the label, and show everything we've added to the dialog. ---*/
	INIT_ARGS;
	SETARG_ULONG( "border_width", 16 );
	SETARG_INT( "spacing", 8 );
	gtk_widget_setv( GTK_WIDGET(GTK_DIALOG(wgt_yesno_dialog )->vbox), nb_args, args );
    gtk_container_add( GTK_CONTAINER( GTK_DIALOG( wgt_yesno_dialog )->vbox), wgt_label );
	gtk_window_set_title( GTK_WINDOW( wgt_yesno_dialog ), title );
	gtk_window_set_modal( GTK_WINDOW( wgt_yesno_dialog ), TRUE );
	gtk_widget_show_all( wgt_yesno_dialog );

	/*--- Wait until user selects either 'yes' or 'no' ---*/
	result = -2;
	while ( result == -2 )
		gtk_main_iteration();
		   	   
	return result;
	
}								// yesno_dialog


static	GtkWidget	*wgt_dialog3;
	
static void
cb_yesno_dialog_btn1( GtkWidget *widget, void *data )
{
	int		*result = (int *)data;

	*result = 1;
	gtk_window_set_modal( GTK_WINDOW(wgt_dialog3), FALSE );
	gtk_widget_destroy(wgt_dialog3);

}								// cb_yesno_dialog_btn1

static void
cb_yesno_dialog_btn2( GtkWidget *widget, void *data )
{
	int		*result = (int *)data;

	*result = 2;
	gtk_window_set_modal( GTK_WINDOW(wgt_dialog3), FALSE );
	gtk_widget_destroy(wgt_dialog3);

}								// cb_yesno_dialog_btn2

static void
cb_yesno_dialog_btn3( GtkWidget *widget, void *data )
{
	int		*result = (int *)data;

	*result = 3;
	gtk_window_set_modal( GTK_WINDOW(wgt_dialog3), FALSE );
	gtk_widget_destroy(wgt_dialog3);

}								// cb_yesno_dialog_btn3

PUBLIC int
dialog3( char *title, int default_val,
	char *btn1, char *btn2, char *btn3,
	char *fmt, ... )
{
	va_list		ap;
	char		buff[ 384 ];
    GtkWidget	*wgt_label, *wgt_btn1, *wgt_btn2, *wgt_btn3;
	int			result;		

	/*--- Develop message to display ---*/ 
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

    /*--- Create the widgets ---*/  
    wgt_dialog3 = gtk_dialog_new();
	gtk_window_set_position( GTK_WINDOW(wgt_dialog3), GTK_WIN_POS_MOUSE );
    wgt_label = gtk_label_new( buff );
    wgt_btn1 = gtk_button_new_with_label( btn1 );
    wgt_btn2 = gtk_button_new_with_label( btn2 );
    wgt_btn3 = gtk_button_new_with_label( btn3 );

    gtk_signal_connect( GTK_OBJECT( wgt_btn1 ), "clicked",
		GTK_SIGNAL_FUNC( cb_yesno_dialog_btn1 ), (gpointer) &result );
    gtk_signal_connect( GTK_OBJECT( wgt_btn2 ), "clicked",
		GTK_SIGNAL_FUNC( cb_yesno_dialog_btn2 ), (gpointer) &result );
    gtk_signal_connect( GTK_OBJECT( wgt_btn3 ), "clicked",
		GTK_SIGNAL_FUNC( cb_yesno_dialog_btn3 ), (gpointer) &result );

    /*--- Ensure that the dialog box is destroyed when the user clicks ok. ---*/   
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_dialog3)->action_area),
		wgt_btn1 );
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_dialog3)->action_area),
		wgt_btn2 );
    gtk_container_add( GTK_CONTAINER(GTK_DIALOG(wgt_dialog3)->action_area),
		wgt_btn3 );
	gtk_window_set_focus( GTK_WINDOW( wgt_dialog3 ), 
		GTK_WIDGET( (default_val == 1) ? wgt_btn1 : 
			((default_val == 2) ? wgt_btn2 : wgt_btn3)) );

    /*--- Add the label, and show everything we've added to the dialog. ---*/
	INIT_ARGS;
	SETARG_ULONG( "border_width", 16 );
	SETARG_INT( "spacing", 8 );
	gtk_widget_setv( GTK_WIDGET(GTK_DIALOG(wgt_dialog3)->vbox), nb_args, args );
    gtk_container_add( GTK_CONTAINER( GTK_DIALOG(wgt_dialog3)->vbox), wgt_label );
	gtk_window_set_title( GTK_WINDOW( wgt_dialog3 ), title );
	gtk_window_set_modal( GTK_WINDOW( wgt_dialog3 ), TRUE );
	gtk_widget_show_all( wgt_dialog3 );

	/*--- Wait until user selects either 'yes' or 'no' ---*/
	result = -2;
	while ( result == -2 )
		gtk_main_iteration();
		   	   
	return result;
	
}								// dialog3


static void
do_block_all_windows( const gboolean f_block )
{
	module_info_t	*module;
	static int		is_blocked = 0;

	if ( f_block && (is_blocked > 0) )
	{
		is_blocked++;
		return;
	}
	if ( !f_block && (is_blocked > 1) )
	{
		is_blocked--;
		return;
	}

	#define BLOCK( W, F ) \
	gtk_widget_set( GTK_WIDGET( W ), "sensitive",  \
		(f_block) ? FALSE : TRUE, \
		NULL )

	/*--- Working windows ---*/
	BLOCK( wgt_wnd_tree, f_block );
	BLOCK( wgt_wnd_resources, f_block );
	BLOCK( wgt_wnd_palette, f_block );

	/*--- Block all modules present ---*/
	module = next_module( FIRST );
	while ( module )
	{
		switch ( module->type )
		{
			case MODULE_TYPE_WINDOW :
			case MODULE_TYPE_DIALOG :
#ifdef USE_GNOME
			case MODULE_TYPE_GNOMEDIALOG_DIALOG :
			case MODULE_TYPE_GNOMEDIALOG_ABOUT :
			case MODULE_TYPE_GNOMEDIALOG_PROPERTYBOX :
			case MODULE_TYPE_GNOMEDIALOG_MESSAGEBOX :
#endif
				if ( module->wgt_window != NULL )
					BLOCK( module->wgt_window, f_block );
				break;
			case MODULE_TYPE_MENU :
				if ( module->u1.menu.menudlg.wgt_showmenu != NULL )
					BLOCK( module->u1.menu.menudlg.wgt_showmenu, f_block );
				if ( module->u1.menu.menued.wgt_menued_window != NULL )
					BLOCK( module->u1.menu.menued.wgt_menued_window, f_block );
				break;
			default :
				assert(0);
		}
		module = next_module( NEXT );
	}
	
	#undef BLOCK

	if ( f_block )
		is_blocked++;
	else
		is_blocked--;

}								// do_block_all_windows


PUBLIC void
push_block_all_windows( void )
{
	do_block_all_windows( TRUE );
}								// push_block_all_windows


PUBLIC void
pop_block_all_windows( void )
{
	do_block_all_windows( FALSE );
}								// pop_block_all_windows


PUBLIC gtk_wgtdef_t *
search_wgtdef( char *class )
{
	int				found;
	gtk_wgtdef_t	**wgtdef, *q;

	for ( found = FALSE, wgtdef = (gtk_wgtdef_t **)gtk_wgtdef; *wgtdef; wgtdef++ )
	{
		q = *wgtdef;
		if ( !strcmp( q->widget_name, class ) )
			return q;
	}

	return NULL;

}								// search_wgtdef


typedef struct

{
	GtkWidget	*initial;
	int			cpt;
	int			depth;
	GtkWidget	*result;
}						_HIERARCHYCOMPOSITE_DATA;

static void
_forall_compute_hierarchy_composite_childs( GtkWidget *widget, _HIERARCHYCOMPOSITE_DATA *data )
{

	if ( IS_WGT_DONTDELETE(widget) )
	{

		if ( !data->result && (widget != data->initial) )
		{
			if (data->cpt++ == data->depth)
			{
				data->result = widget;
				return;
			}
		}
	}							// if


	if ( GTK_IS_CONTAINER(widget) )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)_forall_compute_hierarchy_composite_childs, data );
	}
	
}								// _forall_compute_hierarchy_composite_childs

PUBLIC GtkWidget *
_compute_hierarchy_composite_childs( GtkWidget *wgt_main, int depth )
{
	_HIERARCHYCOMPOSITE_DATA	data;

	data.initial = wgt_main;
	data.cpt	 = 0;
	data.depth	 = depth;
	data.result  = NULL;
	_forall_compute_hierarchy_composite_childs( wgt_main, &data );
	return data.result;
	
}								// _compute_hierarchy_composite_childs


PUBLIC int
search_enum( int value, defclass_t *defclass )
{
	GtkEnumValue const	*penum;
	int					index;

	for ( index = 0, penum = defclass->enum_list; penum->value_name; penum++, index++ )
	{
		if ( penum->value == value )
			return index;
	}							// for
	
	assert( 0 );
	return -1;

}								// search_enum


/*

*/

typedef struct
{
	char	*display;
	guint	value;
}	keyval_t;
	
static keyval_t keyvals[] = {
	
	{ "A", GDK_A },
	{ "B", GDK_B },
	{ "C", GDK_C },
	{ "D", GDK_D },
	{ "E", GDK_E },
	{ "F", GDK_F },
	{ "G", GDK_G },
	{ "H", GDK_H },
	{ "I", GDK_I },
	{ "J", GDK_J },
	{ "K", GDK_K },
	{ "L", GDK_L },
	{ "M", GDK_M },
	{ "N", GDK_N },
	{ "O", GDK_O },
	{ "P", GDK_P },
	{ "Q", GDK_Q },
	{ "R", GDK_R },
	{ "S", GDK_S },
	{ "T", GDK_T },
	{ "U", GDK_U },
	{ "V", GDK_V },
	{ "W", GDK_W },
	{ "X", GDK_X },
	{ "Y", GDK_Y },
	{ "Z", GDK_Z },

#if 0
	{ "a", GDK_a },
	{ "b", GDK_b },
	{ "c", GDK_c },
	{ "d", GDK_d },
	{ "e", GDK_e },
	{ "f", GDK_f },
	{ "g", GDK_g },
	{ "h", GDK_h },
	{ "i", GDK_i },
	{ "j", GDK_j },
	{ "k", GDK_k },
	{ "l", GDK_l },
	{ "m", GDK_m },
	{ "n", GDK_n },
	{ "o", GDK_o },
	{ "p", GDK_p },
	{ "q", GDK_q },
	{ "r", GDK_r },
	{ "s", GDK_s },
	{ "t", GDK_t },
	{ "u", GDK_u },
	{ "v", GDK_v },
	{ "w", GDK_w },
	{ "x", GDK_x },
	{ "y", GDK_y },
	{ "z", GDK_z },
#endif

	{ "F1", GDK_F1 },
	{ "F2", GDK_F2 },
	{ "F3", GDK_F3 },
	{ "F4", GDK_F4 },
	{ "F5", GDK_F5 },
	{ "F6", GDK_F6 },
	{ "F7", GDK_F7 },
	{ "F8", GDK_F8 },
	{ "F9", GDK_F9 },
	{ "F10", GDK_F10 },
	{ "F11", GDK_F11 },
	{ "F12", GDK_F12 },
	
	{ NULL, 0 }

};



PUBLIC guint
key_to_val( char *key )
{
	int			found;
	keyval_t 	*p;

	for ( found = FALSE, p = keyvals; p->display; p++ )
	{
		if ( !strcmp( p->display, key ) )
		{
			found = TRUE;
			break;
		}
	}
	
	return (found) ? p->value : 0;

}								// key_to_val


PUBLIC char *
val_to_key( guint val )
{
	int			found;
	keyval_t 	*p;

	for ( found = FALSE, p = keyvals; p->display; p++ )
	{
		if ( p->value == val )
		{
			found = TRUE;
			break;
		}
	}
	
	return (found) ? p->display : NULL;

}								// val_to_key


PUBLIC GList *
keys_glist( void )
{
	GList		*items = NULL;
	keyval_t 	*p;

	for ( p = keyvals; p->display; p++ )
		items = g_list_append( items, p->display );

	return items;

}								// keys_glist


/*
*/


typedef struct
{
	int			found;
	char		*name;
	GtkWidget	*widget;
}	SEARCHWIDGETBYNAME_DATA;

static void
forall_search_widget_by_name( GtkWidget *widget, SEARCHWIDGETBYNAME_DATA *data )
{
	char	*name;

	name = get_name_wgt( widget );
	if ( !strcmp( name, data->name ) )
	{
		data->found = TRUE;
		data->widget = widget;
	}

	if ( GTK_IS_CONTAINER( widget ) && !data->found )
	{
		gtk_container_forall( GTK_CONTAINER(widget), 
			(GtkCallback)forall_search_widget_by_name, data );
	}

}								// forall_search_widget_by_name

PUBLIC GtkWidget *
search_widget_by_name( GtkWidget *start, char *name )
{
	SEARCHWIDGETBYNAME_DATA	data;

	data.found = FALSE;
	data.widget = NULL;
	data.name = strdup( name );
	gtk_container_forall( GTK_CONTAINER( start ), 
		(GtkCallback)forall_search_widget_by_name, &data );
	free( data.name );
	return (data.found) ? data.widget : NULL;
	
}								// search_widget_by_name
